<?php // $Revision: 1.50 $
/* vim: set expandtab ts=4 sw=4 sts=4: */

/**
 * $Id: library.php,v 1.50 2013/02/19 14:15:27 luiswang Exp $
 *
 * Copyright (c) 2018 by the webTareas developers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

// enable user-defined session save handler
$enableUserDefinedSaveHandler = true;
// system character set
$setCharset = 'utf-8';
// start the benchmark
$parse_start = getmicrotime();
// Session Settings
ini_set('session.use_trans_sid', 0); // Stop adding SID to URLs
ini_set('session.serialize_handler', 'php'); // How to store data
ini_set('session.use_cookies', 1); // Cookies store the session ID

// if the phpversion is >= 4.3.0 then set the 'use_only_cookies' to true
if (version_compare(phpversion(), '4.3.0', '>=')) {
    ini_set('session.use_only_cookies', 1);
}

// set the cache pages expire time in minutes
if(version_compare(phpversion(), '4.2.0', '>=')) {
    session_cache_expire(180);
}

// set more session settings
ini_set('session.name', 'webTareasSID'); // Name of the cookie
ini_set('session.gc_probability', 1); // Garbage Collection
ini_set('session.gc_maxlifetime', 3600); // max session time
ini_set('session.cookie_lifetime', 0); // cookie last browser session
ini_set('session.auto_start', 0); // don't auto start sessions

// how long a session can be inactive
define('SESS_MAXLIFE', 3600);
// {nocache|private|publuc|private_no_cache}
session_cache_limiter('nocache');

// if we should check IPADDR along with SESSID for enhanced security?
if ($enableUserDefinedSaveHandler) {
    define('SESS_IPCHECK', true);
}

// get the remote IP only if needed
define('SESS_REMOTE_ADDR', get_remote_addr());
// database link needs to be initialized
$SESS_DBH = null;
$path_prefix = '';
$name_extension = '';

// determine include path prefix
if ((!isset($indexRedirect)) || ($indexRedirect !== true)) {
    while (!file_exists($path_prefix . 'favicon.ico') && strlen($path_prefix) < 255) {
        $path_prefix .= '../';
    }

    if (preg_match('#.+/extensions/(\w+).*#', str_replace('\\', '/', getcwd()), $matches)) {
        $name_extension = $matches[1];
    }
}

// get configuration settings
require_once($path_prefix . 'includes/settings.php');
require_once($path_prefix . 'includes/defines.lib.php');
// request data class
require_once($path_prefix . 'includes/initrequests.php');
require_once($path_prefix . 'includes/request.class-dbspec.php');
require_once($path_prefix . 'includes/request.class.php');
// database-specific library
require_once($path_prefix . 'includes/library-dbspec.php');
// check ip address black list
$tmpquery = "SELECT ipaddr FROM " . $tablePrefix . "denyips WHERE ipaddr='" . SESS_REMOTE_ADDR . "'";

if (compt($tmpquery) > 0) {
    // need to display nice error
    header('Location: ' . $path_prefix . 'general/error.php?type=denyip');
    exit;
}

// Turn development mode on ?
if ($devMode == 'true') {
    $scriptMinimized = '';
} else {
    $scriptMinimized = '.min';
}

// Set the default timezone to GMT if GMT Timezone enabled
if ($gmtTimezone == 'true' && strcasecmp('GMT', ini_get('date.timezone'))) {
    date_default_timezone_set('GMT');
}

// user call-back functions for session events
// PHP doesn't allow these functions to work within an OO Class Structure, sorry.
if ($enableUserDefinedSaveHandler) {
    session_set_save_handler('_sess_open', '_sess_close', '_sess_read', '_sess_write', '_sess_destroy', '_sess_gc');
}

// PHP version check, force version greater than or equal to 4.1.0
// if you take this out don't even think about asking for help with bugs!!
$phpVersiondata = explode('.', phpversion());

if ($phpVersiondata[0] >= 4) {
    // only need to check subversion if this is php4
    if ($phpVersiondata[0] == 4 && $phpVersiondata[1] < 1) {
        // need to display nice error
        header('Location: ' . $path_prefix . 'general/error.php?type=phpversion');
        exit;
    }
} else {
    // need to display nice error
    header('Location: ' . $path_prefix . 'general/error.php?type=phpversion');
    exit;
}

// OS detection that webTareas is running on
if (strToUpper(substr(PHP_OS, 0, 3) == 'WIN')) {
    define('SYS_OS', 'WIN');
} else {
    define('SYS_OS', strToUpper(PHP_OS));
}

// path used for auth and session cookies
$cookiePath = parse_url($root);

// session cookie save path, more secure this way
ini_set('session.cookie_path', (isset($cookiePath['path'])?$cookiePath['path']:'').'/');

// Kick it, lets get this party started
session_start();

// register globals work around, I don't like this and am working
// to get rid if it
if (((!isset($extractImportVariables)) || ($extractImportVariables != 'false')) && (ini_get('register_globals') == false)) {
    //GET array
    if (!empty($_GET)) {
        extract($_GET);
    } else if (!empty($HTTP_GET_VARS)) {
        extract($HTTP_GET_VARS);
    }

    //POST array
    if (!empty($_POST)) {
        extract($_POST);
    } else if (!empty($HTTP_POST_VARS)) {
        extract($HTTP_POST_VARS);
    }
}

// extract import session variables
$browserSession = isset($_SESSION['browserSession'])?$_SESSION['browserSession']:'';
$idSession = isset($_SESSION['idSession'])?$_SESSION['idSession']:0;
$orgSession = isset($_SESSION['orgSession'])?$_SESSION['orgSession']:0;
$timezoneSession = isset($_SESSION['timezoneSession'])?$_SESSION['timezoneSession']:$defaultTimezone;
$dateformatSession = isset($_SESSION['dateformatSession'])?$_SESSION['dateformatSession']:'';
if ($dateformatSession == '') $dateformatSession = $dateFormat;
$timeformatSession = isset($_SESSION['timeformatSession'])?$_SESSION['timeformatSession']:'';
if ($timeformatSession == '') $timeformatSession = $timeFormat;
$languageSession = isset($_SESSION['languageSession'])?$_SESSION['languageSession']:'';
$loginSession = isset($_SESSION['loginSession'])?$_SESSION['loginSession']:'';
$nameSession = isset($_SESSION['nameSession'])?$_SESSION['nameSession']:'';
$ipSession = isset($_SESSION['ipSession'])?$_SESSION['ipSession']:'';
$dateunixSession = isset($_SESSION['dateunixSession'])?$_SESSION['dateunixSession']:0;
$dateSession = isset($_SESSION['dateSession'])?$_SESSION['dateSession']:'';
$profilSession = isset($_SESSION['profilSession'])?$_SESSION['profilSession']:0;
$logouttimeSession = isset($_SESSION['logouttimeSession'])?$_SESSION['logouttimeSession']:0;
$loginToken = isset($_SESSION['loginToken'])?$_SESSION['loginToken']:'';
$themeSession = isset($_SESSION['themeSession'])?$_SESSION['themeSession']:'';
$paletteSession = isset($_SESSION['paletteSession'])?$_SESSION['paletteSession']:'';
$genderSession = isset($_SESSION['genderSession'])?$_SESSION['genderSession']:'M';
$extSession = isset($_SESSION['extSession'])?$_SESSION['extSession']:'';
$projectSession = isset($_SESSION['projectSession'])?$_SESSION['projectSession']:'';
$expfmtSession = isset($_SESSION['expfmtSession'])?$_SESSION['expfmtSession']:'xls';
$wkcalSession = isset($_SESSION['wkcalSession'])?$_SESSION['wkcalSession']:'1';
$etcSession = isset($_SESSION['etcSession'])?$_SESSION['etcSession']:'0';
$etcgoSession = isset($_SESSION['etcgoSession'])?$_SESSION['etcgoSession']:'0';
$lastvisiteSession = isset($_SESSION['lastvisiteSession'])?$_SESSION['lastvisiteSession']:0;
// clear unnecessary session variables
$currenctBasename = basename($_SERVER['PHP_SELF'], ".php");

if (preg_match('/_serv$/', $currenctBasename) === 0) {  //  skip ajax server-side scripts
    if ((isset($_SESSION['TARGET'])) && ($_SESSION['TARGET'] != $currenctBasename) && (isset($_SESSION['PARAMS']))) {
        unset($_SESSION['PARAMS'], $_SESSION['TARGET']);
    }
}

$urlBornes = '';
// extract import borne variables, compose url borne string
for ($i00=1; $i00<=16; $i00++) {
    $b = 'borne' . $i00;
    
    if (isset($_GET[$b])) {
        ${$b} = $_GET[$b];
        $urlBornes .= '&' . $b . '=' . $_GET[$b];
    }
}

// extract sorting variables
$sor_cible = isset($_POST['sor_cible'])?$_POST['sor_cible']:'';
$sor_champs = isset($_POST['sor_champs'])?$_POST['sor_champs']:'';
$sor_ordre = isset($_POST['sor_ordre'])?$_POST['sor_ordre']:'';
// delete temp directory
function deleteTempDir($dir)
{
    $files = glob($dir . '/*');

    foreach ($files as $file) {
        if (is_file($file)) {
            @unlink($file);
        } else {
            deleteTempDir($file);
        }
    }

    rmdir($dir);
}
// delete temp files
$files = glob($path_prefix . 'files/tmp*');

foreach ($files as $file) {
    $mtime = filemtime($file);

    if ((time() - $mtime) > 3600) {
        if (is_file($file)) {
            @unlink($file);
        } else {
            deleteTempDir($file);
        }
    }
}

// theme
if ($themeSession == '') {
    define('THEME', $defaultTheme);
} else {
    define('THEME', $themeSession);
}

// menu palette
if ($paletteSession == '') {
    define('PALETTE', $defaultPalette);
} else {
    define('PALETTE', $paletteSession);
}

// layout class
require_once($path_prefix . 'includes/block.class.php');

// languages array
require_once($path_prefix . 'languages/languages.inc.php');
// default language selected
$langSel = $langDefault;

// language browser detection
if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
    $plng = preg_split('/,/', strtolower($_SERVER['HTTP_ACCEPT_LANGUAGE']));

    if (count($plng) > 0) {
        while (list($k0, $v) = each($plng)) {
            // does the language file exists and is it in the array?
            $k1 = preg_split('/;/', $v, 1);

            if (@file_exists('../languages/lang_' . $k1[0] . '.php') && array_key_exists($k1[0], $langValue)) {
                $langSel = $k1[0];
                break;
            }

            $k = preg_split('/-/', $k1[0]);

            if (@file_exists('../languages/lang_' . $k[0] . '.php') && array_key_exists($k[0], $langValue)) {
                $langSel = $k[0];
                break;
            }
        }
    }
}

// check session validity
if ($checkSession == true) {
    // a administrator trying to log in turned-off site, if not redirect to the login page
    if (($profilSession != '1') && ($siteOff == 'true')) {
        header('Location: ../index.php?');
        exit;
    }

    if (($orgSession != '1') && (!preg_match('/\/includes\/|\/extensions\/|\/service_site\//', $_SERVER['PHP_SELF']))) {
        // a client user trying to get outside of the "customer service site"
        header('Location: ../service_site/home.php?msg=permissiondenied');
        exit;
    } elseif (($orgSession == '1') && (preg_match('/\/service_site\//', $_SERVER['PHP_SELF']))) {
        // a non-client user trying to get into the "customer service site"
        header('Location: ../general/home.php?msg=permissiondenied');
        exit;
    }

    // if auto logout feature used, check the idle time
    if ($orgSession == '1') {
        if (($logouttimeSession != '0') && ($logouttimeSession != '')) {
            $dateunix = date('U');
            $diff = $dateunix - $dateunixSession;

            if ($diff > $logouttimeSession) {
                // the defined idle time as passed, log'em out
                header('Location: ../general/login.php?logout=true');
                exit;
            } else {
                // update the last activity time in the session
                $_SESSION['dateunixSession'] = $dateunix;
            }
        }
    }

    // verify they have logged in, if not redirect to the login page
    if ($loginToken != md5($loginSession . $cryptKey)) {
        header('Location: ../index.php?session=false');
        exit;
    }

    // query the logs table
    $querymore0 = " WHERE log.login='$loginSession'";
    $checkLog = new request();
    $checkLog->openLogs($querymore0);
    $comptCheckLog = count($checkLog->log_id);

    // make sure there is a row for them
    if ($comptCheckLog != '0') {
        if (session_id() != $checkLog->log_session[0]) {
            header('Location: ../index.php?session=false');
            exit;
        }
    } else {
        header('Location: ../index.php?session=false');
        exit;
    }
}

check_database_connection();

// set language session
if ((isset($_SESSION['languageSession'])) && ($_SESSION['languageSession'] !== '')) {
    $langSel = $_SESSION['languageSession'];
}

/**
 * language include
 */
$lang = '';     // force language text changing

if (selectLanguage($langSel)) {
    // currency matrix
    require($path_prefix . 'includes/currency.inc.php');
    // load array definitions
    require($path_prefix . 'includes/arrays.lib.php');
}

// Defines for graphtasks
if (!defined("GRAPH_TITLE_FONT") && defined("FF_ARIAL")) {
    DEFINE("GRAPH_TITLE_FONT", FF_ARIAL);
    DEFINE("GRAPH_TITLE_FONT_STYLE", FS_BOLD);
    DEFINE("GRAPH_WEEK_FONT", FF_ARIAL);
    DEFINE("GRAPH_ACTINFO_FONT", FF_ARIAL);
    DEFINE("GRAPH_ACTINFO_FONT_STYLE", FS_BOLD);
    DEFINE("ACTIVITY_TITLE_FONT", FF_ARIAL);
    DEFINE("ACTIVITY_CAPTION_FONT", FF_ARIAL);
}

// notifications include
if (!isset($indexRedirect)) {
    require_once($path_prefix . 'includes/notification.class.php');
}

// update sorting table if query sort column
if (!empty($sor_cible) && $sor_champs != 'none') {
    $querymore1 = " WHERE member='$idSession' AND sortid='$sor_cible'";
    $sortingDetail = new request();
    $sortingDetail->openSorting($querymore1);
    $newSortString = "$sor_champs $sor_ordre";

    if (count($sortingDetail->sor_member) == 0) {   // new sorting string
        $tmpsql = 'INSERT INTO ' . $tablePrefix . "sorting (member,sortid,sortstr) VALUES ('$idSession','$sor_cible','$newSortString')";
        connectSql($tmpsql);
    } elseif ($newSortString != $sortingDetail->sor_sortstr[0]) {   // sorting string changed
        $tmpsql = 'UPDATE ' . $tablePrefix . "sorting SET sortstr='$newSortString' WHERE member=$idSession AND sortid='$sor_cible'";
        connectSql($tmpsql);
    }
}

// set all sorting values for logged user
$querymore2 = " WHERE sor.member='$idSession'";
$listSorting = new request();
$listSorting->openSorting($querymore2);

for ($i=0; $i<count($listSorting->sor_member); $i++) {
    $sortingUser[$listSorting->sor_sortid[$i]] = $listSorting->sor_sortstr[$i];
}

// load system-wide permission matrix
if ($profilSession == '0') {
    $querymore3 = " WHERE oge.organization='$orgSession' AND ext.active='" . enumBoolean::bTrue . "' ORDER BY ext.name";
    $listOrganizationExtensions = new request();
    $listOrganizationExtensions->openOrganizationExtensions($querymore3);
    $comptListOrganizationExtensions = count($listOrganizationExtensions->oge_organization);

    for ($i=0; $i<$comptListOrganizationExtensions; $i++) {
        $func = '#' . $listOrganizationExtensions->oge_ext_name[$i];
        $SW_Permissions[$func]['Read']['0'] = ($listOrganizationExtensions->oge_perm_read[$i]== enumBoolean::bTrue);
        $SW_Permissions[$func]['Write']['0'] = ($listOrganizationExtensions->oge_perm_write[$i]== enumBoolean::bTrue);
        $SW_Permissions[$func]['Edit']['0'] = ($listOrganizationExtensions->oge_perm_edit[$i]== enumBoolean::bTrue);
        $SW_Permissions[$func]['Admin']['0'] = ($listOrganizationExtensions->oge_perm_admin[$i]== enumBoolean::bTrue);
    }
} else {
    $querymore3 = " WHERE upd.user_profile='$profilSession'";
    $request1 = new request();
    $request1->openUserProfilesDetail($querymore3);
}

// get unread message count
$tmpquery1 = "SELECT id FROM " . $tablePrefix . "messages WHERE to_id='$idSession' AND message_type<>'M' AND seen='N'";
$unread = compt($tmpquery1);
// copyright
$setCopyright = '<!-- Powered by webTareas v' . $version . ' -->';
// unset array
unset($__default_requireda, $__default_reqtypea, $__default_alphanuma, $__default_numerica, $__default_floata, $__default_timea, $__default_emaila, $__default_urla, $__default_pnuma, $__default_nnnuma, $__default_vernuma);
// check if javascript enabled
$js_disabled = (isset($_POST['js_disabled'])?$_POST['js_disabled']:'');
$js_enabled = (($js_disabled!='1')?'Y':'');

// ---------------------------------------------------------------------------#
// FUNCTIONS
// ---------------------------------------------------------------------------#
/**
 * webTareas authentication routine
 * @global string $tablePrefix
 * @global array $row
 * @global array $strings
 * @global string $error
 * @global string $cryptKey
 * @global string $root
 * @global string $defaultTimezone
 * @global string $dateFormat
 * @global string $timeFormat
 * @global string $langDefault
 * @global int $banThreshold
 * @global int $lockThreshold
 * @global string $detectHijack
 * @param string $login
 * @param string $password
 * @param string $tz
 * @param mixed $token
 * @return mixed
 * @throws Exception
 */
function wtAuthentication($login, $password, $tz, $token = null)
{
    global $tablePrefix, $row, $strings, $error, $cryptKey, $root, $defaultTimezone, $dateFormat, $timeFormat, $langDefault, $banThreshold, $lockThreshold, $detectHijack, $comptRegisterLog;
    $error = '';
    $match = false;
    $comptRegisterLog = '0';
    // query in members table
    $querymore = " WHERE mem.login='$login'";
    $loginUser = new request();
    $loginUser->openMembers($querymore);
    $comptLoginUser = count($loginUser->mem_id);

    // test if user exits
    if ($comptLoginUser == '0') {
        $error = $strings['invalid_login'];
    } else {
        // test account active
        if ($loginUser->mem_active[0] != enumBoolean::bTrue) {
            $error = $strings['account_lockout'];
        } else {
            // test password
            if ($token !== null) {
                if ($password != $loginUser->mem_password[0]) {
                    $error = $strings['invalid_login'];
                } else {
                    // password passed, now test token
                    if ($token != md5($login . $cryptKey)) {
                        $error = $strings['invalid_login'];
                    } else {
                        $match = true;
                    }
                }
            } else {
                if (!is_password_match($password, $loginUser)) {
                    $error = $strings['invalid_login'];
                } else {
                    $match = true;
                }
            }
        }

        if ($match === true) {
            // set session variables
            $_SESSION['browserSession'] = $_SERVER['HTTP_USER_AGENT'];
            $_SESSION['idSession'] = $loginUser->mem_id[0];             // login user id
            $_SESSION['orgSession'] = $loginUser->mem_organization[0];  // login user organization id
            $_SESSION['dateformatSession'] = (($loginUser->mem_date_format[0]=='')?$dateFormat:$loginUser->mem_date_format[0]);
            $_SESSION['timeformatSession'] = (($loginUser->mem_time_format[0]=='')?$timeFormat:$loginUser->mem_time_format[0]);
            $_SESSION['languageSession'] = (($loginUser->mem_language[0]=='')?$langDefault:$loginUser->mem_language[0]);
            $_SESSION['loginSession'] = $login;
            $_SESSION['nameSession'] = $loginUser->mem_name[0];         // login user name
            $_SESSION['ipSession'] = SESS_REMOTE_ADDR;
            $_SESSION['dateunixSession'] = date('U');
            $_SESSION['dateSession'] = date('d-m-Y H:i:s');
            $_SESSION['profilSession'] = $loginUser->mem_profil[0];     // login user profile
            $_SESSION['logouttimeSession'] = $loginUser->mem_logout_time[0];
            $_SESSION['loginToken'] = md5($login . $cryptKey);
            $_SESSION['themeSession'] = $loginUser->mem_theme[0];       // login user theme
            $_SESSION['paletteSession'] = $loginUser->mem_palette[0];   // login user menu palette
            $_SESSION['genderSession'] = $loginUser->mem_gender[0];         // login user gender
            $_SESSION['extSession'] = $loginUser->mem_extension_picture[0]; // user avatar picture file extension
            $_SESSION['expfmtSession'] = $loginUser->mem_export_format[0];  // export file format of login user
            $_SESSION['wkcalSession'] = $loginUser->mem_workcalendar[0];    // workcalendar id of login user
            $_SESSION['etcSession'] = $loginUser->mem_enable_timeclock[0];  // enable timeclock

            if ($_SESSION['etcSession'] == enumBoolean::bTrue) {
                $querymore = " WHERE tck.owner='" . $_SESSION['idSession'] . "' AND tck.finish='" . enumBoolean::bFalse . "'";
                $listTimeClock = new request();
                $listTimeClock->openTimeClock($querymore);
                $_SESSION['etcgoSession'] = ((count($listTimeClock->tck_id)>0)?enumBoolean::bTrue:enumBoolean::bFalse);
            }

            // timezone
            if ($tz == '') {
                $_SESSION['timezoneSession'] = (($loginUser->mem_timezone[0]=='')?$defaultTimezone:$loginUser->mem_timezone[0]);
            } else {
                $_SESSION['timezoneSession'] = $tz;
            }

            // insert into or update log
            $querymore = "WHERE log.login='$login'";
            $registerLog = new request();
            $registerLog->openLogs($querymore);
            $comptRegisterLog = count($registerLog->log_id);
            $session = session_id();
            $requestAct = new request();
            $requestAct->connectClass();     // connect to database server

            try {
                $requestAct->startTransaction();
                $tmpquery = "SELECT login FROM " . $tablePrefix . "loginips WHERE login='$login' AND ipaddr='" . SESS_REMOTE_ADDR . "' AND loginfail='Y'";
                $requestAct->query($tmpquery);

                if ($requestAct->rowCount() > 0) {
                    $tmpsql = "DELETE FROM " . $tablePrefix . "loginips WHERE login='$login' AND ipaddr='" . SESS_REMOTE_ADDR . "' AND loginfail='Y'";

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }
                }

                $tmpquery = "SELECT login FROM " . $tablePrefix . "loginips WHERE login='$login' AND ipaddr='" . SESS_REMOTE_ADDR . "' AND loginfail='N'";
                $requestAct->query($tmpquery);
                $chkHijack = ($detectHijack == 'on');
                $d = explode('.', SESS_REMOTE_ADDR);
                $ipclassb = $d[0] . (isset($d[1])?('.' . $d[1] . '.'):'');
                $approve_string = get_rand_id(32);
                $not_approve_string = get_rand_id(32);

                if ($requestAct->rowCount() > 0) {  // user login from this ip before
                    $tmpsql = "UPDATE " . $tablePrefix . "loginips SET compt=compt+1,last_visite='" . time() . "',approve_string='$approve_string',not_approve_string='$not_approve_string' WHERE login='$login' AND ipaddr='" . SESS_REMOTE_ADDR . "' AND loginfail='N'";
                } else {    // new ip for this user
                    // check if this is the first time for this user to login
                    $tmpquery = "SELECT login FROM " . $tablePrefix . "loginips WHERE login='$login'";
                    $requestAct->query($tmpquery);
                    $compt1 = $requestAct->rowCount();
                    $tmpquery = "SELECT login FROM " . $tablePrefix . "loginips WHERE login='$login' AND ipclassb='$ipclassb' AND loginfail='N'";
                    $requestAct->query($tmpquery);
                    $compt2 = $requestAct->rowCount();

                    if ($compt1 == 0 || $compt2 > 0) {
                        $tmpsql = "INSERT INTO " . $tablePrefix . "loginips (login,ipaddr,loginfail,ipclassb,compt,last_visite,approve_string,not_approve_string) VALUES ('$login','" . SESS_REMOTE_ADDR . "','N','$ipclassb','1','" . time() . "','','')";
                        $chkHijack = false;
                    } else {
                        $tmpsql = "INSERT INTO " . $tablePrefix . "loginips (login,ipaddr,loginfail,ipclassb,compt,last_visite,approve_string,not_approve_string) VALUES ('$login','" . SESS_REMOTE_ADDR . "','N','','1','" . time() . "','$approve_string','$not_approve_string')";
                    }

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }
                }

                // delete old visited webpages (older then 90 days)
                $day90ago = mktime(date('H'), date('i'), date('s'), date('n'), intval(date('j'))-90, date('Y'));
                $tmpsql = "DELETE FROM " . $tablePrefix . "bookmarks WHERE created<" . $day90ago . " AND cuser='" . $loginUser->mem_id[0] . "'";

                if ($requestAct->query($tmpsql) === false) {
                    throw new Exception($tmpsql);
                }

                // archive system notifications read week ago
                $weekago = mktime(date('H'), date('i'), date('s'), date('n'), intval(date('j'))-7, date('Y'));
                $tmpsql = "UPDATE " . $tablePrefix . "messages SET archived='Y' WHERE to_id='" . $loginUser->mem_id[0] . "' AND message_type='S' AND archived='N' AND seen='Y' AND created<=$weekago";

                if ($requestAct->query($tmpsql) === false) {
                    throw new Exception($tmpsql);
                }

                if ($chkHijack) {
                    $tmpquery = "SELECT ipaddr,last_visite,approve_string,not_approve_string FROM " . $tablePrefix . "loginips WHERE login='$login' AND ipclassb='' AND loginfail='N'";
                    $requestAct->query($tmpquery);

                    if ($requestAct->rowCount() > 0) {
                        // change timezone
                        $timezoneSession = $_SESSION['timezoneSession'];
                        // change language
                        if (selectLanguage($_SESSION['languageSession'])) {
                            // currency matrix
                            require('../includes/currency.inc.php');
                            // load array definitions
                            require('../includes/arrays.lib.php');
                        }
                        // send suspicious sign-in list to user
                        $mail = new notification();
                        $mail->getUserinfo('1', 'from');
                        $mail->getUserinfo($_SESSION['idSession'], 'to');
                        $mail->Subject = $strings['noti_suspicious_signin1'];
                        $mail->partMessage = $strings['noti_suspicious_signin2'];
                        $mail->Priority = '1';
                        $mail->openBody();
                        $mail->openContent();

                        while ($requestAct->fetch()) {
                            $ipaddr = ($row[0]);
                            $last_visite = ($row[1]);
                            $astr = ($row[2]);
                            $nstr = ($row[3]);
                            $mail->contentRow('IP:' . $ipaddr . ' ' . $strings["last_visit"] . ':' . createDate($last_visite, 'both', true), '<a href="' . $root . '/general/reporthijack.php?login=' . $login . '&ip=' . ip2long($ipaddr) . '&last=' . $last_visite . '&action=accept&auth=' . $astr . '">' . $strings['noti_suspicious_signin3'] . '</a>&nbsp;&nbsp;<a href="' . $root . '/general/reporthijack.php?login=' . $login . '&ip=' . ip2long($ipaddr) . '&last=' . $last_visite . '&action=reject&auth=' . $nstr . '">' . $strings['noti_suspicious_signin4'] . '</a>', "&nbsp;");
                        }

                        $mail->closeContent();
                        $mail->closeBody();
                        $mail->SendItOut(null, 'e');
                        $error = $strings['login_denied'];
                    }
                }

                if ($error == '') {
                    if ($comptRegisterLog == '0') {
                        $tmpsql = "INSERT INTO " . $tablePrefix . "logs (login,password,ip,session,compt,last_visite) VALUES ('$login','" . get_password($password) . "','" . SESS_REMOTE_ADDR . "','$session','1','" . time() . "')";
                    } else {
                        $_SESSION['lastvisiteSession'] = $registerLog->log_last_visite[0];
                        $increm = $registerLog->log_compt[0] + 1;
                        $tmpsql = "UPDATE " . $tablePrefix . "logs SET ip='" . SESS_REMOTE_ADDR . "',session='$session',compt='$increm',last_visite='" . time() . "' WHERE login='$login'";
                    }

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }

                    $tmpsql = "UPDATE " . $tablePrefix . "members SET invalid_logon='0',modified='" . time() . "',muser='1'";

                    if ($tz != '' && $tz != $loginUser->mem_timezone[0]) {
                        $tmpsql .= ",timezone='$tz'";
                    }

                    $tmpsql .= " WHERE id='" . $loginUser->mem_id[0]. "'";

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }
                }

                $requestAct->endTransaction(enumTransactionEndType::etCommit);
            } catch (Exception $e) {
                $error = 'EXCEPTION CAUGHT! LINE ' . $e->getLine() . ' IN ' . $e->getFile();
                error_log($e->getMessage());
                $requestAct->endTransaction(enumTransactionEndType::etRollBack);
                $match = false;
            }

            $requestAct->close();
        } else {
            // invalid logon attempt
            $requestAct = new request();
            $requestAct->connectClass();     // connect to database server

            try {
                $requestAct->startTransaction();
                $tmpquery = "SELECT compt FROM " . $tablePrefix . "loginips WHERE login='$login' AND ipaddr='" . SESS_REMOTE_ADDR . "' AND loginfail='Y'";
                $requestAct->query($tmpquery);

                if ($requestAct->rowCount() > 0) {
                    if ($requestAct->fetch()) {
                        $comptFail = intval($row[0]) + 1;
                    } else {
                        $comptFail = 1;
                    }


                    if ($banThreshold != '0' && $comptFail >= intval($banThreshold)) {
                        $tmpsql = "DELETE FROM " . $tablePrefix . "loginips WHERE login='$login' AND ipaddr='" . SESS_REMOTE_ADDR . "' AND loginfail='Y'";

                        if ($requestAct->query($tmpsql) === false) {
                            throw new Exception($tmpsql);
                        }

                        $tmpsql = "INSERT INTO " . $tablePrefix . "denyips (ipaddr,reason,created,cuser) VALUES ('" . SESS_REMOTE_ADDR . "','attack','" . time() . "','1')";

                        if ($requestAct->query($tmpsql) === false) {
                            throw new Exception($tmpsql);
                        }
                    } else {
                        $tmpsql = "UPDATE " . $tablePrefix . "loginips SET compt=compt+1,last_visite='" . time() . "' WHERE login='$login' AND ipaddr='" . SESS_REMOTE_ADDR . "' AND loginfail='Y'";

                        if ($requestAct->query($tmpsql) === false) {
                            throw new Exception($tmpsql);
                        }
                    }
                } else {
                    $tmpsql = "INSERT INTO " . $tablePrefix . "loginips (login,ipaddr,loginfail,ipclassb,compt,last_visite,approve_string,not_approve_string) VALUES ('$login','" . SESS_REMOTE_ADDR . "','Y','','1','" . time() . "','','')";

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }
                }

                $invalid_logon = intval($loginUser->mem_invalid_logon[0]) + 1;

                if ($lockThreshold != '0' && $invalid_logon >= intval($lockThreshold)) {
                    $tmpsql = "UPDATE " . $tablePrefix . "members SET active='" . enumBoolean::bFalse . "',invalid_logon='0',modified='" . time() . "',muser='1' WHERE id='" . $loginUser->mem_id[0]. "'";
                } else {
                    $tmpsql = "UPDATE " . $tablePrefix . "members SET invalid_logon=invalid_logon+1,modified='" . time() . "',muser='1' WHERE id='" . $loginUser->mem_id[0]. "'";
                }

                if ($requestAct->query($tmpsql) === false) {
                    throw new Exception($tmpsql);
                }

                $requestAct->endTransaction(enumTransactionEndType::etCommit);
            } catch (Exception $e) {
                $error = 'EXCEPTION CAUGHT! LINE ' . $e->getLine() . ' IN ' . $e->getFile();
                error_log($e->getMessage());
                $requestAct->endTransaction(enumTransactionEndType::etRollBack);
            }

            $requestAct->close();
        }
    }

    if ($match && $error == '') {
        return $loginUser;
    } else {
        return null;
    }
}
/**
 * change language text
 *
 * @global string $lang
 * @global string $path_prefix
 * @global string $langDefault
 * @global string $htmlEntityCharset
 * @global string $devMode Turn development mode on
 * @global string $name_extension
 * @param string $newlang
 * @return boolean
 */
function selectLanguage($newlang) {
    global $lang, $path_prefix, $langDefault, $htmlEntityCharset, $strings, $devMode, $name_extension;

    if ($devMode == 'true') {
        error_log('DEV-LOG:newlang=' . $newlang . ' lang=' . $lang . ' langDefault=' . $langDefault);
    }

    // check if new & current language is the same
    if ($newlang == $lang) {
        return false;       // language not changed
    } else {
        $lang = $newlang;   // change language to $newlang
    }

    // first load the default language file
    if (@file_exists($path_prefix . 'languages/lang_' . $langDefault . '.php')) {
        require($path_prefix . 'languages/lang_' . $langDefault . '.php');
    }
    // extension specified default language file
    if (($name_extension !== '') && (@file_exists($path_prefix . 'extensions/' . $name_extension . '/languages/lang_' . $langDefault . '.php'))) {
        require $path_prefix . 'extensions/' . $name_extension . '/languages/lang_' . $langDefault . '.php';
    }

    // then load the default language user-defined text
    $querymore1 = " WHERE str.language='$langDefault'";
    $listStringsD = new request();
    $listStringsD->openStrings($querymore1);
    $comptListStringsD = count($listStringsD->str_string_key);

    for ($i=0; $i<$comptListStringsD; $i++) {
        $strings[$listStringsD->str_string_key[$i]] = html_entity_decode($listStringsD->str_string_val[$i], ENT_QUOTES, $htmlEntityCharset);
    }

    // then load the language specific file if other than the default language
    if (($lang != $langDefault) && (@file_exists($path_prefix . 'languages/lang_' . $lang . '.php'))) {
        require($path_prefix . 'languages/lang_' . $lang . '.php');
    }
    // extension specified language file
    if (($name_extension !== '') && ($lang != $langDefault) && (@file_exists($path_prefix . 'extensions/' . $name_extension . '/languages/lang_' . $lang . '.php'))) {
        require $path_prefix . 'extensions/' . $name_extension . '/languages/lang_' . $lang . '.php';
    }

    // then load the language specific user-defined text if other than the default language
    if ($lang != $langDefault) {
        $querymore2 = " WHERE str.language='$lang'";
        $listStringsU = new request();
        $listStringsU->openStrings($querymore2);
        $comptListStringsU = count($listStringsU->str_string_key);

        for ($i=0; $i<$comptListStringsU; $i++) {
            $strings[$listStringsU->str_string_key[$i]] = html_entity_decode($listStringsU->str_string_val[$i], ENT_QUOTES, $htmlEntityCharset);
        }
    }

    // load language specific global constant definitions file
    if (@file_exists($path_prefix . 'languages/lang_' . $lang . '_global.php')) {
        require($path_prefix . 'languages/lang_' . $lang . '_global.php');
    }

    return true;
}

/**
 * Checks for webTareas Enhanced releases greater than the current
 * version installed
 *
 * @param string $iCV Version to compare
 * @access public
 */
function updatechecker($iCV) {
    global $strings;
    $checkMsg = '';
    // get latest available version number
    if (($v = @file('http://webtareas.sourceforge.net/version.txt')) === FALSE) {
        return '';
    }

    $iNV = trim(implode('', $v));

    // version comparisions
    // version string order: dev < alpha = a < beta = b < RC < pl
    if (version_compare($iNV, $iCV, '>')) {
        $checkMsg  = '<br><b>' . $strings['update_available'] . '</b> ';
        $checkMsg .= $strings['version_current'] . " $iCV. ";
        $checkMsg .= $strings['version_latest'] . " $iNV.<br>";
        $checkMsg .= '<a href="http://www.sourceforge.net/projects/webtareas" target="_blank">';
        $checkMsg .= $strings['sourceforge_link'] . '</a>.';
    }

    return($checkMsg);
}

/**
 * Calculate time to parse page (used with footer.php)
 *
 * @access public
 */
function getmicrotime() {
    list($usec, $sec) = explode(' ', microtime());
    return((float)$usec + (float)$sec);
}

/**
 * Automatic links
 *
 * @param string $data Text to parse
 * @access public
 */
function autoLinks($data) {
    global $newText;

    $lines = explode("\n", $data);

    while (list($key, $line0) = each($lines)) {
        $line1 = preg_replace_callback("/([ \t]|^)www\./i", function ($matches) { return " http://www."; }, $line0);
        $line2 = preg_replace_callback("/([ \t]|^)ftp\./i", function ($matches) { return " ftp://ftp."; }, $line1);
        $line3 = preg_replace_callback("|(http://[^ )\r\n]+)|i", function ($matches) { return "<a href=\"\\1\" target=\"_blank\">\\1</a>"; }, $line2);
        $line4 = preg_replace_callback("|(https://[^ )\r\n]+)|i", function ($matches) { return "<a href=\"\\1\" target=\"_blank\">\\1</a>"; }, $line3);
        $line5 = preg_replace_callback("|(ftp://[^ )\r\n]+)|i", function ($matches) { return "<a href=\"\\1\" target=\"_blank\">\\1</a>"; }, $line4);
        $line = preg_replace_callback("/([-a-z0-9_]+(\.[_a-z0-9-]+)*@([a-z0-9-]+(\.[a-z0-9-]+)+))/i", function ($matches) { return "<a href=\"mailto:\\1\">\\1</a>"; }, $line5);

        if (empty($newText)) {
            $newText = $line;
        } else {
            $newText .= "\n$line";
        }
    }
}

/**
 * Return number of day between 2 dates
 *
 * @param string $date1 Date(To) to compare
 * @param string $date2 Date(From) to compare
 * @access public
 */
function diff_date($date1, $date2) {
    $dateC = $date2;
    $diff = 0;

    while (date('Ymd',$dateC) <= date('Ymd',$date1)) {
        $dateC = mktime(0,0,0,intval(date('m',$dateC)),intval(date('d',$dateC))+1,intval(date('Y',$dateC)));
        $diff++;
    }

    return($diff);
}

/**
 * Return number of hour between 2 dates
 *
 * @param string $date1 Date(From) to compare
 * @param string $date2 Date(To) to compare
 * @param int $wc The unique id of workcalendar
 * @access public
 */
function diff_hour($date1, $date2, $wc = '1') {
    global $dayHourArray;
    $dateC = $date1;
    $dateE = mktime(0,0,0,intval(date('m',$dateC)),intval(date('d',$dateC)),intval(date('Y',$dateC))+10);
    $diff = 0;
    $querymore = " WHERE wcd.workcalendar='$wc' AND wcd.date>='$dateC' AND wcd.date<'$dateE' ORDER BY wcd.date";
    $listWorkCalendar = new request();
    $listWorkCalendar->openWorkCalendarsDetail($querymore);
    $comptListWorkCalendar = count($listWorkCalendar->wcd_id);

    while ($dateC <= $date2) {
        if ($comptListWorkCalendar == 0) {
            $weekDay = date("w", $dateC);
            $diff += $dayHourArray[$weekDay];
        } else {
            $idx = array_search($dateC, $listWorkCalendar->wcd_date);

            if ($idx === false) {
                $weekDay = date("w", $dateC);
                $diff += $dayHourArray[$weekDay];
            } else {
                $diff += $listWorkCalendar->wcd_hours[$idx];
            }
        }

        $dateC = mktime(0,0,0,intval(date('m',$dateC)),intval(date('d',$dateC))+1,intval(date('Y',$dateC)));
    }

    return($diff);
}

/**
 * Return date some hours after
 *
 * @param string $date1 Date to compare
 * @param string $hour1 Hours after
 * @param int $wc The unique id of workcalendar
 * @access public
 */
function hours_after($date1, $hour1, $wc = '1') {
    global $dayHourArray;
    $dateC = $date1;
    $dateE = mktime(0,0,0,intval(date('m',$date1)),intval(date('d',$date1)),intval(date('Y',$date1))+10);
    $diff = 0;
    $querymore = " WHERE wcd.workcalendar='$wc' AND wcd.date>='$dateC' AND wcd.date<'$dateE' ORDER BY wcd.date";
    $listWorkCalendar = new request();
    $listWorkCalendar->openWorkCalendarsDetail($querymore);
    $comptListWorkCalendar = count($listWorkCalendar->wcd_id);

    while ($diff < $hour1) {
        if ($comptListWorkCalendar == 0) {
            $weekDay = date("w", $dateC);
            $diff += $dayHourArray[$weekDay];
        } else {
            $idx = array_search($dateC, $listWorkCalendar->wcd_date);

            if ($idx === false) {
                $weekDay = date("w", $dateC);
                $diff += $dayHourArray[$weekDay];
            } else {
                $diff += $listWorkCalendar->wcd_hours[$idx];
            }
        }

        if ($diff >= $hour1) {
            break;
        }

        $dateC = mktime(0,0,0,intval(date('m',$dateC)),intval(date('d',$dateC))+1,intval(date('Y',$dateC)));
    }

    return $dateC;
}

/**
 * Return date some hours before
 *
 * @param string $date1 Date to compare
 * @param string $hour1 Hours before
 * @param int $wc The unique id of workcalendar
 * @access public
 */
function hours_before($date1, $hour1, $wc = '1') {
    global $dayHourArray;
    $dateC = $date1;
    list($an1, $mois1, $jour1) = preg_split('/-/', $dateC, 3);
    $dateS = date("Y-m-d", mktime(12,0,0,$mois1,$jour1,$an1-10));
    $diff = 0;
    $querymore = " WHERE wcd.workcalendar='$wc' AND wcd.date>'$dateS' AND wcd.date<='$dateC' ORDER BY wcd.date";
    $listWorkCalendar = new request();
    $listWorkCalendar->openWorkCalendarsDetail($querymore);
    $comptListWorkCalendar = count($listWorkCalendar->wcd_id);

    while ($diff < $hour1) {
        if ($comptListWorkCalendar == 0) {
            $weekDay = date("w", $dateC);
            $diff += $dayHourArray[$weekDay];
        } else {
            $idx = array_search($dateC, $listWorkCalendar->wcd_date);

            if ($idx === false) {
                $weekDay = date("w", $dateC);
                $diff += $dayHourArray[$weekDay];
            } else {
                $diff += $listWorkCalendar->wcd_hours[$idx];
            }
        }

        if ($diff >= $hour1) {
            break;
        }

        list($an1, $mois1, $jour1) = preg_split('/-/', $dateC, 3);
        $dateC = date("Y-m-d", mktime(12,0,0,$mois1,$jour1-1,$an1));
    }

    return $dateC;
}

/**
 * Checks for password match using the globally specified login method
 * @global string $loginMethod
 * @global string $useLDAP
 * @global array $configLDAP
 * @param string $formPassword User name password to test
 * @param string $loginUser User object
 * @return boolean
 */
function is_password_match($formPassword, $loginUser) {
    global $loginMethod, $useLDAP, $configLDAP;

    if ($useLDAP == 'true' && $loginUser->mem_id[0] != '1' && $loginUser->mem_organization[0] == '1') {
        if (($ds = ldap_connect($configLDAP['ldapserver'])) === false) {
            return false;
        }

        ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, $configLDAP['ldapversion']);
        $user_dn = $loginUser->mem_ldap_cn[0] . ',' . $configLDAP['basedn'];

        if (!$bind = ldap_bind($ds, $user_dn, $formPassword)) {
            return(false);
        } else {
            ldap_unbind($ds);
            return(true);
        }
    } else {
        switch ($loginMethod) {
            case 'MD5':
                if (md5($formPassword) == $loginUser->mem_password[0]) {
                    return(true);
                } else {
                    return(false);
                }
            case 'CRYPT':
                $salt = substr($loginUser->mem_password[0], 0, 2);
                if (crypt($formPassword, $salt) == $loginUser->mem_password[0]) {
                    return(true);
                } else {
                    return(false);
                }
            case 'PLAIN':
                if ($formPassword == $loginUser->mem_password[0]) {
                    return(true);
                } else {
                    return(false);
                }

                return(false);
        }
    }
}

/**
 * Return a password using the globally specified method
 *
 * @param string $newPassword Password to transfom
 * @access public
 */
function get_password($newPassword) {
    global $loginMethod;

    switch ($loginMethod) {
        case 'MD5':
            return(md5($newPassword));
        case 'CRYPT':
            $salt = substr($newPassword, 0, 2);
            return(crypt($newPassword, $salt));
        case 'PLAIN':
            return($newPassword);
    }
}

/**
 * Generate a random password
 *
 * @param string $size Size of geenrated password
 * @param boolean $with_numbers Option to use numbers
 * @param boolean $with_tiny_letters Option to use tiny letters
 * @param boolean $with_capital_letters Option to use capital letters
 * @access public
 */
function password_generator($size = 8, $with_numbers = true, $with_tiny_letters = true, $with_capital_letters = true) {
    global $pass_g;

    $pass_g = '';
    $sizeof_lchar = 0;
    $letter = '';
    $letter_tiny = 'abcdefghijklmnopqrstuvwxyz';
    $letter_capital = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $letter_number = '0123456789';

    if ($with_tiny_letters == true) {
        $sizeof_lchar += 26;

        if (isset($letter)) {
            $letter .= $letter_tiny;
        } else {
            $letter = $letter_tiny;
        }
    }

    if ($with_capital_letters == true) {
        $sizeof_lchar += 26;

        if (isset($letter)) {
            $letter .= $letter_capital;
        } else {
            $letter = $letter_capital;
        }
    }

    if ($with_numbers == true) {
        $sizeof_lchar += 10;

        if (isset($letter)) {
            $letter .= $letter_number;
        } else {
            $letter = $letter_number;
        }
    }

    if ($sizeof_lchar > 0) {
        srand((double)microtime() * date("YmdGis"));

        for ($cnt = 0; $cnt < $size; $cnt++) {
            $char_select = rand(0, $sizeof_lchar - 1);
            $pass_g .= $letter[$char_select];
        }
    }

    return($pass_g);
}

/**
 * Move a file in a new destination
 *
 * @param string $source Current path of file
 * @param string $dest New path of file
 * @access public
 */
function moveFile($source, $dest)
{
    global $mkdirMethod, $ftpRoot;

    if ($mkdirMethod == 'FTP') {
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        ftp_rename($ftp, $ftpRoot . '/' . $source, $ftpRoot . '/' . $dest);
        ftp_quit($ftp);
    } else {
        copy('../files/' . $source, '../files/' . $dest);
    }
}

/**
 * Move files in a new destination
 *
 * @param string $sources Current path of files
 * @param string $dest New path of file
 * @access public
 */
function moveFiles($sources, $dest)
{
    global $mkdirMethod, $ftpRoot;

    if ($dest == '') {
        $dest = '.';
    }

    if ($mkdirMethod == 'FTP') {
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);

        foreach ($sources as $source) {
            $filename = strrchr($source, '/');

            if ($filename === false) {
                ftp_rename($ftp, $ftpRoot . '/' . $source, $ftpRoot . '/' . $dest . '/' . $source);
            } else {
                ftp_rename($ftp, $ftpRoot . '/' . $source, $ftpRoot . '/' . $dest . $filename);
            }
        }

        ftp_quit($ftp);
    } else {
        foreach ($sources as $source) {
            $filename = strrchr($source, '/');

            if ($filename === false) {
                rename('../files/' . $source, '../files/' . $dest . '/' . $source);
            } else {
                rename('../files/' . $source, '../files/' . $dest . $filename);
            }
        }
    }
}

/**
 * Copy files in a new destination
 *
 * @param array $sources Current path of files
 * @param array $dests New path of files
 * @access public
 */
function copyFiles($sources, $dests)
{
    global $mkdirMethod, $ftpRoot;

    if ($mkdirMethod == 'FTP') {
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);

        for ($i=0; $i<count($sources); $i++) {
            $source = $sources[$i];
            $dest = $dests[$i];

            if (ftp_chdir($ftp, $source)) {
                ftp_mkdir($ftp, $ftpRoot . '/' . $dest);
            } else {
                $tmpfname = tempnam("../files", "FOO");
                ftp_get($ftp, $tmpfname, $ftpRoot . "/" . $source, FTP_BINARY);
                ftp_put($ftp, $dest, $tmpfname, FTP_BINARY);
                unlink($tmpfname);
            }
        }

        ftp_quit($ftp);
    } else {
        for ($i=0; $i<count($sources); $i++) {
            $source = $sources[$i];
            $dest = $dests[$i];

            if (is_dir($source)) {
                @mkdir('../files/' . $dest, 0755);
            } else {
                @copy('../files/' . $source, '../files/' . $dest);
            }
        }
    }
}

/**
 * Delete a file with a specified path
 *
 * @param string $source Path of file
 * @access public
 */
function deleteFile($source)
{
    global $mkdirMethod, $ftpRoot, $basedir;

    if ($mkdirMethod == 'FTP') {
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        ftp_delete($ftp, preg_replace_callback('#/+#', function ($matches) { return '/'; }, $ftpRoot . '/' . $source));
        ftp_quit($ftp);
    } else {
        unlink(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files/' . $source));
    }
}

/**
 * Delete attach files (transaction)
 * @global string $tablePrefix
 * @param object $requestAct
 * @param string $querymore
 * @throws Exception
 */
function deleteAttachFiles($requestAct, $querymore)
{
    global $tablePrefix;
    $listFiles = clone $requestAct;
    $listFiles->openFiles($querymore);

    if (count($listFiles->fil_id) > 0) {
        $ids = "'" . implode("','", $listFiles->fil_id) . "'";
        $querymore1 = " WHERE fil.id IN ($ids) OR fil.vc_parent IN ($ids)";
        $listFiles1 = clone $requestAct;
        $listFiles1->openFiles($querymore1);
        $ids1 = "'" . implode("','", $listFiles1->fil_id) . "'";
        deleteFiles($listFiles1->fil_fullname);
        copy2HistoryTable($requestAct, 'files', $ids1, 'D');
        $tmpsql = "DELETE FROM " . $tablePrefix . "files WHERE id IN ($ids1)";

        if ($requestAct->query($tmpsql) === false) {
            throw new Exception($tmpsql);
        }
    }
}
/**
 * Delete files
 *
 * @param string $sources Path of files
 * @access public
 */
function deleteFiles($sources)
{
    global $mkdirMethod, $ftpRoot, $basedir;

    if ($mkdirMethod == 'FTP') {
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);

        foreach ($sources as $source) {
            if (ftp_size($ftp, preg_replace_callback('#/+#', function ($matches) { return '/'; }, $ftpRoot . "/" . $source)) > 0) {
                ftp_delete($ftp, preg_replace_callback('#/+#', function ($matches) { return '/'; }, $ftpRoot . '/' . $source));
            }
        }

        ftp_quit($ftp);
    } else {
        foreach ($sources as $source) {
            if (file_exists(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files/' . $source))) {
                unlink(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files/' . $source));
            }
        }
    }
}

/**
 * Checks whether a file exists
 *
 * @param string $source Path of file
 * @return bool true on success or false on failure.
 * @access public
 */
function fileExists($source)
{
    global $mkdirMethod, $ftpRoot, $basedir;

    if ($mkdirMethod == 'FTP') {
        $path = preg_replace_callback('#/+#', function ($matches) { return '/'; }, $ftpRoot . "/" . $source);
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        $listFiles = ftp_nlist($ftp, $path);

        if ($listFiles !== FALSE && in_array($path, $listFiles)) {
            $rc = TRUE;
        } else {
            $rc = FALSE;
        }

        ftp_quit($ftp);
        return $rc;
    } else {
        return file_exists(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . "/files/" . $source));
    }
}

/**
 * Checks whether a folder exists
 *
 * @param string $source Path of file
 * @access public
 */
function dirExists($source)
{
    global $mkdirMethod, $ftpRoot, $basedir;

    if ($mkdirMethod == 'FTP') {
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        $rc = ftp_chdir($ftp, preg_replace_callback('#/+#', function ($matches) { return '/'; }, $ftpRoot . "/" . $source));
        ftp_quit($ftp);
        return $rc;
    } else {
        return file_exists(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . "/files/" . $source));
    }
}

/**
 * Folder creation
 *
 * @param string $path Path to the new directory
 * @access public
 */
function createDir($path)
{
    global $mkdirMethod, $ftpRoot;
    $pathNew = explode('/', $path);

    if ($mkdirMethod == 'FTP') {
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        ftp_chdir($ftp, $ftpRoot);

        foreach ($pathNew as $dir) {
            $create_dir .= $dir . '/';
            @ftp_mkdir($ftp, $create_dir);
        }

        ftp_quit($ftp);
    } else if ($mkdirMethod == 'PHP') {
        foreach ($pathNew as $dir) {
            $create_dir .= $dir . '/';
            @mkdir('../files/' . $create_dir, 0755);
        }
    }
}

/**
 * Folder recursive deletion
 *
 * @param string $location Path of directory to delete
 * @access public
 */
function delDir($location)
{
    global $mkdirMethod, $ftpRoot;

    if ($mkdirMethod == 'FTP') {
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        ftp_chdir($ftp, $ftpRoot);
        ftp_rmdirr($location, $ftp);
        ftp_quit($ftp);
    } else {
        $location = "../files/" . $location;

        if (is_dir($location)) {
            $all = opendir($location);

            while ($file = readdir($all)) {
                if (is_dir($location . '/' . $file) && $file != '..' && $file != '.') {
                    deldir($location . '/' . $file);

                    if (file_exists($location . '/' . $file)) {
                        rmdir($location . '/' . $file);
                    }

                    unset($file);
                } else if (!is_dir($location . '/' . $file)) {
                    if (file_exists($location . '/' . $file)) {
                        unlink("$location/$file");
                    }

                    unset($file);
                }
            }

            closedir($all);
            rmdir($location);
        } else {
            if (file_exists($location)) {
                unlink($location);
            }
        }
    }
}

/**
 * Return recursive folder size
 * @param int $id
 * @return int
 */
function folder_size($id)
{
    $querymore = " WHERE fil.id='$id'";
    $fileDetail = new request();
    $fileDetail->openFiles($querymore);
    $comptFileDetail = count($fileDetail->fil_id);

    if ($comptFileDetail == '0') {
        return 0;
    } else {
        if ($fileDetail->fil_dir[0] == enumBoolean::bTrue) {
            $querymore = " WHERE fil.parent='$id' AND ((fil.vc_parent='0' AND fil.dir='" . enumBoolean::bFalse . "') OR (fil.dir='" . enumBoolean::bTrue . "'))";
            $listFiles = new request();
            $listFiles->openFiles($querymore);
            $comptListFiles = count($listFiles->fil_id);
            $sz = 0;

            for ($i=0; $i<$comptListFiles; $i++) {
                $sz += folder_size($listFiles->fil_id[$i]);
            }

            return $sz;
        } else {
            return ((is_numeric($fileDetail->fil_size[0]))?intval($fileDetail->fil_size[0]):0);
        }
    }
}

/**
 * Return recursive folder usage
 *
 * @param string $location Path of directory to calculate
 * @param boolean $recursive Option to use recursivity
 * @access public
 */
function folder_info_size($path, $recursive = true, $root = true)
{
    global $mkdirMethod, $ftpRoot, $ftpHandler;
    $result = 0;

    if ($mkdirMethod == 'FTP') {
        if ($root) {
            $ftpHandler = ftp_connect(FTPSERVER);
            ftp_login($ftpHandler, FTPLOGIN, FTPPASSWORD);
        }

        $size = ftp_size($ftpHandler, $ftpRoot . '/' . $path);

        if ($size == -1) {
            $list = @ftp_nlist($ftpHandler, $ftpRoot . '/' . $path);

            if (!empty($list)) {
                foreach($list as $value) {
                    $size = ftp_size($ftpHandler, $value);

                    if ($size == -1) {
                        $result += $recursive ? folder_info_size($value, true, false) : 0;
                    } else {
                        $result += $size;
                    }
                }
            }
        }

        if ($root) {
            ftp_quit($ftpHandler);
        }
    } else {
        $path = "../files/" . $path . '/';

        if (is_dir($path) || is_readable($path)) {
            $dir = opendir($path);

            while ($file = readdir($dir)) {
                if ($file != '.' && $file != '..') {
                    if (is_dir($path . $file)) {
                        $result += $recursive ? folder_info_size($path . $file) : 0;
                    } else {
                        $result += filesize($path . $file);
                    }
                }
            }

            closedir($dir);
        }
    }

    return($result);
}

/**
 * Return size converted with units (in the user language)
 *
 * @param string $result Result to convert
 * @access public
 */
function convertSize($result)
{
    global $byteUnits;

    if ($result >= 1073741824) {
        $result = round($result / 1073741824 * 100) / 100 . ' ' . $byteUnits[3];
    } else if ($result >= 1048576) {
        $result = round($result / 1048576 * 100) / 100 . ' ' . $byteUnits[2];
    } else if ($result >= 1024) {
        $result = round($result / 1024 * 100) / 100 . ' ' . $byteUnits[1];
    } else {
        $result = $result . ' ' . $byteUnits[0];
    }

    if ($result == 0) {
        $result = '--';
    }

    return $result;
}

/**
 * convert input date to unix timestamp
 *
 * @param string $inputDate Date input
 * @param string $inputTime Time input (optional)
 * @param boolean $tzEnabled Date/Time input with timezone enabled (optional)
 * @param boolean $forceISO8601 Force ISO 8601 date/time format (optional)
 * @access public
 */
function date2Timestamp($inputDate, $inputTime = '', $tzEnabled = false, $forceISO8601 = false)
{
    global $gmtTimezone, $timezoneSession, $dateformatSession, $timeformatSession;

    if ($forceISO8601) {
        $uformat = 'Y-m-d';
    } else {
        list($uformat, $cformat, $mformat) = preg_split('/\|/', $dateformatSession);
    }

    $serverTimeZone = date('Z');

    if ($inputTime != '') {
        $tmToks = explode(':', $inputTime);

        if (count($tmToks) == 3) {
            $timeSec = $tmToks[2];
        } else {
            $timeSec = '0';
        }

        if (($timeformatSession == '24H') || $forceISO8601) {
            list($timeHour, $timeMin) = explode(':', $inputTime);
        } else {
            $strAM = date('A',mktime(0,0,0,1,1,2000));
            list($inputTime1,$inputTime2) = explode(' ', $inputTime);
            list($timeHour, $timeMin) = explode(':', $inputTime2);

            if ($timeHour == '12') {
                $timeHour = '0';
            }

            $pos = strpos($inputTime, $strAM);

            if ($pos === false) {
                $timeHour = strval(intval($timeHour)+12);
            }
        }
    } else {
        $timeHour = '0';
        $timeMin = '0';
        $timeSec = '0';
    }

    if ($tzEnabled && ($gmtTimezone == 'true')) {
        $hours = ($timezoneSession == '') ? 0 : floor(floatval($timezoneSession));
        $minutes = ($timezoneSession == '') ? 0 : ((floatval($timezoneSession) - $hours) * 60);
        $timeHour = intval($timeHour) - $hours;
        $timeMin = intval($timeMin) - $minutes;
    }

    switch ($uformat) {
        case 'Y-m-d':
        case 'Y.m.d':
            $ts = mktime(intval($timeHour), intval($timeMin), intval($timeSec), intval(substr($inputDate, 5, 2)), intval(substr($inputDate, 8, 2)), intval(substr($inputDate, 0, 4)));
            break;
        case 'd-m-Y':
        case 'd/m/Y':
        case 'd.m.Y':
        case 'd m Y':
            $ts = mktime(intval($timeHour), intval($timeMin), intval($timeSec), intval(substr($inputDate, 3, 2)), intval(substr($inputDate, 0, 2)), intval(substr($inputDate, 6, 4)));
            break;
        case 'd-m-y':
        case 'd/m/y':
        case 'd.m.y':
        case 'd m y':
            $y = substr($inputDate, 6, 2);
            $ts = mktime(intval($timeHour), intval($timeMin), intval($timeSec), intval(substr($inputDate, 3, 2)), intval(substr($inputDate, 0, 2)), intval(($y>=70)?'19'.$y:'20'.$y));
            break;
        case 'j/n/Y':
            list($j, $n, $Y) = preg_split('/\//', $inputDate);
            $ts = mktime(intval($timeHour), intval($timeMin), intval($timeSec), intval($n), intval($j), intval($Y));
            break;
        case 'j/n/y':
            list($j, $n, $y) = preg_split('/\//', $inputDate);
            $ts = mktime(intval($timeHour), intval($timeMin), intval($timeSec), intval($n), intval($j), intval(($y>=70)?'19'.$y:'20'.$y));
            break;
        case 'm/d/Y':
            $ts = mktime(intval($timeHour), intval($timeMin), intval($timeSec), intval(substr($inputDate, 0, 2)), intval(substr($inputDate, 3, 2)), intval(substr($inputDate, 6, 4)));
            break;
        case 'm/d/y':
            $y = substr($inputDate, 6, 2);
            $ts = mktime(intval($timeHour), intval($timeMin), intval($timeSec), intval(substr($inputDate, 0, 2)), intval(substr($inputDate, 3, 2)), intval(($y>=70)?'19'.$y:'20'.$y));
            break;
        case 'j.m.Y':
        case 'j.n.Y':
            list($j, $n, $Y) = preg_split('/./', $inputDate);
            $ts = mktime(intval($timeHour), intval($timeMin), intval($timeSec), intval($n), intval($j), intval($Y));
            break;
        case 'j.n.y':
            list($j, $n, $y) = preg_split('/./', $inputDate);
            $ts = mktime(intval($timeHour), intval($timeMin), intval($timeSec), intval($n), intval($j), intval(($y>=70)?'19'.$y:'20'.$y));
            break;
    }

    if ($tzEnabled && ($gmtTimezone == 'true')) {
        $ts -= $serverTimeZone;
    }

    return $ts;
}

/**
 * Display date/time according to timezone (if timezone enabled)
 *
 * @param mixed $storedDate Date stored in database
 * @param string $mode 'date'-display date only,'time'-display time only,'both'-display both date/time (optional)
 * @param boolean $showSeconds Display seconds (optional)
 * @param boolean $noGmtTZ Don't use gmt timezone (optional)
 * @param boolean $forceISO8601 Force ISO 8601 date/time format (optional)
 * @access public
 */
function createDate($storedDate = '', $mode = 'date', $showSeconds = false, $noGmtTZ = false, $forceISO8601 = false)
{
    global $gmtTimezone, $timezoneSession, $dateformatSession, $timeformatSession;

    if ($storedDate == '0') {    // null value in database
        return '--';
    }

    $timestamp1 = ($storedDate=='')?time():$storedDate;

    if ($forceISO8601) {
        $phpDateFormat = 'Y-m-d';
        $phpTimeFormat = '24H';
        $sep = 'T';
    } else {
        list($phpDateFormat, $jscalendarDateFormat, $yearMonthFormat) = preg_split('/\|/', $dateformatSession);
        $phpTimeFormat = $timeformatSession;
        $sep = ' ';
    }

    $outputFormat = '';

    if ($gmtTimezone == 'true' && $noGmtTZ == false) {  // GMT Timezone Enabled
        $serverTimeZone = date('Z');        // Timezone offset in seconds.
        $timestamp1 += $serverTimeZone;
        $hours = ($timezoneSession=='')?0:floor(floatval($timezoneSession));            // The hours of timezone
        $minutes = ($timezoneSession=='')?0:((floatval($timezoneSession)-$hours)*60);   // The minutes of timezone
        $timestamp1 = mktime(intval(date('H', $timestamp1))+$hours,intval(date('i', $timestamp1))+$minutes,intval(date('s', $timestamp1)),intval(date('n', $timestamp1)),intval(date('j', $timestamp1)),intval(date('Y', $timestamp1)));
    }

    if ($mode != 'time') {   // display mode 'date' & 'both'
        $outputFormat .= $phpDateFormat;
    }

    if ($mode != 'date') {  // display mode 'time' & 'both'
        if ($mode == 'both') {
            $outputFormat .= $sep;
        }

        if ($phpTimeFormat == '24H') {  // 24-hour format
            $outputFormat .= 'H:i';
        } else {                            // 12-hour format
            $outputFormat .= 'A g:i';
        }

        if ($showSeconds) {                  // show seconds
            $outputFormat .= ':s';
        }
    }

    return date($outputFormat, $timestamp1);
}

// Get the (actual) client IP addr, used in combination with
// the SID as the primary key in the database, security.
function get_remote_addr ()
{
    if (isset($_SERVER)) {
        if (isset($_SERVER["HTTP_X_FORWARDED_FOR"])) {
            return trim($_SERVER["HTTP_X_FORWARDED_FOR"]);
        }

        if (isset($_SERVER["HTTP_CLIENT_IP"])) {
            return trim($_SERVER["HTTP_CLIENT_IP"]);
        }

        return trim($_SERVER["REMOTE_ADDR"]);
    }

    if (getenv('HTTP_X_FORWARDED_FOR')) {
        return trim(getenv('HTTP_X_FORWARDED_FOR'));
    }

    if (getenv('HTTP_CLIENT_IP')) {
        return trim(getenv('HTTP_CLIENT_IP'));
    }

    return trim(getenv('REMOTE_ADDR'));
}
/**
 * This function is called when a session try to enter critical section
 *
 * @access public
 */
function enter_critical_section() {
    global $msg;

    if ((function_exists('ftok'))&&(function_exists('sem_get'))&&(function_exists('sem_acquire'))) {
        $key = ftok(realpath('../includes/library.php'), 'N');
        if ($key == -1) {
            $msg = 'ftokError';
            return -1;
        }
        $id = sem_get($key);
        if ($id == FALSE) {
            $msg = 'semgetError';
            return -1;
        }
        if (sem_acquire($id)==FALSE) {
            $msg = 'semacquireError';
            return -1;
        }
        return $id;
    } else {
       ignore_user_abort(1);
       $lockfile = '../files/mutex';

       if (file_exists($lockfile)) {
           if (time() - filemtime($lockfile) > 5) {
               unlink($lockfile);
           }
       }

       $id = @fopen($lockfile, 'x');
       for ($i=0; ($id === false) && ($i < 20); $i++) {
           clearstatcache();
           usleep(rand(9, 999));
           $id = @fopen($lockfile, 'x');
       }

       if ($id == false) {
           $msg = 'semacquireError';
           $id = -1;
       }

       ignore_user_abort(0);
       return $id;
    }
}
/*
 * This function is called when a session try to leave critical section
 *
 * @access public
 */
function leave_critical_section($id) {
    if (function_exists('sem_release')) {
        sem_release($id);
    } else {
       ignore_user_abort(1);
       $lockfile = '../files/mutex';

       fclose($id);
       unlink($lockfile);

       ignore_user_abort(0);
    }
}

/**
 * Delete a file, or a folder and its contents
 *
 * @author      Aidan Lister <aidan@php.net>
 * @version     1.0.2
 * @param       string   $dirname    Directory to delete
 * @return      bool     Returns TRUE on success, FALSE on failure
 */
function rmdirr($dirname)
{
    // Sanity check
    if (!file_exists($dirname)) {
        return false;
    }

    // Simple delete for a file
    if (is_file($dirname)) {
        usleep(10);
        return unlink($dirname);
    }

    // Loop through the folder
    $dir = dir($dirname);
    while (false !== $entry = $dir->read()) {
        // Skip pointers
        if ($entry == '.' || $entry == '..') {
            continue;
        }

        // Recurse
        rmdirr("$dirname/$entry");
    }

    // Clean up
    $dir->close();
    usleep(10);
    return rmdir($dirname);
}

/**
 * Print language strings
 *
 * @param string $key Key of language strings array
 * @access public
 */
function lang_strings($key) {
    global $strings, $profilSession, $enableOnlineTextCustomization, $lang, $SW_Permissions;

    if (!isset($SW_Permissions['TextCustomization']['Admin'][$profilSession]) || !$SW_Permissions['TextCustomization']['Admin'][$profilSession] || $enableOnlineTextCustomization != 'true') {
        return $strings[$key];
    } else {
        $querymore = " WHERE str.string_key='$key' AND str.language='$lang'";
        $stringsDetail = new request();
        $stringsDetail->openStrings($querymore);
        $comptStringsDetail = count($stringsDetail->str_string_key);

        if ($comptStringsDetail == '0') {
            return $strings[$key] . '<a href=\'../administration/updatestrings.php?string_key=' . $key . '\'><img src=\'../themes/' . THEME . '/text_customization.png\'></a>';
        } else {
            return $strings[$key] . '<a href=\'../administration/updatestrings.php?string_key=' . $key . '\'><img src=\'../themes/' . THEME . '/text_customization.png\'></a>';
        }
    }
}

/**
 * Print language help strings
 *
 * @param string $key Key of language help strings array
 * @access public
 */
function lang_help($key) {
    global $strings, $SW_Permissions, $profilSession, $enableOnlineTextCustomization;

    if (!$SW_Permissions['TextCustomization']['Admin'][$profilSession] || $enableOnlineTextCustomization != 'true') {
        return $strings[$key];
    } else {
        return "<strong>" . $key . "</strong><br>" . $strings[$key];
    }
}

/**
 * Convert HTML to text
 *
 * @param string $document html to convert
 * @access public
 */
function convert_html2txt($document) {
    $search = array ('@<script[^>]*?>.*?</script>@si', // Strip out javascript
        '@<[\/\!]*?[^<>]*?>@si',          // Strip out HTML tags
        '@([\r\n])[\s]+@',                // Strip out white space
        '@&(quot|#34);@i',                // Replace HTML entities
        '@&(amp|#38);@i',
        '@&(lt|#60);@i',
        '@&(gt|#62);@i',
        '@&(nbsp|#160);@i',
        '@&(iexcl|#161);@i',
        '@&(cent|#162);@i',
        '@&(pound|#163);@i',
        '@&(copy|#169);@i',
        '@&#(\d+);@');                    // evaluate as php
    $replace = array ('',
        ' ',
        '\1',
        '"',
        '&',
        '<',
        '>',
        ' ',
        chr(161),
        chr(162),
        chr(163),
        chr(169),
        'chr(\1)');
    return preg_replace_callback($search, function ($matches) { return $replace; }, $document);
}

/**
 * E-mail Validation
 *
 * @param string $email e-mail address to validate
 * @access public
 */
function ValidateMail($email) {
  // First, we check that there's one @ symbol, and that the lengths are right
  if (preg_match("/^[^@]{1,64}@[^@]{1,255}$/", $email) !== 1) {
    // Email invalid because wrong number of characters in one section, or wrong number of @ symbols.
    return false;
  }
  // Split it into sections to make life easier
  $email_array = explode("@", $email);
  $local_array = explode(".", $email_array[0]);
  for ($i = 0; $i < sizeof($local_array); $i++) {
     if (preg_match("@^(([A-Za-z0-9!#$%&'*+/=?^_`{|}~-][A-Za-z0-9!#$%&'*+/=?^_`{|}~\.-]{0,63})|(\"[^(\\|\")]{0,62}\"))$@", $local_array[$i]) !== 1) {
      return false;
    }
  }
  if (preg_match("/^\[?[0-9\.]+\]?$/", $email_array[1]) !== 1) { // Check if domain is IP. If not, it should be valid domain name
    $domain_array = explode(".", $email_array[1]);
    if (sizeof($domain_array) < 2) {
        return false; // Not enough parts to domain
    }
    for ($i = 0; $i < sizeof($domain_array); $i++) {
      if (preg_match("/^(([A-Za-z0-9][A-Za-z0-9-]{0,61}[A-Za-z0-9])|([A-Za-z0-9]+))$/", $domain_array[$i]) !== 1) {
        return false;
      }
    }
  }
  return true;
}

function formatted_timestamp($ts)
{
    global $timeformatSession;

    if ($timeformatSession == '12H') {
        $hour = intval(substr($ts,0,2));

        if ($hour == '12') {
            $ts1 = $ts . 'PM';
        } else if ($hour == '24') {
            $ts1 = sprintf("%02d",$hour-24) . substr($ts,2,3) . 'AM';
        } else if ($hour > 12) {
            $ts1 = sprintf("%02d",$hour-12) . substr($ts,2,3) . 'PM';
        } else {
            $ts1 = $ts . 'AM';
        }

        return $ts1;
    } else {
        return $ts;
    }
}

// Random Alphanumeric String Generator
function get_rand_id($len)
{
    $rand_id = '';

    for ($i=0; $i<$len; $i++) {
        mt_srand((double)microtime()*1000000);
        $num = mt_rand(1,36);

        if ($num < 27) {
            $rand_id .= chr($num+96);
        } else {
            $rand_id .= chr($num+21);
        }
    }

    return $rand_id;
}

/**
 *    Delete the provided directory and all its contents from the FTP-server.
 */
function ftp_rmdirr($path, $handle) {
    if (!(@ftp_rmdir($handle, $path) || @ftp_delete($handle, $path))) {
        $list = ftp_rawlist($handle, $path . '/');

        if (!empty($list)) {
            foreach($list as $value) {
                $exp = explode(' ', $value);
                $file = $exp[count($exp)-1];

                if ($file != '.' && $file != '..') {
                    ftp_rmdirr($path . '/' . $file, $handle);
                }
            }
        }
    }

    @ftp_rmdir($handle, $path);
}

/**
 * check if period locked
 *
 * @param int $date1 Timestamp 1
 * @param int $date2 Timestamp 2
 * @access public
 */
function periodLocked($date1, $date2 = "") {
    global $periodUnit, $autoLockPeriod, $period_date_start, $period_date_end;
    $rc = false;

    if ($periodUnit == 'month') {
        $year = date('Y', $date1);
        $month = date('m', $date1);
        $period_date_start = mktime(0, 0, 0, intval($month), 1, intval($year));
        $period_date_end = mktime(0, 0, 0, intval($month)+1, 0, $year);
        $serial_no = intval($month);

        if ($date2 == "") {
            $year1 = $year;
            $serial_no1 = $serial_no;
        } else {
            $year1 = date('Y', $date2);
            $serial_no1 = date('m', $date2);
        }

        while (!$rc && (intval($year)*100+intval($serial_no))<=(intval($year1)*100+intval($serial_no1))) {
            $querymore = " WHERE per.fyear='$year' AND per.serial_no='$serial_no'";
            $periodDetail = new request();
            $periodDetail->openPeriods($querymore);
            $comptPeriodDetail = count($periodDetail->per_id);

            if ($comptPeriodDetail != '0') {
                $result = $periodDetail->per_locked[0];
            } else {
                if ($autoLockPeriod == '0') {
                    $result = 'N';
                } else {
                    $n = time();
                    $t = mktime(0, 0, 0, intval($serial_no)+1, intval($autoLockPeriod), intval($year));
                    $result = ($t>=$n)?'N':'Y';
                }
            }

            $rc = ($result=='Y');
            $serial_no += 1;

            if ($serial_no == '13') {
                $serial_no = '1';
                $year += 1;
            }
        }
    } else {
        $weekNum = getWeekOfYear($date1);
        $year = substr($weekNum, 0, 4);
        $serial_no = sprintf('%d', intval(substr($weekNum, 4)));
        $daysOfWeek = getDaysOfWeek($year, $serial_no);
        $period_date_start = $daysOfWeek[0];
        $period_date_end = $daysOfWeek[6];

        if ($date2 == "") {
            $weekNum1 = $weekNum;
        } else {
            $weekNum1 = getWeekOfYear($date2);
        }

        while (!$rc && (intval($weekNum) <= intval($weekNum1))) {
            $querymore = " WHERE per.fyear='$year' AND per.serial_no='$serial_no'";
            $periodDetail = new request();
            $periodDetail->openPeriods($querymore);
            $comptPeriodDetail = count($periodDetail->per_id);

            if ($comptPeriodDetail != '0') {
                $result = $periodDetail->per_locked[0];
            } else {
                if ($autoLockPeriod == '0') {
                    $result = 'N';
                } else {
                    $n = time();
                    $t = mktime(0, 0, 0, intval(date('n', $daysOfWeek[6])), intval(date('j', $daysOfWeek[6]))+intval($autoLockPeriod), intval(date('Y', $daysOfWeek[6])));
                    $result = ($t>=$n)?'N':'Y';
                }
            }

            $rc = ($result=='Y');
            $weekNum = getWeekOfYear(mktime(0, 0, 0, intval(date('n', $daysOfWeek[0])), intval(date('j', $daysOfWeek[0]))+7, intval(date('Y', $daysOfWeek[0]))));
            $year = substr($weekNum, 0, 4);
            $serial_no = sprintf('%d', intval(substr($weekNum, 4)));
            $daysOfWeek = getDaysOfWeek($year, $serial_no);
        }
    }

    return $rc;
}

/**
 * Convert all HTML entities to their applicable characters
 *
 * @global string $htmlEntityCharset
 * @param string $string The input string
 * @return string
 */
function HtmlEntityDecodeNOD($string) {
    global $htmlEntityCharset;
    return html_entity_decode($string, ENT_QUOTES, $htmlEntityCharset);
}

/**
 * Convert all applicable characters to HTML entities
 *
 * @global string $htmlEntityCharset
 * @param string $string The input string
 * @return string
 */
function HtmlEntitiesNOD($string) {
    global $htmlEntityCharset;
    return htmlentities ($string, ENT_QUOTES, $htmlEntityCharset);
}

/**
 * encrypt password
 * @global string $cryptKey
 * @param string $text
 * @return string
 */
function encrypt_password($text)
{
    global $cryptKey;
    $encrypt_method = "AES-256-CBC";
    // hash
    $key = hash('sha256', DBPASSWORD);
    // iv - encrypt method AES-256-CBC expects 16 bytes - else you will get a warning
    $iv = substr(hash('sha256', $cryptKey), 0, 16);
    $output = openssl_encrypt($text, $encrypt_method, $key, 0, $iv);
    $output = base64_encode($output);
    return $output;
}

/**
 * Convert input value to database stored value
 *
 * @global int $hourAccuracy
 * @global int $quantityAccuracy
 * @global int $measureAccuracy
 * @global int $rateAccuracy
 * @global int $costAccuracy
 * @global int $percentAccuracy
 * @global string $timeformatSession
 * @param string $fieldType
 * @param string $fieldValue
 * @return mixed
 */
function DbValueEncode($fieldType, $fieldValue) {
    global $hourAccuracy, $quantityAccuracy, $measureAccuracy, $rateAccuracy, $costAccuracy, $percentAccuracy, $timeformatSession;
    $dbStoredValue = $fieldValue;

    switch ($fieldType) {
        case enumFieldType::ftText:
        case enumFieldType::ftMemo:
            if ($dbStoredValue != '') {
                $dbStoredValue = HtmlEntitiesNOD($dbStoredValue);
            }
            break;
        case enumFieldType::ftDate:
        case enumFieldType::ftTimestamp:
            if ($dbStoredValue != '') {
                $dbStoredValue = date2Timestamp($dbStoredValue);
            } else {
                $dbStoredValue = '0';
            }

            break;
        case enumFieldType::ftTime:
            if ($dbStoredValue != '') {
                if ($timeformatSession == '24H') {
                    list($h, $m) = preg_split('/:/', $dbStoredValue);
                } else {
                    if (preg_match('/^am/i', $dbStoredValue)) {
                        list($h, $m) = preg_split('/:/', preg_replace_callback('/(am|am )/i', function ($matches) { return ''; }, $dbStoredValue));

                        if ($h == '12') {
                            $h = '0';
                        }
                    } else if (preg_match('/^pm/i', $dbStoredValue)) {
                        list($h, $m) = preg_split('/:/', preg_replace_callback('/(am|am )/i', function ($matches) { return ''; }, $dbStoredValue));

                        if ($h != '12') {
                            $h = intval($h) + 12;
                        }
                    } else {
                        list($h, $m) = preg_split('/:/', $dbStoredValue);

                        if ($h == '12') {
                            $h = '0';
                        }
                    }
                }

                $dbStoredValue = sprintf('%02d%02d', intval($h), intval($m));
            }

            break;
        case enumFieldType::ftHours:
            if ($dbStoredValue != '') {
                $dbStoredValue = round($dbStoredValue, $hourAccuracy);
            } else {
                $dbStoredValue = '0';
            }

            break;
        case enumFieldType::ftQuantity:
            if ($dbStoredValue != '') {
                $dbStoredValue = round($dbStoredValue, $quantityAccuracy);
            } else {
                $dbStoredValue = '0';
            }

            break;
        case enumFieldType::ftMeasure:
            if ($dbStoredValue != '') {
                $dbStoredValue = round($dbStoredValue, $measureAccuracy);
            } else {
                $dbStoredValue = '0';
            }

            break;
        case enumFieldType::ftRate:
            if ($dbStoredValue != '') {
                $dbStoredValue = round($dbStoredValue, $rateAccuracy);
            } else {
                $dbStoredValue = '0';
            }

            break;
        case enumFieldType::ftTotals:
            if ($dbStoredValue != '') {
                $dbStoredValue = round($dbStoredValue, $costAccuracy);
            } else {
                $dbStoredValue = '0';
            }

            break;
        case enumFieldType::ftPercentage:
            if ($dbStoredValue != '') {
                $dbStoredValue = round($dbStoredValue, $percentAccuracy);
            } else {
                $dbStoredValue = '0';
            }

            break;
        case enumFieldType::ftBoolean:
            $dbStoredValue = (($dbStoredValue=='Y')?enumBoolean::bTrue:enumBoolean::bFalse);
            break;
        case enumFieldType::ftPassword:
            if ($dbStoredValue != '') {
                $dbStoredValue = encrypt_password($dbStoredValue);
            }

            break;
    }

    return $dbStoredValue;
}

/**
 * decrypt password
 * @global string $cryptKey
 * @param string $text
 * @return string
 */
function decrypt_password($text)
{
    global $cryptKey;
    $encrypt_method = "AES-256-CBC";
    // hash
    $key = hash('sha256', DBPASSWORD);
    // iv - encrypt method AES-256-CBC expects 16 bytes - else you will get a warning
    $iv = substr(hash('sha256', $cryptKey), 0, 16);
    $output = openssl_decrypt(base64_decode($text), $encrypt_method, $key, 0, $iv);
    return $output;
}

/**
 * Convert database stored value to output value
 * @global int $hourAccuracy
 * @global int $quantityAccuracy
 * @global int $measureAccuracy
 * @global int $rateAccuracy
 * @global int $costAccuracy
 * @global int $percentAccuracy
 * @param string $fieldType
 * @param string $dbStoredValue
 * @param mixed $vv field valid values
 * @return mixed
 */
function DbValueDecode($fieldType, $dbStoredValue, $vv = '') {
    global $hourAccuracy, $quantityAccuracy, $measureAccuracy, $rateAccuracy, $costAccuracy, $percentAccuracy;
    $fieldValue = $dbStoredValue;

    switch ($fieldType) {
        case enumFieldType::ftText:                       // text
            if ($fieldValue != '') {
                $fieldValue = HtmlEntityDecodeNOD($fieldValue);

                if ($vv != '' && is_array($vv) && array_key_exists($fieldValue, $vv)) {
                    $fieldValue = $vv[$fieldValue];
                }
            }

            break;
        case enumFieldType::ftAlphaNumeric:                   // alpha-numeric
        case enumFieldType::ftNumeric:
            if ($fieldValue != '' && $vv != '' && is_array($vv) && array_key_exists($fieldValue, $vv)) {
                $fieldValue = $vv[$fieldValue];
            }

            break;
        case enumFieldType::ftMemo:                       // memo
            if ($fieldValue != '') {
                $fieldValue = HtmlEntityDecodeNOD($fieldValue);
            }

            break;
        case enumFieldType::ftDate:                       // date
        case enumFieldType::ftTimestamp:                  // timestamp
            if ($fieldValue != '0') {
                $fieldValue = createDate($fieldValue, 'date');
            } else {
                $fieldValue = '';
            }

            break;
        case enumFieldType::ftTime:                       // time
            if ($fieldValue != '') {
                $h = substr($fieldValue, 0, 2);
                $m = substr($fieldValue, 2);
                $fieldValue = createDate(mktime(intval($h),intval($m)), 'time', false, true);
            }

            break;
        case enumFieldType::ftHours:                      // hours
            $fieldValue = round($fieldValue, $hourAccuracy);

            if (floatval($fieldValue) == 1) {
                $fieldValue .= " " . lang_strings("hour");
            } else {
                $fieldValue .= " " . lang_strings("hours");
            }

            break;
        case enumFieldType::ftQuantity:                   // quantity
            $fieldValue = round($fieldValue, $quantityAccuracy);
            break;

        case enumFieldType::ftMeasure:                    // measure
            $fieldValue = round($fieldValue, $measureAccuracy);
            break;
        case enumFieldType::ftRate:                       // rate
            $fieldValue = round($fieldValue, $rateAccuracy);
            break;
        case enumFieldType::ftTotals:                     // totals
            $fieldValue = round($fieldValue, $costAccuracy);
            break;
        case enumFieldType::ftPercentage:                    // percentage
            $fieldValue = round($fieldValue, $percentAccuracy);
            break;
        case enumFieldType::ftBoolean:                    // boolean
            $fieldValue = (($fieldValue==enumBoolean::bTrue)?lang_strings('yes'):lang_strings('no'));
            break;
        case enumFieldType::ftEmail:                      // email
            if ($fieldValue != '') {
                $block = new block();
                $fieldValue = $block->buildLink($fieldValue, $fieldValue, 'mail');
            }

            break;
        case enumFieldType::ftUrl:                        // url
            if ($fieldValue != '') {
                $block = new block();
                $fieldValue = $block->buildLink($fieldValue, $fieldValue, 'out');
            }

            break;
        case enumFieldType::ftPassword:
            if ($fieldValue != '') {
                $fieldValue = decrypt_password($fieldValue);
            }

            break;
    }

    return $fieldValue;
}

/**
 * Get table name string
 *
 * @param $table_name The table name
 */
function tableNameString($table_name) {
    global $strings;
    $stringKey = "TBL__" . $table_name;
    return (isset($strings[$stringKey])?$strings[$stringKey]:$table_name);
}

/**
 * Get field name string
 *
 * @param $table_name The table name
 * @param $field_name The field name
 */
function fieldNameString($table_name, $field_name) {
    global $strings;
    $stringKey = "FIE__" . $table_name . "_" . $field_name;
    return (isset($strings[$stringKey])?$strings[$stringKey]:$field_name);
}

/**
 * Get field default value
 *
 * @param $table_name The table name
 * @param $field_name The field name
 */
function fieldDefaultValue($table_name, $field_name) {
    $querymore = " WHERE fdf.tablename='$table_name' AND fdf.fieldname='$field_name'";
    $listFieldsDefinitions = new request();
    $listFieldsDefinitions->openFieldsDefinitions($querymore);
    $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);

    if ($comptListFieldsDefinitions == '0') {
        return '';
    } else {
        return $listFieldsDefinitions->fdf_default_value[0];
    }
}

/**
 * Get field valid value string
 *
 * @global array $strings
 * @param string $table_name The table name
 * @param string $field_name The field name
 * @param string $valid_value The field valid value
 * @return string
 */
function fieldValidValueString($table_name, $field_name, $valid_value) {
    global $strings;

    if ($field_name == 'currency') {
        $stringKey = 'CRC' . $valid_value;
    } else {
        $stringKey = "VV__" . $table_name . "_" . $field_name . "_" . $valid_value;
    }

    return (isset($strings[$stringKey])?$strings[$stringKey]:$valid_value);
}

/**
 * Update string (Transaction)
 *
 * @global string $lang
 * @global string $tablePrefix
 * @global array $strings
 * @param class $requestAct Transaction object
 * @param string $strkey String key
 * @param string $strval String value
 * @param string $strlang String language (default: session language)
 */
function updateString($requestAct, $strkey, $strval, $strlang = '') {
    global $lang, $tablePrefix, $strings;

    if ($strlang == '') {
        $strlang = $lang;
    }

    if ($strlang == $lang) {
        $strings[$strkey] = $strval;
    }

    $querymore = " WHERE str.string_key='$strkey' AND str.language='$strlang'";
    $stringsDetail = clone $requestAct;
    $stringsDetail->openStrings($querymore);
    $comptStringsDetail = $stringsDetail->rowCount();
    $val = HtmlEntitiesNOD($strval);

    if ($comptStringsDetail > 0) {
        $tmpsql = 'UPDATE ' . $tablePrefix . "strings SET string_val='$val' WHERE string_key='$strkey' AND language='$strlang'";
    } else {
        $tmpsql = "INSERT INTO " . $tablePrefix . "strings (string_key,language,string_val) VALUES ('$strkey','$strlang','$val')";
    }

    if ($requestAct->query($tmpsql) === false) {
        throw new Exception($tmpsql);
    }
}

/**
 * Generate selection of options
 *
 * @param string $table_name The table name
 * @param string $field_name The field name
 * @param string $select_name The selection name
 * @param string $default_value The default value
 * @return string
 */
function populateSelect($table_name, $field_name, $select_name, $default_value) {
    $querymore = " WHERE fvd.tablename='$table_name' AND fvd.fieldname='$field_name' ORDER BY fvd.fieldvalue";
    $listFieldsValuesDefinitions = new request();
    $listFieldsValuesDefinitions->openFieldsValuesDefinitions($querymore);
    $comptListFieldsValuesDefinitions = count($listFieldsValuesDefinitions->fvd_id);

    if ($comptListFieldsValuesDefinitions == '0') {
        return '';
    }

    $selectString = '';

    if ($select_name != '') {
        $selectString .= '<select name="' . $select_name . '">';
    }

    if ($default_value == '') {
        $default_value = $listFieldsValuesDefinitions->fvd_fieldvalue[0];
    }

    for ($i=0; $i<$comptListFieldsValuesDefinitions; $i++) {
        if ($listFieldsValuesDefinitions->fvd_fieldvalue[$i] == $default_value) {
            $selectString .= '<option value="' . $listFieldsValuesDefinitions->fvd_fieldvalue[$i] . '" selected>' . $listFieldsValuesDefinitions->fvd_fieldvalue_desc[$i] . '</option>';
        } else {
            $selectString .= '<option value="' . $listFieldsValuesDefinitions->fvd_fieldvalue[$i] . '">' . $listFieldsValuesDefinitions->fvd_fieldvalue_desc[$i] . '</option>';
        }
    }

    if ($select_name != '') {
        $selectString .= '</select>';
    }

    return $selectString;
}

/**
 * Generate document description
 *
 * @global string $tablePrefix
 * @global array $row
 * @param string $tblnam
 * @param int $id
 * @return string
 */
function setupDocDesc($tblnam, $id) {
    global $tablePrefix, $row;
    $desc = '';

    switch ($tblnam) {
        case 'hour_code':
            $querymore = " WHERE hrc.id='$id'";
            $hourCodeDetail = new request();
            $hourCodeDetail->openHourCode($querymore);

            if (count($hourCodeDetail->hrc_id) > 0) {
                $desc = $hourCodeDetail->hrc_name_print[0];
            }

            break;
        case 'hour_type':
            $querymore = " WHERE hrt.id='$id'";
            $hourTypeDetail = new request();
            $hourTypeDetail->openHourType($querymore);

            if (count($hourTypeDetail->hrt_id) > 0) {
                $desc = $hourTypeDetail->hrt_name_print[0];
            }

            break;
        default:
            $querymore1 = " WHERE tdf.tablename='$tblnam'";
            $tableDefinitionDetail = new request();
            $tableDefinitionDetail->openTablesDefinitions($querymore1);

            if (count($tableDefinitionDetail->tdf_tablename) == 0) {
                return '';
            }

            $querymore2 = " WHERE fdf.tablename='$tblnam'";
            $listFieldsDefinitions = new request();
            $listFieldsDefinitions->openFieldsDefinitions($querymore2);
            $dispfields = $tableDefinitionDetail->tdf_dispfields[0];

            if ($dispfields == '') {
                $dispfields = 'id';
            }

            $dispfielda = explode('+', $dispfields);
            $dispstyle = $tableDefinitionDetail->tdf_dispstyle[0];
            $tmpquery = "SELECT " . str_replace('+', ',', $dispfields) . " FROM $tablePrefix" . $tableDefinitionDetail->tdf_dbtablename[0] . " WHERE id='$id'";
            $request1 = new request();
            $request1->connectClass();
            $request1->query($tmpquery);

            if ($request1->fetch()) {
                $j = 0;

                foreach ($dispfielda as $f) {
                    $request1->val[$f] = ($row[$j++]);
                }
            }

            $request1->close();

            foreach ($dispfielda as $f) {
                if ($desc != '') {
                    switch ($dispstyle) {
                        case enumDisplayStyle::dsMultipleRowsWithTitle:
                        case enumDisplayStyle::dsMultipleRowsWithoutTitle:
                            $desc .= '<br>';
                            break;
                        default:
                            $desc .= "&nbsp;&nbsp;";
                    }
                }

                $k = array_search($f, $listFieldsDefinitions->fdf_dbfieldname);
                $querymore = " WHERE fvd.tablename='$tblnam' AND fvd.fieldname='" . $listFieldsDefinitions->fdf_fieldname[$k] . "'";
                $listFieldsValuesDefinitions = new request();
                $listFieldsValuesDefinitions->openFieldsValuesDefinitions($querymore);
                $comptListFieldsValuesDefinitions = count($listFieldsValuesDefinitions->fvd_fieldvalue);

                if ($comptListFieldsValuesDefinitions > 0) {
                    $vv = array();

                    for ($i=0; $i<$comptListFieldsValuesDefinitions; $i++) {
                        $vv[$listFieldsValuesDefinitions->fvd_fieldvalue[$i]] = $listFieldsValuesDefinitions->fvd_fieldvalue_desc[$i];
                    }
                } else {
                    $vv = '';
                }

                $val = DbValueDecode($listFieldsDefinitions->fdf_fieldtype[$k], $request1->val[$f], $vv);

                switch ($dispstyle) {
                    case enumDisplayStyle::dsMultipleRowsWithTitle:
                    case enumDisplayStyle::dsMultipleColumnsWithTitle:
                        $desc .= fieldNameString($tblnam, $listFieldsDefinitions->fdf_fieldname[$k]) . ':' . $val;
                        break;
                    default:
                        $desc .= $val;
                }
            }
    }

    return $desc;
}

/**
 * Include form template javascript
 * @global string $mkdirMethod
 * @global string $ftpRoot
 * @global string $basedir
 * @param string $jspath javascript file path
 * @param integer $ver javascript file version (update timestamp)
 * @return string
 */
function includeJavaScript($jspath, $ver) {
    global $mkdirMethod, $ftpRoot, $basedir;

    // get a copy of javascript file
    if ($mkdirMethod == 'FTP') {
        $path = dirname($jspath);
        $jsnam = basename($jspath);
        $jstmp = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
        $pathNew = $ftpRoot . '/JavaScript/' . $path;
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        ftp_chdir($ftp, $pathNew);
        ftp_get($ftp, $jstmp, $jsnam, FTP_BINARY);
        ftp_quit($ftp);
        $jssrc = '../files/' . basename($jstmp);
    } else {
        $jssrc = '../files/JavaScript/' . $jspath;
    }

    echo '<script type="text/javascript" src="' . $jssrc . '?ver=' . $ver . '"></script>';
}

function file_info_type($extension)
{
    global $type;

    // does the icone file exists?
    if (@file_exists('../interface/icones/' . $extension . '.gif')) {
        $type = $extension . '.gif';
    } else {
        $type = 'fic.gif';
    }

    return $type;
}
/**
 * Generate attachment html
 * @param integer $id
 * @param boolean $latest
 * @return string
 */
function setupAttachHTML($id, $latest = true) {
    $html = '';
    $fileDetail = new request();
    $fileDetail->openFiles(" WHERE fil.id='$id'");
    $comptFileDetail = count($fileDetail->fil_id);

    if ($comptFileDetail > 0) {
        $fileType = file_info_type($fileDetail->fil_extension[0]);

        if (fileExists($fileDetail->fil_fullname[0])) {
            /**
             * get file revision peers
             */
            $querymore1 = " WHERE fil.vc_parent='" . $fileDetail->fil_id[0] . "' AND fil.vc_status='" . enumFileVersionControl::vcPeerReview . "' AND fil.vc_version='" . $fileDetail->fil_vc_version[0] . "' ORDER BY fil.created DESC";
            $listReviews = new request();
            $listReviews->openFiles($querymore1);
            $comptListReviews = count($listReviews->fil_vc_parent);

            if ($comptListReviews > 0) {
                // link to  lastest file revision peer by default
                $idl = $listReviews->fil_id[0];
            } else {
                $idl = $fileDetail->fil_id[0];
            }

            $doc_type = $fileDetail->fil_doc_type[0];
            $doc_type_ex = $fileDetail->fil_doc_type_ex[0];
            $doc_id = $fileDetail->fil_doc_id[0];

            if ($latest) {
                $html .= '<div class="attach-item" data-attach="' . $doc_type . ',' . $doc_type_ex . ',' . $doc_id . ',' . $id . ',' . $idl . ',1"><img src="../interface/icones/' . $fileType . '" border="0" alt=""><label>' . $fileDetail->fil_name[0] . '</label></div>';
            } else {
                $html .= '<div class="attach-item" data-attach="' . $doc_type . ',' . $doc_type_ex . ',' . $doc_id . ',' . $id . ',' . $id . ',0"><img src="../interface/icones/' . $fileType . '" border="0" alt=""><label>' . $fileDetail->fil_name[0] . '</label></div>';
            }
        } else {
            $html .= '<div class="attach-item"><img src="../interface/icones/' . $fileType . '" border="0" alt=""><label class="missing-file">' . $fileDetail->fil_name[0] . '</label></div>';
        }
    }

    return $html;
}
/**
 * Generate attachments html
 * @param string $querymore
 * @return string
 */
function setupAttachListHTML($querymore) {
    $html = '';
    /**
     * get attach list
     */
    $listFiles = new request();
    $listFiles->openFiles($querymore);
    $comptListFiles = count($listFiles->fil_id);

    if ($comptListFiles > 0) {
        for ($i=0; $i<$comptListFiles; $i++) {
            $fileType = file_info_type($listFiles->fil_extension[$i]);

            if (fileExists($listFiles->fil_fullname[$i])) {
                /**
                 * get file revision peers
                 */
                $querymore1 = " WHERE fil.vc_parent='" . $listFiles->fil_id[$i] . "' AND fil.vc_status='" . enumFileVersionControl::vcPeerReview . "' AND fil.vc_version='" . $listFiles->fil_vc_version[$i] . "' ORDER BY fil.created DESC";
                $listReviews = new request();
                $listReviews->openFiles($querymore1);
                $comptListReviews = count($listReviews->fil_vc_parent);

                if ($comptListReviews > 0) {
                    // link to  lastest file revision peer by default
                    $idl = $listReviews->fil_id[0];
                } else {
                    $idl = $listFiles->fil_id[$i];
                }

                $doc_type = $listFiles->fil_doc_type[$i];
                $doc_type_ex = $listFiles->fil_doc_type_ex[$i];
                $doc_id = $listFiles->fil_doc_id[$i];
                $html .= '<div class="attach-item" data-attach="' . $doc_type . ',' . $doc_type_ex . ',' . $doc_id . ',' . $listFiles->fil_id[$i] . ',' . $idl . ',1"><img src="../interface/icones/' . $fileType . '" border="0" alt=""><label>' . $listFiles->fil_name[$i] . '</label></div>';
            } else {
                $html .= '<div class="attach-item"><img src="../interface/icones/' . $fileType . '" border="0" alt=""><label class="missing-file">' . $listFiles->fil_name[$i] . '</label></div>';
            }
        }
    }

    return $html;
}

/**
 * Generate audio field html
 * @global string $mkdirMethod
 * @global string $ftpRoot
 * @global string $basedir
 * @param string $audpath audio file path
 * @return string
 */
function setupAudioHTML($audpath) {
    global $mkdirMethod, $ftpRoot, $basedir;

    // get a copy of audio file
    if ($mkdirMethod == 'FTP') {
        $path = dirname($audpath);
        $audnam = basename($audpath);
        $audtmp = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
        $pathNew = $ftpRoot . '/Media/' . $path;
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        ftp_chdir($ftp, $pathNew);
        ftp_get($ftp, $audtmp, $audnam, FTP_BINARY);
        ftp_quit($ftp);
        $audsrc = '../files/' . basename($audtmp);
    } else {
        $audsrc = '../files/Media/' . $audpath;
    }

    return '<audio controls><source src="' . $audsrc . '" type="audio/mpeg">Your browser does not support the audio element.</audio>';
}

/**
 * Generate video field html
 * @global string $mkdirMethod
 * @global string $ftpRoot
 * @global string $basedir
 * @param string $vidpath video file path
 * @return string
 */
function setupVideoHTML($vidpath) {
    global $mkdirMethod, $ftpRoot, $basedir;

    // get a copy of video file
    if ($mkdirMethod == 'FTP') {
        $path = dirname($vidpath);
        $vidnam = basename($vidpath);
        $vidtmp = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
        $pathNew = $ftpRoot . '/Media/' . $path;
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        ftp_chdir($ftp, $pathNew);
        ftp_get($ftp, $vidtmp, $vidnam, FTP_BINARY);
        ftp_quit($ftp);
        $vidsrc = '../files/' . basename($vidtmp);
    } else {
        $vidsrc = '../files/Media/' . $vidpath;
    }

    return '<video controls><source src="' . $vidsrc . '" type="video/mp4">Your browser does not support the video element.</audio>';
}
/**
 * Generate photo box html
 * @global array $strings
 * @param string $imgsrc
 * @param string $imgnam
 * @param string $subject
 * @return string
 */
function setupPhotoBox($imgsrc, $imgnam='', $subject='')
{
    global $strings;
    $content = '<div class="photo-box">';
    $content .= '<img src="' . $imgsrc . '" onclick="javascript:showFullscreen(this);">';

    if ($imgnam != '') {
        $content .= '<div class="photo-action"><a href="' . $imgsrc . '" download="' . $imgnam . '"><img title="' . $strings['save'] . '" src="../themes/' . THEME . '/btn_download.png"></a></div>';
    }

    if ($subject != '') {
        $content .= '<label>' . $subject . '</label>';
    }

    $content .= '</div>';
    return $content;
}
/**
 * Generate image field html
 *
 * @global string $mkdirMethod
 * @global string $ftpRoot
 * @global string $basedir
 * @global array $strings
 * @param string $imgpath image file path
 * @return string
 */
function setupImageHTML($imgpath)
{
    global $mkdirMethod, $ftpRoot, $basedir;
    $imgnam = basename($imgpath);

    // get a copy of image
    if ($mkdirMethod == 'FTP') {
        $path = dirname($imgpath);
        $imgtmp = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
        $pathNew = $ftpRoot . '/Images/' . $path;
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        ftp_chdir($ftp, $pathNew);
        ftp_get($ftp, $imgtmp, $imgnam, FTP_BINARY);
        ftp_quit($ftp);
        $imgsrc = '../files/' . basename($imgtmp);
    } else {
        $imgsrc = '../files/Images/' . $imgpath;
    }

    return setupPhotoBox($imgsrc, $imgnam);
}

/**
 * Generate single field html
 *
 * @param class $block Block class
 * @param mixed $rawData Raw data
 * @param string $fldtyp Field type
 * @param string $lktbl Linked table
 * @param string $tblnam Table name
 * @param string $fldnam Field name
 * @param boolean $eml parameter $block is actually a notification class
 * @param string $viewurl
  * @return string
 */
function setupFieldHTML($block, $rawData, $fldtyp, $lktbl, $tblnam, $fldnam, $eml = false, $viewurl = '') {
    global $mkdirMethod, $ftpRoot, $basedir, $idSession;

    if ($rawData == '') {
        $result = '';
    } else {
        if ($fldtyp == enumFieldType::ftAvatar) {
            $result = '';

            if (!$eml) {
                list($id, $ext, $gender) = explode(',', $rawData);
                $avatar = getAvatar($id, $ext, $gender);

                if ($avatar != '') {
                    $result = '<img src="' . $avatar . '"' . (($id==$idSession)?'':' data-avatar="' . $id . '"') . '>';
                }
            }
        } else if ($fldtyp == enumFieldType::ftImage) {    // image
            if ($eml) {
                $path = dirname(HtmlEntityDecodeNOD($rawData));
                $imgnam = basename(HtmlEntityDecodeNOD($rawData));
                $imgtmp = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
                $imgsrc = '../files/' . basename($imgtmp);

                // get a copy of image
                if ($mkdirMethod == 'FTP') {
                    $pathNew = $ftpRoot . '/Images/' . $path;
                    $ftp = ftp_connect(FTPSERVER);
                    ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
                    ftp_chdir($ftp, $pathNew);
                    ftp_get($ftp, $imgtmp, $imgnam, FTP_BINARY);
                    ftp_quit($ftp);
                } else {
                    copy('../files/Images/' . HtmlEntityDecodeNOD($rawData), $imgtmp);
                }

                $block->AddEmbeddedImage($imgtmp, $fldnam);
                $result = '<img src="cid:' . $fldnam . '">';
            } else {
                $result = setupImageHTML(HtmlEntityDecodeNOD($rawData));
            }
        } else if ($fldtyp == enumFieldType::ftAudio) {   // audio
            if ($eml) {
                $path = dirname(HtmlEntityDecodeNOD($rawData));
                $audnam = basename(HtmlEntityDecodeNOD($rawData));
                $audtmp = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
                $audsrc = '../files/' . basename($audtmp);

                // get a copy of audio
                if ($mkdirMethod == 'FTP') {
                    $pathNew = $ftpRoot . '/Media/' . $path;
                    $ftp = ftp_connect(FTPSERVER);
                    ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
                    ftp_chdir($ftp, $pathNew);
                    ftp_get($ftp, $audtmp, $audnam, FTP_BINARY);
                    ftp_quit($ftp);
                } else {
                    copy('../files/Media/' . HtmlEntityDecodeNOD($rawData), $audtmp);
                }

                $block->AddAttachment($audtmp, $fldnam);
            } else {
                $result = setupAudioHTML(HtmlEntityDecodeNOD($rawData));
            }
        } else if ($fldtyp == enumFieldType::ftVideo) {   // video
            if ($eml) {
                $path = dirname(HtmlEntityDecodeNOD($rawData));
                $vidnam = basename(HtmlEntityDecodeNOD($rawData));
                $vidtmp = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
                $vidsrc = '../files/' . basename($vidtmp);

                // get a copy of video
                if ($mkdirMethod == 'FTP') {
                    $pathNew = $ftpRoot . '/Media/' . $path;
                    $ftp = ftp_connect(FTPSERVER);
                    ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
                    ftp_chdir($ftp, $pathNew);
                    ftp_get($ftp, $vidtmp, $vidnam, FTP_BINARY);
                    ftp_quit($ftp);
                } else {
                    copy('../files/Media/' . HtmlEntityDecodeNOD($rawData), $vidtmp);
                }

                $block->AddAttachment($vidtmp, $fldnam);
            } else {
                $result = setupVideoHTML(HtmlEntityDecodeNOD($rawData));
            }
        } else if ($fldtyp == enumFieldType::ftPassword) {   // password
            $result = (($rawData=='')?'':'********');
        } else {
            if ($lktbl != '') {
                if ($rawData == '0') {
                    $result = '';
                } else {
                    $result = setupDocDesc($lktbl, $rawData);

                    if (!$eml) {
                        $result = setupLinkedTableURL($block, $lktbl, $rawData, $result, $viewurl);
                    }
                }
            } else {
                if ($fldtyp == enumFieldType::ftText || $fldtyp == enumFieldType::ftAlphaNumeric || $fldtyp == enumFieldType::ftNumeric) {
                    // load field valid values
                    $querymore = " WHERE fvd.tablename='$tblnam' AND fvd.fieldname='$fldnam' ORDER BY fvd.fieldvalue";
                    $listFieldsValuesDefinitions = new request();
                    $listFieldsValuesDefinitions->openFieldsValuesDefinitions($querymore);
                    $comptListFieldsValuesDefinitions = count($listFieldsValuesDefinitions->fvd_tablename);
                } else {
                    $comptListFieldsValuesDefinitions = 0;
                }

                if ($comptListFieldsValuesDefinitions == 0) {
                    $result = DbValueDecode($fldtyp, $rawData);
                } else {
                    $vv = array();

                    for ($i=0; $i<$comptListFieldsValuesDefinitions; $i++) {
                        $vv[$listFieldsValuesDefinitions->fvd_fieldvalue[$i]] = $listFieldsValuesDefinitions->fvd_fieldvalue_desc[$i];
                    }

                    $result = DbValueDecode($fldtyp, $rawData, $vv);
                }
            }
        }
    }

    return $result;
}

/**
 * Generate custom fields html
 *
 * @global string $tablePrefix
 * @global array $row
 * @param class $block Block class
 * @param string $tblnam Table name
 * @param int $id Record id
 * @param boolean $pub Published fields only
 * @param boolean $eml parameter $block is actually a notification class
 */
function setupCustomFieldsHTML($block, $tblnam, $id, $pub = false, $eml = false) {
    global $tablePrefix, $row;
    // open fields definitions
    $querymore = " WHERE fdf.tablename='$tblnam' AND fdf.sysdef='N'";

    if ($pub) {   // published fields only
        $querymore .= " AND fdf.published='" . enumPublished::pbYes . "'";
    }

    $querymore .= " ORDER BY fdf.disporder";
    $listFieldsDefinitions = new request();
    $listFieldsDefinitions->openFieldsDefinitions($querymore);
    $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);

    if (preg_match('/@/', $tblnam) == 0) {
        $dbtblnam = $tablePrefix . $tblnam;
    } else {
        $dbtblnam = $tablePrefix . preg_replace_callback('/@/', function ($matches) { return 'FORM_'; }, $tblnam);
    }

    if ($comptListFieldsDefinitions != '0') {
        // fetch data
        $flds = 'U_' . implode(',U_', $listFieldsDefinitions->fdf_fieldname);
        $sql = "SELECT $flds FROM $dbtblnam WHERE id='$id'";
        $request1 = new request();
        $request1->connectClass();
        $request1->query($sql);

        if ($request1->fetch()) {
            for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
                $listFieldsDefinitions->rawData[$i] = ($row[$i]);
            }
        }

        $request1->close();

        for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
            $fieldValue = setupFieldHTML($block, $listFieldsDefinitions->rawData[$i], $listFieldsDefinitions->fdf_fieldtype[$i], $listFieldsDefinitions->fdf_linkedtable[$i], $tblnam, $listFieldsDefinitions->fdf_fieldname[$i], $eml, $listFieldsDefinitions->fdf_tdf_viewurl[$i]);
            // generate html
            $block->contentRow(fieldNameString($tblnam, $listFieldsDefinitions->fdf_fieldname[$i]), $fieldValue);
        }
    }
}

/**
 * Generate currency & float value input html
 *
 * @global array $__default_requireda
 * @global array $__default_reqtypea
 * @global array $__default_floata
 * @global array $__default_pnuma
 * @global string $currencyOnRight
 * @global array $CurrencyMatrix
 * @param object $block Block class reference
 * @param string $label Label
 * @param string $cuid Currency input field id
 * @param string $cuval Currency field value
 * @param string $amid Float input field id
 * @param float $amval Float field value
 * @param string $req Required field
 * @param int $acc Float field value accuracy
 * @param boolean $hl Highlight field
 */
function setupCurrencyInputHTML($block, $label, $cuid, $cuval, $amid, $amval, $req, $acc, $hl = false)
{
    global $__default_requireda, $__default_reqtypea, $__default_floata, $__default_pnuma, $currencyOnRight, $CurrencyMatrix;
    /**
     * get currency valid values & help
     */
    $currHelp = '<table>';
    $currVV = array();

    foreach ($CurrencyMatrix['active'] as $currCode) {
        $currHelp .= '<tr><th>' . $currCode . '</th><td>' . $CurrencyMatrix['name'][$currCode] . '</td></tr>';
        $currVV[$currCode] = $currCode;
    }

    $currHelp .= '</table>';
    /**
     * currency field input html
     */
    $cuFieldValue = '<select id="' . $cuid . '" name="' . $cuid . '">';

    foreach ($currVV as $k => $d) {
        $cuFieldValue .= '<option value="' . $k . '" ' . (($k==$cuval)?'selected':'') . '>' . $d . '</option>';
    }

    $cuFieldValue .= '</select>';
    /**
     * float field input html
     */
    $dftval = (($amval=='')?'':round($amval, $acc));
    $amFieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="text" id="' . $amid . '" name="' . $amid . '" value="' . $dftval . '" maxlength="30">';
    $__default_floata[] = $amid;
    $__default_pnuma[] = $amid;

    // required field
    if ($req == 'Y') {
        $__default_requireda[] = $amid;
        $__default_reqtypea[] = enumFieldType::ftTotals;
    }

    // generate input html string and output it
    if ($currencyOnRight == 'true') {
        $block->contentRow((($req=='Y')?'* ':'') . $label . ' ' . $block->printHelp($currHelp, true), $amFieldValue . $cuFieldValue . '<strong id="' . $amid . 'S"></strong>');
    } else {
        $block->contentRow((($req=='Y')?'* ':'') . $label . ' ' . $block->printHelp($currHelp, true), $cuFieldValue . $amFieldValue . '<strong id="' . $amid . 'S"></strong>');
    }
}
/**
 * Generate date range input html
 * @global array $strings
 * @global array $__default_requireda
 * @global array $__default_reqtypea
 * @param object $block Block class reference
 * @param type $label Label
 * @param type $id Date input field id prefix
 * @param type $val1 Date from field value
 * @param type $val2 Date to field value
 * @param type $req Required field
 * @param type $quickdates Quick dates
 * @return string
 */
function setupDateRangeInput($block, $label, $id, $val1 = '', $val2 = '', $req = false, $quickdates = 'backwards') {
    global $strings, $__default_requireda, $__default_reqtypea;
    $dtval1 = ($val1==='')?'':createDate($val1, 'date', false, true);
    $dtval2 = ($val2==='')?'':createDate($val2, 'date', false, true);
    $y = intval(date('Y'));
    $n = intval(date('n'));
    $j = intval(date('j'));
    $fieldValue = '<ol><li><input accept="date" type="text" id="' . $id . '1" name="' . $id . '1" value="' . $dtval1 . '"><strong id="' . $id . '1S"></strong> - <input accept="date" type="text" id="' . $id . '2" name="' . $id . '2" value="' . $dtval2 . '">' . '<strong id="' . $id . '2S"></strong></li>';

    if ($quickdates == 'backwards') {
        $fieldValue .= '<li><div class="leftValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate('', 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate('', 'date', false, true) . '\');">' . $strings['today'] . '</a></div>'
            . '<div class="rightValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n-1,1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n,0,$y), 'date', false, true) . '\');">' . $strings['last_month'] . '</a></div></li>'
            . '<li><div class="leftValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n,$j-1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n,$j-1,$y), 'date', false, true) . '\');">' . $strings['yesterday'] . '</a></div>'
            . '<div class="rightValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n-3,1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n,0,$y), 'date', false, true) . '\');">' . $strings['last3months'] . '</a></div></li>'
            . '<li><div class="leftValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n,$j-7,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n,$j-1,$y), 'date', false, true) . '\');">' . $strings['last7days'] . '</a></div>'
            . '<div class="rightValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,1,1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,12,31,$y), 'date', false, true) . '\');">' . $strings['this_year'] . '</a></div></li>'
            . '<li><div class="leftValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n,$j-30,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n,$j-1,$y), 'date', false, true) . '\');">' . $strings['last30days'] . '</a></div>'
            . '<div class="rightValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,1,1,$y-1), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,12,31,$y-1), 'date', false, true) . '\');">' . $strings['last_year'] . '</a></div></li>'
            . '<li><div class="leftValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n,1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n+1,0,$y), 'date', false, true) . '\');">' . $strings['this_month'] . '</a></div>'
            . '<div class="rightValue"><a href="javascript:$(\'#' . $id . '1\').val(\'\');$(\'#' . $id . '2\').val(\'\');">' . $strings['all_dates'] . '</a></div></li>'
            . '</ol>';
    } else if ($quickdates == 'forwards') {
        $fieldValue .= '<li><div class="leftValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate('', 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate('', 'date', false, true) . '\');">' . $strings['today'] . '</a></div>'
            . '<div class="rightValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n+1,1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n+2,0,$y), 'date', false, true) . '\');">' . $strings['next_month'] . '</a></div></li>'
            . '<li><div class="leftValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n,$j+1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n,$j+1,$y), 'date', false, true) . '\');">' . $strings['tomorrow'] . '</a></div>'
            . '<div class="rightValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n+1,1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n+4,0,$y), 'date', false, true) . '\');">' . $strings['next3months'] . '</a></div></li>'
            . '<li><div class="leftValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n,$j+1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n,$j+7,$y), 'date', false, true) . '\');">' . $strings['next7days'] . '</a></div>'
            . '<div class="rightValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,1,1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,12,31,$y), 'date', false, true) . '\');">' . $strings['this_year'] . '</a></div></li>'
            . '<li><div class="leftValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n,$j+1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n,$j+30,$y), 'date', false, true) . '\');">' . $strings['next30days'] . '</a></div>'
            . '<div class="rightValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,1,1,$y+1), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,12,31,$y+1), 'date', false, true) . '\');">' . $strings['next_year'] . '</a></div></li>'
            . '<li><div class="leftValue"><a href="javascript:$(\'#' . $id . '1\').val(\'' . createDate(mktime(0,0,0,$n,1,$y), 'date', false, true) . '\');$(\'#' . $id . '2\').val(\'' . createDate(mktime(0,0,0,$n+1,0,$y), 'date', false, true) . '\');">' . $strings['this_month'] . '</a></div>'
            . '<div class="rightValue"><a href="javascript:$(\'#' . $id . '1\').val(\'\');$(\'#' . $id . '2\').val(\'\');">' . $strings['all_dates'] . '</a></div></li>'
            . '</ol>';
    } else {
        $fieldValue .= '<li><div class="leftValue"><a href="javascript:$(\'#' . $id . '1\').val(\'\');$(\'#' . $id . '2\').val(\'\');">' . $strings['all_dates'] . '</a></div></li></ol>';
    }

    // required field
    if ($req == 'Y') {
        $__default_requireda[] = $id . '1';
        $__default_reqtypea[] = enumFieldType::ftDate;
        $__default_requireda[] = $id . '2';
        $__default_reqtypea[] = enumFieldType::ftDate;
    }

    if ($block === null) {  // generate input html string only
        return $fieldValue;
    } else {                // generate input html string and output it
        $block->contentRow((($req=='Y')?'* ':'') . $label, $fieldValue);
    }
}
/**
 * Generate input html
 *
 * @global int $hourAccuracy
 * @global int $quantityAccuracy
 * @global int $measureAccuracy
 * @global int $rateAccuracy
 * @global int $costAccuracy
 * @global int $percentAccuracy
 * @global string $timeformatSession
 * @global array $__default_requireda
 * @global array $__default_reqtypea
 * @global array $__default_alphanuma
 * @global array $__default_numerica
 * @global array $__default_floata
 * @global array $__default_timea
 * @global array $__default_emaila
 * @global array $__default_urla
 * @global array $__default_pnuma
 * @global array $__default_nnnuma
 * @global array $__default_vernuma
 * @global array $upIdIndex upload file element id
 * @param object $block Block class reference
 * @param string $label Lable
 * @param string $id Input field id
 * @param mixed $typo Field type & extra options
 * @param string $req Required field
 * @param int $len Maximum input length(Text,Alpha-Num,Numeric)/Memo textarea size('6'-6 rows,'9'-9 rows,others-3 rows)
 * @param string $val Field value
 * @param array $vv Field valid values
 * @param string $lt Linked table
 * @param boolean $hl Highlight field
 * @param string $info Additional information
 * @param string $events HTML DOM Events
 * @return mixed
 */
function setupInputHTML($block, $label, $id, $typo, $req = '', $len = '0', $val = '', $vv = '', $lt = '', $hl = false, $info = '', $events = '') {
    global $hourAccuracy, $quantityAccuracy, $measureAccuracy, $rateAccuracy, $costAccuracy, $percentAccuracy, $timeformatSession, $__default_requireda, $__default_reqtypea, $__default_alphanuma, $__default_numerica, $__default_floata, $__default_timea, $__default_emaila, $__default_urla, $__default_pnuma, $__default_nnnuma, $__default_vernuma, $upIdIndex;

    if (is_array($typo)) {
        $typ = $typo[0];

        for ($i=1; $i<count($typo); $i++) {
            switch ($typo[$i]) {
                case enumFieldOption::foPositiveNumber:
                    $__default_pnuma[] = $id;
                    break;
                case enumFieldOption::foNonNegativeNumber:
                    $__default_nnnuma[] = $id;
                    break;
                case enumFieldOption::foVersionNumber:
                    $__default_vernuma[] = $id;
                    break;
            }
        }
    } else {
        $typ = $typo;
    }

    if ($typ == enumFieldType::ftImage) {         // image
        if (!isset($upIdIndex)) {
            $upIdIndex = '1';
        }

        $fieldValue = setupUploadFileHTML($upIdIndex, "image/*", ($req=='Y'));
        $fieldValue .= '<input type="hidden" id="Idx' . $id . '" name="Idx' . $id . '" value="' . $upIdIndex . '">';
        $upIdIndex = strval(intval($upIdIndex) + 1);

        if ($val != '') {
            if ($req != 'Y') {
                $fieldValue .= '<br><input type="checkbox" name="Del' . $id . '" value="on">' . lang_strings('delete');
            }

            $fieldValue .= '<br>' . setupImageHTML($val);
        }
    } else if ($typ == enumFieldType::ftAudio) {  // audio
        if (!isset($upIdIndex)) {
            $upIdIndex = '1';
        }

        $fieldValue = setupUploadFileHTML($upIdIndex, "audio/*", ($req=='Y'));
        $fieldValue .= '<input type="hidden" id="Idx' . $id . '" name="Idx' . $id . '" value="' . $upIdIndex . '">';
        $upIdIndex = strval(intval($upIdIndex) + 1);

        if ($val != '') {
            if ($req != 'Y') {
                $fieldValue .= '<br><input type="checkbox" name="Del' . $id . '" value="on">' . lang_strings('delete');
            }

            $fieldValue .= '<br>' . setupAudioHTML($val);
        }
    } else if ($typ == enumFieldType::ftVideo) {  // video
        if (!isset($upIdIndex)) {
            $upIdIndex = '1';
        }

        $fieldValue = setupUploadFileHTML($upIdIndex, "video/*", ($req=='Y'));
        $fieldValue .= '<input type="hidden" id="Idx' . $id . '" name="Idx' . $id . '" value="' . $upIdIndex . '">';
        $upIdIndex = strval(intval($upIdIndex) + 1);

        if ($val != '') {
            if ($req != 'Y') {
                $fieldValue .= '<br><input type="checkbox" name="Del' . $id . '" value="on">' . lang_strings('delete');
            }

            $fieldValue .= '<br>' . setupVideoHTML($val);
        }
    } else {
        if ($req == 'Y') {
            $__default_requireda[] = $id;
            $__default_reqtypea[] = $typ;
        }

        switch ($typ) {
            case enumFieldType::ftText:
                if ($vv == '') {
                    $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="text" id="' . $id . '" name="' . $id . '" value="' . $val . '" maxlength="' . $len . '" ' . $events . '>';
                } else {
                    $fieldValue = '<select id="' . $id . '" name="' . $id . '" ' . $events . '>';

                    foreach ($vv as $k => $d) {
                        $fieldValue .= '<option value="' . $k . '" ' . (($k==$val)?'selected':'') . '>' . $d . '</option>';
                    }

                    $fieldValue .= '</select>';
                }
                break;
            case enumFieldType::ftAlphaNumeric:
                if ($vv == '') {
                    $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="text" id="' . $id . '" name="' . $id . '" value="' . $val . '" maxlength="' . $len . '" ' . $events . '>';
                    $__default_alphanuma[] = $id;
                } else {
                    $fieldValue = '<select id="' . $id . '" name="' . $id . '" ' . $events . '>';

                    foreach ($vv as $k => $d) {
                        $fieldValue .= '<option value="' . $k . '" ' . (($k==$val)?'selected':'') . '>' . $d . '</option>';
                    }

                    $fieldValue .= '</select>';
                }

                break;
            case enumFieldType::ftNumeric:
                if ($vv == '') {
                    $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="text" id="' . $id . '" name="' . $id . '" value="' . $val . '" maxlength="' . $len . '" ' . $events . '>';

                    if ($lt != '') {
                        $fieldValue .= '<input type="button" value="..." onclick="window.open(\'../includes/chooselist.php?TBLNAM=' . $lt . '&ELEMID=' . $id . '\',\'_blank\')">';
                    }

                    $__default_numerica[] = $id;
                } else {
                    $fieldValue = '<select id="' . $id . '" name="' . $id . '" ' . $events . '>';

                    foreach ($vv as $k => $d) {
                        $fieldValue .= '<option value="' . $k . '" ' . (($k==$val)?'selected':'') . '>' . $d . '</option>';
                    }

                    $fieldValue .= '</select>';
                }

                break;
            case enumFieldType::ftMemo:
                if ($len != '6' && $len != '9') {
                    $len = '3';
                }

                $fieldValue = '<textarea rows="' . $len . '"' . (($hl)?' class="highlight"':'') . ' id="' . $id . '" name="' . $id . '" cols="47" ' . $events . '>' . $val . '</textarea>';
                break;
            case enumFieldType::ftDate:
            case enumFieldType::ftTimestamp:
                if ($val == '' || $val == '0') {
                    $dftval = '';
                } else {
                    $dftval = createDate($val, 'date', false, true);
                }

                $fieldValue = '<input class="' . (($hl)?'highlight':'') . '" accept="date" type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" ' . $events . '>';
                break;
            case enumFieldType::ftTime:
                if ($val == '') {
                    $dftval = '';
                } else {
                    $h = substr($val, 0, 2);
                    $m = substr($val, 2);
                    $t = mktime(intval($h),intval($m));

                    if ($timeformatSession == '24H') {
                        $dftval = date('H:i', $t);
                    } else {
                        $dftval = date('A g:i', $t);
                    }
                }

                $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' accept="time" type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="' . (($timeformatSession=='24H')?'5':'8') .'" ' . $events . '>';
                $__default_timea[] = $id;
                break;
            case enumFieldType::ftHours:
                $dftval = (($val=='')?'':round($val, $hourAccuracy));
                $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' accept="hours" type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30" ' . $events . '>';
                $__default_floata[] = $id;
                break;
            case enumFieldType::ftQuantity:
                $dftval = (($val=='')?'':round($val, $quantityAccuracy));
                $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30" ' . $events . '>';
                $__default_floata[] = $id;
                break;
            case enumFieldType::ftMeasure:
                $dftval = (($val=='')?'':round($val, $measureAccuracy));
                $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30" ' . $events . '>';
                $__default_floata[] = $id;
                break;
            case enumFieldType::ftRate:
                $dftval = (($val=='')?'':round($val, $rateAccuracy));
                $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30" ' . $events . '>';
                $__default_floata[] = $id;
                break;
            case enumFieldType::ftTotals:
                $dftval = (($val=='')?'':round($val, $costAccuracy));
                $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30" ' . $events . '>';
                $__default_floata[] = $id;
                break;
            case enumFieldType::ftPercentage:
                $dftval = (($val=='')?'':round($val, $percentAccuracy));
                $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30" ' . $events . '>';
                $__default_floata[] = $id;
                $__default_nnnuma[] = $id;
                break;
            case enumFieldType::ftBoolean:
                $fieldValue = '<input type="checkbox" id="' . $id . '" name="' . $id . '" value="Y" ' . (($val==enumBoolean::bTrue)?'checked':'') . ' ' . $events . '>';
                break;
            case enumFieldType::ftEmail:
                $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="text" id="' . $id . '" name="' . $id . '" value="' . $val . '" maxlength="255" ' . $events . '>';
                $__default_emaila[] = $id;
                break;
            case enumFieldType::ftUrl:
                $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="text" id="' . $id . '" name="' . $id . '" value="' . $val . '" maxlength="255" ' . $events . '>';
                $__default_urla[] = $id;
                break;
            case enumFieldType::ftPassword:
                $fieldValue = '<input' . (($hl)?' class="highlight"':'') . ' type="password" id="' . $id . '" name="' . $id . '" value="" maxlength="255" ' . $events . '>';
                break;
        }
    }

    if ($block === null) {  // generate input html string only
        return $fieldValue . '<strong id="' . $id . 'S"></strong>' . (($info=='')?'':'<i>' . $info . '</i>');
    } else {                // generate input html string and output it
        $block->contentRow((($req=='Y')?'* ':'') . $label, $fieldValue . '<strong id="' . $id . 'S"></strong>' . (($info=='')?'':'<i>' . $info . '</i>'));
    }
}

/**
 * Get unix timestamp for a date/date offset
 * @param string $dateStr
 * @return int
 */
function mktimeEx($dateStr) {
    $oa = '';
    $val = 0;

    if (preg_match('/^(\d{4})(0\d|10|11|12)([0-2]\d|30|31)$/', $dateStr, $oa)) {
        $val = mktime(0,0,0,intval($oa[2]),intval($oa[3]),intval($oa[1]));
    } else if (preg_match('/^(\+|\-)(\d+)$/', $dateStr, $oa)) {
        if ($oa[1] == '+') {
            $val = mktime(0,0,0,intval(date('n')),intval(date('j'))+intval($oa[2]),intval(date('Y')));
        } else {
            $val = mktime(0,0,0,intval(date('n')),intval(date('j'))-intval($oa[2]),intval(date('Y')));
        }
    } else if (preg_match('/^(\+|\-)(\d+y|)(\d+m|)(\d+w|)(\d+d|)$/', $dateStr, $oa)) {
        $y = ($oa[2]=='')?0:intval(substr($oa[2], 0, -1));
        $n = ($oa[3]=='')?0:intval(substr($oa[3], 0, -1));
        $w = ($oa[4]=='')?0:intval(substr($oa[4], 0, -1));
        $j = ($oa[5]=='')?0:intval(substr($oa[5], 0, -1));

        if ($oa[1] == '+') {
            $val = mktime(0,0,0,intval(date('n'))+$n,intval(date('j'))+7*$w+$j,intval(date('Y'))+$y);
        } else {
            $val = mktime(0,0,0,intval(date('n'))-$n,intval(date('j'))-7*$w-$j,intval(date('Y'))-$y);
        }
    }

    return $val;
}

/*
 * Generate custom fields input html
 *
 * @param class $block Block class
 * @param string $tblnam Table name
 * @param string $id Record id
 * @param string $groupid Custom fields group id
 * @param boolean $pub Published fields only
 * @param boolean $cpy Copy form
 */
function setupCustomFieldsInputHTML($block, $tblnam, $id = '', $groupid = '0', $pub = false, $cpy = false) {
    global $tablePrefix, $row;
    $querymore = " WHERE fdf.tablename='$tblnam' AND fdf.sysdef='N'";

    if ($pub) {
        $querymore .= " AND fdf.published='" . enumPublished::pbYes . "'";
    }

    $querymore .= " ORDER BY fdf.disporder";
    $listFieldsDefinitions = new request();
    $listFieldsDefinitions->openFieldsDefinitions($querymore);
    $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);

    if (preg_match('/@/', $tblnam) == 0) {
        $dbtblnam = $tablePrefix . $tblnam;
    } else {
        $dbtblnam = $tablePrefix . preg_replace_callback('/@/', function ($matches) { return 'FORM_'; }, $tblnam);
    }

    if ($comptListFieldsDefinitions > 0) {
        for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
            $val = $listFieldsDefinitions->fdf_default_value[$i];

            switch ($listFieldsDefinitions->fdf_fieldtype[$i]) {
                case enumFieldType::ftText:
                case enumFieldType::ftMemo:
                    $val = HtmlEntityDecodeNOD($val);
                    break;
                case enumFieldType::ftDate:
                    $val = mktimeEx($val);
                    break;
            }

            $listFieldsDefinitions->org_value[$i] = $val;
        }

        if ($id != '') {
            $flds = 'U_' . implode(',U_', $listFieldsDefinitions->fdf_fieldname);
            $sql = "SELECT $flds FROM $dbtblnam WHERE id='$id'";
            $request1 = new request();
            $request1->connectClass();
            $request1->query($sql);

            if ($request1->fetch()) {
                for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
                    $val = ($row[$i]);

                    if ($listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftText || $listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftMemo || $listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftImage || $listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftAudio || $listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftVideo) {
                        $val = HtmlEntityDecodeNOD($val);
                    }

                    if ($cpy && ($listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftImage || $listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftAudio || $listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftVideo)) {  // no original value for image/audio/video when copying form
                        $val = '';
                    }

                    $listFieldsDefinitions->org_value[$i] = $val;
                }
            }

            $request1->close();
        }

        for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
            if ($listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftSerial) {
                $label = fieldNameString($tblnam, $listFieldsDefinitions->fdf_fieldname[$i]);

                if ($id == '' || $cpy) {    // generate next serial number when adding/copying form
                    switch ($listFieldsDefinitions->fdf_default_value[$i]) {    // serial format

                        case '1':
                            $prefix = date('Y');
                            $sl = intval($listFieldsDefinitions->fdf_length[$i]) - 4;
                            break;
                        case '2':
                            $prefix = date('Y') . strval(intval((intval(date('n'))-1)/3)+1);
                            $sl = intval($listFieldsDefinitions->fdf_length[$i]) - 5;
                            break;
                        case '3':
                            $prefix = date('Ym');
                            $sl = intval($listFieldsDefinitions->fdf_length[$i]) - 6;
                            break;
                        case '4':
                            $y = intval(date('Y'));
                            $m = intval(date('n'));
                            $w = intval(date('W'));

                            if ($m == 1 && $w > 8) {
                                $y -= 1;
                            }

                            $prefix = strval($y) . date('W');
                            $sl = intval($listFieldsDefinitions->fdf_length[$i]) - 6;
                            break;
                        case '5':
                            $prefix = date('Ymd');
                            $sl = intval($listFieldsDefinitions->fdf_length[$i]) - 8;
                            break;
                        default:
                            $prefix = '';
                    }

                    $fieldName = 'U_' . $listFieldsDefinitions->fdf_fieldname[$i];
                    $tmpquery = "SELECT * FROM $dbtblnam WHERE $fieldName LIKE '" . $prefix . "%'";
                    $s = compt($tmpquery) + 1;
                    $fieldValue = sprintf("%s%0" . $sl . "d", $prefix, $s);
                } else {    // serial number is read-only
                    $fieldValue = $listFieldsDefinitions->org_value[$i];
                }

                $block->contentRow($label, $fieldValue);
            } else {
                if ($listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftText || $listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftAlphaNumeric || $listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftNumeric) {
                    $querymore = " WHERE fvd.tablename='$tblnam' AND fvd.fieldname='" . $listFieldsDefinitions->fdf_fieldname[$i] . "' ORDER BY fvd.fieldvalue";
                    $listFieldsValuesDefinitions = new request();
                    $listFieldsValuesDefinitions->openFieldsValuesDefinitions($querymore);
                    $comptListFieldsValuesDefinitions = count($listFieldsValuesDefinitions->fvd_tablename);

                    if ($comptListFieldsValuesDefinitions == '0') {
                        $vv = '';
                    } else {
                        unset($vv);

                        for ($j=0; $j<$comptListFieldsValuesDefinitions; $j++) {
                            $vv[$listFieldsValuesDefinitions->fvd_fieldvalue[$j]] = $listFieldsValuesDefinitions->fvd_fieldvalue_desc[$j];
                        }
                    }
                } else {
                    $vv = '';
                }

                setupInputHTML($block, fieldNameString($tblnam, $listFieldsDefinitions->fdf_fieldname[$i]), 'U' . $groupid . $listFieldsDefinitions->fdf_fieldname[$i], $listFieldsDefinitions->fdf_fieldtype[$i], $listFieldsDefinitions->fdf_required[$i], $listFieldsDefinitions->fdf_length[$i], $listFieldsDefinitions->org_value[$i], $vv, $listFieldsDefinitions->fdf_linkedtable[$i]);
            }
        }
    }
}

/**
 * Generate form detail row input html
 *
 * @global int $hourAccuracy
 * @global int $rateAccuracy
 * @global int $quantityAccuracy
 * @global int $measureAccuracy
 * @global int $costAccuracy
 * @global int $percentAccuracy
 * @global string $timeformatSession
 * @global array $__default_requireda
 * @global array $__default_reqtypea
 * @global array $__default_alphanuma
 * @global array $__default_numerica
 * @global array $__default_floata
 * @global array $__default_timea
 * @global array $__default_emaila
 * @global array $__default_urla
 * @global array $__default_nnnuma
 * @param string $tblid Table element id
 * @param object $fieldsDef Fields definition object
 * @param array $titles Titles
 * @param array $rowData Row data
 * @param string $isNew New row
 * @param string $cpy Copy row
 * @return string
 */
function setupFormDetailRowInputHTML($tblid, $fieldsDef, $titles, $rowData, $isNew = 'false', $cpy = '') {
    global $hourAccuracy, $quantityAccuracy, $measureAccuracy, $rateAccuracy, $costAccuracy, $percentAccuracy, $timeformatSession, $__default_requireda, $__default_reqtypea, $__default_alphanuma, $__default_numerica, $__default_floata, $__default_timea, $__default_emaila, $__default_urla, $__default_nnnuma;
    $newscripts = array();
    $strongs = '';
    $trid = $tblid . 'r' . $rowData['id'];
    echo '<tr id="' . $trid . '"><td><a id="' . $trid . 'delrow" href="javascript:if (typeof onBeforeDelRow===\'function\') { if (onBeforeDelRow($(this))!==true) exit; } delRow(\'' . $tblid . '\',\'' . $trid . '\',' . (($isNew=='true' || $cpy=='true')?'true':'false') . '); if (typeof onAfterDelRow===\'function\') { onAfterDelRow($(this)); }">X</a></td>';

    for ($i=0,$t=0; $i<count($fieldsDef->fdf_tablename); $i++) {
        $id = $trid . 'f' . sprintf('%02d', $i);
        $typ = $fieldsDef->fdf_fieldtype[$i];
        $len = $fieldsDef->fdf_length[$i];
        $val = $rowData[$fieldsDef->fdf_fieldname[$i]];

        if ($typ == enumFieldType::ftImage || $typ == enumFieldType::ftAudio || $typ == enumFieldType::ftVideo) {
            continue;
        }

        if ($typ == enumFieldType::ftText || $typ == enumFieldType::ftAlphaNumeric || $typ == enumFieldType::ftNumeric) {
            $querymore = " WHERE fvd.tablename='" . $fieldsDef->fdf_tablename[$i] . "' AND fvd.fieldname='" . $fieldsDef->fdf_fieldname[$i] . "' ORDER BY fvd.fieldvalue";
            $listFieldsValuesDefinitions = new request();
            $listFieldsValuesDefinitions->openFieldsValuesDefinitions($querymore);
            $comptListFieldsValuesDefinitions = count($listFieldsValuesDefinitions->fvd_tablename);

            if ($comptListFieldsValuesDefinitions == '0') {
                $vv = '';
            } else {
                unset($vv);

                for ($j=0; $j<$comptListFieldsValuesDefinitions; $j++) {
                    $vv[$listFieldsValuesDefinitions->fvd_fieldvalue[$j]] = $listFieldsValuesDefinitions->fvd_fieldvalue_desc[$j];
                }
            }
        } else {
            $vv = '';
        }

        echo '<td data-title="' . $titles[$t++] . '">';

        if ($fieldsDef->fdf_required[$i] == 'Y') {
            if ($isNew == 'true') {
                $newscripts[] = "required=$id,$typ";

                if (!isset($__default_requireda)) {
                    $__default_requireda = array();
                    $__default_reqtypea = array();
                }
            } else {
                $__default_requireda[] = $id;
                $__default_reqtypea[] = $typ;
            }
        }

        switch ($typ) {
            case enumFieldType::ftText:
                if ($vv == '') {
                    $fieldValue = '<input type="text" id="' . $id . '" name="' . $id . '" value="' . $val . '" maxlength="' . $len . '">';
                } else {
                    $fieldValue = '<select id="' . $id . '" name="' . $id . '">';

                    foreach ($vv as $k => $d) {
                        $fieldValue .= '<option value="' . $k . '" ' . (($k==$val)?'selected':'') . '>' . $d . '</option>';
                    }

                    $fieldValue .= '</select>';
                }
                break;
            case enumFieldType::ftAlphaNumeric:
                if ($vv == '') {
                    $fieldValue = '<input type="text" id="' . $id . '" name="' . $id . '" value="' . $val . '" maxlength="' . $len . '">';

                    if ($isNew == 'true') {
                        $newscripts[] = "alphanum=$id";

                        if (!isset($__default_alphanuma)) {
                            $__default_alphanuma = array();
                        }
                    } else {
                        $__default_alphanuma[] = $id;
                    }
                } else {
                    $fieldValue = '<select id="' . $id . '" name="' . $id . '">';

                    foreach ($vv as $k => $d) {
                        $fieldValue .= '<option value="' . $k . '" ' . (($k==$val)?'selected':'') . '>' . $d . '</option>';
                    }

                    $fieldValue .= '</select>';
                }

                break;
            case enumFieldType::ftNumeric:
                if ($vv == '') {
                    $fieldValue = '<input type="text" id="' . $id . '" name="' . $id . '" value="' . $val . '" maxlength="' . $len . '">';

                    if ($fieldsDef->fdf_linkedtable[$i] != '') {
                        $fieldValue .= '<input type="button" value="..." onclick="window.open(\'../includes/chooselist.php?TBLNAM=' . $fieldsDef->fdf_linkedtable[$i] . '&ELEMID=' . $id . '\',\'_blank\')">';
                    }

                    if ($isNew == 'true') {
                        $newscripts[] = "numeric=$id";

                        if (!isset($__default_numerica)) {
                            $__default_numerica = array();
                        }
                    } else {
                        $__default_numerica[] = $id;
                    }
                } else {
                    $fieldValue = '<select id="' . $id . '" name="' . $id . '">';

                    foreach ($vv as $k => $d) {
                        $fieldValue .= '<option value="' . $k . '" ' . (($k==$val)?'selected':'') . '>' . $d . '</option>';
                    }

                    $fieldValue .= '</select>';
                }

                break;
            case enumFieldType::ftMemo:
                $fieldValue = '<input class="colmun_memo" type="text" id="' . $id . '" name="' . $id . '" value="' . $val . '">';
                break;
            case enumFieldType::ftDate:
                if ($val == '') {
                    $dftval = '';
                } else {
                    $dftval = createDate($val, 'date', false, true);
                }

                $fieldValue = '<input accept="date" type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '">';
                break;
            case enumFieldType::ftTime:
                if ($val == '') {
                    $dftval = '';
                } else {
                    $h = substr($val, 0, 2);
                    $m = substr($val, 2);
                    $t = mktime(intval($h),intval($m));

                    if ($timeformatSession == '24H') {
                        $dftval = date('H:i', $t);
                    } else {
                        $dftval = date('A g:i', $t);
                    }
                }

                $fieldValue = '<input accept="time" type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="' . (($timeformatSession=='24H')?'5':'8') .'">';

                if ($isNew == 'true') {
                    $newscripts[] = "time=$id";

                    if (!isset($__default_timea)) {
                        $__default_timea = array();
                    }
                } else {
                    $__default_timea[] = $id;
                }

                break;
            case enumFieldType::ftHours:
                $dftval = (($val=='')?'':round($val, $hourAccuracy));
                $fieldValue = '<input type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30">&nbsp;' . lang_strings("hours");

                if ($isNew == 'true') {
                    $newscripts[] = "float=$id";

                    if (!isset($__default_floata)) {
                        $__default_floata = array();
                    }
                } else {
                    $__default_floata[] = $id;
                }

                break;
            case enumFieldType::ftQuantity:
                $dftval = (($val=='')?'':round($val, $quantityAccuracy));
                $fieldValue = '<input type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30">';

                if ($isNew == 'true') {
                    $newscripts[] = "float=$id";

                    if (!isset($__default_floata)) {
                        $__default_floata = array();
                    }
                } else {
                    $__default_floata[] = $id;
                }

                break;
            case enumFieldType::ftMeasure:
                $dftval = (($val=='')?'':round($val, $measureAccuracy));
                $fieldValue = '<input type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30">';

                if ($isNew == 'true') {
                    $newscripts[] = "float=$id";

                    if (!isset($__default_floata)) {
                        $__default_floata = array();
                    }
                } else {
                    $__default_floata[] = $id;
                }

                break;
            case enumFieldType::ftRate:
                $dftval = (($val=='')?'':round($val, $rateAccuracy));
                $fieldValue = '<input type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30">';

                if ($isNew == 'true') {
                    $newscripts[] = "float=$id";

                    if (!isset($__default_floata)) {
                        $__default_floata = array();
                    }
                } else {
                    $__default_floata[] = $id;
                }

                break;
            case enumFieldType::ftTotals:
                $dftval = (($val=='')?'':round($val, $costAccuracy));
                $fieldValue = '<input type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30">';

                if ($isNew == 'true') {
                    $newscripts[] = "float=$id";

                    if (!isset($__default_floata)) {
                        $__default_floata = array();
                    }
                } else {
                    $__default_floata[] = $id;
                }

                break;
            case enumFieldType::ftPercentage:
                $dftval = (($val=='')?'':round($val, $percentAccuracy));
                $fieldValue = '<input type="text" id="' . $id . '" name="' . $id . '" value="' . $dftval . '" maxlength="30">';

                if ($isNew == 'true') {
                    $newscripts[] = "float=$id";
                    $newscripts[] = "nnnum=$id";

                    if (!isset($__default_floata)) {
                        $__default_floata = array();
                    }

                    if (!isset($__default_nnnuma)) {
                        $__default_nnnuma = array();
                    }
                } else {
                    $__default_floata[] = $id;
                    $__default_nnnuma[] = $id;
                }

                break;
            case enumFieldType::ftBoolean:
                $fieldValue = '<input type="checkbox" id="' . $id . '" name="' . $id . '" value="Y" ' . (($val==enumBoolean::bTrue)?'checked':'') . '>';
                break;
            case enumFieldType::ftEmail:
                $fieldValue = '<input type="text" id="' . $id . '" name="' . $id . '" value="' . $val . '" maxlength="255">';

                if ($isNew == 'true') {
                    $newscripts[] = "email=$id";

                    if (!isset($__default_emaila)) {
                        $__default_emaila = array();
                    }
                } else {
                    $__default_emaila[] = $id;
                }

                break;
            case enumFieldType::ftUrl:
                $fieldValue = '<input type="text" id="' . $id . '" name="' . $id . '" value="' . $val . '" maxlength="255">';

                if ($isNew == 'true') {
                    $newscripts[] = "url=$id";

                    if (!isset($__default_urla)) {
                        $__default_urla = array();
                    }
                } else {
                    $__default_urla[] = $id;
                }

                break;
            case enumFieldType::ftPassword:
                $fieldValue = '<input type="password" id="' . $id . '" name="' . $id . '" value="" maxlength="255">';
                break;
        }

        echo $fieldValue . '</td>';
        $strongs .= '<strong id="' . $id . 'S"></strong>';
    }

    echo '<td>' . $strongs . '</td></tr>';
    return implode('++', $newscripts);
}
/**
 * Generate linked table url
 * @param class $block Block class
 * @param string $lktbl
 * @param string $id
 * @param string $desc
 * @param string $viewurl
 * @global string $path_prefix
 * @return string
 */
function setupLinkedTableURL($block, $lktbl, $id, $desc, $viewurl = '')
{
    global $path_prefix;

    switch ($lktbl) {
        case 'claim_type':
            $url = $block->buildLink($path_prefix . 'expenses/claim_type.php?', $desc, 'out');
            break;
        case 'departments':
            $url = $block->buildLink($path_prefix . 'administration/departments.php?', $desc, 'out');
            break;
        case 'equipment_category':
            $url = $block->buildLink($path_prefix . 'equipments/equipment_category.php?', $desc, 'out');
            break;
        case 'equipment_type':
            $url = $block->buildLink($path_prefix . 'equipments/equipment_type.php?', $desc, 'out');
            break;
        case 'equipments':
            $url = $block->buildLink($path_prefix . 'equipments/viewequipment.php?id=' . $id, $desc, 'out');
            break;
        case 'expense_type':
            $url = $block->buildLink($path_prefix . 'expenses/expense_type.php?', $desc, 'out');
            break;
        case 'expenses':
            $url = $block->buildLink($path_prefix . 'expenses/viewexpense.php?id=' . $id, $desc, 'out');
            break;
        case 'files':
            $url = $block->buildLink($path_prefix . 'linkedcontent/viewfile.php?id=' . $id, $desc, 'out');
            break;
        case 'hour_code':
            $url = $block->buildLink($path_prefix . 'timesheet/hour_code.php?', $desc, 'out');
            break;
        case 'hour_type':
            $url = $block->buildLink($path_prefix . 'timesheet/hour_type.php?', $desc, 'out');
            break;
        case 'invoices':
            $url = $block->buildLink($path_prefix . 'invoices/viewinvoice.php?id=' . $id, $desc, 'out');
            break;
        case 'locations':
            $url = $block->buildLink($path_prefix . 'administration/locations.php', $desc, 'out');
            break;
        case 'materials':
            $url = $block->buildLink($path_prefix . 'materials/viewmaterial.php?id=' . $id, $desc, 'out');
            break;
        case 'meetings':
            $url = $block->buildLink($path_prefix . 'meetings/viewmeeting.php?id=' . $id, $desc, 'out');
            break;
        case 'members':
            $url = $block->buildLink($id, '', 'message');
            break;
        case 'notes':
            $url = $block->buildLink($path_prefix . 'notes/viewnote.php?id=' . $id, $desc, 'out');
            break;
        case 'organizations':
            $url = $block->buildLink($path_prefix . 'clients/viewclient.php?id=' . $id, $desc, 'out');
            break;
        case 'payment_methods':
            $url = $block->buildLink($path_prefix . 'administration/payment_method.php', $desc, 'out');
            break;
        case 'payment_terms':
            $url = $block->buildLink($path_prefix . 'administration/payment_terms.php', $desc, 'out');
            break;
        case 'payments':
            $url = $block->buildLink($path_prefix . 'payments/viewpayment.php?id=' . $id, $desc, 'out');
            break;
        case 'projects':
            $url = $block->buildLink($path_prefix . 'projects/viewproject.php?id=' . $id, $desc, 'out');
            break;
        case 'services':
            $url = $block->buildLink($path_prefix . 'services/viewservice.php?id=' . $id, $desc, 'out');
            break;
        case 'support_requests':
            $url = $block->buildLink($path_prefix . 'support/viewrequest.php?id=' . $id, $desc, 'out');
            break;
        case 'tasks':
            $url = $block->buildLink($path_prefix . 'tasks/viewtask.php?id=' . $id, $desc, 'out');
            break;
        case 'tax_codes':
            $url = $block->buildLink($path_prefix . 'invoices/tax_code.php', $desc, 'out');
            break;
        case 'tax_groups':
            $url = $block->buildLink($path_prefix . 'invoices/tax_group.php', $desc, 'out');
            break;
        case 'topics':
            $url = $block->buildLink($path_prefix . 'topics/viewtopic.php?id=' . $id, $desc, 'out');
            break;
        case 'user_profiles':
            $url = $block->buildLink($path_prefix . 'userprofiles/viewuserprofile.php?id=' . $id, $desc, 'out');
            break;
        default:
            if ($viewurl == '') {
                $url = $block->buildLink($path_prefix . 'forms/viewform.php?tblnam=' . $lktbl . '&id=' . $id, $desc, 'out');
            } else {
                $url = $block->buildLink($path_prefix . $viewurl . '?id=' . $id, $desc, 'out');
            }
    }

    return $url;
}
/**
 * Generate form details html
 *
 * @global string $tablePrefix
 * @global array $row
 * @param class $block Block class
 * @param string $tblnam Table name
 * @param int $id Record id
 * @param boolean $pub Published fields only
 * @param boolean $eml parameter $block is actually a notification class
 * @param string $formPrefix
 */
function setupFormDetailsHTML($block, $tblnam, $id, $pub = false, $eml = false, $formPrefix = '') {
    global $tablePrefix, $row, $sortingUser;
    $querymore = " WHERE tdf.fathertable='$tblnam'";

    if ($pub) {
        $querymore .= " AND tdf.publishenabled='Y'";
    }

    $querymore .= " ORDER BY tdf.tablename";
    $listChildTableDefinitions = new request();
    $listChildTableDefinitions->openTablesDefinitions($querymore);
    $comptListChildTableDefinitions = count($listChildTableDefinitions->tdf_tablename);

    for ($i=0; $i<$comptListChildTableDefinitions; $i++) {
        $viewurl = $listChildTableDefinitions->tdf_viewurl[$i];
        $querymore = " WHERE fdf.tablename='" . $listChildTableDefinitions->tdf_tablename[$i] . "' AND fdf.sysdef='N'";

        if ($pub) {
            $querymore .= " AND fdf.published='" . enumPublished::pbYes . "'";
        }

        $querymore .= " ORDER BY fdf.disporder";
        $listFieldsDefinitions = new request();
        $listFieldsDefinitions->openFieldsDefinitions($querymore);
        $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);

        if ($comptListFieldsDefinitions == '0') {
            continue;
        }

        $sortingFields = array();

        for ($j=0; $j<$comptListFieldsDefinitions; $j++) {
            $sortingFields[] = $listFieldsDefinitions->fdf_dbfieldname[$j];
        }

        if (!$eml) {
            $block1 = new block();
            $block1->form = $block->form . $listChildTableDefinitions->tdf_id[$i];
            $block1->openForm($formPrefix . $block1->form . 'Anchor');
            $sortRef = $listChildTableDefinitions->tdf_tablename[$i];
            $block1->sorting($sortRef, (isset($sortingUser[$sortRef])?$sortingUser[$sortRef]:''), 'id', $sortingFields);
            $sf = str_replace(array('ASC', 'DESC', ' '), '', $block1->sortingValue);
            $sortingValue = (in_array($sf, $listFieldsDefinitions->fdf_dbfieldname)?$block1->sortingValue:'id');
        } else {
            $block1 = $block;
            $sortingValue = 'id';
        }

        $dbtblnam = $tablePrefix . $listChildTableDefinitions->tdf_dbtablename[$i];
        $flds = implode(',', $listFieldsDefinitions->fdf_dbfieldname);
        $sql = "SELECT $flds FROM $dbtblnam WHERE pid='$id' ORDER BY $sortingValue";
        $request1 = new request();
        $request1->connectClass();
        $request1->query($sql);
        $comptRequest1 = 0;

        while ($request1->fetch()) {
            for ($j=0; $j<$comptListFieldsDefinitions; $j++) {
                $request1->data[$comptRequest1][$j] = ($row[$j]);
            }

            $comptRequest1++;
        }

        $request1->close();
        $block1->headingToggle(tableNameString($listChildTableDefinitions->tdf_tablename[$i]));

        if ($comptRequest1 != '0') {
            if ($eml) {
                $block1->openResults();
                $block1->labels($listFieldsDefinitions->fdf_fielddesc);
            } else {
                $block1->openResults(false);
                $block1->labels($listFieldsDefinitions->fdf_fielddesc, 'false');
            }

            for ($j=0; $j<$comptRequest1; $j++) {
                $block1->openRow();

                for ($k=0; $k<$comptListFieldsDefinitions; $k++) {
                    if ($listFieldsDefinitions->fdf_linkedtable[$k] == '') {
                        if ($listFieldsDefinitions->fdf_fieldtype[$k] == enumFieldType::ftPassword) {
                            $fieldValue = (($request1->data[$j][$k]=='')?'':'********');
                        } else {
                            if ($listFieldsDefinitions->fdf_fieldtype[$k] == enumFieldType::ftText || $listFieldsDefinitions->fdf_fieldtype[$k] == enumFieldType::ftAlphaNumeric || $listFieldsDefinitions->fdf_fieldtype[$k] == enumFieldType::ftNumeric) {
                                $querymore = " WHERE fvd.tablename='" . $listChildTableDefinitions->tdf_tablename[$i] . "' AND fvd.fieldname='" . $listFieldsDefinitions->fdf_fieldname[$k] . "' ORDER BY fvd.fieldvalue";
                                $listFieldsValuesDefinitions = new request();
                                $listFieldsValuesDefinitions->openFieldsValuesDefinitions($querymore);
                                $comptListFieldsValuesDefinitions = count($listFieldsValuesDefinitions->fvd_tablename);
                            } else {
                                $comptListFieldsValuesDefinitions = 0;
                            }

                            if ($comptListFieldsValuesDefinitions > 0) {
                                $vv = array();

                                for ($l=0; $l<$comptListFieldsValuesDefinitions; $l++) {
                                    $vv[$listFieldsValuesDefinitions->fvd_fieldvalue[$l]] = $listFieldsValuesDefinitions->fvd_fieldvalue_desc[$l];
                                }
                            } else {
                                $vv = '';
                            }

                            $fieldValue = DbValueDecode($listFieldsDefinitions->fdf_fieldtype[$k], $request1->data[$j][$k], $vv);
                        }
                    } else {
                        if ($request1->data[$j][$k] == '0') {
                            $fieldValue = '';
                        } else {
                            $fieldValue = setupDocDesc($listFieldsDefinitions->fdf_linkedtable[$k], $request1->data[$j][$k]);

                            if (!$eml) {
                                $fieldValue = setupLinkedTableURL($block, $listFieldsDefinitions->fdf_linkedtable[$k], $request1->data[$j][$k], $fieldValue, $viewurl);
                            }
                        }
                    }

                    $block1->cellRow($fieldValue);
                }

                $block1->closeRow();
            }

            $block1->closeResults();

            if (!$eml) {
                $block1->closeFormResults();
            }
        } else {
            $block1->noresults();
        }

        $block1->closeToggle();
    }
}

/**
 * Generate form details input html
 *
 * @global string $tablePrefix
 * @global array $row
 * @param string $tblnam Parent table name
 * @param string $id Record id
 * @param boolean $pub Published fields only
 * @param string $cpy Copy form data
 */
function setupFormDetailsInputHTML($tblnam, $id = '', $pub = false, $cpy = '') {
    global $tablePrefix, $row, $strings, $htmlEntityCharset;
    $querymore = " WHERE tdf.fathertable='$tblnam'";

    if ($pub) {
        $querymore .= " AND tdf.publishenabled='Y'";
    }

    $querymore .= " ORDER BY tdf.tablename";
    $listChildTableDefinitions = new request();
    $listChildTableDefinitions->openTablesDefinitions($querymore);
    $comptListChildTableDefinitions = count($listChildTableDefinitions->tdf_tablename);
    echo '<input type="hidden" name="tblc" value="' . $comptListChildTableDefinitions . '">';

    for ($i=0; $i<$comptListChildTableDefinitions; $i++) {
        $tblid = 'tbl' . sprintf('%02d', $i);
        $querymore = " WHERE fdf.tablename='" . $listChildTableDefinitions->tdf_tablename[$i] . "' AND fdf.sysdef='N'";

        if ($pub) {
            $querymore .= " AND fdf.published='" . enumPublished::pbYes . "'";
        }

        $querymore .= " ORDER BY fdf.disporder";
        $listFieldsDefinitions = new request();
        $listFieldsDefinitions->openFieldsDefinitions($querymore);
        $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);

        if ($comptListFieldsDefinitions == '0') {
            continue;
        }

        $dbtblnam = $tablePrefix . $listChildTableDefinitions->tdf_dbtablename[$i];
        echo '<input type="hidden" id="' . $tblid . 'nam" name="' . $tblid . 'nam" value="' . $listChildTableDefinitions->tdf_dbtablename[$i] . '">';
        echo '<input type="hidden" id="' . $tblid . 'fc" name="' . $tblid . 'fc" value="' . $comptListFieldsDefinitions . '">';
        echo '<input type="hidden" id="' . $tblid . 'del" name="' . $tblid . 'del" value="">';
        $headers = '<thead><tr><th>&nbsp;</th>';
        $titles = array();
        $newRow = array('id' => '**n');

        for ($j=0; $j<$comptListFieldsDefinitions; $j++) {
            echo '<input type="hidden" name="' . $tblid . 'f' . sprintf('%02d', $j) . '" value="' . $listFieldsDefinitions->fdf_dbfieldname[$j] . '">';
            echo '<input type="hidden" name="' . $tblid . 'f' . sprintf('%02d', $j) . 't" value="' . $listFieldsDefinitions->fdf_fieldtype[$j] . '">';
            $val = $listFieldsDefinitions->fdf_default_value[$j];

            switch ($listFieldsDefinitions->fdf_fieldtype[$j]) {
                case enumFieldType::ftText:
                case enumFieldType::ftMemo:
                    $val = HtmlEntityDecodeNOD($val);
                    break;
                case enumFieldType::ftDate:
                    if (!preg_match('/^(\+|\-)\d+$/', $val)) {
                        $val = mktime(0,0,0,intval(substr($val,4,2)),intval(substr($val,6,2)),intval(substr($val,0,4)));
                    } else if (substr($val,0,1) == '+') {
                        $val = mktime(0,0,0,intval(date('n')),intval(date('j'))+intval(substr($val,1)),intval(date('Y')));
                    } else {
                        $val = mktime(0,0,0,intval(date('n')),intval(date('j'))-intval(substr($val,1)),intval(date('Y')));
                    }

                    break;
            }

            $newRow[$listFieldsDefinitions->fdf_fieldname[$j]] = $val;
            $headers .= '<th>' . (($listFieldsDefinitions->fdf_required[$j]=='Y')?'*':'') . $listFieldsDefinitions->fdf_fielddesc[$j] . '</th>';
            $titles[] = (($listFieldsDefinitions->fdf_required[$j]=='Y')?'*':'') . $listFieldsDefinitions->fdf_fielddesc[$j];
        }

        $headers .= '<th></th></tr></thead>';
        $orgIdx = 0;
        $rowIdCollect = array();

        if ($id != '') {
            $flds = implode(',', $listFieldsDefinitions->fdf_dbfieldname);
            $sql = "SELECT id,$flds FROM $dbtblnam WHERE pid='$id' ORDER BY id";
            $request1 = new request();
            $request1->connectClass();
            $request1->query($sql);
            $orgRows = array();

            while ($request1->fetch()) {
                if ($cpy == 'true') {
                    $rowIdCollect[] = $tblid . 'r' . $orgIdx . 'n';
                    $orgRows[$orgIdx]['id'] = $orgIdx . 'n';
                } else {
                    $rowIdCollect[] = $tblid . 'r' . ($row[0]);
                    $orgRows[$orgIdx]['id'] = ($row[0]);
                }

                for ($j=0; $j<$comptListFieldsDefinitions; $j++) {
                    $val = ($row[$j+1]);

                    if ($listFieldsDefinitions->fdf_fieldtype[$j] == enumFieldType::ftText || $listFieldsDefinitions->fdf_fieldtype[$j] == enumFieldType::ftMemo || $listFieldsDefinitions->fdf_fieldtype[$j] == enumFieldType::ftImage || $listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftAudio || $listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftVideo) {
                        $val = HtmlEntityDecodeNOD($val);
                    }

                    $orgRows[$orgIdx][$listFieldsDefinitions->fdf_fieldname[$j]] = $val;
                }

                $orgIdx++;
            }

            $request1->close();
        }

        ob_start();
        $newscripts = setupFormDetailRowInputHTML($tblid, $listFieldsDefinitions, $titles, $newRow, 'true');
        $newRowContent = ob_get_contents();
        ob_end_clean();
        echo '<input type="hidden" id="' . $tblid . 'nrc" name="' . $tblid . 'nrc" value="' . (($cpy=='true')?$orgIdx:'0') . '">';
        echo '<input type="hidden" id="' . $tblid . 'newrow" value="' . htmlentities($newRowContent, ENT_QUOTES, $htmlEntityCharset) . '">';
        echo '<input type="hidden" id="' . $tblid . 'newscript" value="' . $newscripts . '">';
        echo '<input type="hidden" id="' . $tblid . 'rowids" name="' . $tblid . 'rowids" value="' . implode(',', $rowIdCollect) . '">';
        echo '<div class="heading"><h1>' . tableNameString($listChildTableDefinitions->tdf_tablename[$i]) . '</h1></div>';
        echo '<div class="listing"><table id="' . $tblid . '">' . $headers . '<tbody>';

        if ($orgIdx > 0) {
            for ($j=0; $j<$orgIdx; $j++) {
                setupFormDetailRowInputHTML($tblid, $listFieldsDefinitions, $titles, $orgRows[$j], 'false', $cpy);
            }
        }

        echo '</tbody></table></div><div class="clear"><a id="' . $tblid . 'addrow" href="javascript:if (typeof onBeforeAddRow===\'function\') { if (onBeforeAddRow($(this))!==true) exit; } var x=addRow(\'' . $tblid . '\'); if (typeof onAfterAddRow===\'function\') { onBeforeAddRow($(this)); }">' . $strings['more_row'] . '</a></div>';
    }
}

/*
 * Generate default form data check scripts
 */
function setupDefaultFormDataCheckScripts() {
    global $strings, $timeformatSession, $__default_requireda, $__default_reqtypea, $__default_alphanuma, $__default_numerica, $__default_floata, $__default_timea, $__default_emaila, $__default_urla, $__default_pnuma, $__default_nnnuma, $__default_vernuma, $quantityAccuracy, $rateAccuracy, $costAccuracy, $percentAccuracy;
    echo '<input type="hidden" id="__qty_accuracy" value="' . $quantityAccuracy . '">'
        . '<input type="hidden" id="__rate_accuracy" value="' . $rateAccuracy . '">'
        . '<input type="hidden" id="__cost_accuracy" value="' . $costAccuracy . '">'
        . '<input type="hidden" id="__percent_accuracy" value="' . $percentAccuracy . '">'
        . '<input type="hidden" id="__default_vernume" value="' . $strings['alert_versionnumber_field'] . '">';

    if (isset($__default_requireda)) {
        echo '
<input type="hidden" id="__default_requiredl" value="' . implode(',', $__default_requireda) . '">
<input type="hidden" id="__default_reqtypel" value="' . implode(',', $__default_reqtypea) . '">
<input type="hidden" id="__default_requirede" value="' . $strings['alert_mandatory_field'] . '">
';
    }

    if (isset($__default_alphanuma)) {
        echo '
<input type="hidden" id="__default_alphanuml" value="' . implode(',', $__default_alphanuma) . '">
<input type="hidden" id="__default_alphanume" value="' . $strings['alert_alphanum_field'] . '">
';
    }

    if (isset($__default_numerica)) {
        echo '
<input type="hidden" id="__default_numericl" value="' . implode(',', $__default_numerica) . '">
<input type="hidden" id="__default_numerice" value="' . $strings['alert_numerical_field'] . '">
';
    }

    if (isset($__default_floata)) {
        echo '
<input type="hidden" id="__default_floatl" value="' . implode(',', $__default_floata) . '">
<input type="hidden" id="__default_floate" value="' . $strings['alert_float_field'] . '">
';
    }

    if (isset($__default_timea)) {
        echo '
<input type="hidden" id="__default_timel" value="' . implode(',', $__default_timea) . '">
<input type="hidden" id="__default_timee" value="' . $strings['alert_time_field'] . '">
<input type="hidden" id="__default_timef" value="' . $timeformatSession . '">
';
    }

    if (isset($__default_emaila)) {
        echo '
<input type="hidden" id="__default_emaill" value="' . implode(',', $__default_emaila) . '">
<input type="hidden" id="__default_emaile" value="' . $strings['alert_email_field'] . '">
';
    }

    if (isset($__default_urla)) {
        echo '
<input type="hidden" id="__default_urll" value="' . implode(',', $__default_urla) . '">
<input type="hidden" id="__default_urle" value="' . $strings['alert_url_field'] . '">
';
    }

    if (isset($__default_pnuma)) {
        echo '
<input type="hidden" id="__default_pnuml" value="' . implode(',', $__default_pnuma) . '">
<input type="hidden" id="__default_pnume" value="' . $strings['alert_positivenum_field'] . '">
';
    }

    if (isset($__default_nnnuma)) {
        echo '
<input type="hidden" id="__default_nnnuml" value="' . implode(',', $__default_nnnuma) . '">
<input type="hidden" id="__default_nnnume" value="' . $strings['alert_nonnegativenum_field'] . '">
';
    }

    if (isset($__default_vernuma)) {
        echo '<input type="hidden" id="__default_vernuml" value="' . implode(',', $__default_vernuma) . '">';
    } else {
        echo '<input type="hidden" id="__default_vernuml" value="">';
    }
}

/*
 * Generate attach files html
 *
 * @param $opt_publish Display publish checkbox
 */
function setupAttachFilesHTML($opt_publish) {
    global $strings;
    $innerHTML = '<div class="fileInput">'
        . '<div class="contentRow"><div class="leftValue"></div><div class="rightValue"><div data-progress-bar="x"><div class="percent">0%</div></div></div></div>'
        . '<div class="contentRow"><div class="leftValue">' . $strings['comments'] . '</div><div class="rightValue"><input type="text" name="attdesc[]" maxlength="255"></div></div>'
        . '<div class="contentRow"><div class="leftValue">' . $strings['vc_version'] . '</div><div class="rightValue"><input data-version="" type="text" name="attver[]" value="1.0" maxlength="255"><strong data-version=""></strong></div></div>';

    if ($opt_publish) {
        $innerHTML .= '<div class="contentRow"><div class="leftValue">' . $strings['publish'] . '</div><div class="rightValue"><input type="checkbox" name="attpub[]" value="Y"></div></div>';
    }

    $innerHTML .= '</div>';
    echo "<input type='hidden' id='attachContent' value='$innerHTML'>";
    echo "<input type='hidden' id='deleteBtn' value='<input class=\"right\" type=\"button\" value=\"" . $strings['delete'] . "\" onclick=\"RemoveAttach($(this));\">'>";
    echo "<input type='hidden' id='nextAttID' value='0'>";
    $block1 = new block();
    $block1->openContent();
    $block1->contentTitle(lang_strings("attach_documents"));
    $block1->contentRow('<label class="custom-file-input" data-title="' . $strings['choose'] . '"><input id="fileselect" name="fileselect[]" type="file" multiple="multiple"></label>');
    echo '<div id="filelist" class="clear"></div><script>$("#fileselect").change(function(e) {var files = e.target.files || e.dataTransfer.files; UploadFiles(files);});</script>';
    $block1->closeContent();
}

/*
 * Generate projects site attach files html
 */
function setupProjectsSiteAttachFilesHTML() {
    global $strings;
    $innerHTML = '<div class="fileInput">'
        . '<div class="contentRow"><div class="leftValue"></div><div class="rightValue"><div data-progress-bar="x"><div class="percent">0%</div></div></div></div>'
        . '<div class="contentRow"><div class="leftValue">' . $strings['comments'] . '</div><div class="rightValue"><input type="text" name="attdesc[]" maxlength="255"></div></div>'
        . '<div class="contentRow"><div class="leftValue">' . $strings['vc_version'] . '</div><div class="rightValue"><input data-version="" type="text" name="attver[]" value="1.0" maxlength="255"><strong data-version=""></strong></div></div>'
        . '<input type="hidden" name="attpub[]" value="Y">'
        . '</div>';
    echo "<input type='hidden' id='attachContent' value='$innerHTML'>";
    echo "<input type='hidden' id='deleteBtn' value='<input class=\"right\" type=\"button\" value=\"" . $strings['delete'] . "\" onclick=\"RemoveAttach($(this));\">'>";
    echo "<input type='hidden' id='nextAttID' value='0'>";
    $block1 = new block();
    $block1->openContent();
    $block1->contentTitle(lang_strings("attach_documents"));
    $block1->contentRow('<label class="custom-file-input" data-title="' . $strings['choose'] . '"><input id="fileselect" name="fileselect[]" type="file" multiple="multiple"></label>');
    echo '<div id="filelist" class="clear"></div><script>$("#fileselect").change(function(e) {var files = e.target.files || e.dataTransfer.files; UploadFiles(files);});</script>';
    $block1->closeContent();
}

/*
 * Generate upload file html
 *
 * @param string $id Element ID
 * @param string $accept specifies the types of file that the server accepts
 * @param boolean $required required field
 */
function setupUploadFileHTML($id, $accept='', $required=false) {
    global $strings, $__default_requireda, $__default_reqtypea;

    if ($required) {
        $__default_requireda[] = 'attnam' . $id;
        $__default_reqtypea[] = enumFieldType::ftText;
    }

    $result = '<label class="custom-file-input" data-title="' . $strings['choose'] . '"><input id="file' . $id . '" name="file' . $id . '" type="file"' . (($accept=='')?'':' accept="' . $accept . '"') . '></label> <strong id="attnam' . $id . 'S"></strong><div id="progress' . $id . '"><div data-progress-bar="x"><div class="percent">0%</div></div></div><div data-selected-file="x" id="selected' . $id . '" class="hide"><span></span><input type="hidden" id="attnam' . $id . '" name="attnam' . $id . '" value=""><input type="hidden" id="atttmp' . $id . '" name="atttmp' . $id . '" value=""><input type="hidden" id="attreq' . $id . '" value="' . (($required)?'1':'0') . '"></div><script>$("#file' . $id . '").change(function(e) {var files = e.target.files || e.dataTransfer.files; UploadFile("' . $id . '",files);});</script>';
    return $result;
}

/**
 * Check custom fields
 *
 * @global string $tablePrefix
 * @global array $row
 * @global int $hourAccuracy
 * @global int $quantityAccuracy
 * @global int $measureAccuracy
 * @global int $rateAccuracy
 * @global int $costAccuracy
 * @global int $percentAccuracy
 * @global string $timeformatSession
 * @param string $tblnam Table name
 * @param string $id Record id
 * @param string $groupid Custom fields group id
 * @param boolean $pub Published fields only
 * @param boolean $cpy Copy form
 * @return string
 */
function checkCustomFields($tblnam, $id, $groupid = '0', $pub = false, $cpy = false) {
    global $tablePrefix, $hourAccuracy, $quantityAccuracy, $measureAccuracy, $rateAccuracy, $costAccuracy, $percentAccuracy, $timeformatSession;
    $querymore = " WHERE fdf.tablename='$tblnam' AND fdf.sysdef='N' AND fdf.unique_field='Y'";

    if ($pub) {
        $querymore .= " AND fdf.published='" . enumPublished::pbYes . "'";
    }

    $listFieldsDefinitions = new request();
    $listFieldsDefinitions->openFieldsDefinitions($querymore);
    $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);

    if (preg_match('/@/', $tblnam) == 0) {
        $dbtblnam = $tablePrefix . $tblnam;
    } else {
        $dbtblnam = $tablePrefix . preg_replace_callback('/@/', function ($matches) { return 'FORM_'; }, $tblnam);
    }

    if ($comptListFieldsDefinitions != '0') {
        for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
            $fieldName = 'U_' . $listFieldsDefinitions->fdf_fieldname[$i];
            $fldnam = 'U' . $groupid . $listFieldsDefinitions->fdf_fieldname[$i];
            $fieldValue = isset($_POST[$fldnam])?$_POST[$fldnam]:'';
            $tmpquery = "SELECT id FROM $dbtblnam WHERE $fieldName=";
            $skip = false;

            switch ($listFieldsDefinitions->fdf_fieldtype[$i]) {
                case enumFieldType::ftText:
                case enumFieldType::ftNumeric:
                case enumFieldType::ftAlphaNumeric:
                case enumFieldType::ftEmail:
                case enumFieldType::ftUrl:
                    $tmpquery .= "'$fieldValue'";
                    break;
                case enumFieldType::ftDate:
                    $t = date2Timestamp($fieldValue);
                    $fieldValue = date('Ymd', $t);
                    $tmpquery .= "'$fieldValue'";
                    break;
                case enumFieldType::ftTime:
                    if ($timeformatSession == '24H') {
                        list($h, $m) = preg_split('/:/', $fieldValue);
                    } else {
                        if (preg_match('/^am/i', $fieldValue)) {
                            list($h, $m) = preg_split('/:/', preg_replace_callback('/(am|am )/i', function ($matches) { return ''; }, $fieldValue));

                            if ($h == '12') {
                                $h = '0';
                            }

                        } else if (preg_match('/^pm/i', $fieldValue)) {
                            list($h, $m) = preg_split('/:/', preg_replace_callback('/(am|am )/i', function ($matches) { return ''; }, $fieldValue));

                            if ($h != '12') {
                                $h = intval($h) + 12;
                            }
                        } else {
                            list($h, $m) = preg_split('/:/', $fieldValue);

                            if ($h == '12') {
                                $h = '0';
                            }
                        }
                    }

                    $fieldValue = sprintf('%02d%02d', intval($h), intval($m));
                    $tmpquery .= "'$fieldValue'";
                    break;
                case enumFieldType::ftHours:
                    $fieldValue = round($fieldValue, $hourAccuracy);
                    $tmpquery .= $fieldValue;
                    break;
                case enumFieldType::ftQuantity:
                    $fieldValue = round($fieldValue, $quantityAccuracy);
                    $tmpquery .= $fieldValue;
                    break;
                case enumFieldType::ftMeasure:
                    $fieldValue = round($fieldValue, $measureAccuracy);
                    $tmpquery .= $fieldValue;
                    break;
                case enumFieldType::ftRate:
                    $fieldValue = round($fieldValue, $rateAccuracy);
                    $tmpquery .= $fieldValue;
                    break;
                case enumFieldType::ftTotals:
                    $fieldValue = round($fieldValue, $costAccuracy);
                    $tmpquery .= $fieldValue;
                    break;
                case enumFieldType::ftPercentage:
                    $fieldValue = round($fieldValue, $percentAccuracy);
                    $tmpquery .= $fieldValue;
                    break;
                default:
                    $skip = true;
            }

            if ($skip === true) {
                continue;
            }

            if ($id != '' && $cpy == false) {
                $tmpquery .= " AND id<>'$id'";
            }

            $duplicateCheck = new request();
            $duplicateCheck->connectClass();
            $duplicateCheck->query($tmpquery);

            if ($duplicateCheck->fetch()) {
                $dup = true;
            } else {
                $dup = false;
            }

            $duplicateCheck->close();

            if ($dup) {
                $tbldispnam = tableNameString($tblnam);

                if (substr($tblnam,0,1)=='@') {
                    $tbldispnam .= '(' . lang_strings('form_template') . ')';
                }

                $flddispnam = fieldNameString($tblnam, $listFieldsDefinitions->fdf_fieldname[$i]);
                return lang_strings('duplicate_entry_found') . ' ' . $tbldispnam . '.' . $flddispnam;
            }
        }
    }

    return '';
}

/**
 * Upload custom fields images/audio/video & generate custom fields update sql
 *
 * @param string $tblnam Table name
 * @param string $id Record id
 * @param string $groupid Custom fields group id
 * @param boolean $pub Published fields only
 * @return string
 */
function setupCustomMediaFieldsUpdateSQL($tblnam, $id, $groupid = '0', $pub = false) {
    $querymore = " WHERE fdf.tablename='$tblnam' AND fdf.sysdef='N' AND fdf.fieldtype IN ('" . enumFieldType::ftImage . "','" . enumFieldType::ftAudio . "','" . enumFieldType::ftVideo . "')";

    if ($pub) {
        $querymore .= " AND fdf.published='" . enumPublished::pbYes . "'";
    }

    $listFieldsDefinitions = new request();
    $listFieldsDefinitions->openFieldsDefinitions($querymore);
    $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);
    $result = '';

    for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
        $fldnam = 'U' . $groupid . $listFieldsDefinitions->fdf_fieldname[$i];
        $imgidx = isset($_POST['Idx' . $fldnam])?$_POST['Idx' . $fldnam]:'';
        $imgtmp = isset($_POST['atttmp' . $imgidx])?$_POST['atttmp' . $imgidx]:'';
        $imgdel = isset($_POST['Del' . $fldnam])?$_POST['Del' . $fldnam]:'';

        if ($imgdel != 'on' && $imgtmp != '') {
            if ($result !== '') {
                $result .= ',';
            }

            if ($listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftImage) {
                $fieldValue = uploadImage($tblnam, $id, $fldnam, $imgtmp);
            } else {
                $fieldValue = uploadMediaFile($tblnam, $id, $fldnam, $imgtmp);
            }

            $result .= "U_" . $listFieldsDefinitions->fdf_fieldname[$i] . "='$fieldValue'";
        }
    }

    return $result;
}

/**
 * Generate custom fields insert sql
 *
 * @global string $tablePrefix
 * @param string $tblnam Table name
 * @param string $groupid Custom fields group id
 * @param boolean $pub Published fields only
 * @return array 0 - field list, 1 - value list
 */
function setupCustomFieldsInsertSQL($tblnam, $groupid = '0', $pub = false) {
    global $tablePrefix;
    $querymore = " WHERE fdf.tablename='$tblnam' AND fdf.sysdef='N'";

    if ($pub) {
        $querymore .= " AND fdf.published='" . enumPublished::pbYes . "'";
    }

    $listFieldsDefinitions = new request();
    $listFieldsDefinitions->openFieldsDefinitions($querymore);
    $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);
    $results = array(0=>'',1=>'');

    if (preg_match('/@/', $tblnam) == 0) {
        $dbtblnam = $tablePrefix . $tblnam;
    } else {
        $dbtblnam = $tablePrefix . preg_replace_callback('/@/', function ($matches) { return 'FORM_'; }, $tblnam);
    }

    for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
        $results[0] .= ',U_' . $listFieldsDefinitions->fdf_fieldname[$i];
        $fieldName = 'U_' . $listFieldsDefinitions->fdf_fieldname[$i];
        $fldnam = 'U' . $groupid . $listFieldsDefinitions->fdf_fieldname[$i];

        switch ($listFieldsDefinitions->fdf_fieldtype[$i]) {
            case enumFieldType::ftImage:
            case enumFieldType::ftAudio:
            case enumFieldType::ftVideo:
                $results[1] .= ",''";
                break;
            case enumFieldType::ftSerial:
                switch ($listFieldsDefinitions->fdf_default_value[$i]) {    // serial format
                    case '1':
                        $prefix = date('Y');
                        $sl = intval($listFieldsDefinitions->fdf_length[$i]) - 4;
                        break;
                    case '2':
                        $prefix = date('Y') . strval(intval((intval(date('n'))-1)/3)+1);
                        $sl = intval($listFieldsDefinitions->fdf_length[$i]) - 5;
                        break;
                    case '3':
                        $prefix = date('Ym');
                        $sl = intval($listFieldsDefinitions->fdf_length[$i]) - 6;
                        break;
                    case '4':
                        $y = intval(date('Y'));
                        $m = intval(date('n'));
                        $w = intval(date('W'));

                        if ($m == 1 && $w > 8) {
                            $y -= 1;
                        }

                        $prefix = strval($y) . date('W');
                        $sl = intval($listFieldsDefinitions->fdf_length[$i]) - 6;
                        break;
                    case '5':
                        $prefix = date('Ymd');
                        $sl = intval($listFieldsDefinitions->fdf_length[$i]) - 8;
                        break;
                    default:
                        $prefix = '';
                }

                $tmpquery = "SELECT * FROM $dbtblnam WHERE $fieldName LIKE '" . $prefix . "%'";
                $s = compt($tmpquery) + 1;
                $results[1] .= ",'" . sprintf("%s%0" . $sl . "d", $prefix, $s) . "'";
                break;
            default:
                $val = isset($_POST[$fldnam])?$_POST[$fldnam]:'';
                $fieldValue = DbValueEncode($listFieldsDefinitions->fdf_fieldtype[$i], $val);
                $results[1] .= ",'$fieldValue'";
        }
    }

    return $results;
}

/**
 * Generate custom fields update sql
 *
 * @global string $tablePrefix
 * @global array $row
 * @param string $tblnam Table name
 * @param string $id Record id
 * @param string $groupid Custom fields group id
 * @param boolean $pub Published fields only
 * @return string
 */
function setupCustomFieldsUpdateSQL($tblnam, $id, $groupid = '0', $pub = false) {
    global $tablePrefix, $row;
    $querymore = " WHERE fdf.tablename='$tblnam' AND fdf.sysdef='N'";

    if ($pub) {
        $querymore .= " AND fdf.published='" . enumPublished::pbYes . "'";
    }

    $listFieldsDefinitions = new request();
    $listFieldsDefinitions->openFieldsDefinitions($querymore);
    $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);
    $result = '';

    if (preg_match('/@/', $tblnam) == 0) {
        $dbtblnam = $tablePrefix . $tblnam;
    } else {
        $dbtblnam = $tablePrefix . preg_replace_callback('/@/', function ($matches) { return 'FORM_'; }, $tblnam);
    }

    if ($comptListFieldsDefinitions != '0') {
        for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
            $val = $listFieldsDefinitions->fdf_default_value[$i];

            switch ($listFieldsDefinitions->fdf_fieldtype[$i]) {
                case enumFieldType::ftText:
                case enumFieldType::ftMemo:
                    $val = HtmlEntityDecodeNOD($val);
                    break;
                case enumFieldType::ftDate:
                    if (!preg_match('/^(\+|\-)\d+$/', $val)) {
                        $val = mktime(0,0,0,intval(substr($val,4,2)),intval(substr($val,6,2)),intval(substr($val,0,4)));
                    } else if (substr($val,0,1) == '+') {
                        $val = mktime(0,0,0,intval(date('n')),intval(date('j'))+intval(substr($val,1)),intval(date('Y')));
                    } else {
                        $val = mktime(0,0,0,intval(date('n')),intval(date('j'))-intval(substr($val,1)),intval(date('Y')));
                    }

                    break;
            }

            $listFieldsDefinitions->org_value[$i] = $val;
        }

        $flds = 'U_' . implode(',U_', $listFieldsDefinitions->fdf_fieldname);
        $sql = "SELECT $flds FROM $dbtblnam WHERE id='$id'";
        $request1 = new request();
        $request1->connectClass();
        $request1->query($sql);

        if ($request1->fetch()) {
            for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
                $val = ($row[$i]);

                if ($listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftText || $listFieldsDefinitions->fdf_fieldtype[$i] == enumFieldType::ftMemo) {
                    $val = HtmlEntityDecodeNOD($val);
                }

                $listFieldsDefinitions->org_value[$i] = $val;
            }
        }

        $request1->close();

        for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
            $fieldName = 'U_' . $listFieldsDefinitions->fdf_fieldname[$i];
            $fldnam = 'U' . $groupid . $listFieldsDefinitions->fdf_fieldname[$i];

            switch ($listFieldsDefinitions->fdf_fieldtype[$i]) {
                case enumFieldType::ftImage:
                    $imgidx = isset($_POST['Idx' . $fldnam])?$_POST['Idx' . $fldnam]:'';
                    $imgtmp = isset($_POST['atttmp' . $imgidx])?$_POST['atttmp' . $imgidx]:'';
                    $imgdel = isset($_POST['Del' . $fldnam])?$_POST['Del' . $fldnam]:'';

                    if ($imgdel == 'on') {
                        $result .= ",$fieldName=''";
                    } else if ($imgtmp != '') {
                        $fieldValue = uploadImage($tblnam, $id, $fldnam, $imgtmp);
                        $result .= ",$fieldName='$fieldValue'";
                    }

                    break;
                case enumFieldType::ftAudio:
                case enumFieldType::ftVideo:
                    $imgidx = isset($_POST['Idx' . $fldnam])?$_POST['Idx' . $fldnam]:'';
                    $imgtmp = isset($_POST['atttmp' . $imgidx])?$_POST['atttmp' . $imgidx]:'';
                    $imgdel = isset($_POST['Del' . $fldnam])?$_POST['Del' . $fldnam]:'';

                    if ($imgdel == 'on') {
                        $result .= ",$fieldName=''";
                    } else if ($imgtmp != '') {
                        $fieldValue = uploadMediaFile($tblnam, $id, $fldnam, $imgtmp);
                        $result .= ",$fieldName='$fieldValue'";
                    }

                    break;
                case enumFieldType::ftSerial:
                    break;
                case enumFieldType::ftPassword:
                    $fieldValue = isset($_POST[$fldnam])?$_POST[$fldnam]:'';

                    if ($fieldValue != '') {
                        $fieldValue = DbValueEncode($listFieldsDefinitions->fdf_fieldtype[$i], $fieldValue);
                        $result .= ",$fieldName='$fieldValue'";
                    }

                    break;
                default:
                    $fieldValue = isset($_POST[$fldnam])?$_POST[$fldnam]:'';

                    if ($fieldValue != $listFieldsDefinitions->org_value[$i]) {
                        $fieldValue = DbValueEncode($listFieldsDefinitions->fdf_fieldtype[$i], $fieldValue);
                        $result .= ",$fieldName='$fieldValue'";
                    }
            }
        }
    }

    return $result;
}

/**
 * Update form details
 *
 * @global string $tablePrefix
 * @global int $idSession
 * @param class $requestAct Transaction object
 * @param int $id Record id
 * @param int $pro Project id
 */
function updateFormDetails($requestAct, $id, $pro) {
    global $tablePrefix, $idSession;
    $tblc = isset($_POST['tblc'])?$_POST['tblc']:'';    // table counter

    if ($tblc == '') {
        $tblc = 0;
    }

    for ($i=0; $i<$tblc; $i++) {
        $tblid = 'tbl' . sprintf('%02d', $i);   // table element id
        $tblnam = $_POST[$tblid . 'nam'];   // table name
        $fc = $_POST[$tblid . 'fc'];    // field counter
        $del = $_POST[$tblid . 'del'];  // old row id list to be deleted
        $rowids = $_POST[$tblid . 'rowids'];    // row element id list
        $tablename = str_replace('FORM_', '@', $tblnam);
        $querymore = " WHERE tdf.tablename='$tablename'";
        $tableDefinitionDetail = clone $requestAct;
        $tableDefinitionDetail->openTablesDefinitions($querymore);
        $his = $tableDefinitionDetail->tdf_history[0];
        $querymore1 = " WHERE fdf.tablename='$tablename' AND fdf.fieldtype='" . enumFieldType::ftPassword . "'";
        $listPasswordFields = clone $requestAct;
        $listPasswordFields->openFieldsDefinitions($querymore1);
        $comptListPasswordFields = count($listPasswordFields->fdf_tablename);
        // get db field name list & field type list
        unset($f, $ft);

        for ($j=0; $j<$fc; $j++) {
            $f[$j] = $_POST[$tblid . 'f' . sprintf('%02d', $j)];
            $ft[$j] = $_POST[$tblid . 'f' . sprintf('%02d', $j) . 't'];
        }

        // delete old rows
        if ($del != '') {
            $delA = preg_split('/,/', str_replace($tblid . 'r', '', $del));

            foreach ($delA as $del) {
                if ($his == 'Y') {
                    copy2HistoryTable($requestAct, $tablename, $del, 'D');
                }

                $tmpsql = "DELETE FROM " . $tablePrefix . $tblnam . " WHERE id='$del'";

                if ($requestAct->query($tmpsql) === false) {
                    throw new Exception($tmpsql);
                }
            }
        }

        if ($rowids != '') {
            $rowidsA = preg_split('/,/', $rowids);

            foreach ($rowidsA as $rowid) {  // get each row inputs
                unset($fv);

                for ($j=0; $j<$fc; $j++) {  // get each field inputs
                    $fv[$j] = $_POST[$rowid . 'f' . sprintf('%02d', $j)];
                    $fv[$j] = DbValueEncode($ft[$j], $fv[$j]);
                }

                if (preg_match('/^' . $tblid . 'r.+n$/', $rowid)) {
                    // add new row
                    $tmpsql = "INSERT INTO " . $tablePrefix . $tblnam . " (pid,project," . implode(',', $f) . ",created,cuser) VALUES ('$id','$pro','" . implode("','", $fv) . "','" . time() . "','$idSession')";

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }

                    if ($his == 'Y') {
                        $num1 = $requestAct->newId($tablename);
                        copy2HistoryTable($requestAct, $tablename, $num1, 'A');
                    }
                } else {
                    // update old row
                    $rowid = str_replace($tblid . 'r', '', $rowid);
                    $tmpsql = "UPDATE " . $tablePrefix . $tblnam . " SET modified='" . time() . "',muser='$idSession'";

                    for ($j=0; $j<$fc; $j++) {
                        if ($comptListPasswordFields > 0 && in_array($f[$j], $listPasswordFields->fdf_dbfieldname)) {
                            if ($fv[$j] != '') {
                                $tmpsql .= "," . $f[$j] . "='" . $fv[$j] . "'";
                            }
                        } else {
                            $tmpsql .= "," . $f[$j] . "='" . $fv[$j] . "'";
                        }
                    }

                    $tmpsql .= " WHERE id='$rowid'";

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }

                    if ($his == 'Y') {
                        copy2HistoryTable($requestAct, $tablename, $rowid, 'U');
                    }
                }
            }
        }
    }
}

/**
 * Upload attach files (transaction)
 *
 * @global int $idSession
 * @global string $tablePrefix
 * @param class $requestAct Transaction object
 * @param string $doc_type Linked document type
 * @param string $doc_id Linked document id
 * @param string $status linked document status
 * @param int $dir Upload folder id
 * @param string $path Upload folder path
 * @param string $doc_type_ex Linked form name
 */
function uploadAttachFiles($requestAct, $doc_type, $doc_id, $status, $dir, $path, $doc_type_ex = '') {
    global $idSession, $tablePrefix, $row;
    $attnam = isset($_POST['attnam'])?$_POST['attnam']:'';
    $atttmp = isset($_POST['atttmp'])?$_POST['atttmp']:'';
    $attdesc = isset($_POST['attdesc'])?$_POST['attdesc']:'';
    $attver = isset($_POST['attver'])?$_POST['attver']:'';
    $attpub = isset($_POST['attpub'])?$_POST['attpub']:'';

    if (is_array($attnam) && count($attnam) > 0) {
        for ($attIdx=0; $attIdx<count($attnam); $attIdx++) {
            // get project id
            switch ($doc_type) {
                case enumFileRelatedDocumentType::dtProject:
                    $tmpquery = "SELECT id FROM " . $tablePrefix . "projects WHERE id='$doc_id'";
                    break;
                case enumFileRelatedDocumentType::dtPhase:
                    $tmpquery = "SELECT project FROM " . $tablePrefix . "phases WHERE id='$doc_id'";
                    break;
                case enumFileRelatedDocumentType::dtTask:
                    $tmpquery = "SELECT project FROM " . $tablePrefix . "tasks WHERE id='$doc_id'";
                    break;
                case enumFileRelatedDocumentType::dtMeeting:
                    $tmpquery = "SELECT project FROM " . $tablePrefix . "meetings WHERE id='$doc_id'";
                    break;
                case enumFileRelatedDocumentType::dtDiscussion:
                    $tmpquery = "SELECT topic.project FROM " . $tablePrefix . "posts pos INNER JOIN " . $tablePrefix . "topics topic ON topic.id=pos.topic WHERE pos.id='$doc_id'";
                    break;
                case enumFileRelatedDocumentType::dtNote:
                    $tmpquery = "SELECT project FROM " . $tablePrefix . "notes WHERE id='$doc_id'";
                    break;
                case enumFileRelatedDocumentType::dtExpense:
                    $tmpquery = "SELECT project FROM " . $tablePrefix . "expenses WHERE id='$doc_id'";
                    break;
                case enumFileRelatedDocumentType::dtSupportRequest:
                    $tmpquery = "SELECT req.project FROM " . $tablePrefix . "support_requests req WHERE req.id='$doc_id'";
                    break;
                case enumFileRelatedDocumentType::dtSupportPost:
                    $tmpquery = "SELECT req.project FROM " . $tablePrefix . "support_posts pos INNER JOIN " . $tablePrefix . "support_requests req ON req.id=pos.request_id WHERE pos.id='$doc_id'";
                    break;
                case enumFileRelatedDocumentType::dtForm:
                    $tmpquery = "SELECT project FROM " . $tablePrefix . preg_replace_callback("/@/", function ($matches) { return "FORM_"; }, $doc_type_ex) . " WHERE id='$doc_id'";
                    break;
                default:
                    $tmpquery = '';
            }

            if ($tmpquery != '') {
                $request1 = clone $requestAct;
                $request1->query($tmpquery);

                if ($request1->fetch()) {
                    $project = ($row[0]);
                }
            } else {
                $project = '0';
            }

            $attsiz = filesize($atttmp[$attIdx]);
            $nam = DbValueEncode(enumFieldType::ftText, $attnam[$attIdx]);
            $pub = DbValueEncode(enumFieldType::ftBoolean, $attpub[$attIdx]);
            $desc = DbValueEncode(enumFieldType::ftMemo, $attdesc[$attIdx]);
            $ver = ($attver[$attIdx]=='')?'0.0':$attver[$attIdx];
            $tmpsql = "INSERT INTO " . $tablePrefix . "files (owner,doc_type,doc_type_ex,doc_id,name,comments,published,status,vc_version,vc_parent,dir,parent,fullname,project,created,cuser) VALUES ('$idSession','$doc_type','$doc_type_ex','$doc_id','$nam','$desc','$pub','$status','$ver','0','" . enumBoolean::bFalse . "','$dir','','$project','" . time() . "','$idSession')";

            if ($requestAct->query($tmpsql) === false) {
                throw new Exception($tmpsql);
            }

            $num = $requestAct->newId("files");
            $extension = strtolower(substr(strrchr($attnam[$attIdx], ".") , 1));
            $extension_len = strlen($extension) + 1;
            $filename_len = strlen($attnam[$attIdx]);
            $filename_prefix = substr($attnam[$attIdx], 0, $filename_len - $extension_len);
            $atttmp1 = str_replace('\\', '/', $atttmp[$attIdx]);
            $atttmp2 = substr($atttmp1, strpos($atttmp1, str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT'])));
            $realname = uploadFile($path, $atttmp2, $filename_prefix . '--' . $num . '.v' . $ver . '.' . $extension);
            $fullname = HtmlEntitiesNOD(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $path . '/' . $realname));
            $tmpsql = 'UPDATE ' . $tablePrefix . "files SET size='$attsiz',extension='$extension',fullname='$fullname' WHERE id='$num'";

            if ($requestAct->query($tmpsql) === false) {
                throw new Exception($tmpsql);
            }

            copy2HistoryTable($requestAct, 'files', $num, 'A');
        }
    }
}

/**
 * Upload single file (transaction)
 *
 * @global int $idSession
 * @global string $mkdirMethod
 * @global string $ftpRoot
 * @global string $tablePrefix
 * @global array $strings
 * @global string $root
 * @global int $approvalStringLifeTime
 * @param class $requestAct Transaction object
 * @param string $id Element ID
 * @param string $doc_type Linked document type
 * @param string $doc_type_ex Linked form name
 * @param string $doc_id Linked document id
 * @param string $dir Upload folder id
 * @param string $path Upload folder path
 * @param string $vc_version File version
 * @param int $vc_parent Original file id
 * @param string $desc File comments
 * @param string $pub Publish flag
 * @param string $groupid Custom fields group id
 * @param boolean $pubOnly Published fields only
 * @param boolean $upd Update flag
 * @return int Record id
 */
function uploadSingleFile($requestAct, $id, $doc_type, $doc_type_ex, $doc_id, $dir, $path, $vc_version, $vc_parent, $desc, $pub, $groupid = '0', $pubOnly = false, $upd = false) {
    global $idSession, $tablePrefix, $row;
    $atttmp0 = $_POST['atttmp' . $id];
    $atttmp1 = str_replace('\\', '/', $atttmp0);
    $atttmp = substr($atttmp1, strpos($atttmp1, str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT'])));
    $attnam = $_POST['attnam' . $id];
    $attsiz = filesize($atttmp0);
    $status = enumFileStatus::stNoApprovalsNeeded;

    if ($upd) {
        $querymore = " WHERE fdf.tablename='files' AND fdf.sysdef<>'Y'";
        $listFieldsDefinitions = clone $requestAct;
        $listFieldsDefinitions->openFieldsDefinitions($querymore);
        $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);
        $cfds =(($comptListFieldsDefinitions != '0')?',U_' . implode(',U_', $listFieldsDefinitions->fdf_fieldname):'');
        $tmpsql = "INSERT INTO " . $tablePrefix . "files (owner,doc_type,doc_type_ex,doc_id,name,size,extension,comments,published,status,vc_status,vc_version,vc_parent,dir,parent,fullname,project,created,cuser,modified,muser$cfds) SELECT owner,doc_type,doc_type_ex,doc_id,name,size,extension,comments,published,status,'" . enumFileVersionControl::vcVersionUpdate . "',vc_version,id,dir,parent,fullname,project,created,cuser,'" . time() . "','$idSession'$cfds FROM " . $tablePrefix . "files WHERE id='$vc_parent'";
    } else {
        // get project id
        switch ($doc_type) {
            case enumFileRelatedDocumentType::dtProject:
                $tmpquery = "SELECT id FROM " . $tablePrefix . "projects WHERE id='$doc_id'";
                break;
            case enumFileRelatedDocumentType::dtPhase:
                $tmpquery = "SELECT project FROM " . $tablePrefix . "phases WHERE id='$doc_id'";
                break;
            case enumFileRelatedDocumentType::dtTask:
                $tmpquery = "SELECT project FROM " . $tablePrefix . "tasks WHERE id='$doc_id'";
                break;
            case enumFileRelatedDocumentType::dtMeeting:
                $tmpquery = "SELECT project FROM " . $tablePrefix . "meetings WHERE id='$doc_id'";
                break;
            case enumFileRelatedDocumentType::dtDiscussion:
                $tmpquery = "SELECT topic.project FROM " . $tablePrefix . "posts pos INNER JOIN " . $tablePrefix . "topics topic ON topic.id=pos.topic WHERE pos.id='$doc_id'";
                break;
            case enumFileRelatedDocumentType::dtNote:
                $tmpquery = "SELECT project FROM " . $tablePrefix . "notes WHERE id='$doc_id'";
                break;
            case enumFileRelatedDocumentType::dtExpense:
                $tmpquery = "SELECT project FROM " . $tablePrefix . "expenses WHERE id='$doc_id'";
                break;
            case enumFileRelatedDocumentType::dtSupportRequest:
                $tmpquery = "SELECT req.project FROM " . $tablePrefix . "support_requests req WHERE req.id='$doc_id'";
                break;
            case enumFileRelatedDocumentType::dtSupportPost:
                $tmpquery = "SELECT req.project FROM " . $tablePrefix . "support_posts pos INNER JOIN " . $tablePrefix . "support_requests req ON req.id=pos.request_id WHERE pos.id='$doc_id'";
                break;
            case enumFileRelatedDocumentType::dtForm:
                $tmpquery = "SELECT project FROM " . $tablePrefix . preg_replace_callback("/@/", function ($matches) { return "FORM_"; }, $doc_type_ex) . " WHERE id='$doc_id'";
                break;
            default:
                $tmpquery = '';
        }

        if ($tmpquery != '') {
            $request1 = clone $requestAct;
            $request1->query($tmpquery);

            if ($request1->fetch()) {
                $project = ($row[0]);
            }
        } else {
            $project = '0';
        }

        $cfsqls = setupCustomFieldsInsertSQL('files', $groupid, $pubOnly);  // Generate custom fields insert sql
        $tmpsql = "INSERT INTO " . $tablePrefix . "files (owner,doc_type,doc_type_ex,doc_id,name,comments,published,status,vc_status,vc_version,vc_parent,dir,parent,fullname,project,created,cuser" . $cfsqls[0] . ") VALUES ('$idSession','$doc_type','$doc_type_ex','$doc_id','" . HtmlEntitiesNOD($attnam) . "','" . HtmlEntitiesNOD($desc) . "','$pub','$status','" . enumFileVersionControl::vcPeerReview . "','$vc_version','$vc_parent','" . enumBoolean::bFalse . "','$dir','','$project','" . time() . "','$idSession'" . $cfsqls[1] . ")";
    }

    if ($requestAct->query($tmpsql) === false) {
        throw new Exception($tmpsql);
    }

    $num = $requestAct->newId("files");

    if (!$upd) {
        $cfsql = setupCustomMediaFieldsUpdateSQL('files', $num);    // Upload custom fields images & generate custom fields update sql

        if ($cfsql !== '') {
            $tmpsql = "UPDATE " . $tablePrefix . "files SET $cfsql WHERE id='$num'";

            if ($requestAct->query($tmpsql) === false) {
                throw new Exception($tmpsql);
            }
        }
    }

    $result = $num;
    $extension = strtolower(substr(strrchr($attnam, ".") , 1));
    $extension_len = strlen($extension) + 1;
    $filename_len = strlen($attnam);
    $filename_prefix = substr($attnam, 0, $filename_len - $extension_len);
    $realname = uploadFile($path, $atttmp, $filename_prefix . '--' . $num . '.v' . $vc_version . '.' . $extension);
    $fullname = HtmlEntitiesNOD(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $path . '/' . $realname));

    if ($upd) {
        $cfsql = setupCustomFieldsUpdateSQL('files', $vc_parent, $groupid, $pubOnly);  // Generate custom fields update sql
        $tmpsql = 'UPDATE ' . $tablePrefix . "files SET owner='$idSession',name='" . HtmlEntitiesNOD($attnam) . "',size='$attsiz',extension='$extension',comments='" . HtmlEntitiesNOD($desc) . "',status='$status',vc_version='$vc_version',fullname='$fullname',created='" . time() . "',cuser='$idSession',modified=0,muser=0$cfsql WHERE id='$vc_parent'";
        $num = $vc_parent;
    } else {
        $tmpsql = 'UPDATE ' . $tablePrefix . "files SET size='$attsiz',extension='$extension',fullname='$fullname' WHERE id='$num'";
    }

    if ($requestAct->query($tmpsql) === false) {
        throw new Exception($tmpsql);
    }

    $producer = $idSession;
    $doctype = 'files';

    if (!defined('INNER_ADDDOCUMENT')) {
        define('INNER_ADDDOCUMENT', 1);
    }

    require('../approvals/adddocument.php');

    if ($upd) {
        copy2HistoryTable($requestAct, 'files', $result, 'U');
        copy2HistoryTable($requestAct, 'files', $vc_parent, 'A');
    } else {
        copy2HistoryTable($requestAct, 'files', $result, 'A');
    }

    return $result;
}

/**
 * Upload an image
 *
 * @param $tblnam Table name
 * @param $id Record id
 * @param $fldnam Field name
 * @param $imgtmp Image temp file
 * @return string
 */
function uploadImage($tblnam, $id, $fldnam, $imgtmp) {
    $path = date('Ym');     // group images by month
    $imgnam = str_replace('@', 'form', $tblnam) . '-' . $id . '-' . $fldnam;
    uploadFile('Images/' . $path, $imgtmp, $imgnam);
    return HtmlEntitiesNOD($path . '/' . $imgnam);
}

/**
 * Upload a media file
 *
 * @param $tblnam Table name
 * @param $id Record id
 * @param $fldnam Field name
 * @param $fldtmp Media temp file
 * @return string
 */
function uploadMediaFile($tblnam, $id, $fldnam, $fldtmp) {
    $path = date('Ym');     // group files by month
    $nam0 = str_replace('@', 'form', $tblnam) . '-' . $id . '-' . $fldnam;
    uploadFile('Media/' . $path, $fldtmp, $nam0);
    return HtmlEntitiesNOD($path . '/' . $nam0);
}

/**
 * Upload a file
 *
 * @global string $mkdirMethod
 * @global string $ftpRoot
 * @global string $basedir
 * @param string $path path
 * @param string $atttmp temp file
 * @param string $attnam file name
 * @return string URL-encodes file name
 */
function uploadFile($path, $atttmp, $attnam) {
    global $mkdirMethod, $ftpRoot, $basedir;
    $attnamNew = urlencode($attnam);

    if ($mkdirMethod == 'FTP') {
        $pathNew = preg_replace_callback('#/+#', function ($matches) { return '/'; }, $ftpRoot . '/' . $path);
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);

        if (ftp_chdir($ftp, $pathNew) == FALSE) {
            ftp_mkdir($ftp, $pathNew);
            ftp_chdir($ftp, $pathNew);
        }

        ftp_put($ftp, $attnamNew, $atttmp, FTP_BINARY);
        ftp_quit($ftp);
    } else {
        $pathNew = preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files/' . $path);

        if (!file_exists($pathNew)) {
            mkdir($pathNew, 0700, true);
        }

        copy($atttmp, preg_replace_callback('#/+#', function ($matches) { return '/'; }, $pathNew . '/' . $attnamNew));
    }

    @unlink($atttmp);   // delete temp file
    return $attnamNew;
}

/**
 * Add hours for leave request
 *
 * @global string $tablePrefix
 * @global int $idSession
 * @global array $strings
 * @param class $requestAct Transaction object
 * @param int $num The unique id of leave request
 * @param int $applicant The owner of leave request
 * @param int $ty The type of leave request
 * @param timestamp $date_start The start date of leave request
 * @param timestamp $date_end The end date of leave request
 * @param float $hours_start The hours on start date
 * @param float $hours_end The hours on end date
 */
function AddLeaveRequestHours($requestAct, $num, $applicant, $ty, $date_start, $date_end, $hours_start, $hours_end)
{
    global $tablePrefix, $idSession, $strings;
    $comm = HtmlEntitiesNOD($strings['leave_request'] . ' ' . $num);
    $tmpsql = "INSERT INTO " . $tablePrefix . "work_hours (project,work,owner,date,hours,comments,billing,baseref,created,cuser) VALUES ('0','$ty','$applicant','$date_start','$hours_start','$comm','" . enumBoolean::bFalse . "','$num','" . time() . "','$idSession')";

    if ($requestAct->query($tmpsql) === false) {
        throw new Exception($tmpsql);
    }

    if ($date_end != $date_start && $hours_end > 0) {
        $tmpsql = "INSERT INTO " . $tablePrefix . "work_hours (project,work,owner,date,hours,comments,billing,baseref,created,cuser) VALUES ('0','$ty','$applicant','$date_end','$hours_end','$comm','" . enumBoolean::bFalse . "','$num','" . time() . "','$idSession')";

        if ($requestAct->query($tmpsql) === false) {
            throw new Exception($tmpsql);
        }
    }

    $querymore = " WHERE mem.id='$applicant'";
    $memberDetail = clone $requestAct;
    $memberDetail->openMembers($querymore);
    $querymore = " WHERE wcd.workcalendar='" . $memberDetail->mem_workcalendar[0] . "' AND wcd.date>'$date_start' AND wcd.date<'$date_end' ORDER BY wcd.date";
    $listWorkCalendarDetail = clone $requestAct;
    $listWorkCalendarDetail->openWorkCalendarsDetail($querymore);
    $comptListWorkCalendarDetail = count($listWorkCalendarDetail->wcd_id);

    for ($i=0; $i<$comptListWorkCalendarDetail; $i++) {
        $tmpsql = "INSERT INTO " . $tablePrefix . "work_hours (project,work,owner,date,hours,comments,billing,baseref,created,cuser) VALUES ('0','$ty','$applicant','" . $listWorkCalendarDetail->wcd_date[$i] . "','" . $listWorkCalendarDetail->wcd_hours[$i] . "','$comm','" . enumBoolean::bFalse . "','$num','" . time() . "','$idSession')";

        if ($requestAct->query($tmpsql) === false) {
            throw new Exception($tmpsql);
        }
    }
}

/**
 * Format a number with grouped thousands
 *
 * @global string $decimalPoint Separator for the decimal point.
 * @global string $thousandsSeparator   Thousands separator.
 * @param float $number The number being formatted.
 * @param int $decimals Sets the number of decimal points.
 * @param boolean $right_justification   right-justification.
 * @return string   A formatted version of number.
 */
function NumberFormatNOD($number, $decimals, $right_justification = true) {
    global $decimalPoint, $thousandsSeparator;
    $rc = number_format($number, $decimals, $decimalPoint, $thousandsSeparator);

    if ($right_justification) {
        $rc = '<div class="right">' . $rc . '</div>';
    }

    return $rc;
}

/**
 * Format a number with grouped thousands and currency
 *
 * @global string $decimalPoint Separator for the decimal point.
 * @global string $thousandsSeparator   Thousands separator.
 * @global string $currencyOnRight
 * @param float $number
 * @param string $currency
 * @param int $decimals
 * @param bool $right_justification
 */
function CurrencyFormatNOD($number, $currency, $decimals, $right_justification = false) {
    global $decimalPoint, $thousandsSeparator, $currencyOnRight;
    $rc = number_format($number, $decimals, $decimalPoint, $thousandsSeparator);

    if ($currencyOnRight == 'true') {
        $rc = $rc . "&nbsp;" . $currency;
    } else {
        $rc = $currency . "&nbsp;" . $rc;
    }

    if ($right_justification) {
        $rc = '<div class="right">' . $rc . '</div>';
    }

    return $rc;
}

/**
 * Quote string with slashes and strip -- ({ /* to prevent sql injection
 * @param string $input
 * @return string
 */
function AddSlashesNOD($input) {
    $r1 = str_replace(array('--', '({', '/*'), array('', '', ''), $input);
    $r = addslashes($r1);
    return $r;
}

/**
 * Adds an amount of days to a timestamp
 *
 * @param timestamp $ts
 * @param int $n
 * @param bool $dateOnly
 * @return timestamp
 */
function addDays($ts, $n, $dateOnly = true) {
    if ($dateOnly) {
        return mktime(0, 0, 0, intval(date('n', $ts)), intval(date('j', $ts))+$n, intval(date('Y', $ts)));
    } else {
        return mktime(intval(date('H', $ts)), intval(date('i', $ts)), intval(date('s', $ts)), intval(date('n', $ts)), intval(date('j', $ts))+$n, intval(date('Y', $ts)));
    }
}

/**
 * Return the first day of the first week
 *
 * @global string $weekRule
 * @global string $firstDayOfWeek
 * @param int $y
 * @return timestamp
 */
function getDayOfFirstWeek($y) {
    global $weekRule, $firstDayOfWeek;
    $dayOfFirstWeek = mktime(0, 0, 0, 1, 1, $y);
    $N = date('N', $dayOfFirstWeek);

    switch ($weekRule) {
        case '0':   //First day
            while (intval($N) <> intval($firstDayOfWeek)) {
                if (intval($N) > intval($firstDayOfWeek)) {
                    $dayOfFirstWeek = addDays($dayOfFirstWeek, -1);
                } else {
                    $dayOfFirstWeek = addDays($dayOfFirstWeek, 1);
                }

                $N = date('N', $dayOfFirstWeek);
            }

            break;
        case '1':   //First full week
            while (intval($N) <> intval($firstDayOfWeek)) {
                $dayOfFirstWeek = addDays($dayOfFirstWeek, 1);
                $N = date('N', $dayOfFirstWeek);
            }

            break;
        case '2':   //First four day week
            $dayInWeek = 1;

            while (intval($N) <> intval($firstDayOfWeek)) {
                $dayOfFirstWeek = addDays($dayOfFirstWeek, 1);
                $N = date('N', $dayOfFirstWeek);
                $dayInWeek++;
            }

            if ($dayInWeek > 4) {
                $dayOfFirstWeek = addDays($dayOfFirstWeek, -7);
            }
    }

    return $dayOfFirstWeek;
}

/**
 * Returns the week of the year that includes the date in the specified $d value
 *
 * @param timestamp $ts
 * @return string
 */
function getWeekOfYear($ts) {
    $beginOfDay = mktime(0, 0, 0, intval(date('n', $ts)), intval(date('j', $ts)), intval(date('Y', $ts)));
    $y = intval(date('Y', $ts));
    $dayOfFirstWeek = getDayOfFirstWeek($y);
    $dayOfFirstWeekPrev = getDayOfFirstWeek($y-1);
    $dayOfFirstWeekNext = getDayOfFirstWeek($y+1);

    if ($beginOfDay >= $dayOfFirstWeekNext) {
        return sprintf("%04d01", $y+1);
    } else {
        if ($beginOfDay < $dayOfFirstWeek) {
            $y--;
            $lastDayOfWeek = addDays($dayOfFirstWeekPrev, 6);
        } else {
            $lastDayOfWeek = addDays($dayOfFirstWeek, 6);
        }

        $w = 1;

        while ($lastDayOfWeek < $beginOfDay) {
            $lastDayOfWeek = addDays($lastDayOfWeek, 7);
            $w++;
        }

        return sprintf("%04d%02d", $y, $w);
    }
}

/**
 * Returns the days in the specified the week of the year value
 *
 * @param string $fyear year
 * @param string $serialno  the week of the year
 * @return array
 */
function getDaysOfWeek($fyear, $serialno) {
    $dayOfFirstWeek = getDayOfFirstWeek(intval($fyear));
    $dayOfWeek = addDays($dayOfFirstWeek, (intval($serialno)-1)*7);
    $daysOfWeek = array();

    for ($i=0; $i<7; $i++) {
        $daysOfWeek[$i] = addDays($dayOfWeek, $i);
    }

    return $daysOfWeek;
}

/**
 * Initial periods data
 *
 * @global string $periodUnit
 * @global int $autoLockPeriod
 * @global string $tablePrefix
 * @param string $fyear
 * @return string error message
 * @throws Exception
 */
function initialPeriodsOfYear($fyear) {
    global $periodUnit, $autoLockPeriod, $tablePrefix, $idSession;
    $error = '';
    $needCommit = false;
    $requestAct = new request();
    $requestAct->connectClass();     // connect to database server

    try {
        $requestAct->startTransaction();

        if ($periodUnit == 'month') {
            for ($i=1; $i<=12; $i++) {
                $tmpquery = "SELECT * FROM " . $tablePrefix . "periods WHERE fyear='$fyear' AND serial_no='$i'";
                $requestAct->query($tmpquery);

                if ($requestAct->rowCount() > 0) {
                    continue;
                }

                $needCommit = true;
                $date_start = mktime(0, 0, 0, $i, 1, $fyear);
                $period_date_start = mktime(0, 0, 0, $i, 1, intval($fyear));
                $period_date_end = mktime(0, 0, 0, $i+1, 0, intval($fyear));

                if ($autoLockPeriod == '0') {
                    $locked = 'N';
                } else {
                    $n = time();
                    $t = mktime(0, 0, 0, intval($i)+1, intval($autoLockPeriod), intval($fyear));
                    $locked = ($t>=$n)?'N':'Y';
                }

                $tmpsql = "INSERT INTO " . $tablePrefix . "periods (fyear,serial_no,date_start,date_end,locked,created,cuser) VALUES ('$fyear','$i','$period_date_start','$period_date_end','$locked','" . time() . "','$idSession')";

                if ($requestAct->query($tmpsql) === false) {
                    throw new Exception($tmpsql);
                }
            }
        } else {
            $daysOfWeek = getDaysOfWeek($fyear, '1');
            $weekNum = getWeekOfYear($daysOfWeek[0]);

            while (intval(substr($weekNum, 0, 4)) == intval($fyear)) {
                if ($autoLockPeriod == '0') {
                    $locked = 'N';
                } else {
                    $n = time();
                    $t = addDays($daysOfWeek[6], intval($autoLockPeriod));
                    $locked = ($t>=$n)?'N':'Y';
                }

                $W = ltrim(substr($weekNum, 4), '0');
                $tmpquery = "SELECT * FROM " . $tablePrefix . "periods WHERE fyear='$fyear' AND serial_no='$W'";
                $requestAct->query($tmpquery);

                if ($requestAct->rowCount() <= 0) {
                    $needCommit = true;
                    $tmpsql = "INSERT INTO " . $tablePrefix . "periods (fyear,serial_no,date_start,date_end,locked,created,cuser) VALUES ('$fyear','$W','" . $daysOfWeek[0] . "','" . $daysOfWeek[6] . "','$locked','" . time() . "','$idSession')";

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }
                }

                $weekNum = getWeekOfYear(addDays($daysOfWeek[0], 7));   // next week
                $daysOfWeek = getDaysOfWeek(substr($weekNum, 0, 4), substr($weekNum, 4));
            }
        }

        if ($needCommit) {
            $requestAct->endTransaction(enumTransactionEndType::etCommit);
        } else {
            $requestAct->endTransaction(enumTransactionEndType::etRollBack);
        }
    } catch (Exception $e) {
        $error = 'EXCEPTION CAUGHT! LINE ' . $e->getLine() . ' IN ' . $e->getFile();
        error_log($e->getMessage());
        $requestAct->endTransaction(enumTransactionEndType::etRollBack);
    }

    $requestAct->close();
    return $error;
}

/**
 * Return the file name with view & save urls
 *
 * @param string $doc_type
 * @param int $doc_id
 * @param int $id
 * @return string
 */
function fileNameWithURLs($doc_type, $doc_id, $id) {
    $fn = '';
    $block0 = new block();
    /**
     * get file details
     */
    $querymore = " WHERE fil.id='$id'";
    $fileDetail = new request();
    $fileDetail->openFiles($querymore);

    if (count($fileDetail->fil_id) > 0) {
        $fn .= $fileDetail->fil_name[0];
        /**
         * get file revision peers
         */
        $querymore1 = " WHERE fil.vc_parent='$id' AND fil.vc_status='" . enumFileVersionControl::vcPeerReview . "' AND fil.vc_version='" . $fileDetail->fil_vc_version[0] . "' ORDER BY fil.created DESC";
        $listReviews = new request();
        $listReviews->openFiles($querymore1);
        $comptListReviews = count($listReviews->fil_vc_parent);

        if ($comptListReviews > 0) {
            // link to  lastest file revision peer by default
            $id = $listReviews->fil_id[0];
        }

        $fn .= "&nbsp;" . $block0->buildLink("../linkedcontent/accessfile.php?doc_type=$doc_type&amp;doc_id=$doc_id&amp;mode=view&amp;id=$id", lang_strings("view"), 'in');
        $fn .= "&nbsp;" . $block0->buildLink("../linkedcontent/accessfile.php?doc_type=$doc_type&amp;doc_id=$doc_id&amp;mode=download&amp;id=$id", lang_strings("save"), 'in');
    } else {
        $fn = '!!!';
    }

    return $fn;
}

/**
 * Return the status of equipment
 *
 * @global string $tablePrefix
 * @param int $id equipment id
 * @param timestamp $dt
 * @return string
 */
function equipmentStatus($id, $dt = '') {
    global $tablePrefix;
    $querymore = " WHERE equ.id='$id'";
    $equipmentDetail = new request();
    $equipmentDetail->openEquipments($querymore);
    // equipment broken or sold
    if ($equipmentDetail->equ_status[0] == enumEquipmentStatus::stBroken || $equipmentDetail->equ_status[0] == enumEquipmentStatus::stSold) {
        return $equipmentDetail->equ_status[0];
    }

    if ($dt === '') {
        $dt = mktime(0, 0, 0, intval(date('m')), intval(date('d')), intval(date('Y')));  // The timestamp of today
    }

    $tmpquery = "SELECT res.id FROM " . $tablePrefix . "tasks_resource res INNER JOIN " . $tablePrefix . "tasks tas ON tas.id=res.task WHERE res.resource='$id' AND res.type='" . enumResourceType::rtEquipment . "' AND tas.start_date<='$dt' AND ((tas.status='" . enumStatus::stCompleted . "' AND tas.complete_date>='$dt') OR ((tas.status='" . enumStatus::stNotStarted . "' OR tas.status='" . enumStatus::stOpen . "') AND tas.due_date>='$dt'))";

    if (compt($tmpquery) > 0) {
        return enumEquipmentStatus::stOccupied;        // occupied
    } else {
        return enumEquipmentStatus::stAvailable;       // available
    }
}

/**
 * Check if the workcalendar of the current login user has been initialized in the specified period
 *
 * @global string $periodUnit
 * @global string $tablePrefix
 * @global int $wkcalSession
 * @param string $fyear
 * @param string $serialno
 * @return boolean
 */
function workCalendarAvailable($fyear = '', $serialno = '')
{
    global $periodUnit, $tablePrefix, $wkcalSession;

    if ($serialno == "") {
        if ($periodUnit == 'month') {
            $fyear = date("Y");
            $serialno = date("n");
        } else {
            $x = getWeekOfYear(time());
            $fyear = substr($x, 0, 4);
            $serialno = ltrim(substr($x, 4), '0');
        }
    }

    if ($periodUnit == 'month') {
        $date_start_period = mktime(0, 0, 0, intval($serialno), 1, intval($fyear));
        $date_end_period = mktime(0, 0, 0, intval($serialno)+1, 0, intval($fyear));
    } else {
        $daysOfWeek = getDaysOfWeek($fyear, $serialno);
        $date_start_period = $daysOfWeek[0];
        $date_end_period = $daysOfWeek[6];
    }

    $tmpquery = "SELECT wcd.id FROM " . $tablePrefix . "workcalendars_detail wcd WHERE wcd.workcalendar='$wkcalSession' AND wcd.year='$fyear' AND wcd.date>='$date_start_period' AND wcd.date<='$date_end_period'";

    if (compt($tmpquery) === 0) {
        return false;
    } else {
        return true;
    }
}

/**
 * Convert Data Type
 *
 * @global array $fieldDataType
 * @param string $fieldtype1
 * @param int $length1
 * @param string $fieldtype2
 * @param int $length2
 * @return string
 */
function DbCastType($fieldtype1, $length1, $fieldtype2, $length2)
{
    global $fieldDataType;

    if (($fieldtype1 == enumFieldType::ftMemo) || ($fieldtype2 == enumFieldType::ftMemo)) {
        $typ = $fieldDataType[enumFieldType::ftMemo];
    } else {
        switch ($fieldtype1) {
            case enumFieldType::ftText: case enumFieldType::ftAlphaNumeric: case enumFieldType::ftNumeric: case enumFieldType::ftSerial:
                $len1 = $length1;
                break;
            case enumFieldType::ftEmail:
                $len1 = 255;
                break;
            case enumFieldType::ftImage: case enumFieldType::ftAudio: case enumFieldType::ftVideo: case enumFieldType::ftUrl:
                $len1 = 1024;
                break;
            case enumFieldType::ftTime:
                $len1 = 4;
                break;
            case enumFieldType::ftBoolean:
                $len1 = 1;
                break;
            default:
                $len1 = 50;
        }

        switch ($fieldtype2) {
            case enumFieldType::ftText: case enumFieldType::ftAlphaNumeric: case enumFieldType::ftNumeric: case enumFieldType::ftSerial:
                $len2 = $length2;
                break;
            case enumFieldType::ftEmail:
                $len2 = 255;
                break;
            case enumFieldType::ftImage: case enumFieldType::ftAudio: case enumFieldType::ftVideo: case enumFieldType::ftUrl:
                $len2 = 1024;
                break;
            case enumFieldType::ftTime:
                $len2 = 4;
                break;
            case enumFieldType::ftBoolean:
                $len2 = 1;
                break;
            default:
                $len2 = 50;
        }

        $len = (($len1>$len2)?$len1:$len2);
        $typ = 'char(' . $len . ')';
    }

    return $typ;
}

/**
 * Generate form navigation items
 *
 * @global array $SW_Permissions
 * @global string $profilSession
 * @global array $strings
 * @return array
 */
function setupFormNavigationItems()
{
    global $SW_Permissions, $profilSession, $strings;
    $rc = array();
    $querymore = " WHERE tdf.sysdef='N' AND tdf.mainmenu='Y' AND tdf.fathertable='' ORDER BY tdf.tablename";
    $listTablesDefinitions = new request();
    $listTablesDefinitions->openTablesDefinitions($querymore);
    $comptListTablesDefinitions = count($listTablesDefinitions->tdf_tablename);

    for ($i=0; $i<$comptListTablesDefinitions; $i++) {
        $tblnam = $listTablesDefinitions->tdf_tablename[$i];

        if ($SW_Permissions[$tblnam]['Read'][$profilSession]) {
            $x['url'] = "forms/listforms.php?tblnam=$tblnam";
            $x['label'] = $strings['form'] . ' : ' . tableNameString($tblnam);
            $rc[] = $x;
        }
    }

    return $rc;
}

/**
 * Generate extension navigation items
 *
 * @global array $SW_Permissions
 * @global string $profilSession
 * @return array
 */
function setupExtensionNavigationItems()
{
    global $SW_Permissions, $profilSession;
    $rc = array();
    $querymore = " WHERE ext.active='" . enumBoolean::bTrue . "' ORDER BY ext.name";
    $listExtensions = new request();
    $listExtensions->openExtensions($querymore);
    $comptListExtensions = count($listExtensions->ext_name);

    for ($i=0; $i<$comptListExtensions; $i++) {
        $name = $listExtensions->ext_name[$i];
        $func = '#' . $name;

        if ($SW_Permissions[$func]['Read'][$profilSession]) {
            $x['url'] = "extensions/$name/index.php";
            $x['label'] = $listExtensions->ext_title[$i];
            $rc[] = $x;
        }
    }

    return $rc;
}

/**
 * calculate document next recurring date
 * @param int $recurring_start
 * @param int $recurring_finish
 * @param string $recurring_type
 * @param int $recurring_frequency
 * @param boolean $from_tomorrow
 * @return int
 */
function nextRecurringDate($recurring_start, $recurring_finish, $recurring_type, $recurring_frequency, $from_tomorrow=true)
{
    if ($from_tomorrow) {
        $from = mktime(0, 0, 0, date('n'), intval(date('j'))+1, date('Y'));
    } else {
        $from = mktime(0, 0, 0, date('n', $recurring_start), intval(date('j', $recurring_start))+1, date('Y', $recurring_start));
    }

    if (date('Ymd', $from) > date('Ymd', $recurring_finish)) {
        return 0;
    }

    if (date('Ymd', $from) <= date('Ymd', $recurring_start)) {
        return $recurring_start;
    }

    switch ($recurring_type) {
        case 'D':   // daily
            $recurring_next = $recurring_start;

            while (date('Ymd', $recurring_next) < date('Ymd', $from)) {
                $recurring_next = mktime(0, 0, 0, date('n', $recurring_next), intval(date('j', $recurring_next)) + $recurring_frequency, date('Y', $recurring_next));
            }

            break;
        case 'W':   // weekly
            $recurring_next = $recurring_start;

            while (date('Ymd', $recurring_next) < date('Ymd', $from)) {
                $recurring_next = mktime(0, 0, 0, date('n', $recurring_next), intval(date('j', $recurring_next)) + ($recurring_frequency * 7), date('Y', $recurring_next));
            }

            break;
        case 'M':   // monthly
            $recurring_next = $recurring_start;

            while (date('Ymd', $recurring_next) < date('Ymd', $from)) {
                $recurring_next = mktime(0, 0, 0, intval(date('n', $recurring_next)) + $recurring_frequency, date('j', $recurring_next), date('Y', $recurring_next));
            }

            break;
        default:
            $recurring_next = 0;
    }

    if (date('Ymd', $recurring_next) > date('Ymd', $recurring_finish)) {
        return 0;
    } else {
        return $recurring_next;
    }
}

//Function to check if the request is an AJAX request
function is_ajax() {
  return isset($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest';
}

/**
 * Get new invoice number
 * @param class $requestAct Transaction object
 * @param int $dt invoice date timestamp
 * @param int $id excluded invoice id (optional)
 * @return string
 */
function newInvoiceNo($requestAct, $dt, $id='')
{
    $dtF = date('Ym000', $dt);
    $dtT = date('Ym999', $dt);
    $querymore = " WHERE inv.status<>'" . enumDocumentStatus::stDraft . "' AND inv.invoice_no>='$dtF' AND inv.invoice_no<='$dtT' AND LENGTH(inv.invoice_no)=9";

    if ($id != '') {
        $querymore .= " AND inv.id<>'$id'";
    }

    $querymore .= " ORDER BY inv.invoice_no DESC";
    $listInvoices = clone $requestAct;
    $listInvoices->openInvoices($querymore, 0, 1);

    if (count($listInvoices->inv_id) > 0) {
        $n = intval(substr($listInvoices->inv_invoice_no[0], 6,3)) + 1;
    } else {
        $n = 1;
    }

    $invno = date('Ym', $dt) . sprintf('%03s', $n);
    return $invno;
}

/**
 * unzip print layout file
 * @global string $mkdirMethod
 * @global string $ftpRoot
 * @global string $basedir
 * @param string $path
 * @return mixed
 */
function unzipPrintLayout($path)
{
    global $mkdirMethod, $ftpRoot, $basedir;
    $tmpdir = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
    unlink($tmpdir);
    mkdir($tmpdir);

    if ($mkdirMethod == 'FTP') {
        $zipfile = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        $rc = @ftp_get($ftp, $zipfile, preg_replace_callback('#/+#', function ($matches) { return '/'; }, $ftpRoot . "/PrintLayouts/" . $path), FTP_BINARY);
        ftp_quit($ftp);
    } else {
        $zipfile = preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files/PrintLayouts/' . $path);
    }

    $zip = new ZipArchive();

    if ($zip->open($zipfile) !== true) {
        return false;
    }

    $zip->extractTo($tmpdir);
    $zip->close();
    return $tmpdir;
}

/**
 * setup table references
 * @param string $html
 * @param int $tblcnt
 * @param array $names
 * @param array $tablenames
 * @param array $aliases
 * @return mixed
 */
function setupTableReferences($html, &$tblcnt, $names, $tablenames, $aliases)
{
    $newnames = array();
    $newtablenames = array();
    $newdbtablenames = array();
    $newaliases = array();
    $newrelations = array();

    for ($i=0; $i<count($names); $i++) {
        $querymore = " WHERE fdf.tablename='" . $tablenames[$i] . "' AND fdf.linkedtable<>''";
        $listFieldDefinitions = new request();
        $listFieldDefinitions->openFieldsDefinitions($querymore);
        $comptListFieldDefinitions = count($listFieldDefinitions->fdf_tablename);

        for ($j=0; $j<$comptListFieldDefinitions; $j++) {
            if (preg_match('/\[\[' . str_replace('.', '\.', $names[$i]) . '\.' . $listFieldDefinitions->fdf_fieldname[$j] . '\..+\]\]/i', $html)) {
                $newnames[] = $names[$i] . '.' . $listFieldDefinitions->fdf_fieldname[$j];
                $newtablenames[] = $listFieldDefinitions->fdf_linkedtable[$j];
                $newdbtablenames[] = $listFieldDefinitions->fdf_dblinkedtable[$j];
                $newaliases[] = 't' . $tblcnt;
                $newrelations[] = 't' . $tblcnt . '.id=' . $aliases[$i] . '.' . $listFieldDefinitions->fdf_dbfieldname[$i];
                $tblcnt++;
            }
        }
    }

    if (count($newnames) > 0) {
        $more = setupTableReferences($html, $tblcnt, $newnames, $newtablenames, $newaliases);

        if ($more !== false) {
            return array('name' => array_merge($newnames, $more['name']), 'tablename' => array_merge($newtablenames, $more['tablename']), 'dbtablename' => array_merge($newdbtablenames, $more['dbtablename']), 'alias' => array_merge($newaliases, $more['alias']), 'relation' => array_merge($newrelations, $more['relation']));
        } else {
            return array('name' => $newnames, 'tablename' => $newtablenames, 'dbtablename' => $newdbtablenames, 'alias' => $newaliases, 'relation' => $newrelations);
        }
    } else {
        return false;
    }
}

/**
 * prepare user query sql
 * @global string $tablePrefix
 * @param string $userquery
 * @param mixed $num
 * @return string
 */
function prepareUserQuery($userquery, $num = '')
{
    global $tablePrefix;
    $listTableDefinitions = new request();
    $listTableDefinitions->openTablesDefinitions('');

    if ($num === '') {
        $tmpquery = $userquery;
    } else {
        $tmpquery = preg_replace_callback('/\[\[id\]\]/i', function ($matches) { return $num; }, $userquery);
    }

    foreach (array('idSession', 'orgSession', 'timezoneSession', 'dateformatSession', 'timeformatSession', 'languageSession', 'loginSession', 'nameSession', 'ipSession', 'profilSession', 'genderSession', 'wkcalSession') as $sv) {
        $tmpquery = preg_replace_callback('/\[\[' . $sv . '\]\]/i', function ($matches) { return $_SESSION[$sv]; }, $tmpquery);
    }

    $numm = preg_match_all('/\[\[([@|\w]\w+)\]\]/i', $tmpquery, $matches);

    if ($numm > 0) {
        for ($jfe=0; $jfe<$numm; $jfe++) {
            $kfe = array_search($matches[1][$jfe], $listTableDefinitions->tdf_tablename);

            if ($kfe !== FALSE) {
                $tmpquery = str_replace($matches[0][$jfe], $tablePrefix . $listTableDefinitions->tdf_dbtablename[$kfe], $tmpquery);
            }
        }
    }

    return $tmpquery;
}

/**
 * load report built-in variables
 * @param string $html
 * @return string
 */
function loadReportBuiltInVariables($html)
{
    $html0 = preg_replace_callback('/\[\[date\]\]/i', function ($matches) { return createDate(); }, $html);
    $html1 = preg_replace_callback('/\[\[time\]\]/i', function ($matches) { return createDate('', 'time'); }, $html0);
    return $html1;
}

/**
 * load report data sources
 * @global string $tablePrefix
 * @global array $row
 * @param string $html
 * @param string $name
 * @param string $tablename
 * @param string $where_condition
 * @param boolean $multi_rows
 * @return string
 */
function loadReportDataSources($html, $name, $tablename, $where_condition, $multi_rows=false)
{
    global $tablePrefix, $row;
    $matches = array();
    $querymore0 = " WHERE tdf.tablename='$tablename'";
    $tableDefinitionDetail = new request();
    $tableDefinitionDetail->openTablesDefinitions($querymore0);

    if (count($tableDefinitionDetail->tdf_id) == 0) {
        return $html;
    }

    if (preg_match_all('/\[\[' . $name . '\..+\]\]/i', $html, $matches) > 0) {
        $names = array($name);
        $tablenames = array($tablename);
        $dbtablenames = array($tableDefinitionDetail->tdf_dbtablename[0]);
        $aliases = array('t0');
        $where_condition = str_replace($name, 't0', $where_condition);
        $tblcnt = 1;
        $more = setupTableReferences($html, $tblcnt, $names, $tablenames, $aliases);
        $tblnams = array();     // table names
        $fldnams = array();     // field names
        $fields = array();      // field names for sql generation
        $fldtyps = array();     // field types
        $searches = array();    // patterns for final value replacement
        $from = ' FROM ' . $tablePrefix . $tableDefinitionDetail->tdf_dbtablename[0] . ' t0';
        //error_log(print_r($more, true));
        if ($more !== false) {
            for ($i=0; $i<count($more['name']); $i++) {
                $from .= " LEFT OUTER JOIN " . $tablePrefix . $more['dbtablename'][$i] . " " . $more['alias'][$i] . " ON " . $more['relation'][$i];
                $names[] = $more['name'][$i];
                $tablenames[] = $more['tablename'][$i];
                $dbtablenames[] = $more['dbtablename'][$i];
                $aliases[] = $more['alias'][$i];
            }
        }

        for ($i=count($names)-1; $i>=0; $i--) {
            $matches1 = array();

            if (preg_match_all('/\[\[' . str_replace('.', '\.', $names[$i]) . '\..+\]\]/i', $html, $matches1) <= 0) {
                continue;
            }
//error_log(print_r($matches1, true));
            foreach ($matches1[0] as $match) {
                $fieldname0 = str_replace(']]', '', str_replace('[[' . $names[$i] . '.', '', strtolower($match)));

                if (preg_match('/.+::.+/', $fieldname0) == 1) {
                    $array0 = explode('::', $fieldname0);
                    $fieldname = $array0[0];
                    $format = $array0[1];
                } else {
                    $fieldname = $fieldname0;
                    $format = '';
                }

                if ($fieldname == 'id') {
                    $tblnams[] = $tablenames[$i];
                    $fldnams[] = $fieldname;
                    $fields[] = $aliases[$i] . '.id';
                    $fldtyps[] = enumFieldType::ftSerial;
                    $searches[] = '[[' . $aliases[$i] . '.' . $fieldname0 . ']]';
                    $formats[] = $format;
                } else {
                    $querymore = " WHERE fdf.tablename='" . $tablenames[$i] . "' AND fdf.fieldname='$fieldname'";
                    $fieldDefinitionDetail = new request();
                    $fieldDefinitionDetail->openFieldsDefinitions($querymore);

                    if (count($fieldDefinitionDetail->fdf_tablename) > 0) {
                        $tblnams[] = $tablenames[$i];
                        $fldnams[] = $fieldname;
                        $fields[] = $aliases[$i] . '.' . $fieldDefinitionDetail->fdf_dbfieldname[0];
                        $fldtyps[] = $fieldDefinitionDetail->fdf_fieldtype[0];
                        $searches[] = '[[' . $aliases[$i] . '.' . $fieldname0 . ']]';
                        $formats[] = $format;
                    }
                }
            }

            $html = str_replace($names[$i] . '.', $aliases[$i] . '.', $html);
        }

        $tmpquery = "SELECT " . implode(',', $fields) . $from . " WHERE $where_condition";
        $request1 = new request();
        $request1->connectClass();
        $request1->query($tmpquery);
        $request1->data = array();

        while ($request1->fetch()) {
            $request1->data[] = $row;
        }

        $request1->close();
        $htmla = array();

        for ($j=0; $j<count($request1->data); $j++) {
            $html1 = $html;

            for ($i=0; $i<count($fields); $i++) {
                $vv = '';

                if ($fldtyps[$i] == enumFieldType::ftText || $fldtyps[$i] == enumFieldType::ftAlphaNumeric || $fldtyps[$i] == enumFieldType::ftNumeric) {
                    $querymore1 = " WHERE fvd.tablename='" . $tblnams[$i] . "' AND fvd.fieldname='" . $fldnams[$i] . "'";
                    $listFieldsValuesDefinitions = new request();
                    $listFieldsValuesDefinitions->openFieldsValuesDefinitions($querymore1);

                    if (count($listFieldsValuesDefinitions->fvd_id) > 0) {
                        $vv = array();

                        for ($l=0; $l<count($listFieldsValuesDefinitions->fvd_id); $l++) {
                            $vv[$listFieldsValuesDefinitions->fvd_fieldvalue[$l]] = $listFieldsValuesDefinitions->fvd_fieldvalue_desc[$l];
                        }
                    }
                }

                $val = DbValueDecode($fldtyps[$i], $request1->data[$j][$i], $vv);

                if ($formats[$i] !== '') {
                    $val = sprintf($formats[$i], $val);
                }

                $html1 = str_ireplace($searches[$i], $val, $html1);
            }

            if ($multi_rows) {
                $html1 = preg_replace_callback('/\[\[row_number\]\]/i', function ($matches) { return ($j+1); }, $html1);
            }

            $htmla[] = $html1;
        }

        if (count($htmla) > 0) {
            $html = implode('', $htmla);
        }
    }

    return $html;
}

/**
 * load report multi-rows data sources
 * @param string $html
 * @param string $id
 * @return string
 */
function loadReportMultiRowsDataSources($html, $id)
{
    $matches = array();

    if (preg_match_all('/<!--repeat-begin-->(.+)<!--repeat-end-->/si', $html, $matches)) {

        foreach ($matches[0] as $html1) {
            $html2 = str_ireplace('<!--repeat-end-->', '', str_ireplace('<!--repeat-begin-->', '', $html1));
            $matches1 = array();

            if (preg_match_all('/<!--key=(.+)-->/si', $html2, $matches1) !== 1) {
                continue;
            }

            $key = str_ireplace('-->', '', str_ireplace('<!--key=', '', $matches1[0][0]));
            $res = explode('.', $key);

            if (count($res) !== 2) {
                continue;
            }

            $html3 = loadReportDataSources($html2, $res[0], $res[0], "$key='$id'", true);
            $html = str_ireplace($html1, $html3, $html);
        }
    }

    return $html;
}

/**
 * Copy data to history table
 * @global string $tablePrefix
 * @global integer $idSession
 * @param class $requestAct Transaction object
 * @param string $tblnam Table name
 * @param string $ids Record IDs
 * @param string $action A - Add, U - Update, D - Delete
 * @throws Exception
 */
function copy2HistoryTable($requestAct, $tblnam, $ids, $action)
{
    global $tablePrefix, $idSession;

    switch ($tblnam) {
        case 'claim_type':
            $fields = 'name,locked';
            break;
        case 'contacts':
            $fields = 'salutation,first_name,last_name,title,organization,department,do_not_call,email_work,email_home,email_other,phone_work,phone_home,phone_other,mobile,fax,address_street,address_city,address_state,address_postalcode,address_country,alt_address_street,alt_address_city,alt_address_state,alt_address_postalcode,alt_address_country,assistant,assistant_phone,description,shared,member';
            break;
        case 'departments':
            $fields = 'organization,name,location,locked';
            break;
        case 'equipment_category':
            $fields = 'name,locked';
            break;
        case 'equipment_type':
            $fields = 'name,locked';
            break;
        case 'equipments':
            $fields = 'equipment_id,equipment_category,equipment_type,name,owner,location,status,manufacturer,model,serial_number,purchase_date,warranty,cost_currency,cost_price,sell_currency,sell_price,photo,locked';
            break;
        case 'expense_type':
            $fields = 'name,claimtype,locked';
            break;
        case 'expenses':
            $fields = 'project,status,status_approval,printed,owner,description,docdate,billable,claimtype,expensetype,amount,currency,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled';
            break;
        case 'files':
            $fields = 'owner,doc_type,doc_type_ex,doc_id,name,size,extension,comments,published,status,vc_status,vc_version,vc_parent,dir,parent,fullname,project';
            break;
        case 'flextime_adjustment':
            $fields = 'owner,refdate,hours,comments';
            break;
        case 'invoices':
            $fields = 'invoice_no,organization,owner,status,status_approval,printed,doc_date,due_date,payment_terms,payment_method,currency,before_discount,discount_percent,discount_sum,freight,tax_sum,amount,paid,notes,bill_to,ship_to,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,tax_group,tax_group2,tax_group3';
            break;
        case 'invoice_rows':
            $fields = 'invoice,description,project,quantity,price,unit,discount_percent,discount_sum,tax_code,tax_sum,tax_code2,tax_sum2,tax_code3,tax_sum3,line_total';
            break;
        case 'leaves':
            $fields = 'owner,year,type,hours,due_date,comments';
            break;
        case 'leave_requests':
            $fields = 'owner,type,date_start,date_end,time_start,time_end,hours_total,hours_start,hours_end,status,comments';
            break;
        case 'locations':
            $fields = 'name,timezone,language,locked';
            break;
        case 'materials':
            $fields = 'name,unit,currency,costrate,locked';
            break;
        case 'material_issue':
            $fields = 'project,task,owner,material,date,quantity';
            break;
        case 'meetings':
            $fields = 'project,priority,status,owner,name,agenda,location,minutes,duration,start_time,reminder,reminder_time1,reminder_time2,confirmed,allreplied';
            break;
        case 'members':
            $fields = 'organization,department,login,password,name,title,email_work,email_home,phone_work,phone_home,mobile,fax,comments,profil,language,logout_time,start_page,location,date_format,time_format,manager,deputy,theme,active,workcalendar,enable_flextime,begin_flextime,initial_flexibalance,invalid_logon,export_format,ldap_cn,enable_timeclock';
            break;
        case 'extensions':
            $fields = 'name,description,version,active';
            break;
        case 'notes':
            $fields = 'project,owner,topic,subject,description,date,published';
            break;
        case 'organizations':
            $fields = 'name,address,zip_code,city,country,phone,fax,url,email,comments,sscomments,extension_logo,owner,currency,workcalendar,payment_method,payment_terms,locked';
            break;
        case 'payment_methods':
            $fields = 'name,locked';
            break;
        case 'payment_terms':
            $fields = 'name,start_from,extra_months,extra_days,locked';
            break;
        case 'payments':
            $fields = 'invoice,currency,amount,doc_date,payment_method,notes,printed';
            break;
        case 'phases':
            $fields = 'project,phase_set,order_num,status,date_start,date_end,comments';
            break;
        case 'posts':
            $fields = 'topic,member,message';
            break;
        case 'projects':
            $fields = 'organization,owner,priority,status,status_next,status_approval,open_date,complete_date,name,description,published,phase_set,type,upload_dir,currency,permission_set,cost_method,workcalendar';
            break;
        case 'projects_budget':
            $fields = 'project,costtype,currency,amount';
            break;
        case 'project_permissions':
            $fields = 'project,profil,view_project,create_customer_service_site,copy_project,import_project,export_project,modify_project_properties,delete_project,view_project_budget,view_project_cost,view_phase,modify_phase_properties,view_task,create_task,copy_task,modify_task_status,log_task_hours,material_issue,arrange_task_resource,modify_task_properties,edit_task_relationships,delete_task,configure_task_security,publish_task,view_milestone,create_milestone,modify_milestone_properties,delete_milestone,configure_milestone_security,publish_milestone,view_meeting,create_meeting,copy_meeting,modify_meeting_status,log_meeting_hours,arrange_meeting_resource,modify_meeting_properties,delete_meeting,configure_meeting_security,view_support_request,add_support_response,assign_support_request,close_support_request,delete_support_request,view_discussion,add_discussion_topic,post_reply,close_discussion_topic,publish_discussion_topic,delete_discussion_topic,view_team,send_message,add_team_member,publish_team_member,modify_team_member_properties,delete_team_member,view_file,add_file,update_file,publish_file,delete_file,view_note,add_note,modify_note_properties,delete_note,publish_note,view_calendar,add_calendar_event,modify_calendar_event,delete_calendar_event,view_role,configure_role_security,view_expense,create_expense,modify_expense_properties,delete_expense';
            break;
        case 'services':
            $fields = 'name,hourly_rate,hourly_rate_i,currency,locked';
            break;
        case 'support_posts':
            $fields = 'request_id,message,date,owner,project';
            break;
        case 'support_requests':
            $fields = 'owner,status,reproducibility,severity,priority,subject,message,date_open,date_close,project,assigned_to,resolution,auto_close,published';
            break;
        case 'tasks':
            $fields = 'project,priority,status,owner,assigned_to,name,description,start_date,due_date,estimated_time,actual_time,comments,completion,assigned,published,parent_phase,complete_date,service,fixed_duration,milestone,type,task_group,level,parent_task,subtask,support_request';
            break;
        case 'tasks_predecessor':
            $fields = 'task,predecessor,type,lag';
            break;
        case 'tasks_resource':
            $fields = 'project,task,resource,type,quantity';
            break;
        case 'tax_codes':
            $fields = 'tax_group,name,rate';
            break;
        case 'tax_groups':
            $fields = 'name,locked';
            break;
        case 'teams':
            $fields = 'project,member,published,authorized,profil,service,comments';
            break;
        case 'timesheet':
            $fields = 'owner,period,standard_hours,actual_hours,billable_hours,overtime_hours,status,comments';
            break;
        case 'topics':
            $fields = 'project,owner,subject,status,last_post,posts,published';
            break;
        default:
            $querymore1 = " WHERE tdf.tablename='$tblnam'";
            $tableDefinitionDetail = clone $requestAct;
            $tableDefinitionDetail->openTablesDefinitions($querymore1);
            $fields = 'status,printed,project' . (($tableDefinitionDetail->tdf_fathertable[0]=='')?'':',pid') . (($tableDefinitionDetail->tdf_publishenabled[0]=='Y')?',published':'');
    }

    $dbtblnam = $tablePrefix . preg_replace_callback('/@/', function ($matches) { return 'FORM_'; }, $tblnam);
    $querymore2 = " WHERE fdf.tablename='$tblnam' AND fdf.sysdef='N'";
    $listFieldsDefinitions = clone $requestAct;
    $listFieldsDefinitions->openFieldsDefinitions($querymore2);
    $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);

    for ($i=0; $i<$comptListFieldsDefinitions; $i++) {
        $fields .= ',U_' . $listFieldsDefinitions->fdf_fieldname[$i];
    }

    if ($action == 'D') {
        $tmpsql = "INSERT INTO " . $dbtblnam . "_hist (action,id,$fields,created,cuser,modified,muser) SELECT '$action',id,$fields,created,cuser,'" . time() . "','$idSession' FROM " . $dbtblnam . " WHERE id IN ($ids)";
    } else {
        $tmpsql = "INSERT INTO " . $dbtblnam . "_hist (action,id,$fields,created,cuser,modified,muser) SELECT '$action',id,$fields,created,cuser,modified,muser FROM " . $dbtblnam . " WHERE id IN ($ids)";
    }

    if ($requestAct->query($tmpsql) === false) {
        throw new Exception($tmpsql);
    }
}

/**
 * Save current url to `bookmarks` table
 * @global int $idSession
 * @global string $tablePrefix
 * @global string $root
 * @param string $title
 * @param string $url
 * @throws Exception
 */
function logURL($title, $url)
{
    global $idSession, $tablePrefix, $root;
    $logNeeded = true;
    $requestAct = new request();
    $requestAct->connectClass();     // connect to database server
    $url = $root . '/' . str_replace('../', '', $url);

    try {
        $requestAct->startTransaction();
        $querymore = " WHERE boo.cuser='$idSession' ORDER BY boo.created DESC";
        $lastBookmark = clone $requestAct;
        $lastBookmark->openBookmarks($querymore, 0, 1);

        if (count($lastBookmark->boo_title) > 0) {
            if (($url == $lastBookmark->boo_url[0]) && ((time() - $lastBookmark->boo_created[0]) < 300)) {
                $logNeeded = false;
            }
        }

        if ($logNeeded) {
            list($usec, $sec) = explode(" ", microtime());
            $id = sprintf("%024d", $idSession) . sprintf("%024d", $sec) . sprintf("%020d", $usec);
            $title0 = HtmlEntitiesNOD($title);
            $url0 = HtmlEntitiesNOD($url);
            $tmpsql = "INSERT INTO " . $tablePrefix . "bookmarks (id,title,url,created,cuser) VALUES ('$id','$title0','$url0','" . time() . "','$idSession')";

            if ($requestAct->query($tmpsql) === false) {
                throw new Exception($tmpsql);
            }

            $requestAct->endTransaction(enumTransactionEndType::etCommit);
        } else {
            $requestAct->endTransaction(enumTransactionEndType::etRollBack);
        }
    } catch (Exception $e) {
        error_log($e->getMessage());
        $requestAct->endTransaction(enumTransactionEndType::etRollBack);
    }

    $requestAct->close();
}

/**
 * get company logo image path
 * @global string $mkdirMethod
 * @global string $ftpRoot
 * @global string $basedir
 * @param int $id
 * @param string $ext
 * @return string
 */
function getLogo($id, $ext)
{
    global $mkdirMethod, $ftpRoot, $basedir;
    $logosrc = '';

    if ($mkdirMethod == 'FTP') {
        $path = $ftpRoot . '/Logos/' . $id . '.' . $ext;
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        $listFiles = ftp_nlist($ftp, $path);

        if ($listFiles !== FALSE && in_array($path, $listFiles)) {
            $logotmp = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
            ftp_get($ftp, $logotmp, $path, FTP_BINARY);
            $logosrc = 'files/' . basename($logotmp);
        }

        ftp_quit($ftp);
    } else if (file_exists(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files/Logos/' . $id . '.' . $ext))) {
        $logosrc = 'files/Logos/' . $id . '.' . $ext;
    }

    if ($logosrc == '') {
        return '';
    } else {
        return verSource($logosrc);
    }
}

/**
 * get avatar image path
 * @global string $mkdirMethod
 * @global string $ftpRoot
 * @global string $basedir
 * @param int $id user id
 * @param string $ext avatar picture file extension
 * @param string $gender user gender
 * @param boolean $defaultAvatar
 * @return string
 */
function getAvatar($id, $ext, $gender, $defaultAvatar = true)
{
    global $mkdirMethod, $ftpRoot, $basedir;
    $avasrc = '';

    if ($mkdirMethod == 'FTP') {
        $path = $ftpRoot . '/Avatars/' . $id . '.' . $ext;
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        $listFiles = ftp_nlist($ftp, $path);

        if ($listFiles !== FALSE && in_array($path, $listFiles)) {
            $avatmp = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
            ftp_get($ftp, $avatmp, $path, FTP_BINARY);
            $avasrc = 'files/' . basename($avatmp);
        }

        ftp_quit($ftp);
    } else if (file_exists(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files/Avatars/' . $id . '.' . $ext))) {
        $avasrc = 'files/Avatars/' . $id . '.' . $ext;
    }

    if ($avasrc == '' && $defaultAvatar) {
        $i = (intval($id) % 6) + 1;
        $avasrc = 'includes/avatars/' . strtolower($gender) . $i . '.png';
    }

    if ($avasrc == '') {
        return '';
    } else {
        return verSource($avasrc);
    }
}

/**
 * append file modification time to image path/url
 * @global string $path_prefix
 * @param string $path
 * @return string
 */
function verSource($path)
{
    global $path_prefix;
    return $path_prefix . $path . '?ver=' . filemtime($path_prefix . $path);
}

/**
 * get message photo image path
 * @global string $mkdirMethod
 * @global string $ftpRoot
 * @global string $basedir
 * @param int $id
 * @param string $ext
 * @return string
 */
function getMsgPhoto($id, $ext)
{
    global $mkdirMethod, $ftpRoot, $basedir;
    $photosrc = '';

    if ($mkdirMethod == 'FTP') {
        $path = $ftpRoot . '/Messages/' . $id . '.' . $ext;
        $ftp = ftp_connect(FTPSERVER);
        ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
        $listFiles = ftp_nlist($ftp, $path);

        if ($listFiles !== FALSE && in_array($path, $listFiles)) {
            $phototmp = tempnam(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files'), 'tmp');
            ftp_get($ftp, $phototmp, $path, FTP_BINARY);
            $photosrc = '../files/' . basename($phototmp);
        }

        ftp_quit($ftp);
    } else if (file_exists(preg_replace_callback('#/+#', function ($matches) { return '/'; }, $basedir . '/files/Messages/' . $id . '.' . $ext))) {
        $photosrc = '../files/Messages/' . $id . '.' . $ext;
    }

    return $photosrc;
}

/**
 * update chat session data
 * @global integer $idSession
 * @global string $tablePrefix
 * @param string $chatType
 * @param integer $chatTo
 */
function updateChatSession($chatType, $chatTo)
{
    global $idSession, $tablePrefix;
    $tmpquery1 = "SELECT member FROM " . $tablePrefix . "chat_sessions WHERE member='$idSession'";

    if (compt($tmpquery1) > 0) {
        $tmpsql = "UPDATE " . $tablePrefix . "chat_sessions SET chat_type='$chatType',chat_to='$chatTo',last_access='" . time() . "' WHERE member='$idSession'";
    } else {
        $tmpsql = "INSERT INTO " . $tablePrefix . "chat_sessions (member,chat_type,chat_to,last_access) VALUES ('$idSession','$chatType','$chatTo','" . time() . "')";
    }

    connectSql($tmpsql);
}

/**
 * delete chat session data
 * @global integer $idSession
 * @global string $tablePrefix
 */
function clearChatSession()
{
    global $idSession, $tablePrefix;
    $tmpquery1 = "SELECT member FROM " . $tablePrefix . "chat_sessions WHERE member='$idSession'";

    if (compt($tmpquery1) > 0) {
        $tmpsql = "DELETE FROM " . $tablePrefix . "chat_sessions WHERE member='$idSession'";
        connectSql($tmpsql);
    }
}

/**
 * create custom field XML document
 * @param requestObject $listFieldsDefinitions
 * @return \DOMDocument
 */
function exportCustomFields($listFieldsDefinitions)
{
    $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);
    // create XML document
    $doc = new DOMDocument('1.0', 'UTF-8');
    $doc->formatOutput = true;
    // The collection of custom fields
    $custom_fields = $doc->createElement('custom_fields');
    $doc->appendChild($custom_fields);

    for ($j=0; $j<$comptListFieldsDefinitions; $j++) {
        $custom_field = $doc->createElement('custom_field');    // custom field
        $custom_fields->appendChild($custom_field);
        $custom_field->appendChild($doc->createElement('table', $listFieldsDefinitions->fdf_tablename[$j])); // The name of system table
        $custom_field->appendChild($doc->createElement('name', $listFieldsDefinitions->fdf_fieldname[$j])); // The name of custom field
        $custom_field->appendChild($doc->createElement('description', $listFieldsDefinitions->fdf_fielddesc[$j]));  // The description of custom field
        $custom_field->appendChild($doc->createElement('type', $listFieldsDefinitions->fdf_fieldtype[$j])); // The type of custom field
        $custom_field->appendChild($doc->createElement('length', $listFieldsDefinitions->fdf_length[$j]));  // The length of custom field
        $custom_field->appendChild($doc->createElement('default_value', $listFieldsDefinitions->fdf_default_value[$j]));    // The default value of custom field
        $custom_field->appendChild($doc->createElement('required', $listFieldsDefinitions->fdf_required[$j]));  // Indicate a required custom field
        $custom_field->appendChild($doc->createElement('unique', $listFieldsDefinitions->fdf_unique_field[$j]));    // Indicate a unique custom field
        $custom_field->appendChild($doc->createElement('create_index', $listFieldsDefinitions->fdf_create_index[$j]));  // Create an index for custom field
        $custom_field->appendChild($doc->createElement('published', $listFieldsDefinitions->fdf_published[$j]));    // Indicate the custom field published
        $custom_field->appendChild($doc->createElement('linked_table', $listFieldsDefinitions->fdf_linkedtable[$j]));   // The name of linked table
        // find valid values of custom field
        $querymore = " WHERE fvd.tablename='" . $listFieldsDefinitions->fdf_tablename[$j] . "' AND fvd.fieldname='" . $listFieldsDefinitions->fdf_fieldname[$j] . "'";
        $listFieldsValuesDefinitions = new request();
        $listFieldsValuesDefinitions->openFieldsValuesDefinitions($querymore);
        $comptListFieldsValuesDefinitions = count($listFieldsValuesDefinitions->fvd_id);

        if ($comptListFieldsValuesDefinitions != '0') {
            // The collection of valid values
            $valid_values = $doc->createElement('valid_values');
            $custom_field->appendChild($valid_values);

            for ($k=0; $k<$comptListFieldsValuesDefinitions; $k++) {
                $valid_value = $doc->createElement('valid_value');  // valid value
                $valid_values->appendChild($valid_value);
                $valid_value->appendChild($doc->createElement('value', $listFieldsValuesDefinitions->fvd_fieldvalue[$k]));  // The value of valid value
                $valid_value->appendChild($doc->createElement('description', $listFieldsValuesDefinitions->fvd_fieldvalue_desc[$k]));   // The description of valid value
            }
        }
    }

    return $doc;
}

/**
 * create form template XML document
 * @param requestObject $listTablesDefinitions
 * @return \DOMDocument
 */
function exportFormTemplates($listTablesDefinitions)
{
    $comptListTablesDefinitions = count($listTablesDefinitions->tdf_tablename);
    // create XML document
    $doc = new DOMDocument('1.0', 'UTF-8');
    $doc->formatOutput = true;
    // The collection of form templates
    $form_templates = $doc->createElement("form_templates");
    $doc->appendChild($form_templates);

    for ($i=0; $i<$comptListTablesDefinitions; $i++) {
        // find the custom fields of form template
        $querymore = " WHERE fdf.tablename='" . $listTablesDefinitions->tdf_tablename[$i] . "' AND fdf.sysdef='N'";
        $listFieldsDefinitions = new request();
        $listFieldsDefinitions->openFieldsDefinitions($querymore);
        $comptListFieldsDefinitions = count($listFieldsDefinitions->fdf_tablename);

        if ($comptListFieldsDefinitions == '0' && $listTablesDefinitions->tdf_sysdef[$i] == 'Y') {
            // skip if this is a system table without custom fields
            continue;
        }

        $form_template = $doc->createElement('form_template');  // form template
        $form_templates->appendChild($form_template);
        $form_template->appendChild($doc->createElement('name', $listTablesDefinitions->tdf_tablename[$i]));    // The name of form template

        if ($listTablesDefinitions->tdf_sysdef[$i] == 'N') {    // this is not a system table
            $form_template->appendChild($doc->createElement('description', tableNameString($listTablesDefinitions->tdf_tablename[$i])));    // The description of form template
            $form_template->appendChild($doc->createElement('printable', $listTablesDefinitions->tdf_printable[$i]));   // printable table
            $form_template->appendChild($doc->createElement('mainmenu', $listTablesDefinitions->tdf_mainmenu[$i]));     // create main menu item
            $form_template->appendChild($doc->createElement('history', $listTablesDefinitions->tdf_history[$i]));       // create history log table
            $form_template->appendChild($doc->createElement('view_url', $listTablesDefinitions->tdf_viewurl[$i]));      // custom view url
            $form_template->appendChild($doc->createElement('key_fields', $listTablesDefinitions->tdf_keyfields[$i]));  // The key fields of form template
            $form_template->appendChild($doc->createElement('display_fields', $listTablesDefinitions->tdf_dispfields[$i])); // The fields shown in search results
            $form_template->appendChild($doc->createElement('display_style', $listTablesDefinitions->tdf_dispstyle[$i]));   // The display style of key fields
            $form_template->appendChild($doc->createElement('form_fields', $listTablesDefinitions->tdf_formfields[$i]));    // The fields shown in form list
            $form_template->appendChild($doc->createElement('list_fields', $listTablesDefinitions->tdf_listfields[$i]));    // The fields shown in choose list
            $form_template->appendChild($doc->createElement('primary_fields', $listTablesDefinitions->tdf_primfields[$i])); // The primary search fields
            $form_template->appendChild($doc->createElement('father_table', $listTablesDefinitions->tdf_fathertable[$i]));  // The parent form template
            $form_template->appendChild($doc->createElement('approval_enabled', $listTablesDefinitions->tdf_approvalenabled[$i]));  // Enable approvals
            $form_template->appendChild($doc->createElement('publish_enabled', $listTablesDefinitions->tdf_publishenabled[$i]));    // Enable publish to customer service site
        }

        if ($comptListFieldsDefinitions != '0') {
            // The collection of custom fields
            $custom_fields = $doc->createElement('custom_fields');
            $form_template->appendChild($custom_fields);

            for ($j=0; $j<$comptListFieldsDefinitions; $j++) {
                $custom_field = $doc->createElement('custom_field');    // custom field
                $custom_fields->appendChild($custom_field);
                $custom_field->appendChild($doc->createElement('name', $listFieldsDefinitions->fdf_fieldname[$j])); // The name of custom field
                $custom_field->appendChild($doc->createElement('description', $listFieldsDefinitions->fdf_fielddesc[$j]));  // The description of custom field
                $custom_field->appendChild($doc->createElement('type', $listFieldsDefinitions->fdf_fieldtype[$j])); // The type of custom field
                $custom_field->appendChild($doc->createElement('length', $listFieldsDefinitions->fdf_length[$j]));  // The length of custom field
                $custom_field->appendChild($doc->createElement('default_value', $listFieldsDefinitions->fdf_default_value[$j]));    // The default value of custom field
                $custom_field->appendChild($doc->createElement('required', $listFieldsDefinitions->fdf_required[$j]));  // Indicate a required custom field
                $custom_field->appendChild($doc->createElement('unique', $listFieldsDefinitions->fdf_unique_field[$j]));    // Indicate a unique custom field
                $custom_field->appendChild($doc->createElement('create_index', $listFieldsDefinitions->fdf_create_index[$j]));  // Create an index for custom field
                $custom_field->appendChild($doc->createElement('published', $listFieldsDefinitions->fdf_published[$j]));    // Indicate the custom field published
                $custom_field->appendChild($doc->createElement('linked_table', $listFieldsDefinitions->fdf_linkedtable[$j]));   // The name of linked table
                $custom_field->appendChild($doc->createElement('display_order', $listFieldsDefinitions->fdf_disporder[$j]));    // The display order of custom field
                // find valid values of custom field
                $querymore = " WHERE fvd.tablename='" . $listTablesDefinitions->tdf_tablename[$i] . "' AND fvd.fieldname='" . $listFieldsDefinitions->fdf_fieldname[$j] . "'";
                $listFieldsValuesDefinitions = new request();
                $listFieldsValuesDefinitions->openFieldsValuesDefinitions($querymore);
                $comptListFieldsValuesDefinitions = count($listFieldsValuesDefinitions->fvd_id);

                if ($comptListFieldsValuesDefinitions != '0') {
                    // The collection of valid values
                    $valid_values = $doc->createElement('valid_values');
                    $custom_field->appendChild($valid_values);

                    for ($k=0; $k<$comptListFieldsValuesDefinitions; $k++) {
                        $valid_value = $doc->createElement('valid_value');  // valid value
                        $valid_values->appendChild($valid_value);
                        $valid_value->appendChild($doc->createElement('value', $listFieldsValuesDefinitions->fvd_fieldvalue[$k]));  // The value of valid value
                        $valid_value->appendChild($doc->createElement('description', $listFieldsValuesDefinitions->fvd_fieldvalue_desc[$k]));   // The description of valid value
                    }
                }
            }
        }
    }

    return $doc;
}

/**
 * import custom fields
 * @global object $requestAct
 * @global string $tablePrefix
 * @global int $idSession
 * @global string $db_mediumint_auto
 * @global string $db_char1
 * @global string $db_mediumint
 * @global string $db_char1default0
 * @global string $db_timestamp
 * @global array $fieldDataType
 * @param DOMDocument $doc
 * @throws Exception
 */
function importCustomFields($doc)
{
    global $requestAct, $tablePrefix, $idSession, $db_mediumint_auto, $db_char1, $db_mediumint, $db_char1default0, $db_timestamp, $fieldDataType, $row;

    // The collection of custom fields
    $custom_fields = $doc->getElementsByTagName('custom_field');

    if ($custom_fields->length > 0) {
        foreach ($custom_fields as $custom_field) {
            $elements = $custom_field->getElementsByTagName("table");   // The name of system table
            $tblnam = $elements->item(0)->nodeValue;
            // get system table definition
            $querymore = " WHERE tdf.tablename='$tblnam'";
            $systemTableDefinition = clone $requestAct;
            $systemTableDefinition->openTablesDefinitions($querymore);

            if (count($systemTableDefinition->tdf_tablename) == '0') {
                throw new Exception('system table ' . $tblnam . ' not exists');
            }

            $his = $systemTableDefinition->tdf_history[0];
            $dbtblnam = $tablePrefix . $tblnam;
            $elements = $custom_field->getElementsByTagName("name");    // The name of custom field
            $fldnam = $elements->item(0)->nodeValue;
            // check if custom field already exists
            $tmpquery = "SELECT tablename FROM " . $tablePrefix . "fields_def WHERE tablename='$tblnam' AND fieldname='$fldnam'";
            $requestAct->query($tmpquery);
            $comptFieldDefinitionDetail = $requestAct->rowCount();

            if ($comptFieldDefinitionDetail == '0') {
                // determine custom field display order
                $tmpquery = "SELECT MAX(disporder) FROM " . $tablePrefix . "fields_def WHERE tablename='$tblnam'";
                $requestAct->query($tmpquery);
                $disporder = '0';

                if ($requestAct->fetch()) {
                    $disporder = intval($row[0]) + 1;
                }

                $elements = $custom_field->getElementsByTagName("description"); // The description of custom field
                $flddes = $elements->item(0)->nodeValue;
                $elements = $custom_field->getElementsByTagName("type");        // The type of custom field
                $fldtyp = $elements->item(0)->nodeValue;
                $elements = $custom_field->getElementsByTagName("length");      // The length of custom field
                $len = $elements->item(0)->nodeValue;
                $elements = $custom_field->getElementsByTagName("default_value");   // The default value of custom field
                $defval = $elements->item(0)->nodeValue;
                $elements = $custom_field->getElementsByTagName("required");    // Indicate a required custom field
                $req = $elements->item(0)->nodeValue;
                $elements = $custom_field->getElementsByTagName("unique");      // Indicate a unique custom field
                $uni = $elements->item(0)->nodeValue;
                $elements = $custom_field->getElementsByTagName("create_index");    // Create an index for custom field
                $idx = $elements->item(0)->nodeValue;
                $elements = $custom_field->getElementsByTagName("published");   // Indicate the custom field published
                $pub = $elements->item(0)->nodeValue;
                $elements = $custom_field->getElementsByTagName("linked_table");    // The name of linked table
                $lkt = $elements->item(0)->nodeValue;
                $len = ((is_numeric($len))?$len:'0');

                if ($fldtyp == enumFieldType::ftText || $fldtyp == enumFieldType::ftMemo) {
                    $defval = HtmlEntitiesNOD($defval);
                }

                $req = (($req=='Y')?'Y':'N');
                $uni = (($uni=='Y')?'Y':'N');
                $idx = (($idx=='Y')?'Y':'N');
                $pub = (($pub=='Y')?'Y':'N');
                // it's impossible to add a serial custom field if the table is not empty
                if ($fldtyp == enumFieldType::ftSerial) {
                    $tmpquery = "SELECT id FROM $dbtblnam";
                    $requestAct->query($tmpquery);
                    $cnt = $requestAct->rowCount();

                    if ($cnt != '0') {
                        continue;
                    }
                }

                $tmpsql = "INSERT INTO " . $tablePrefix . "fields_def (tablename,fieldname,fieldtype,length,default_value,required,unique_field,create_index,sysdef,published,linkedtable,disporder) VALUES ('$tblnam','$fldnam','$fldtyp','$len','$defval','$req','$uni','$idx','N','$pub','$lkt','$disporder')";

                if ($requestAct->query($tmpsql) === false) {
                    throw new Exception($tmpsql);
                }

                $tmpsql = "ALTER TABLE $dbtblnam ADD U_$fldnam";

                if ($fldtyp == enumFieldType::ftText) {
                    $tmpsql .= ' ' . str_replace('%length%', intval($len)*4, $fieldDataType[$fldtyp]);
                } else if ($fldtyp == enumFieldType::ftNumeric || $fldtyp == enumFieldType::ftAlphaNumeric || $fldtyp == enumFieldType::ftSerial) {
                    $tmpsql .= ' ' . str_replace('%length%', $len, $fieldDataType[$fldtyp]);
                } else {
                    $tmpsql .= ' ' . $fieldDataType[$fldtyp];
                }

                // required field
                if ($req == 'Y') {
                    $tmpsql .= ' NOT NULL';
                }

                // default value
                if ($defval != '') {
                    switch ($fldtyp) {
                        case enumFieldType::ftText:
                        case enumFieldType::ftMemo:
                        case enumFieldType::ftAlphaNumeric:
                        case enumFieldType::ftNumeric:
                        case enumFieldType::ftTime:
                        case enumFieldType::ftEmail:
                        case enumFieldType::ftUrl:
                            $tmpsql .= " DEFAULT '$defval'";
                            break;
                        case enumFieldType::ftDate:
                            if (!preg_match('/^(\+|\-)\d+$/', $defval)) {
                                $tmpsql .= " DEFAULT '" . mktime(0,0,0,intval(substr($defval,4,2)),intval(substr($defval,6,2)),intval(substr($defval,0,4))) . "'";
                            }

                            break;
                        case enumFieldType::ftHours:
                        case enumFieldType::ftQuantity:
                        case enumFieldType::ftMeasure:
                        case enumFieldType::ftRate:
                        case enumFieldType::ftTotals:
                        case enumFieldType::ftPercentage:
                            $tmpsql .= " DEFAULT $defval";
                            break;
                        case enumFieldType::ftBoolean:
                            $tmpsql .= " DEFAULT '" . (($defval=='1')?enumBoolean::bTrue:enumBoolean::bFalse) . "'";
                            break;
                    }
                }

                if ($requestAct->query($tmpsql) === false) {
                    throw new Exception($tmpsql);
                }

                // add history field in the database
                if ($his == 'Y') {
                    $tmpsql = "ALTER TABLE " . $dbtblnam . "_hist ADD U_$fldnam";

                    if ($fldtyp == enumFieldType::ftText) {
                        $tmpsql .= ' ' . str_replace('%length%', intval($len)*4, $fieldDataType[$fldtyp]);
                    } else if ($fldtyp == enumFieldType::ftNumeric || $fldtyp == enumFieldType::ftAlphaNumeric || $fldtyp == enumFieldType::ftSerial) {
                        $tmpsql .= ' ' . str_replace('%length%', $len, $fieldDataType[$fldtyp]);
                    } else {
                        $tmpsql .= ' ' . $fieldDataType[$fldtyp];
                    }

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }
                }

                // create index
                if ($idx == 'Y') {
                    $tmpsql = "CREATE " . (($uni=='Y')?'UNIQUE':'') . " INDEX idxu" . $dbtblnam . $fldnam . " ON $dbtblnam (U_$fldnam)";

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }
                }

                // update field description
                if ($flddes != fieldNameString($tblnam, $fldnam)) {
                    updateString($requestAct, "FIE__" . $tblnam . "_" . $fldnam, $flddes);
                }
            } else {
                $fldtyp = $fieldDefinitionDetail->fdf_fieldtype[0];
            }

            // The collection of valid values
            $valid_values = $custom_field->getElementsByTagName('valid_value');

            if ($valid_values->length > 0) {
                // add valid values
                foreach ($valid_values as $valid_value) {
                    $elements = $valid_value->getElementsByTagName("value");        // The value of valid value
                    $vv = $elements->item(0)->nodeValue;
                    $elements = $valid_value->getElementsByTagName("description");  // The description of valid value
                    $vd = $elements->item(0)->nodeValue;
                    $fv = (($fldtyp == enumFieldType::ftText)?HtmlEntitiesNOD($vv):$vv);
                    // check if valid value already exists
                    $tmpquery = "SELECT tablename FROM " . $tablePrefix . "fields_valdef WHERE tablename='$tblnam' AND fieldname='$fldnam' AND fieldvalue='$fv'";
                    $requestAct->query($tmpquery);
                    $comptValidValueDetail = $requestAct->rowCount();

                    if ($comptValidValueDetail == '0') {
                        $tmpsql = "INSERT INTO " . $tablePrefix . "fields_valdef (tablename,fieldname,fieldvalue,sysdef) VALUES ('$tblnam','$fldnam','$fv','N')";

                        if ($requestAct->query($tmpsql) === false) {
                            throw new Exception($tmpsql);
                        }
                    }

                    // update valid value description
                    updateString($requestAct, "VV__" . $tblnam . "_" . $fldnam . "_" . $vv, $vd);
                }
            }
        }
    }
}

/**
 * import form templates
 * @global object $requestAct
 * @global string $tablePrefix
 * @global int $idSession
 * @global string $db_mediumint_auto
 * @global string $db_char1
 * @global string $db_mediumint
 * @global string $db_char1default0
 * @global string $db_timestamp
 * @global array $fieldDataType
 * @param DOMDocument $doc
 * @throws Exception
 */
function importFormTemplates($doc)
{
    global $requestAct, $tablePrefix, $idSession, $db_mediumint_auto, $db_char1, $db_mediumint, $db_char1default0, $db_timestamp, $fieldDataType;
    // The collection of form templates
    $form_templates = $doc->getElementsByTagName("form_template");

    if ($form_templates->length > 0) {
        foreach ($form_templates as $form_template) {
            $elements = $form_template->getElementsByTagName("name");   // The name of form template
            $tblnam = $elements->item(0)->nodeValue;
            // check if form template already exists
            $tmpquery = "SELECT id FROM " . $tablePrefix . "tables_def WHERE UPPER(tablename)='" . strtoupper($tblnam) . "'";
            $requestAct->query($tmpquery);
            $comptTableDefinitionDetail = $requestAct->rowCount();

            if (preg_match('/@/', $tblnam) == 0) {
                $dbtblnam = $tablePrefix . $tblnam;
            } else {
                $dbtblnam = $tablePrefix . preg_replace_callback('/@/', function ($matches) { return 'FORM_'; }, $tblnam);
            }

            if ($comptTableDefinitionDetail == '0') {
                $elements = $form_template->getElementsByTagName("description");    // The description of form template
                $tbldes = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("printable");      // printable table
                $printable = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("mainmenu");       // create main menu item
                $mainmenu = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("history");        // create history log table
                $history = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("view_url");       // custom view url
                $viewurl = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("key_fields");     // The key fields of form template
                $keyfields = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("display_fields"); // The fields shown in search results
                $dispfields = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("display_style");  // The display style of key fields
                $dispstyle = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("form_fields");    // The fields shown in form list
                $formfields = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("list_fields");    // The fields shown in choose list
                $listfields = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("primary_fields"); // The primary search fields
                $primfields = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("father_table");   // The parent form template
                $ftbl = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("approval_enabled");   // Enable approvals
                $approvable = $elements->item(0)->nodeValue;
                $elements = $form_template->getElementsByTagName("publish_enabled");    // Enable publish to customer service site
                $publishe = $elements->item(0)->nodeValue;
                $searchable = (($ftbl=='')?'Y':'N');
                $pr = (($printable=='Y')?'Y':'N');
                $mm = (($mainmenu=='Y')?'Y':'N');
                $ae = (($approvable=='Y')?'Y':'N');
                $pe = (($publishe=='Y')?'Y':'N');
                $his = (($history=='Y')?'Y':'N');
                $tmpsql = "INSERT INTO " . $tablePrefix . "tables_def (tablename,sysdef,cfenabled,searchable,printable,mainmenu,keyfields,formfields,dispfields,dispstyle,listfields,primfields,fathertable,linkedfield,approvalenabled,publishenabled,history,viewurl,created,cuser) VALUES ('$tblnam','N','Y','$searchable','$pr','$mm','$keyfields','$formfields','$dispfields','$dispstyle','$listfields','$primfields','$ftbl','','$ae','$pe','$his','$viewurl','" . time() . "','$idSession')";

                if ($requestAct->query($tmpsql) === false) {
                    throw new Exception($tmpsql);
                }

                $tmpsql = "CREATE TABLE $dbtblnam (id $db_mediumint_auto" . (($ftbl=="")?"":",pid $db_mediumint") . ",status $db_mediumint,printed $db_char1default0,project $db_mediumint" . (($pe=='Y')?",published $db_char1":"") . ",created $db_timestamp,cuser $db_mediumint,modified $db_timestamp,muser $db_mediumint,PRIMARY KEY (id)) $db_create_table_postfix";

                if ($requestAct->query($tmpsql) === false) {
                    throw new Exception($tmpsql);
                }

                if ($tbldes != $strings['TBL__' . $tblnam]) {
                    updateString($requestAct, "TBL__" . $tblnam, $tbldes);
                }

                if ($ftbl != '') {
                    $tmpsql = "CREATE INDEX idxu" . $dbtblnam . "pid ON $dbtblnam (pid)";   // create index

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }
                }

                if ($his == 'Y') {
                    // create history log table
                    $tmpsql = "CREATE TABLE " . $dbtblnam . "_hist (hid $db_mediumint_auto,action $db_char1,id $db_mediumint" . (($ftbl=="")?"":",pid $db_mediumint") . ",status $db_mediumint,printed $db_char1default0,project $db_mediumint" . (($pe=='Y')?",published $db_char1":"") . ",created $db_timestamp,cuser $db_mediumint,modified $db_timestamp,muser $db_mediumint,PRIMARY KEY (hid)) $db_create_table_postfix";

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }
                    // create index
                    $tmpsql = "CREATE INDEX idxu" . $dbtblnam . "_histid ON " . $dbtblnam . "_hist (id)";

                    if ($requestAct->query($tmpsql) === false) {
                        throw new Exception($tmpsql);
                    }
                }
                // full control by default for all user profiles
                $listUserProfiles = clone $requestAct;
                $listUserProfiles->openUserProfiles('');
                $comptListUserProfiles = count($listUserProfiles->upf_id);

                for ($i=0; $i<$comptListUserProfiles; $i++) {
                    foreach (array('Read','Write','Edit','Admin') as $p) {
                        $tmpsql = "INSERT INTO " . $tablePrefix . "user_profiles_detail (user_profile,function_name,access_level,granted) VALUES ('" . $listUserProfiles->upf_id[$i] . "','$tblnam','$p','" . enumBoolean::bTrue . "')";

                        if ($requestAct->query($tmpsql) === false) {
                            throw new Exception($tmpsql);
                        }
                    }
                }
            }

            // The collection of custom fields
            $custom_fields = $form_template->getElementsByTagName('custom_field');

            if ($custom_fields->length > 0) {
                foreach ($custom_fields as $custom_field) {
                    $elements = $custom_field->getElementsByTagName("name");    // The name of custom field
                    $fldnam = $elements->item(0)->nodeValue;
                    // check if custom field already exists
                    $tmpquery = "SELECT tablename FROM " . $tablePrefix . "fields_def WHERE tablename='$tblnam' AND fieldname='$fldnam'";
                    $requestAct->query($tmpquery);
                    $comptFieldDefinitionDetail = $requestAct->rowCount();

                    if ($comptFieldDefinitionDetail == '0') {
                        $elements = $custom_field->getElementsByTagName("description"); // The description of custom field
                        $flddes = $elements->item(0)->nodeValue;
                        $elements = $custom_field->getElementsByTagName("type");        // The type of custom field
                        $fldtyp = $elements->item(0)->nodeValue;
                        $elements = $custom_field->getElementsByTagName("length");      // The length of custom field
                        $len = $elements->item(0)->nodeValue;
                        $elements = $custom_field->getElementsByTagName("default_value");   // The default value of custom field
                        $defval = $elements->item(0)->nodeValue;
                        $elements = $custom_field->getElementsByTagName("required");    // Indicate a required custom field
                        $req = $elements->item(0)->nodeValue;
                        $elements = $custom_field->getElementsByTagName("unique");      // Indicate a unique custom field
                        $uni = $elements->item(0)->nodeValue;
                        $elements = $custom_field->getElementsByTagName("create_index");    // Create an index for custom field
                        $idx = $elements->item(0)->nodeValue;
                        $elements = $custom_field->getElementsByTagName("published");   // Indicate the custom field published
                        $pub = $elements->item(0)->nodeValue;
                        $elements = $custom_field->getElementsByTagName("linked_table");    // The name of linked table
                        $lkt = $elements->item(0)->nodeValue;
                        $elements = $custom_field->getElementsByTagName("display_order");   // The display order of custom field
                        $disporder = $elements->item(0)->nodeValue;
                        $len = ((is_numeric($len))?$len:'0');

                        if ($fldtyp == enumFieldType::ftText || $fldtyp == enumFieldType::ftMemo) {
                            $defval = HtmlEntitiesNOD($defval);
                        }

                        $req = (($req=='Y')?'Y':'N');
                        $uni = (($uni=='Y')?'Y':'N');
                        $idx = (($idx=='Y')?'Y':'N');
                        $pub = (($pub=='Y')?'Y':'N');
                        $disporder = ((is_numeric($disporder))?$disporder:'0');
                        // it's impossible to add a serial custom field if the table is not empty
                        if ($fldtyp == enumFieldType::ftSerial) {
                            $tmpquery = "SELECT id FROM $dbtblnam";
                            $requestAct->query($tmpquery);
                            $cnt = $requestAct->rowCount();

                            if ($cnt != '0') {
                                continue;
                            }
                        }

                        $tmpsql = "INSERT INTO " . $tablePrefix . "fields_def (tablename,fieldname,fieldtype,length,default_value,required,unique_field,create_index,sysdef,published,linkedtable,disporder) VALUES ('$tblnam','$fldnam','$fldtyp','$len','$defval','$req','$uni','$idx','N','$pub','$lkt','$disporder')";

                        if ($requestAct->query($tmpsql) === false) {
                            throw new Exception($tmpsql);
                        }

                        $tmpsql = "ALTER TABLE $dbtblnam ADD U_$fldnam";

                        if ($fldtyp == enumFieldType::ftText) {
                            $tmpsql .= ' ' . str_replace('%length%', intval($len)*4, $fieldDataType[$fldtyp]);
                        } else if ($fldtyp == enumFieldType::ftNumeric || $fldtyp == enumFieldType::ftAlphaNumeric || $fldtyp == enumFieldType::ftSerial) {
                            $tmpsql .= ' ' . str_replace('%length%', $len, $fieldDataType[$fldtyp]);
                        } else {
                            $tmpsql .= ' ' . $fieldDataType[$fldtyp];
                        }

                        // required field
                        if ($req == 'Y') {
                            $tmpsql .= ' NOT NULL';
                        }

                        // default value
                        if ($defval != '') {
                            switch ($fldtyp) {
                                case enumFieldType::ftText:
                                case enumFieldType::ftMemo:
                                case enumFieldType::ftAlphaNumeric:
                                case enumFieldType::ftNumeric:
                                case enumFieldType::ftTime:
                                case enumFieldType::ftEmail:
                                case enumFieldType::ftUrl:
                                    $tmpsql .= " DEFAULT '$defval'";
                                    break;
                                case enumFieldType::ftDate:
                                    if (!preg_match('/^(\+|\-)\d+$/', $defval)) {
                                        $tmpsql .= " DEFAULT '" . mktime(0,0,0,intval(substr($defval,4,2)),intval(substr($defval,6,2)),intval(substr($defval,0,4))) . "'";
                                    }

                                    break;
                                case enumFieldType::ftHours:
                                case enumFieldType::ftQuantity:
                                case enumFieldType::ftMeasure:
                                case enumFieldType::ftRate:
                                case enumFieldType::ftTotals:
                                case enumFieldType::ftPercentage:
                                    $tmpsql .= " DEFAULT $defval";
                                    break;
                                case enumFieldType::ftBoolean:
                                    $tmpsql .= " DEFAULT '" . (($defval=='1')?enumBoolean::bTrue:enumBoolean::bFalse) . "'";
                                    break;
                            }
                        }

                        if ($requestAct->query($tmpsql) === false) {
                            throw new Exception($tmpsql);
                        }

                        // add history field in the database
                        if ($his == 'Y') {
                            $tmpsql = "ALTER TABLE " . $dbtblnam . "_hist ADD U_$fldnam";

                            if ($fldtyp == enumFieldType::ftText) {
                                $tmpsql .= ' ' . str_replace('%length%', intval($len)*4, $fieldDataType[$fldtyp]);
                            } else if ($fldtyp == enumFieldType::ftNumeric || $fldtyp == enumFieldType::ftAlphaNumeric || $fldtyp == enumFieldType::ftSerial) {
                                $tmpsql .= ' ' . str_replace('%length%', $len, $fieldDataType[$fldtyp]);
                            } else {
                                $tmpsql .= ' ' . $fieldDataType[$fldtyp];
                            }

                            if ($requestAct->query($tmpsql) === false) {
                                throw new Exception($tmpsql);
                            }
                        }

                        // create index
                        if ($idx == 'Y') {
                            $tmpsql = "CREATE " . (($uni=='Y')?'UNIQUE':'') . " INDEX idxu" . $dbtblnam . $fldnam . " ON $dbtblnam (U_$fldnam)";

                            if ($requestAct->query($tmpsql) === false) {
                                throw new Exception($tmpsql);
                            }
                        }

                        // update field description
                        if ($flddes != fieldNameString($tblnam, $fldnam)) {
                            updateString($requestAct, "FIE__" . $tblnam . "_" . $fldnam, $flddes);
                        }
                    } else {
                        $fldtyp = $fieldDefinitionDetail->fdf_fieldtype[0];
                    }

                    // The collection of valid values
                    $valid_values = $custom_field->getElementsByTagName('valid_value');

                    if ($valid_values->length > 0) {
                        // add valid values
                        foreach ($valid_values as $valid_value) {
                            $elements = $valid_value->getElementsByTagName("value");        // The value of valid value
                            $vv = $elements->item(0)->nodeValue;
                            $elements = $valid_value->getElementsByTagName("description");  // The description of valid value
                            $vd = $elements->item(0)->nodeValue;
                            $fv = (($fldtyp == enumFieldType::ftText)?HtmlEntitiesNOD($vv):$vv);
                            // check if valid value already exists
                            $tmpquery = "SELECT tablename FROM " . $tablePrefix . "fields_valdef WHERE tablename='$tblnam' AND fieldname='$fldnam' AND fieldvalue='$fv'";
                            $requestAct->query($tmpquery);
                            $comptValidValueDetail = $requestAct->rowCount();

                            if ($comptValidValueDetail == '0') {
                                $tmpsql = "INSERT INTO " . $tablePrefix . "fields_valdef (tablename,fieldname,fieldvalue,sysdef) VALUES ('$tblnam','$fldnam','$fv','N')";

                                if ($requestAct->query($tmpsql) === false) {
                                    throw new Exception($tmpsql);
                                }
                            }

                            // update valid value description
                            updateString($requestAct, "VV__" . $tblnam . "_" . $fldnam . "_" . $vv, $vd);
                        }
                    }
                }
            }
        }
    }
}

/**
 * show admin link on main menu
 * @global array $SW_Permissions
 * @global string $profilSession
 * @global string $enableTimesheet
 * @global string $enableEquipment
 * @global string $enableMaterial
 * @return boolean
 */
function showAdminMainMenu()
{
    global $SW_Permissions, $profilSession, $enableTimesheet, $enableEquipment, $enableMaterial;
    if ($SW_Permissions['Currency Rate']['Read'][$profilSession]) return true;
    if ($SW_Permissions['Accounting']['Read'][$profilSession]) return true;
    if ($SW_Permissions['Service']['Read'][$profilSession]) return true;
    if ($SW_Permissions['Company Detail']['Read'][$profilSession]) return true;
    if ($SW_Permissions['User']['Read'][$profilSession]) return true;
    if (($enableTimesheet == 'true') && ($SW_Permissions['Leave']['Read'][$profilSession])) return true;
    if ($SW_Permissions['Locations']['Read'][$profilSession]) return true;
    if ($SW_Permissions['Custom Fields']['Read'][$profilSession]) return true;
    if ($SW_Permissions['Form Template']['Read'][$profilSession]) return true;
    if ($SW_Permissions['TextCustomization']['Admin'][$profilSession]) return true;
    if ($SW_Permissions['Print Layout']['Admin'][$profilSession]) return true;
    if ($SW_Permissions['System Settings']['Read'][$profilSession]) return true;
    if ($SW_Permissions['Extension']['Admin'][$profilSession]) return true;
    if ($SW_Permissions['Profile']['Admin'][$profilSession]) return true;
    if ($SW_Permissions['Period']['Read'][$profilSession]) return true;
    if ($SW_Permissions['Phase Set']['Read'][$profilSession]) return true;
    if ($SW_Permissions['Work Calendar']['Read'][$profilSession]) return true;
    if ($SW_Permissions['Deny IP']['Read'][$profilSession]) return true;
    if ($SW_Permissions['System Log']['Read'][$profilSession]) return true;
    if ($enableEquipment == 'true' && $SW_Permissions['Equipment']['Read'][$profilSession]) return true;
    if ($enableMaterial == 'true' && $SW_Permissions['Material']['Read'][$profilSession]) return true;
    if ($SW_Permissions['Approval Procedure']['Read'][$profilSession]) return true;
    return false;
}
