<?php // $Revision: 1.13 $
/* vim: set expandtab ts=4 sw=4 sts=4: */

/**
 * $Id: upgrade_db.php,v 1.13 2013/02/01 02:25:46 luiswang Exp $
 *
 * Copyright (c) 2018 by the webTareas developers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

// check MySQL database
function check_mysql($dbserver, $dblogin, $dbpasswd, $dbname) {
    global $block1, $error;
    $mysqli = new mysqli($dbserver, $dblogin, $dbpasswd);
    $error = false;
    /*
     * Use this instead of $connect_error if you need to ensure
     * compatibility with PHP versions prior to 5.2.9 and 5.3.0.
     */
    if (mysqli_connect_error()) {
        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", "Could not connect to the database server.", "false", "");
        $block1->contentRow("", mysqli_connect_error() . "(" . mysqli_connect_errno() . ")");
        $error = true;
    } else {
        $block1->contentRow("<img src=\"../themes/" . THEME . "/success.png\">", "Connection to database server is OK.", "false", "");

        if ($mysqli->select_db($dbname)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/success.png\">", "Database $dbname exists and connectable.", "false", "");
        } else {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", "Database $dbname does not exists.", "false", "");
            $error = true;
        }

        $mysqli->close();
    }
}

// check PostgreSQL database
function check_postgres($dbserver, $dblogin, $dbpasswd, $dbname)
{
    global $block1, $error;

    if (($pg = pg_connect("host=" . $dbserver . " dbname=" . $dbname . " user=" . $dblogin . " password=" . $dbpasswd))) {
        $block1->contentRow("<img src=\"../themes/" . THEME . "/success.png\">", "Database $dbname exists and connectable.", "false", "");
        pg_close($pg);
    } else {
        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", "Database $dbname does not exists.", "false", "");
        $error = true;
    }
}

function setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, $tblnam)
{
    global $block1;
    $rc = '';
    $sql = "SELECT fieldname FROM " . $prevtbprefix . "fields_def WHERE tablename='$tblnam' AND sysdef='N' ORDER BY disporder";

    if (!($result = $mysqlip->query($sql))) {
        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
        $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
        throw new Exception('', 1);
    } else {
        while (($row = $result->fetch_row())) {
            $rc .= ',U_' . $row[0];
        }
    }

    return $rc;
}

function setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, $tblnam, &$qrysql)
{
    global $block1;
    $rc = '';
    $qrysql = '';
    $sql = "SELECT fieldname,fieldtype,default_value FROM " . $prevtbprefix . "fields_def WHERE tablename='$tblnam' AND sysdef='N' ORDER BY disporder";

    if (!($result = pg_query($pgsqlp, $sql))) {
        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
        $block1->contentRow("", pg_last_notice($pgsqlp));
        throw new Exception('', 1);
    } else {
        while (($row = pg_fetch_row($result))) {
            $dftval = trim($row[2]);    // default value

            if ($dftval === '') {
                switch ($row[1]) {
                    case '0':   // text
                    case '2':   // memo
                    case '4':   // time
                    case '8':   // image
                    case 'E':   // e-mail
                    case 'U':   // url
                    case 'A':   // alpha-numeric
                    case 'D':   // audio
                    case 'V':   // video
                    case 'W':   // password
                        break;
                    default:
                        $dftval = '0';
                }
            }

            $qrysql .= ",CASE WHEN U_" . $row[0] . " IS NULL THEN '" . $dftval . "' ELSE U_" . $row[0] . " END";
            $rc .= ',U_' . $row[0];
        }
    }

    return $rc;
}

// data migration (MySQL -> MySQL)
function upgrade_database_my2my($prevdbserver, $prevdblogin, $prevdbpasswd, $prevdbname, $prevtbprefix, $dbserver, $dblogin, $dbpasswd, $dbname, $tbprefix)
{
    global $block1, $error, $prevroot, $prevlangDefault, $strings;
    $lang = (file_exists($prevroot . '/languages/lang_' . $prevlangDefault . '.php'))?$prevlangDefault:'en';
    require $prevroot . '/languages/lang_' . $lang . '.php';
    $mysqlip = new mysqli($prevdbserver, $prevdblogin, $prevdbpasswd, $prevdbname);

    if (mysqli_connect_error()) {
        die('Connect Error (' . mysqli_connect_errno() . ') ' . mysqli_connect_error());
    }

    $mysqlin = new mysqli($dbserver, $dblogin, $dbpasswd, $dbname);

    if (mysqli_connect_error()) {
        die('Connect Error (' . mysqli_connect_errno() . ') ' . mysqli_connect_error());
    }

    $error = false;
    set_time_limit(0);

    try {
        @$mysqlin->autocommit(false);    // Turns off auto-committing database modifications
        // strings
        $sql = "SELECT string_key,language,string_val FROM " . $prevtbprefix . "strings";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $string_key = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "strings (string_key,language,string_val) VALUES ('$string_key','" . $row[1] . "','" . $row[2] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // re-index table definitions
        $sql = "DELETE FROM " . $tbprefix . "tables_def";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $hisA = array();    // history table index
        $sql = "SELECT id,tablename,sysdef,cfenabled,searchable,printable,mainmenu,keyfields,formfields,dispfields,dispstyle,listfields,primfields,fathertable,linkedfield,approvalenabled,publishenabled,javascript,projectrelated,history,viewurl FROM " . $prevtbprefix . "tables_def ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;
            $tblA = array();

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $tblA[] = $row[1];
                $sql = "INSERT INTO " . $tbprefix . "tables_def (id,tablename,sysdef,cfenabled,searchable,printable,mainmenu,keyfields,formfields,dispfields,dispstyle,listfields,primfields,fathertable,linkedfield,approvalenabled,publishenabled,javascript,projectrelated,history,viewurl,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "','" . time() . "','1','" . time() . "','1')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "tables_def AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }

            $h = fopen('tables_def.csv', 'r');
            $hdr = fgets($h);

            while (!feof($h)) {
                $r = fgets($h);
                $r = str_replace(array('"', ' ', "\r", "\n"), '', $r);

                if ($r == '') {
                    continue;
                }

                $ra = explode(',', $r);
                $sysdef = $ra[1];

                if ($sysdef == 'Y') {
                    $tblnam = $ra[0];
                } else {
                    $tblnam = '@' . $ra[0];
                }

                if ($ra[15] == 'Y') {
                    $hisA[] = $tblnam;
                }

                if (array_search($tblnam, $tblA) !== false) {
                    $sql = "UPDATE " . $tbprefix . "tables_def SET sysdef='$sysdef',cfenabled='" . $ra[2] . "',searchable='" . $ra[3] . "',printable='" . $ra[4] . "',keyfields='" . $ra[5] . "',dispfields='" . $ra[6] . "',dispstyle='" . $ra[7] . "',listfields='" . $ra[8] . "',primfields='" . $ra[9] . "',fathertable='" . $ra[10] . "',linkedfield='" . $ra[11] . "',approvalenabled='" . $ra[12] . "',publishenabled='" . $ra[13] . "',projectrelated='" . $ra[14] . "',history='" . $ra[15] . "' WHERE tablename='$tblnam'";
                } else {
                    $sql = "INSERT INTO " . $tbprefix . "tables_def (tablename,sysdef,cfenabled,searchable,printable,keyfields,dispfields,dispstyle,listfields,primfields,fathertable,linkedfield,approvalenabled,publishenabled,projectrelated,history,viewurl,created,cuser,modified,muser) VALUES ('$tblnam','$sysdef','" . $ra[2] . "','" . $ra[3] . "','" . $ra[4] . "','" . $ra[5] . "','" . $ra[6] . "','" . $ra[7] . "','" . $ra[8] . "','" . $ra[9] . "','" . $ra[10] . "','" . $ra[11] . "','" . $ra[12] . "','" . $ra[13] . "','" . $ra[14] . "','" . $ra[15] . "','','" . time() . "','1','" . time() . "','1')";
                }

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            fclose($h);
        }

        // create form templates
        $sql = "SELECT tablename,fathertable,publishenabled,history FROM " . $prevtbprefix . "tables_def WHERE sysdef='N'";
        $ftA = array();
        $fdA = array();
        $ffA = array();

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $tblnam = $row[0];
                $ftA[] = $tblnam;
                $dbtblnam = str_replace('@', 'FORM_', $tblnam);
                $fdA[] = $dbtblnam;
                $ftbl = $row[1];
                $pe = $row[2];
                $his = $row[3];
                $sql = "CREATE TABLE " . $tbprefix . $dbtblnam . " (id int unsigned NOT NULL auto_increment" . (($ftbl=="")?"":",pid int unsigned NOT NULL default '0'") . ",status int unsigned NOT NULL default '0',printed char(1) NOT NULL default '0',project int unsigned NOT NULL default '0'" . (($pe=='Y')?",published char(1) NOT NULL default ''":"") . ",created int NOT NULL default '0',cuser int unsigned NOT NULL default '0',modified int NOT NULL default '0',muser int unsigned NOT NULL default '0',PRIMARY KEY (id)) ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_unicode_ci";
                $ffA[] = "id" . (($ftbl=='')?'':',pid') . ",status,project" . (($pe=='Y')?',published':'') . ",created,cuser,modified,muser";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }

                if ($his == 'Y') {
                    $sql = "CREATE TABLE " . $tbprefix . $dbtblnam . "_hist (hid int unsigned NOT NULL auto_increment,action char(1) NOT NULL default '',id int unsigned NOT NULL default '0'" . (($ftbl=="")?"":",pid int unsigned NOT NULL default '0'") . ",status int unsigned NOT NULL default '0',printed char(1) NOT NULL default '0',project int unsigned NOT NULL default '0'" . (($pe=='Y')?",published char(1) NOT NULL default ''":"") . ",created int NOT NULL default '0',cuser int unsigned NOT NULL default '0',modified int NOT NULL default '0',muser int unsigned NOT NULL default '0',PRIMARY KEY (hid)) ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_unicode_ci";
                    $hisA[] = $tblnam;

                    if (!$mysqlin->query($sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                        throw new Exception('', 1);
                    }
                }
            }
        }

        // create custom fields
        $sql = "SELECT fdf.tablename FROM " . $prevtbprefix . "fields_def fdf WHERE fdf.sysdef='N' GROUP BY fdf.tablename";
        $tblnamA = array();
        $fieldDataType = array( enumFieldType::ftText => 'varchar(%length%)', enumFieldType::ftAlphaNumeric => 'varchar(%length%)', enumFieldType::ftNumeric => 'varchar(%length%)', enumFieldType::ftMemo => 'text', enumFieldType::ftDate => "int NOT NULL default '0'", enumFieldType::ftTime => 'char(4)', enumFieldType::ftHours => 'decimal(19,6)', enumFieldType::ftQuantity => 'decimal(19,6)', enumFieldType::ftMeasure => 'decimal(19,6)', enumFieldType::ftRate => 'decimal(19,6)', enumFieldType::ftTotals => 'decimal(19,6)', enumFieldType::ftImage => 'varchar(1024)', enumFieldType::ftAudio => 'varchar(1024)', enumFieldType::ftVideo => 'varchar(1024)', enumFieldType::ftBoolean => 'char(1)', enumFieldType::ftEmail => 'varchar(255)', enumFieldType::ftUrl => 'varchar(1024)', enumFieldType::ftSerial => 'varchar(%length%)', enumFieldType::ftTimestamp => "int NOT NULL default '0'", enumFieldType::ftPercentage => 'decimal(19,6)', enumFieldType::ftPassword => 'varchar(1024)' );

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $tblnamA[] = $row[0];
            }
        }

        if (count($tblnamA) > 0) {
            for ($j=0; $j<count($tblnamA); $j++) {
                $tblnam = $tblnamA[$j];
                $sql = "SELECT tablename,fieldname,fieldtype,length,default_value,required,unique_field,create_index,sysdef,published,linkedtable,disporder FROM " . $prevtbprefix . "fields_def WHERE tablename='$tblnam' AND sysdef='N' ORDER BY disporder";

                if (!($result = $mysqlip->query($sql))) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
                    throw new Exception('', 1);
                } else {
                    $fldnamA = array();
                    $fldtypA = array();
                    $lenA = array();
                    $defvalA = array();
                    $reqA = array();
                    $uniA = array();
                    $idxA = array();

                    while (($row = $result->fetch_row())) {
                        $sql = "INSERT INTO " . $tbprefix . "fields_def (tablename,fieldname,fieldtype,length,default_value,required,unique_field,create_index,sysdef,published,linkedtable,disporder) VALUES ('" . implode("','", $row) . "')";

                        if (!$mysqlin->query($sql)) {
                            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                            throw new Exception('', 1);
                        }

                        $fldnamA[] = $row[1];
                        $fldtypA[] = $row[2];
                        $lenA[] = $row[3];
                        $defvalA[] = $row[4];
                        $reqA[] = $row[5];
                        $uniA[] = $row[6];
                        $idxA[] = $row[7];
                    }

                    for ($i=0; $i<count($fldnamA); $i++) {
                        $dbtblnam = $tbprefix . str_replace('@', 'FORM_', $tblnam);
                        $fldnam = $fldnamA[$i];
                        $fldtyp = $fldtypA[$i];
                        $len = $lenA[$i];
                        $req = $reqA[$i];
                        $defval = $defvalA[$i];
                        $idx = $idxA[$i];
                        $sql = "ALTER TABLE $dbtblnam ADD U_$fldnam";

                        if ($fldtyp == enumFieldType::ftText) {
                            $sql .= ' ' . str_replace('%length%', intval($len)*4, $fieldDataType[$fldtyp]);
                        } else if ($fldtyp == enumFieldType::ftNumeric || $fldtyp == enumFieldType::ftAlphaNumeric || $fldtyp == enumFieldType::ftSerial) {
                            $sql .= ' ' . str_replace('%length%', intval($len), $fieldDataType[$fldtyp]);
                        } else {
                            $sql .= ' ' . $fieldDataType[$fldtyp];
                        }

                        // required field
                        if ($req == 'Y') {
                            $sql .= ' NOT NULL';
                        }

                        // default value
                        if ($defval != '') {
                            switch ($fldtyp) {
                                case enumFieldType::ftText:
                                case enumFieldType::ftMemo:
                                    $sql .= " DEFAULT '" . HtmlEntitiesNOD($defval) . "'";
                                    break;
                                case enumFieldType::ftAlphaNumeric:
                                case enumFieldType::ftNumeric:
                                case enumFieldType::ftTime:
                                case enumFieldType::ftEmail:
                                case enumFieldType::ftUrl:
                                    $sql .= " DEFAULT '$defval'";
                                    break;
                                case enumFieldType::ftDate:
                                    if (!preg_match('/^(\+|\-)\d+$/', $defval)) {
                                        $sql .= " DEFAULT '" . mktime(0,0,0,intval(substr($defval,4,2)),intval(substr($defval,6,2)),intval(substr($defval,0,4))) . "'";
                                    }

                                    break;
                                case enumFieldType::ftHours:
                                case enumFieldType::ftQuantity:
                                case enumFieldType::ftMeasure:
                                case enumFieldType::ftRate:
                                case enumFieldType::ftTotals:
                                case enumFieldType::ftPercentage:
                                    $sql .= " DEFAULT $defval";
                                    break;
                                case enumFieldType::ftBoolean:
                                    $sql .= " DEFAULT '" . (($defval=='1')?enumBoolean::bTrue:enumBoolean::bFalse) . "'";
                                    break;
                            }
                        }

                        if (!$mysqlin->query($sql)) {
                            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                            throw new Exception('', 1);
                        }

                        if (array_search($tblnam, $hisA) !== false) {   // create custom field in history table as well
                            $sql = "ALTER TABLE " . $dbtblnam . "_hist ADD U_$fldnam";

                            if ($fldtyp == enumFieldType::ftText) {
                                $sql .= ' ' . str_replace('%length%', intval($len)*4, $fieldDataType[$fldtyp]);
                            } else if ($fldtyp == enumFieldType::ftNumeric || $fldtyp == enumFieldType::ftAlphaNumeric || $fldtyp == enumFieldType::ftSerial) {
                                $sql .= ' ' . str_replace('%length%', intval($len), $fieldDataType[$fldtyp]);
                            } else {
                                $sql .= ' ' . $fieldDataType[$fldtyp];
                            }

                            if (!$mysqlin->query($sql)) {
                                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                                throw new Exception('', 1);
                            }
                        }

                        // create index
                        if ($idx == 'Y') {
                            $sql = "CREATE " . (($uni=='Y')?'UNIQUE':'') . " INDEX idxu" . $dbtblnam . $fldnam . " ON $dbtblnam (U_$fldnam)";

                            if (!$mysqlin->query($sql)) {
                                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                                throw new Exception('', 1);
                            }
                        }

                        // add valid values
                        $sql = "SELECT fieldvalue FROM " . $prevtbprefix . "fields_valdef WHERE tablename='$tblnam' AND fieldname='$fldnam' AND sysdef='N'";

                        if (!($result = $mysqlip->query($sql))) {
                            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
                            throw new Exception('', 1);
                        } else {
                            while (($row = $result->fetch_row())) {
                                $sql = "INSERT INTO " . $tbprefix . "fields_valdef (tablename,fieldname,fieldvalue,sysdef) VALUES ('$tblnam','$fldnam','" . $row[0] . "','N')";

                                if (!$mysqlin->query($sql)) {
                                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                                    throw new Exception('', 1);
                                }
                            }
                        }
                    }
                }
            }
        }

        // copy custom form templates data
        if (count($ftA) > 0) {
            for ($i=0; $i<count($ftA); $i++) {
                $sql = "SELECT fieldname FROM " . $prevtbprefix . "fields_def WHERE tablename='" . $ftA[$i] . "' AND sysdef='N'";

                if (!($result = $mysqlip->query($sql))) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
                    throw new Exception('', 1);
                } else {
                    while (($row = $result->fetch_row())) {
                        $ffA[$i] .= ",U_" . $row[0];
                    }
                }

                $sql = "SELECT " . $ffA[$i] . " FROM " . $prevtbprefix . $fdA[$i] . " ORDER BY id";

                if (!($result = $mysqlip->query($sql))) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
                    throw new Exception('', 1);
                } else {
                    $id = 0;

                    while (($row = $result->fetch_row())) {
                        for ($x=0; $x<count($row); $x++) $row[$x] = addslashes($row[$x]);
                        $id = $row[0];
                        $sql = "INSERT INTO " . $tbprefix . $fdA[$i] . " (" . $ffA[$i] . ") VALUES ('" . implode("','", $row) . "')";

                        if (!$mysqlin->query($sql)) {
                            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                            throw new Exception('', 1);
                        }
                    }

                    $id++;
                    $sql = "ALTER TABLE " . $tbprefix . $fdA[$i] . " AUTO_INCREMENT=$id";

                    if (!$mysqlin->query($sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                        throw new Exception('', 1);
                    }
                }

                if (array_search($ftA[$i], $hisA) !== false) {
                    $sql = "SELECT hid,action," . $ffA[$i] . " FROM " . $prevtbprefix . $fdA[$i] . "_hist ORDER BY hid";

                    if (!($result = $mysqlip->query($sql))) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
                        throw new Exception('', 1);
                    } else {
                        $hid = 0;

                        while (($row = $result->fetch_row())) {
                            for ($x=0; $x<count($row); $x++) $row[$x] = addslashes($row[$x]);
                            $hid = $row[0];
                            $sql = "INSERT INTO " . $tbprefix . $fdA[$i] . "_hist (hid,action," . $ffA[$i] . ") VALUES ('" . implode("','", $row) . "')";

                            if (!$mysqlin->query($sql)) {
                                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                                throw new Exception('', 1);
                            }
                        }

                        $hid++;
                        $sql = "ALTER TABLE " . $tbprefix . $fdA[$i] . "_hist AUTO_INCREMENT=$hid";

                        if (!$mysqlin->query($sql)) {
                            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                            throw new Exception('', 1);
                        }
                    }
                }
            }
        }

        // approvals
        $sql = "DELETE FROM " . $tbprefix . "approvals";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,doctype,conditions,active,comments,userquery FROM " . $prevtbprefix . "approvals ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "approvals (id,name,doctype,conditions,active,comments,userquery) VALUES ('$id','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "approvals AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // approvals_producers
        $sql = "DELETE FROM " . $tbprefix . "approvals_producers";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT approval,producer_type,producer,exclude FROM " . $prevtbprefix . "approvals_producers";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "approvals_producers (approval,producer_type,producer,exclude) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // approvals_stages
        $sql = "DELETE FROM " . $tbprefix . "approvals_stages";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT approval,confirmation_level,sortid FROM " . $prevtbprefix . "approvals_stages";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "approvals_stages (approval,confirmation_level,sortid) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // approvals_terms
        $sql = "DELETE FROM " . $tbprefix . "approvals_terms";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,approval,fieldname,ratio,value,compare_fields FROM " . $prevtbprefix . "approvals_terms ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "approvals_terms (id,approval,fieldname,ratio,value,compare_fields) VALUES ('$id','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "approvals_terms AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // assignments
        $sql = "DELETE FROM " . $tbprefix . "assignments";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,task,owner,assigned_to,comments,assigned FROM " . $prevtbprefix . "assignments ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "assignments (id,task,owner,assigned_to,comments,assigned) VALUES ('$id','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "assignments AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // attendees
        $sql = "DELETE FROM " . $tbprefix . "attendees";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT meeting,member,attended,authorized,required,schedule_string,string_validto FROM " . $prevtbprefix . "attendees";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "attendees (meeting,member,attended,authorized,required,schedule_string,string_validto) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // bookmarks
        $sql = "DELETE FROM " . $tbprefix . "bookmarks";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT title,url,created,cuser FROM " . $prevtbprefix . "bookmarks";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $id = sprintf("%024d", $row[3]) . sprintf("%024d", $row[2]) . sprintf("%020d", 0);
                $sql = "INSERT INTO " . $tbprefix . "bookmarks (id,title,url,created,cuser) VALUES ('$id','" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // calendar
        $sql = "DELETE FROM " . $tbprefix . "calendars";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT owner,target,target_id FROM " . $prevtbprefix . "calendars";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "calendars (owner,target,target_id) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // calendar_events
        $sql = "DELETE FROM " . $tbprefix . "calendar_events";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,target,target_id,subject,description,time_start,time_end,reminder_time1,reminder_time2,created,cuser,modified,muser FROM " . $prevtbprefix . "calendar_events";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "calendar_events (id,target,target_id,subject,description,time_start,time_end,reminder_time1,reminder_time2,created,cuser,modified,muser) VALUES ('$id','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "','" . $row[9] . "','" . $row[10] . "','" . $row[11] . "','" . $row[12] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "calendar_events AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // calendar_events - History
        $sql = "DELETE FROM " . $tbprefix . "calendar_events_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,target,target_id,subject,description,time_start,time_end,reminder_time1,reminder_time2,created,cuser,modified,muser FROM " . $prevtbprefix . "calendar_events_hist";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "calendar_events_hist (hid,action,id,target,target_id,subject,description,time_start,time_end,reminder_time1,reminder_time2,created,cuser,modified,muser) VALUES ('$hid','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "','" . $row[9] . "','" . $row[10] . "','" . $row[11] . "','" . $row[12] . "','" . $row[13] . "','" . $row[14] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "calendar_events_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // claim_type
        $sql = "DELETE FROM " . $tbprefix . "claim_type";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "claim_type ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "claim_type (id,name,locked,created,cuser,modified,muser) VALUES ('$id','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "claim_type AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // claim_type_hist
        $sql = "DELETE FROM " . $tbprefix . "claim_type_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "claim_type_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "claim_type_hist (hid,action,id,name,locked,created,cuser,modified,muser) VALUES ('$hid','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "claim_type_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // confirmation_levels
        $sql = "DELETE FROM " . $tbprefix . "confirmation_levels";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,comments,max_requirements FROM " . $prevtbprefix . "confirmation_levels ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "confirmation_levels (id,name,comments,max_requirements) VALUES ('$id','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "confirmation_levels AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // confirmation_levels_detail
        $sql = "DELETE FROM " . $tbprefix . "confirmation_levels_detail";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT confirmation_level,authorizer_type,authorizer,required FROM " . $prevtbprefix . "confirmation_levels_detail";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "confirmation_levels_detail (confirmation_level,authorizer_type,authorizer,required) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // contacts
        $sql = "DELETE FROM " . $tbprefix . "contacts";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,salutation,first_name,last_name,title,organization,department,do_not_call,email_work,email_home,email_other,phone_work,phone_home,phone_other,mobile,fax,address_street,address_city,address_state,address_postalcode,address_country,alt_address_street,alt_address_city,alt_address_state,alt_address_postalcode,alt_address_country,assistant,assistant_phone,birthdate,description,shared,member,created,cuser,modified,muser FROM " . $prevtbprefix . "contacts ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "contacts (id,salutation,first_name,last_name,title,organization,department,do_not_call,email_work,email_home,email_other,phone_work,phone_home,phone_other,mobile,fax,address_street,address_city,address_state,address_postalcode,address_country,alt_address_street,alt_address_city,alt_address_state,alt_address_postalcode,alt_address_country,assistant,assistant_phone,birthdate,description,shared,member,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "contacts AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // contacts_hist
        $sql = "DELETE FROM " . $tbprefix . "contacts_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,salutation,first_name,last_name,title,organization,department,do_not_call,email_work,email_home,email_other,phone_work,phone_home,phone_other,mobile,fax,address_street,address_city,address_state,address_postalcode,address_country,alt_address_street,alt_address_city,alt_address_state,alt_address_postalcode,alt_address_country,assistant,assistant_phone,birthdate,description,shared,member,created,cuser,modified,muser FROM " . $prevtbprefix . "contacts_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "contacts_hist (hid,action,id,salutation,first_name,last_name,title,organization,department,do_not_call,email_work,email_home,email_other,phone_work,phone_home,phone_other,mobile,fax,address_street,address_city,address_state,address_postalcode,address_country,alt_address_street,alt_address_city,alt_address_state,alt_address_postalcode,alt_address_country,assistant,assistant_phone,birthdate,description,shared,member,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "contacts_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // currency_rates
        $sql = "DELETE FROM " . $tbprefix . "currency_rates";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT currency_from,currency_to,reference_date,exchange_rate,created,cuser,modified,muser FROM " . $prevtbprefix . "currency_rates";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "currency_rates (currency_from,currency_to,reference_date,exchange_rate,created,cuser,modified,muser) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // denyips
        $sql = "DELETE FROM " . $tbprefix . "denyips";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT ipaddr,reason,created,cuser FROM " . $prevtbprefix . "denyips";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "denyips (ipaddr,reason,created,cuser) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // departments
        $sql = "DELETE FROM " . $tbprefix . "departments";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,organization,name,location,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "departments ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "departments (id,organization,name,location,locked,created,cuser,modified,muser) VALUES ('$id','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "departments AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // departments_hist
        $sql = "DELETE FROM " . $tbprefix . "departments_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,organization,name,location,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "departments_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "departments_hist (hid,action,id,organization,name,location,locked,created,cuser,modified,muser) VALUES ('$hid','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "','" . $row[9] . "','" . $row[10] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "departments_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // documents_approval
        $sql = "DELETE FROM " . $tbprefix . "documents_approval";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,approval,owner,docid,doctype,docdate,current_stage,status,canceled,closed,comments,userquery,created,cuser,modified,muser FROM " . $prevtbprefix . "documents_approval ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "documents_approval (id,approval,owner,docid,doctype,docdate,current_stage,status,canceled,closed,comments,userquery,created,cuser,modified,muser) VALUES ('$id','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "','" . $row[9] . "','" . $row[10] . "','" . $row[11] . "','" . $row[12] . "','" . $row[13] . "','" . $row[14] . "','" . $row[15] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "documents_approval AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // documents_approval_authorizers
        $sql = "DELETE FROM " . $tbprefix . "documents_approval_authorizers";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT document_approval,confirmation_level,authorizer,required,stage,status,approve_string,not_approve_string,string_validto,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "documents_approval_authorizers";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "documents_approval_authorizers (document_approval,confirmation_level,authorizer,required,stage,status,approve_string,not_approve_string,string_validto,comments,created,cuser,modified,muser) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "','" . $row[9] . "','" . $row[10] . "','" . $row[11] . "','" . $row[12] . "','" . $row[13] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // documents_approval_terms
        $sql = "DELETE FROM " . $tbprefix . "documents_approval_terms";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,document_approval,fieldname,ratio,value,compare_fields FROM " . $prevtbprefix . "documents_approval_terms ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "documents_approval_terms (id,document_approval,fieldname,ratio,value,compare_fields) VALUES ('$id','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "documents_approval_terms AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // equipment_category
        $sql = "DELETE FROM " . $tbprefix . "equipment_category";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "equipment_category ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "equipment_category (id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "equipment_category AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // equipment_category_hist
        $sql = "DELETE FROM " . $tbprefix . "equipment_category_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "equipment_category_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "equipment_category_hist (hid,action,id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "equipment_category_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // equipment_type
        $sql = "DELETE FROM " . $tbprefix . "equipment_type";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "equipment_type ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "equipment_type (id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "equipment_type AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // equipment_type_hist
        $sql = "DELETE FROM " . $tbprefix . "equipment_type_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "equipment_type_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "equipment_type_hist (hid,action,id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "equipment_type_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // equipments
        $sql = "DELETE FROM " . $tbprefix . "equipments";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'equipments');
        $sql = "SELECT id,equipment_id,equipment_category,equipment_type,name,owner,location,status,manufacturer,model,serial_number,purchase_date,warranty,cost_price,cost_currency,sell_price,sell_currency,photo,locked,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "equipments ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "equipments (id,equipment_id,equipment_category,equipment_type,name,owner,location,status,manufacturer,model,serial_number,purchase_date,warranty,cost_price,cost_currency,sell_price,sell_currency,photo,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "equipments AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // equipments_hist
        $sql = "DELETE FROM " . $tbprefix . "equipments_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'equipments');
        $sql = "SELECT hid,action,id,equipment_id,equipment_category,equipment_type,name,owner,location,status,manufacturer,model,serial_number,purchase_date,warranty,cost_price,cost_currency,sell_price,sell_currency,photo,locked,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "equipments_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "equipments_hist (hid,action,id,equipment_id,equipment_category,equipment_type,name,owner,location,status,manufacturer,model,serial_number,purchase_date,warranty,cost_price,cost_currency,sell_price,sell_currency,photo,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "equipments_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // expense_type
        $sql = "DELETE FROM " . $tbprefix . "expense_type";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,claimtype,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "expense_type ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "expense_type (id,name,claimtype,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "expense_type AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // expense_type_hist
        $sql = "DELETE FROM " . $tbprefix . "expense_type_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,claimtype,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "expense_type_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "expense_type_hist (hid,action,id,name,claimtype,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "expense_type_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // expenses
        $sql = "DELETE FROM " . $tbprefix . "expenses";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'expenses');
        $sql = "SELECT id,project,status,status_approval,printed,owner,description,docdate,billable,claimtype,expensetype,amount,currency,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "expenses ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "expenses (id,project,status,status_approval,printed,owner,description,docdate,billable,claimtype,expensetype,amount,currency,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "expenses AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // expenses_hist
        $sql = "DELETE FROM " . $tbprefix . "expenses_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'expenses');
        $sql = "SELECT hid,action,id,project,status,status_approval,printed,owner,description,docdate,billable,claimtype,expensetype,amount,currency,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "expenses_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "expenses_hist (hid,action,id,project,status,status_approval,printed,owner,description,docdate,billable,claimtype,expensetype,amount,currency,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "expenses_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // extensions
        $sql = "DELETE FROM " . $tbprefix . "extensions";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,description,version,active,created,cuser,modified,muser FROM " . $prevtbprefix . "extensions ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "extensions (id,name,description,version,active,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "extensions AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // extensions_hist
        $sql = "DELETE FROM " . $tbprefix . "extensions_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,description,version,active,created,cuser,modified,muser FROM " . $prevtbprefix . "extensions_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "extensions_hist (hid,action,id,name,description,version,active,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "extensions_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // field_filters
        $sql = "DELETE FROM " . $tbprefix . "field_filters";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT member,tablename,dbfieldname,pattern_begin,pattern_mid,pattern_end FROM " . $prevtbprefix . "field_filters";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "field_filters (member,tablename,dbfieldname,pattern_begin,pattern_mid,pattern_end) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // files
        $sql = "DELETE FROM " . $tbprefix . "files";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'files');
        $sql = "SELECT id,owner,doc_type,doc_type_ex,doc_id,name,size,extension,comments,published,status,vc_status,vc_version,vc_parent,dir,parent,fullname,project,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "files ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "files (id,owner,doc_type,doc_type_ex,doc_id,name,size,extension,comments,published,status,vc_status,vc_version,vc_parent,dir,parent,fullname,project,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "files AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // files_hist
        $sql = "DELETE FROM " . $tbprefix . "files_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'files');
        $sql = "SELECT hid,action,id,owner,doc_type,doc_type_ex,doc_id,name,size,extension,comments,published,status,vc_status,vc_version,vc_parent,dir,parent,fullname,project,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "files_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "files_hist (hid,action,id,owner,doc_type,doc_type_ex,doc_id,name,size,extension,comments,published,status,vc_status,vc_version,vc_parent,dir,parent,fullname,project,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "files_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // flextime_adjustment
        $sql = "DELETE FROM " . $tbprefix . "flextime_adjustment";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,owner,refdate,hours,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "flextime_adjustment ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "flextime_adjustment (id,owner,refdate,hours,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "flextime_adjustment AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // flextime_adjustment_hist
        $sql = "DELETE FROM " . $tbprefix . "flextime_adjustment_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,owner,refdate,hours,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "flextime_adjustment_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "flextime_adjustment_hist (hid,action,id,owner,refdate,hours,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "flextime_adjustment_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // history
        $sql = "DELETE FROM " . $tbprefix . "history";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,title,uri,created,cuser FROM " . $prevtbprefix . "history ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "history (id,title,uri,created,cuser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "history AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // hour_code
        $sql = "DELETE FROM " . $tbprefix . "hour_code";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,hourtype,sysdef,locked,service,created,cuser,modified,muser FROM " . $prevtbprefix . "hour_code ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "hour_code (id,hourtype,sysdef,locked,service,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // hour_type
        $sql = "DELETE FROM " . $tbprefix . "hour_type";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,sysdef,locked,service,created,cuser,modified,muser FROM " . $prevtbprefix . "hour_type ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "hour_type (id,sysdef,locked,service,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // invoices
        $sql = "DELETE FROM " . $tbprefix . "invoices";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'invoices');
        $sql = "SELECT id,invoice_no,organization,owner,status,status_approval,printed,doc_date,due_date,payment_terms,payment_method,currency,before_discount,discount_percent,discount_sum,freight,tax_sum,amount,paid,notes,bill_to,ship_to,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,tax_group,tax_group2,tax_group3,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "invoices ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "invoices (id,invoice_no,organization,owner,status,status_approval,printed,doc_date,due_date,payment_terms,payment_method,currency,before_discount,discount_percent,discount_sum,freight,tax_sum,amount,paid,notes,bill_to,ship_to,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,tax_group,tax_group2,tax_group3,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "invoices AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // invoices_hist
        $sql = "DELETE FROM " . $tbprefix . "invoices_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'invoices');
        $sql = "SELECT hid,action,id,invoice_no,organization,owner,status,status_approval,printed,doc_date,due_date,payment_terms,payment_method,currency,before_discount,discount_percent,discount_sum,freight,tax_sum,amount,paid,notes,bill_to,ship_to,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,tax_group,tax_group2,tax_group3,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "invoices_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "invoices_hist (hid,action,id,invoice_no,organization,owner,status,status_approval,printed,doc_date,due_date,payment_terms,payment_method,currency,before_discount,discount_percent,discount_sum,freight,tax_sum,amount,paid,notes,bill_to,ship_to,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,tax_group,tax_group2,tax_group3,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "invoices_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // invoice_rows
        $sql = "DELETE FROM " . $tbprefix . "invoice_rows";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,invoice,description,project,quantity,price,unit,discount_percent,discount_sum,tax_code,tax_sum,tax_code2,tax_sum2,tax_code3,tax_sum3,line_total,created,cuser,modified,muser FROM " . $prevtbprefix . "invoice_rows ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "invoice_rows (id,invoice,description,project,quantity,price,unit,discount_percent,discount_sum,tax_code,tax_sum,tax_code2,tax_sum2,tax_code3,tax_sum3,line_total,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "invoice_rows AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // invoice_rows_hist
        $sql = "DELETE FROM " . $tbprefix . "invoice_rows_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,invoice,description,project,quantity,price,unit,discount_percent,discount_sum,tax_code,tax_sum,tax_code2,tax_sum2,tax_code3,tax_sum3,line_total,created,cuser,modified,muser FROM " . $prevtbprefix . "invoice_rows_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "invoice_rows_hist (hid,action,id,invoice,description,project,quantity,price,unit,discount_percent,discount_sum,tax_code,tax_sum,tax_code2,tax_sum2,tax_code3,tax_sum3,line_total,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "invoice_rows_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // leaves
        $sql = "DELETE FROM " . $tbprefix . "leaves";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,owner,year,type,hours,due_date,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "leaves ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "leaves (id,owner,year,type,hours,due_date,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "leaves AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // leaves_hist
        $sql = "DELETE FROM " . $tbprefix . "leaves_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,owner,year,type,hours,due_date,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "leaves_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "leaves_hist (hid,action,id,owner,year,type,hours,due_date,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "leaves_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // leave_requests
        $sql = "DELETE FROM " . $tbprefix . "leave_requests";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'leave_requests');
        $sql = "SELECT id,owner,type,date_start,date_end,time_start,time_end,hours_total,hours_start,hours_end,status,comments,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "leave_requests ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "leave_requests (id,owner,type,date_start,date_end,time_start,time_end,hours_total,hours_start,hours_end,status,comments,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "leave_requests AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // leave_requests_hist
        $sql = "DELETE FROM " . $tbprefix . "leave_requests_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'leave_requests');
        $sql = "SELECT hid,action,id,owner,type,date_start,date_end,time_start,time_end,hours_total,hours_start,hours_end,status,comments,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "leave_requests_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "leave_requests_hist (hid,action,id,owner,type,date_start,date_end,time_start,time_end,hours_total,hours_start,hours_end,status,comments,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "leave_requests_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // locations
        $sql = "DELETE FROM " . $tbprefix . "locations";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,timezone,language,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "locations ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $n = $row[1];
                $sql = "INSERT INTO " . $tbprefix . "locations (id,name,timezone,language,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "locations AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // locations_hist
        $sql = "DELETE FROM " . $tbprefix . "locations_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,timezone,language,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "locations_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "locations_hist (hid,action,id,name,timezone,language,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "locations_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // loginips
        $sql = "DELETE FROM " . $tbprefix . "loginips";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT login,ipaddr,loginfail,ipclassb,compt,last_visite,approve_string,not_approve_string FROM " . $prevtbprefix . "loginips";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "loginips (login,ipaddr,loginfail,ipclassb,compt,last_visite,approve_string,not_approve_string) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // materials
        $sql = "DELETE FROM " . $tbprefix . "materials";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'materials');
        $sql = "SELECT id,name,unit,currency,costrate,locked,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "materials ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "materials (id,name,unit,currency,costrate,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "materials AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // materials_hist
        $sql = "DELETE FROM " . $tbprefix . "materials_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'materials');
        $sql = "SELECT hid,action,id,name,unit,currency,costrate,locked,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "materials_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "materials_hist (hid,action,id,name,unit,currency,costrate,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "materials_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // material_issue
        $sql = "DELETE FROM " . $tbprefix . "material_issue";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,task,owner,material,date,quantity,created,cuser,modified,muser FROM " . $prevtbprefix . "material_issue ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "material_issue (id,project,task,owner,material,date,quantity,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "material_issue AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // material_issue_hist
        $sql = "DELETE FROM " . $tbprefix . "material_issue_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,project,task,owner,material,date,quantity,created,cuser,modified,muser FROM " . $prevtbprefix . "material_issue_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "material_issue_hist (hid,action,id,project,task,owner,material,date,quantity,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "material_issue_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // meetings
        $sql = "DELETE FROM " . $tbprefix . "meetings";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'meetings');
        $sql = "SELECT id,project,priority,status,owner,name,agenda,location,minutes,duration,start_time,reminder,reminder_time1,reminder_time2,confirmed,allreplied,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "meetings ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "meetings (id,project,priority,status,owner,name,agenda,location,minutes,duration,start_time,reminder,reminder_time1,reminder_time2,confirmed,allreplied,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "meetings AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // meetings_hist
        $sql = "DELETE FROM " . $tbprefix . "meetings_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'meetings');
        $sql = "SELECT hid,action,id,project,priority,status,owner,name,agenda,location,minutes,duration,start_time,reminder,reminder_time1,reminder_time2,confirmed,allreplied,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "meetings_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "meetings_hist (hid,action,id,project,priority,status,owner,name,agenda,location,minutes,duration,start_time,reminder,reminder_time1,reminder_time2,confirmed,allreplied,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "meetings_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // meetings_available
        $sql = "DELETE FROM " . $tbprefix . "meetings_available";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT meeting,member,schedule FROM " . $prevtbprefix . "meetings_available";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "meetings_available (meeting,member,schedule) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // meetings_schedule
        $sql = "DELETE FROM " . $tbprefix . "meetings_schedule";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,meeting,start_time,canceled FROM " . $prevtbprefix . "meetings_schedule ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "meetings_schedule (id,meeting,start_time,canceled) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "meetings_schedule AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // meetings_time
        $sql = "DELETE FROM " . $tbprefix . "meetings_time";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,meeting,owner,date,hours,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "meetings_time ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "meetings_time (id,project,meeting,owner,date,hours,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "meetings_time AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // members
        $sql = "DELETE FROM " . $tbprefix . "members";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'members');
        $sql = "SELECT id,organization,department,login,password,name,title,email_work,email_home,phone_work,phone_home,mobile,fax,comments,profil,language,logout_time,start_page,location,timezone,date_format,time_format,manager,deputy,theme,palette,gender,extension_picture,active,workcalendar,enable_flextime,begin_flextime,initial_flexibalance,invalid_logon,export_format,ldap_cn,enable_timeclock,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "members ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "members (id,organization,department,login,password,name,title,email_work,email_home,phone_work,phone_home,mobile,fax,comments,profil,language,logout_time,start_page,location,timezone,date_format,time_format,manager,deputy,theme,palette,gender,extension_picture,active,workcalendar,enable_flextime,begin_flextime,initial_flexibalance,invalid_logon,export_format,ldap_cn,enable_timeclock,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "members AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // members_hist
        $sql = "DELETE FROM " . $tbprefix . "members_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'members');
        $sql = "SELECT hid,action,id,organization,department,login,password,name,title,email_work,email_home,phone_work,phone_home,mobile,fax,comments,profil,language,logout_time,start_page,location,timezone,date_format,time_format,manager,deputy,theme,palette,gender,extension_picture,active,workcalendar,enable_flextime,begin_flextime,initial_flexibalance,invalid_logon,export_format,ldap_cn,enable_timeclock,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "members_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "members_hist (hid,action,id,organization,department,login,password,name,title,email_work,email_home,phone_work,phone_home,mobile,fax,comments,profil,language,logout_time,start_page,location,timezone,date_format,time_format,manager,deputy,theme,palette,gender,extension_picture,active,workcalendar,enable_flextime,begin_flextime,initial_flexibalance,invalid_logon,export_format,ldap_cn,enable_timeclock,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "members_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // messages
        $sql = "DELETE FROM " . $tbprefix . "messages";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,from_id,to_id,message_type,message,extension_picture,subject,seen,archived,created FROM " . $prevtbprefix . "messages ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "messages (id,from_id,to_id,message_type,message,extension_picture,subject,seen,archived,created) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "messages AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // notes
        $sql = "DELETE FROM " . $tbprefix . "notes";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'notes');
        $sql = "SELECT id,project,owner,topic,subject,description,date,published,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "notes ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "notes (id,project,owner,topic,subject,description,date,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "notes AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // notes_hist
        $sql = "DELETE FROM " . $tbprefix . "notes_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'notes');
        $sql = "SELECT hid,action,id,project,owner,topic,subject,description,date,published,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "notes_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "notes_hist (hid,action,id,project,owner,topic,subject,description,date,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "notes_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // notifications
        $sql = "DELETE FROM " . $tbprefix . "notifications";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT member,notiid,block,block_string,string_validto FROM " . $prevtbprefix . "notifications";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "notifications (member,notiid,block,block_string,string_validto) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // organizations
        $sql = "DELETE FROM " . $tbprefix . "organizations";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'organizations');
        $sql = "SELECT id,name,address,zip_code,city,country,phone,fax,url,email,comments,sscomments,extension_logo,owner,currency,workcalendar,payment_method,payment_terms,locked,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "organizations ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "organizations (id,name,address,zip_code,city,country,phone,fax,url,email,comments,sscomments,extension_logo,owner,currency,workcalendar,payment_method,payment_terms,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "organizations AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // organizations_hist
        $sql = "DELETE FROM " . $tbprefix . "organizations_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'organizations');
        $sql = "SELECT hid,action,id,name,address,zip_code,city,country,phone,fax,url,email,comments,sscomments,extension_logo,owner,currency,workcalendar,payment_method,payment_terms,locked,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "organizations_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "organizations_hist (hid,action,id,name,address,zip_code,city,country,phone,fax,url,email,comments,sscomments,extension_logo,owner,currency,workcalendar,payment_method,payment_terms,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "organizations_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // organizations_extension
        $sql = "DELETE FROM " . $tbprefix . "organizations_extension";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT organization,extension,perm_read,perm_write,perm_edit,perm_admin,created,cuser FROM " . $prevtbprefix . "organizations_extension";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "organizations_extension (organization,extension,perm_read,perm_write,perm_edit,perm_admin,created,cuser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // organizations_form
        $sql = "DELETE FROM " . $tbprefix . "organizations_form";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,organization,tablename,addenabled,modenabled,created,cuser,modified,muser FROM " . $prevtbprefix . "organizations_form ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "organizations_form (id,organization,tablename,addenabled,modenabled,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "organizations_form AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // payment_methods
        $sql = "DELETE FROM " . $tbprefix . "payment_methods";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "payment_methods ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "payment_methods (id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "payment_methods AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // payment_methods_hist
        $sql = "DELETE FROM " . $tbprefix . "payment_methods_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "payment_methods_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "payment_methods_hist (hid,action,id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "payment_methods_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // payment_terms
        $sql = "DELETE FROM " . $tbprefix . "payment_terms";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,start_from,extra_months,extra_days,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "payment_terms ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "payment_terms (id,name,start_from,extra_months,extra_days,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "payment_terms AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // payment_terms_hist
        $sql = "DELETE FROM " . $tbprefix . "payment_terms_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,start_from,extra_months,extra_days,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "payment_terms_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "payment_terms_hist (hid,action,id,name,start_from,extra_months,extra_days,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "payment_terms_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // payments
        $sql = "DELETE FROM " . $tbprefix . "payments";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'payments');
        $sql = "SELECT id,invoice,currency,amount,doc_date,payment_method,notes,printed,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "payments ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "payments (id,invoice,currency,amount,doc_date,payment_method,notes,printed,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "payments AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // payments_hist
        $sql = "DELETE FROM " . $tbprefix . "payments_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'payments');
        $sql = "SELECT hid,action,id,invoice,currency,amount,doc_date,payment_method,notes,printed,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "payments_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "payments_hist (hid,action,id,invoice,currency,amount,doc_date,payment_method,notes,printed,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "payments_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // periods
        $sql = "DELETE FROM " . $tbprefix . "periods";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,fyear,serial_no,date_start,date_end,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "periods ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "periods (id,fyear,serial_no,date_start,date_end,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "periods AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // phase_sets
        $sql = "DELETE FROM " . $tbprefix . "phase_sets";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,phase_num,created,cuser,modified,muser FROM " . $prevtbprefix . "phase_sets ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "phase_sets (id,name,phase_num,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "phase_sets AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // phases
        $sql = "DELETE FROM " . $tbprefix . "phases";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'phases');
        $sql = "SELECT id,project,phase_set,order_num,status,date_start,date_end,comments,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "phases ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "phases (id,project,phase_set,order_num,status,date_start,date_end,comments,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "phases AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // phases_hist
        $sql = "DELETE FROM " . $tbprefix . "phases_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'phases');
        $sql = "SELECT hid,action,id,project,phase_set,order_num,status,date_start,date_end,comments,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "phases_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "phases_hist (hid,action,id,project,phase_set,order_num,status,date_start,date_end,comments,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "phases_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // posts
        $sql = "DELETE FROM " . $tbprefix . "posts";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'posts');
        $sql = "SELECT id,topic,member,message,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "posts ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "posts (id,topic,member,message,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "posts AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // posts_hist
        $sql = "DELETE FROM " . $tbprefix . "posts_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'posts');
        $sql = "SELECT hid,action,id,topic,member,message,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "posts_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "posts_hist (hid,action,id,topic,member,message,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "posts_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // print_layouts
        $sql = "DELETE FROM " . $tbprefix . "print_layouts";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,doc_type,doc_type_ex,description,path,start_page,defaults,created,cuser,modified,muser FROM " . $prevtbprefix . "print_layouts ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "print_layouts (id,doc_type,doc_type_ex,description,path,start_page,defaults,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "print_layouts AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // projects
        $sql = "DELETE FROM " . $tbprefix . "projects";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'projects');
        $sql = "SELECT id,organization,owner,priority,status,status_next,status_approval,open_date,complete_date,name,description,published,phase_set,type,upload_dir,currency,permission_set,cost_method,workcalendar,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "projects ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "projects (id,organization,owner,priority,status,status_next,status_approval,open_date,complete_date,name,description,published,phase_set,type,upload_dir,currency,permission_set,cost_method,workcalendar,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "projects AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // projects_hist
        $sql = "DELETE FROM " . $tbprefix . "projects_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'projects');
        $sql = "SELECT hid,action,id,organization,owner,priority,status,status_next,status_approval,open_date,complete_date,name,description,published,phase_set,type,upload_dir,currency,permission_set,cost_method,workcalendar,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "projects_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "projects_hist (hid,action,id,organization,owner,priority,status,status_next,status_approval,open_date,complete_date,name,description,published,phase_set,type,upload_dir,currency,permission_set,cost_method,workcalendar,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "projects_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // projects_budget
        $sql = "DELETE FROM " . $tbprefix . "projects_budget";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,costtype,currency,amount,created,cuser,modified,muser FROM " . $prevtbprefix . "projects_budget ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "projects_budget (id,project,costtype,currency,amount,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "projects_budget AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // projects_budget_hist
        $sql = "DELETE FROM " . $tbprefix . "projects_budget_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,project,costtype,currency,amount,created,cuser,modified,muser FROM " . $prevtbprefix . "projects_budget_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "projects_budget_hist (hid,action,id,project,costtype,currency,amount,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "projects_budget_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // projects_form
        $sql = "DELETE FROM " . $tbprefix . "projects_form";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT project,tablename,publishenabled,created,cuser,modified,muser FROM " . $prevtbprefix . "projects_form";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "projects_form (project,tablename,publishenabled,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // project_permissions
        $sql = "DELETE FROM " . $tbprefix . "project_permissions";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,profil,view_project,create_customer_service_site,copy_project,import_project,export_project,modify_project_properties,delete_project,view_project_budget,view_project_cost,view_phase,modify_phase_properties,view_task,create_task,copy_task,modify_task_status,log_task_hours,material_issue,arrange_task_resource,modify_task_properties,edit_task_relationships,delete_task,configure_task_security,publish_task,view_milestone,create_milestone,modify_milestone_properties,delete_milestone,configure_milestone_security,publish_milestone,view_meeting,create_meeting,copy_meeting,modify_meeting_status,log_meeting_hours,arrange_meeting_resource,modify_meeting_properties,delete_meeting,configure_meeting_security,view_support_request,add_support_response,assign_support_request,close_support_request,delete_support_request,view_discussion,add_discussion_topic,post_reply,close_discussion_topic,publish_discussion_topic,delete_discussion_topic,view_team,send_message,add_team_member,publish_team_member,modify_team_member_properties,delete_team_member,view_file,add_file,update_file,publish_file,delete_file,view_note,add_note,modify_note_properties,delete_note,publish_note,view_calendar,add_calendar_event,modify_calendar_event,delete_calendar_event,view_role,configure_role_security,view_expense,create_expense,modify_expense_properties,delete_expense,created,cuser,modified,muser FROM " . $prevtbprefix . "project_permissions ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "project_permissions (id,project,profil,view_project,create_customer_service_site,copy_project,import_project,export_project,modify_project_properties,delete_project,view_project_budget,view_project_cost,view_phase,modify_phase_properties,view_task,create_task,copy_task,modify_task_status,log_task_hours,material_issue,arrange_task_resource,modify_task_properties,edit_task_relationships,delete_task,configure_task_security,publish_task,view_milestone,create_milestone,modify_milestone_properties,delete_milestone,configure_milestone_security,publish_milestone,view_meeting,create_meeting,copy_meeting,modify_meeting_status,log_meeting_hours,arrange_meeting_resource,modify_meeting_properties,delete_meeting,configure_meeting_security,view_support_request,add_support_response,assign_support_request,close_support_request,delete_support_request,view_discussion,add_discussion_topic,post_reply,close_discussion_topic,publish_discussion_topic,delete_discussion_topic,view_team,send_message,add_team_member,publish_team_member,modify_team_member_properties,delete_team_member,view_file,add_file,update_file,publish_file,delete_file,view_note,add_note,modify_note_properties,delete_note,publish_note,view_calendar,add_calendar_event,modify_calendar_event,delete_calendar_event,view_role,configure_role_security,view_expense,create_expense,modify_expense_properties,delete_expense,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "project_permissions AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // project_permissions_hist
        $sql = "DELETE FROM " . $tbprefix . "project_permissions_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,project,profil,view_project,create_customer_service_site,copy_project,import_project,export_project,modify_project_properties,delete_project,view_project_budget,view_project_cost,view_phase,modify_phase_properties,view_task,create_task,copy_task,modify_task_status,log_task_hours,material_issue,arrange_task_resource,modify_task_properties,edit_task_relationships,delete_task,configure_task_security,publish_task,view_milestone,create_milestone,modify_milestone_properties,delete_milestone,configure_milestone_security,publish_milestone,view_meeting,create_meeting,copy_meeting,modify_meeting_status,log_meeting_hours,arrange_meeting_resource,modify_meeting_properties,delete_meeting,configure_meeting_security,view_support_request,add_support_response,assign_support_request,close_support_request,delete_support_request,view_discussion,add_discussion_topic,post_reply,close_discussion_topic,publish_discussion_topic,delete_discussion_topic,view_team,send_message,add_team_member,publish_team_member,modify_team_member_properties,delete_team_member,view_file,add_file,update_file,publish_file,delete_file,view_note,add_note,modify_note_properties,delete_note,publish_note,view_calendar,add_calendar_event,modify_calendar_event,delete_calendar_event,view_role,configure_role_security,view_expense,create_expense,modify_expense_properties,delete_expense,created,cuser,modified,muser FROM " . $prevtbprefix . "project_permissions_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "project_permissions_hist (hid,action,id,project,profil,view_project,create_customer_service_site,copy_project,import_project,export_project,modify_project_properties,delete_project,view_project_budget,view_project_cost,view_phase,modify_phase_properties,view_task,create_task,copy_task,modify_task_status,log_task_hours,material_issue,arrange_task_resource,modify_task_properties,edit_task_relationships,delete_task,configure_task_security,publish_task,view_milestone,create_milestone,modify_milestone_properties,delete_milestone,configure_milestone_security,publish_milestone,view_meeting,create_meeting,copy_meeting,modify_meeting_status,log_meeting_hours,arrange_meeting_resource,modify_meeting_properties,delete_meeting,configure_meeting_security,view_support_request,add_support_response,assign_support_request,close_support_request,delete_support_request,view_discussion,add_discussion_topic,post_reply,close_discussion_topic,publish_discussion_topic,delete_discussion_topic,view_team,send_message,add_team_member,publish_team_member,modify_team_member_properties,delete_team_member,view_file,add_file,update_file,publish_file,delete_file,view_note,add_note,modify_note_properties,delete_note,publish_note,view_calendar,add_calendar_event,modify_calendar_event,delete_calendar_event,view_role,configure_role_security,view_expense,create_expense,modify_expense_properties,delete_expense,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "project_permissions_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // report_category
        $sql = "DELETE FROM " . $tbprefix . "report_category";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,created,cuser,modified,muser FROM " . $prevtbprefix . "report_category ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "report_category (id,name,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "report_category AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // reports
        $sql = "DELETE FROM " . $tbprefix . "reports";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,category,name,url,tablename,sysdef,userquery,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "reports ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "reports (id,category,name,url,tablename,sysdef,userquery,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "reports AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // reports_rel_tables
        $sql = "DELETE FROM " . $tbprefix . "reports_rel_tables";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,report,tablename,table_left FROM " . $prevtbprefix . "reports_rel_tables ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "reports_rel_tables (id,report,tablename,table_left) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "reports_rel_tables AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // reports_rel_fields
        $sql = "DELETE FROM " . $tbprefix . "reports_rel_fields";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,report,rel_table,field_left,field_right FROM " . $prevtbprefix . "reports_rel_fields ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "reports_rel_fields (id,report,rel_table,field_left,field_right) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "reports_rel_fields AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // reports_criteria
        $sql = "DELETE FROM " . $tbprefix . "reports_criteria";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,report,rel_table,fieldname,fieldtype,ratio,value,table_right,compare_fields,parameter FROM " . $prevtbprefix . "reports_criteria ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "reports_criteria (id,report,rel_table,fieldname,fieldtype,ratio,value,table_right,compare_fields,parameter) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "reports_criteria AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // reports_output
        $sql = "DELETE FROM " . $tbprefix . "reports_output";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,report,rel_table,fieldname,fieldtype,title,disp_order,sort_order,sort_type,field_group,field_function FROM " . $prevtbprefix . "reports_output ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "reports_output (id,report,rel_table,fieldname,fieldtype,title,disp_order,sort_order,sort_type,field_group,field_function) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "reports_output AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // services
        $sql = "DELETE FROM " . $tbprefix . "services";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'services');
        $sql = "SELECT id,name,hourly_rate,hourly_rate_i,currency,locked,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "services ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "services (id,name,hourly_rate,hourly_rate_i,currency,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "services AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // services_hist
        $sql = "DELETE FROM " . $tbprefix . "services_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'services');
        $sql = "SELECT hid,action,id,name,hourly_rate,hourly_rate_i,currency,locked,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "services_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "services_hist (hid,action,id,name,hourly_rate,hourly_rate_i,currency,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "services_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // sorting
        $sql = "DELETE FROM " . $tbprefix . "sorting";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        // support_posts
        $sql = "DELETE FROM " . $tbprefix . "support_posts";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'support_posts');
        $sql = "SELECT id,request_id,message,date,owner,project,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "support_posts ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "support_posts (id,request_id,message,date,owner,project,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "support_posts AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // support_posts_hist
        $sql = "DELETE FROM " . $tbprefix . "support_posts_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'support_posts');
        $sql = "SELECT hid,action,id,request_id,message,date,owner,project,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "support_posts_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "support_posts_hist (hid,action,id,request_id,message,date,owner,project,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "support_posts_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // support_requests
        $sql = "DELETE FROM " . $tbprefix . "support_requests";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'support_requests');
        $sql = "SELECT id,owner,status,reproducibility,severity,priority,subject,message,date_open,date_close,project,assigned_to,resolution,auto_close,published,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "support_requests ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "support_requests (id,owner,status,reproducibility,severity,priority,subject,message,date_open,date_close,project,assigned_to,resolution,auto_close,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "support_requests AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // support_requests_hist
        $sql = "DELETE FROM " . $tbprefix . "support_requests_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'support_requests');
        $sql = "SELECT hid,action,id,owner,status,reproducibility,severity,priority,subject,message,date_open,date_close,project,assigned_to,resolution,auto_close,published,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "support_requests_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "support_requests_hist (hid,action,id,owner,status,reproducibility,severity,priority,subject,message,date_open,date_close,project,assigned_to,resolution,auto_close,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "support_requests_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // tasks
        $sql = "DELETE FROM " . $tbprefix . "tasks";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'tasks');
        $sql = "SELECT id,project,priority,status,owner,assigned_to,name,description,start_date,due_date,estimated_time,actual_time,comments,completion,assigned,published,parent_phase,complete_date,service,fixed_duration,milestone,type,task_group,level,parent_task,subtask,support_request,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "tasks ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "tasks (id,project,priority,status,owner,assigned_to,name,description,start_date,due_date,estimated_time,actual_time,comments,completion,assigned,published,parent_phase,complete_date,service,fixed_duration,milestone,type,task_group,level,parent_task,subtask,support_request,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "tasks AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // tasks_hist
        $sql = "DELETE FROM " . $tbprefix . "tasks_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'tasks');
        $sql = "SELECT hid,action,id,project,priority,status,owner,assigned_to,name,description,start_date,due_date,estimated_time,actual_time,comments,completion,assigned,published,parent_phase,complete_date,service,fixed_duration,milestone,type,task_group,level,parent_task,subtask,support_request,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "tasks_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "tasks_hist (hid,action,id,project,priority,status,owner,assigned_to,name,description,start_date,due_date,estimated_time,actual_time,comments,completion,assigned,published,parent_phase,complete_date,service,fixed_duration,milestone,type,task_group,level,parent_task,subtask,support_request,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "tasks_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // tasks_predecessor
        $sql = "DELETE FROM " . $tbprefix . "tasks_predecessor";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,task,predecessor,type,lag,created,cuser,modified,muser FROM " . $prevtbprefix . "tasks_predecessor ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "tasks_predecessor (id,task,predecessor,type,lag,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "tasks_predecessor AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // tasks_predecessor_hist
        $sql = "DELETE FROM " . $tbprefix . "tasks_predecessor_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,task,predecessor,type,lag,created,cuser,modified,muser FROM " . $prevtbprefix . "tasks_predecessor_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "tasks_predecessor_hist (hid,action,id,task,predecessor,type,lag,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "tasks_predecessor_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // tasks_resource
        $sql = "DELETE FROM " . $tbprefix . "tasks_resource";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,task,resource,type,quantity,created,cuser,modified,muser FROM " . $prevtbprefix . "tasks_resource ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "tasks_resource (id,project,task,resource,type,quantity,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "tasks_resource AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // tasks_resource_hist
        $sql = "DELETE FROM " . $tbprefix . "tasks_resource_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,project,task,resource,type,quantity,created,cuser,modified,muser FROM " . $prevtbprefix . "tasks_resource_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "tasks_resource_hist (hid,action,id,project,task,resource,type,quantity,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "tasks_resource_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // tasks_time
        $sql = "DELETE FROM " . $tbprefix . "tasks_time";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,task,owner,date,hours,comments,billing,service,created,cuser,modified,muser FROM " . $prevtbprefix . "tasks_time ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "tasks_time (id,project,task,owner,date,hours,comments,billing,service,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "tasks_time AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // tax_codes
        $sql = "DELETE FROM " . $tbprefix . "tax_codes";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,tax_group,name,rate,created,cuser,modified,muser FROM " . $prevtbprefix . "tax_codes ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "tax_codes (id,tax_group,name,rate,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "tax_codes AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // tax_codes_hist
        $sql = "DELETE FROM " . $tbprefix . "tax_codes_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,tax_group,name,rate,created,cuser,modified,muser FROM " . $prevtbprefix . "tax_codes_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "tax_codes_hist (hid,action,id,tax_group,name,rate,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "tax_codes_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // tax_groups
        $sql = "DELETE FROM " . $tbprefix . "tax_groups";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "tax_groups ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "tax_groups (id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "tax_groups AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // tax_groups_hist
        $sql = "DELETE FROM " . $tbprefix . "tax_groups_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "tax_groups_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "tax_groups_hist (hid,action,id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "tax_groups_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // teams
        $sql = "DELETE FROM " . $tbprefix . "teams";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,member,published,authorized,profil,service,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "teams ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "teams (id,project,member,published,authorized,profil,service,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "teams AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // teams_hist
        $sql = "DELETE FROM " . $tbprefix . "teams_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,project,member,published,authorized,profil,service,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "teams_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "teams_hist (hid,action,id,project,member,published,authorized,profil,service,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "teams_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // timeclock
        $sql = "DELETE FROM " . $tbprefix . "timeclock";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,owner,finish,project,task,meeting,work,clockin,clockout,hours,comments,billing,service FROM " . $prevtbprefix . "timeclock ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "timeclock (id,owner,finish,project,task,meeting,work,clockin,clockout,hours,comments,billing,service) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "timeclock AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // timesheet
        $sql = "DELETE FROM " . $tbprefix . "timesheet";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,owner,period,standard_hours,actual_hours,billable_hours,overtime_hours,status,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "timesheet ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "timesheet (id,owner,period,standard_hours,actual_hours,billable_hours,overtime_hours,status,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "timesheet AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // timesheet_hist
        $sql = "DELETE FROM " . $tbprefix . "timesheet_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,owner,period,standard_hours,actual_hours,billable_hours,overtime_hours,status,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "timesheet_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "timesheet_hist (hid,action,id,owner,period,standard_hours,actual_hours,billable_hours,overtime_hours,status,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "timesheet_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // topics
        $sql = "DELETE FROM " . $tbprefix . "topics";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'topics');
        $sql = "SELECT id,project,owner,subject,status,last_post,posts,published,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "topics ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "topics (id,project,owner,subject,status,last_post,posts,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "topics AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // topics_hist
        $sql = "DELETE FROM " . $tbprefix . "topics_hist";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $cfs = setupCustomFieldsSQLmy($mysqlip, $prevtbprefix, 'topics');
        $sql = "SELECT hid,action,id,project,owner,subject,status,last_post,posts,published,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "topics_hist ORDER BY hid";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $hid = 0;

            while (($row = $result->fetch_row())) {
                $hid = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "topics_hist (hid,action,id,project,owner,subject,status,last_post,posts,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $hid++;
            $sql = "ALTER TABLE " . $tbprefix . "topics_hist AUTO_INCREMENT=$hid";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // updates
        $sql = "DELETE FROM " . $tbprefix . "updates";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,type,item,member,comments,created FROM " . $prevtbprefix . "updates ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "updates (id,type,item,member,comments,created) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "updates AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // user_profiles
        $sql = "DELETE FROM " . $tbprefix . "user_profiles";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,department,location,language,date_format,time_format,workcalendar,enable_flextime,readonly,sysdef,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "user_profiles ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "user_profiles (id,name,department,location,language,date_format,time_format,workcalendar,enable_flextime,readonly,sysdef,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "user_profiles AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // user_profiles_detail
        $sql = "DELETE FROM " . $tbprefix . "user_profiles_detail";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT user_profile,function_name,access_level,granted FROM " . $prevtbprefix . "user_profiles_detail";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            while (($row = $result->fetch_row())) {
                $sql = "INSERT INTO " . $tbprefix . "user_profiles_detail (user_profile,function_name,access_level,granted) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }
        }

        // work_hours
        $sql = "DELETE FROM " . $tbprefix . "work_hours";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,work,owner,date,hours,comments,billing,service,baseref,created,cuser,modified,muser FROM " . $prevtbprefix . "work_hours ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "work_hours (id,project,work,owner,date,hours,comments,billing,service,baseref,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "work_hours AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // workcalendars
        $sql = "DELETE FROM " . $tbprefix . "workcalendars";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,description,created,cuser,modified,muser FROM " . $prevtbprefix . "workcalendars ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "workcalendars (id,name,description,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "workcalendars AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // workcalendars_weekday
        $sql = "DELETE FROM " . $tbprefix . "workcalendars_weekday";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,workcalendar,year,day_type,hours,start_time,end_time,created,cuser,modified,muser FROM " . $prevtbprefix . "workcalendars_weekday ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "workcalendars_weekday (id,workcalendar,year,day_type,hours,start_time,end_time,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "workcalendars_weekday AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        // workcalendars_detail
        $sql = "DELETE FROM " . $tbprefix . "workcalendars_detail";

        if (!$mysqlin->query($sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
            throw new Exception('', 1);
        }

        $sql = "SELECT id,workcalendar,year,date,type,hours,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "workcalendars_detail ORDER BY id";

        if (!($result = $mysqlip->query($sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", $mysqlip->error . '(' . $mysqlip->errno . ')');
            throw new Exception('', 1);
        } else {
            $id = 0;

            while (($row = $result->fetch_row())) {
                $id = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "workcalendars_detail (id,workcalendar,year,date,type,hours,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!$mysqlin->query($sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                    throw new Exception('', 1);
                }
            }

            $id++;
            $sql = "ALTER TABLE " . $tbprefix . "workcalendars_detail AUTO_INCREMENT=$id";

            if (!$mysqlin->query($sql)) {
                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                $block1->contentRow("", $mysqlin->error . '(' . $mysqlin->errno . ')');
                throw new Exception('', 1);
            }
        }

        $mysqlin->commit();
    } catch (Exception $e) {
        $error = true;

        if ($e->getCode() == '0') {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $e->getMessage(), "false", "");
        }

        $mysqlin->rollback();
    }

    $mysqlin->autocommit(true);
    $mysqlin->close();
    $mysqlip->close();
}

// data migration (PostgreSQL -> PostgreSQL)
function upgrade_database_pg2pg($prevdbserver, $prevdblogin, $prevdbpasswd, $prevdbname, $prevtbprefix, $dbserver, $dblogin, $dbpasswd, $dbname, $tbprefix)
{
    global $block1, $error, $prevroot, $prevlangDefault, $strings;
    $lang = (file_exists($prevroot . '/languages/lang_' . $prevlangDefault . '.php'))?$prevlangDefault:'en';
    require $prevroot . '/languages/lang_' . $lang . '.php';
    $pgsqlp = pg_connect('host=' . $prevdbserver . ' dbname=' . $prevdbname . ' user=' . $prevdblogin . ' password=' . $prevdbpasswd,  PGSQL_CONNECT_FORCE_NEW);

    if ($pgsqlp == false) {
        die('Connect Error');
    }

    $pgsqln = pg_connect('host=' . $dbserver . ' dbname=' . $dbname . ' user=' . $dblogin . ' password=' . $dbpasswd,  PGSQL_CONNECT_FORCE_NEW);

    if ($pgsqln == false) {
        die('Connect Error');
    }

    $error = false;
    set_time_limit(0);

    try {
        @pg_query($pgsqln, 'BEGIN');    // start a new transaction for the database connection
        // strings
        $sql = "SELECT string_key,language,string_val FROM " . $prevtbprefix . "strings";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $string_key = $row[0];
                $sql = "INSERT INTO " . $tbprefix . "strings (string_key,language,string_val) VALUES ('$string_key','" . $row[1] . "','" . $row[2] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // re-index table definitions
        $sql = "DELETE FROM " . $tbprefix . "tables_def";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $hisA = array();    // history table index
        $sql = "SELECT id,tablename,sysdef,cfenabled,searchable,printable,mainmenu,keyfields,formfields,dispfields,dispstyle,listfields,primfields,fathertable,linkedfield,approvalenabled,publishenabled,javascript,projectrelated,history,viewurl FROM " . $prevtbprefix . "tables_def ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;
            $tblA = array();

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tables_def_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $tblA[] = $row[1];
                $sql = "INSERT INTO " . $tbprefix . "tables_def (tablename,sysdef,cfenabled,searchable,printable,mainmenu,keyfields,formfields,dispfields,dispstyle,listfields,primfields,fathertable,linkedfield,approvalenabled,publishenabled,javascript,projectrelated,history,viewurl,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "','" . time() . "','1','" . time() . "','1')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);

            $h = fopen('tables_def.csv', 'r');
            $hdr = fgets($h);

            while (!feof($h)) {
                $r = fgets($h);
                $r = str_replace(array('"', ' ', "\r", "\n"), '', $r);

                if ($r == '') {
                    continue;
                }

                $ra = explode(',', $r);
                $sysdef = $ra[1];

                if ($sysdef == 'Y') {
                    $tblnam = $ra[0];
                } else {
                    $tblnam = '@' . $ra[0];
                }

                if ($ra[15] == 'Y') {
                    $hisA[] = $tblnam;
                }

                if (array_search($tblnam, $tblA) !== false) {
                    $sql = "UPDATE " . $tbprefix . "tables_def SET sysdef='$sysdef',cfenabled='" . $ra[2] . "',searchable='" . $ra[3] . "',printable='" . $ra[4] . "',keyfields='" . $ra[5] . "',dispfields='" . $ra[6] . "',dispstyle='" . $ra[7] . "',listfields='" . $ra[8] . "',primfields='" . $ra[9] . "',fathertable='" . $ra[10] . "',linkedfield='" . $ra[11] . "',approvalenabled='" . $ra[12] . "',publishenabled='" . $ra[13] . "',projectrelated='" . $ra[14] . "',history='" . $ra[15] . "'  WHERE tablename='$tblnam'";
                } else {
                    $sql = "INSERT INTO " . $tbprefix . "tables_def (tablename,sysdef,cfenabled,searchable,printable,keyfields,dispfields,dispstyle,listfields,primfields,fathertable,linkedfield,approvalenabled,publishenabled,projectrelated,history,viewurl,created,cuser,modified,muser) VALUES ('$tblnam','$sysdef','" . $ra[2] . "','" . $ra[3] . "','" . $ra[4] . "','" . $ra[5] . "','" . $ra[6] . "','" . $ra[7] . "','" . $ra[8] . "','" . $ra[9] . "','" . $ra[10] . "','" . $ra[11] . "','" . $ra[12] . "','" . $ra[13] . "','" . $ra[14] . "','" . $ra[15] . "','','" . time() . "','1','" . time() . "','1')";
                }

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            fclose($h);
        }

        // create form templates
        $sql = "SELECT tablename,fathertable,publishenabled,history FROM " . $prevtbprefix . "tables_def WHERE sysdef='N'";
        $ftA = array();
        $fdA = array();
        $ffA = array();

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $tblnam = $row[0];
                $ftA[] = $tblnam;
                $dbtblnam = str_replace('@', 'FORM_', $tblnam);
                $fdA[] = $dbtblnam;
                $ftbl = $row[1];
                $pe = $row[2];
                $his = $row[3];
                $sql = "CREATE TABLE " . $tbprefix . $dbtblnam . " (id serial" . (($ftbl=="")?"":",pid integer NOT NULL default '0'") . ",status integer NOT NULL default '0',printed char(1) NOT NULL default '0',project integer NOT NULL default '0'" . (($pe=='Y')?",published char(1) NOT NULL default ''":"") . ",created integer NOT NULL default '0',cuser integer NOT NULL default '0',modified integer NOT NULL default '0',muser integer NOT NULL default '0',PRIMARY KEY (id))";
                $ffA[] = (($ftbl=='')?'':'pid,') . "status,project" . (($pe=='Y')?',published':'') . ",created,cuser,modified,muser";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                if ($his == 'Y') {
                    $sql = "CREATE TABLE " . $tbprefix . $dbtblnam . "_hist (hid serial,action char(1) NOT NULL default '',id integer NOT NULL default '0'" . (($ftbl=="")?"":",pid integer NOT NULL default '0'") . ",status integer NOT NULL default '0',printed char(1) NOT NULL default '0',project integer NOT NULL default '0'" . (($pe=='Y')?",published char(1) NOT NULL default ''":"") . ",created integer NOT NULL default '0',cuser integer NOT NULL default '0',modified integer NOT NULL default '0',muser integer NOT NULL default '0',PRIMARY KEY (hid))";
                    $hisA[] = $tblnam;

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }
            }

            @pg_free_result($result);
        }

        // create custom fields
        $sql = "SELECT fdf.tablename FROM " . $prevtbprefix . "fields_def fdf WHERE fdf.sysdef='N' GROUP BY fdf.tablename";
        $tblnamA = array();
        $fieldDataType = array( enumFieldType::ftText => 'varchar(%length%)', enumFieldType::ftAlphaNumeric => 'varchar(%length%)', enumFieldType::ftNumeric => 'varchar(%length%)', enumFieldType::ftMemo => 'text', enumFieldType::ftDate => "integer NOT NULL default '0'", enumFieldType::ftTime => 'char(4)', enumFieldType::ftHours => 'decimal(19,6)', enumFieldType::ftQuantity => 'decimal(19,6)', enumFieldType::ftMeasure => 'decimal(19,6)', enumFieldType::ftRate => 'decimal(19,6)', enumFieldType::ftTotals => 'decimal(19,6)', enumFieldType::ftImage => 'varchar(1024)', enumFieldType::ftAudio => 'varchar(1024)', enumFieldType::ftVideo => 'varchar(1024)', enumFieldType::ftBoolean => 'char(1)', enumFieldType::ftEmail => 'varchar(255)', enumFieldType::ftUrl => 'varchar(1024)', enumFieldType::ftSerial => 'varchar(%length%)', enumFieldType::ftTimestamp => "integer NOT NULL default '0'", enumFieldType::ftPercentage => 'decimal(19,6)', enumFieldType::ftPassword => 'varchar(1024)' );

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $tblnamA[] = $row[0];
            }

            @pg_free_result($result);
        }

        if (count($tblnamA) > 0) {
            for ($j=0; $j<count($tblnamA); $j++) {
                $tblnam = $tblnamA[$j];
                $sql = "SELECT tablename,fieldname,fieldtype,length,default_value,required,unique_field,create_index,sysdef,published,linkedtable,disporder FROM " . $prevtbprefix . "fields_def WHERE tablename='$tblnam' AND sysdef='N' ORDER BY disporder";

                if (!($result = pg_query($pgsqlp, $sql))) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqlp));
                    throw new Exception('', 1);
                } else {
                    $fldnamA = array();
                    $fldtypA = array();
                    $lenA = array();
                    $defvalA = array();
                    $reqA = array();
                    $uniA = array();
                    $idxA = array();

                    while (($row = pg_fetch_row($result))) {
                        $sql = "INSERT INTO " . $tbprefix . "fields_def (tablename,fieldname,fieldtype,length,default_value,required,unique_field,create_index,sysdef,published,linkedtable,disporder) VALUES ('" . implode("','", $row) . "')";

                        if (!pg_query($pgsqln, $sql)) {
                            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                            $block1->contentRow("", pg_last_notice($pgsqln));
                            throw new Exception('', 1);
                        }

                        $fldnamA[] = $row[1];
                        $fldtypA[] = $row[2];
                        $lenA[] = $row[3];
                        $defvalA[] = $row[4];
                        $reqA[] = $row[5];
                        $uniA[] = $row[6];
                        $idxA[] = $row[7];
                    }

                    @pg_free_result($result);

                    for ($i=0; $i<count($fldnamA); $i++) {
                        $dbtblnam = $tbprefix . str_replace('@', 'FORM_', $tblnam);
                        $fldnam = $fldnamA[$i];
                        $fldtyp = $fldtypA[$i];
                        $len = $lenA[$i];
                        $req = $reqA[$i];
                        $defval = $defvalA[$i];
                        $sql = "ALTER TABLE $dbtblnam ADD U_$fldnam";

                        if ($fldtyp == enumFieldType::ftText) {
                            $sql .= ' ' . str_replace('%length%', intval($len)*4, $fieldDataType[$fldtyp]);
                        } else if ($fldtyp == enumFieldType::ftNumeric || $fldtyp == enumFieldType::ftAlphaNumeric || $fldtyp == enumFieldType::ftSerial) {
                            $sql .= ' ' . str_replace('%length%', intval($len), $fieldDataType[$fldtyp]);
                        } else {
                            $sql .= ' ' . $fieldDataType[$fldtyp];
                        }

                        // required field
                        if ($req == 'Y') {
                            $sql .= ' NOT NULL';
                        }

                        // default value
                        if ($defval != '') {
                            switch ($fldtyp) {
                                case enumFieldType::ftText:
                                case enumFieldType::ftMemo:
                                    $sql .= " DEFAULT '" . HtmlEntitiesNOD($defval) . "'";
                                    break;
                                case enumFieldType::ftAlphaNumeric:
                                case enumFieldType::ftNumeric:
                                case enumFieldType::ftTime:
                                case enumFieldType::ftEmail:
                                case enumFieldType::ftUrl:
                                    $sql .= " DEFAULT '$defval'";
                                    break;
                                case enumFieldType::ftDate:
                                    if (!preg_match('/^(\+|\-)\d+$/', $defval)) {
                                        $sql .= " DEFAULT '" . mktime(0,0,0,intval(substr($defval,4,2)),intval(substr($defval,6,2)),intval(substr($defval,0,4))) . "'";
                                    }

                                    break;
                                case enumFieldType::ftHours:
                                case enumFieldType::ftQuantity:
                                case enumFieldType::ftMeasure:
                                case enumFieldType::ftRate:
                                case enumFieldType::ftTotals:
                                case enumFieldType::ftPercentage:
                                    $sql .= " DEFAULT $defval";
                                    break;
                                case enumFieldType::ftBoolean:
                                    $sql .= " DEFAULT '" . (($defval=='1')?enumBoolean::bTrue:enumBoolean::bFalse) . "'";
                                    break;
                            }
                        }

                        if (!pg_query($pgsqln, $sql)) {
                            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                            $block1->contentRow("", pg_last_notice($pgsqln));
                            throw new Exception('', 1);
                        }

                        if (array_search($tblnam, $hisA) !== false) {   // create custom field in history table as well
                            $sql = "ALTER TABLE " . $dbtblnam . "_hist ADD U_$fldnam";

                            if ($fldtyp == enumFieldType::ftText) {
                                $sql .= ' ' . str_replace('%length%', intval($len)*4, $fieldDataType[$fldtyp]);
                            } else if ($fldtyp == enumFieldType::ftNumeric || $fldtyp == enumFieldType::ftAlphaNumeric || $fldtyp == enumFieldType::ftSerial) {
                                $sql .= ' ' . str_replace('%length%', intval($len), $fieldDataType[$fldtyp]);
                            } else {
                                $sql .= ' ' . $fieldDataType[$fldtyp];
                            }

                            if (!pg_query($pgsqln, $sql)) {
                                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                                $block1->contentRow("", pg_last_notice($pgsqln));
                                throw new Exception('', 1);
                            }
                        }

                        // create index
                        if ($idx == 'Y') {
                            $sql = "CREATE " . (($uni=='Y')?'UNIQUE':'') . " INDEX idxu" . $dbtblnam . $fldnam . " ON $dbtblnam (U_$fldnam)";

                            if (!pg_query($pgsqln, $sql)) {
                                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                                $block1->contentRow("", pg_last_notice($pgsqln));
                                throw new Exception('', 1);
                            }
                        }

                        // add valid values
                        $sql = "SELECT fieldvalue FROM " . $prevtbprefix . "fields_valdef WHERE tablename='$tblnam' AND fieldname='$fldnam' AND sysdef='N'";

                        if (!($result = pg_query($pgsqlp, $sql))) {
                            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                            $block1->contentRow("", pg_last_notice($pgsqlp));
                            throw new Exception('', 1);
                        } else {
                            while (($row = pg_fetch_row($result))) {
                                $sql = "INSERT INTO " . $tbprefix . "fields_valdef (tablename,fieldname,fieldvalue,sysdef) VALUES ('$tblnam','$fldnam','" . $row[0] . "','N')";

                                if (!pg_query($pgsqln, $sql)) {
                                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                                    $block1->contentRow("", pg_last_notice($pgsqln));
                                    throw new Exception('', 1);
                                }
                            }

                            @pg_free_result($result);
                        }
                    }
                }
            }
        }

        // copy custom form templates data
        if (count($ftA) > 0) {
            for ($i=0; $i<count($ftA); $i++) {
                $sql = "SELECT fieldname FROM " . $prevtbprefix . "fields_def WHERE tablename='" . $ftA[$i] . "' AND sysdef='N'";

                if (!($result = pg_query($pgsqlp, $sql))) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqlp));
                    throw new Exception('', 1);
                } else {
                    while (($row = pg_fetch_row($result))) {
                        $ffA[$i] .= ",U_" . $row[0];
                    }

                    @pg_free_result($result);
                }

                $sql = "SELECT id," . $ffA[$i] . " FROM " . $prevtbprefix . $fdA[$i] . " ORDER BY id";

                if (!($result = pg_query($pgsqlp, $sql))) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqlp));
                    throw new Exception('', 1);
                } else {
                    $id = -1;

                    while (($row = pg_fetch_row($result))) {
                        if ($id != $row[0]) {
                            $id = $row[0];
                            $sql = "ALTER SEQUENCE " . $tbprefix . $fdA[$i] . "_id_seq RESTART WITH " . $row[0];

                            if (!pg_query($pgsqln, $sql)) {
                                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                                $block1->contentRow("", pg_last_notice($pgsqln));
                                throw new Exception('', 1);
                            }
                        }

                        for ($x=0; $x<count($row); $x++) $row[$x] = addslashes($row[$x]);
                        unset($row[0]);
                        $sql = "INSERT INTO " . $tbprefix . $fdA[$i] . " (" . $ffA[$i] . ") VALUES ('" . implode("','", $row) . "')";

                        if (!pg_query($pgsqln, $sql)) {
                            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                            $block1->contentRow("", pg_last_notice($pgsqln));
                            throw new Exception('', 1);
                        }

                        $id++;
                    }

                    @pg_free_result($result);
                }

                if (array_search($ftA[$i], $hisA) !== false) {
                    $sql = "SELECT hid,action,id," . $ffA[$i] . " FROM " . $prevtbprefix . $fdA[$i] . "_hist ORDER BY hid";

                    if (!($result = pg_query($pgsqlp, $sql))) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqlp));
                        throw new Exception('', 1);
                    } else {
                        $hid = -1;

                        while (($row = pg_fetch_row($result))) {
                            if ($hid != $row[0]) {
                                $hid = $row[0];
                                $sql = "ALTER SEQUENCE " . $tbprefix . $fdA[$i] . "_hist_hid_seq RESTART WITH " . $row[0];

                                if (!pg_query($pgsqln, $sql)) {
                                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                                    $block1->contentRow("", pg_last_notice($pgsqln));
                                    throw new Exception('', 1);
                                }
                            }

                            for ($x=0; $x<count($row); $x++) $row[$x] = addslashes($row[$x]);
                            unset($row[0]);
                            $sql = "INSERT INTO " . $tbprefix . $fdA[$i] . "_hist (action,id," . $ffA[$i] . ") VALUES ('" . implode("','", $row) . "')";

                            if (!pg_query($pgsqln, $sql)) {
                                $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                                $block1->contentRow("", pg_last_notice($pgsqln));
                                throw new Exception('', 1);
                            }

                            $hid++;
                        }

                        @pg_free_result($result);
                    }
                }
            }
        }

        // approvals
        $sql = "DELETE FROM " . $tbprefix . "approvals";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,doctype,conditions,active,comments,userquery FROM " . $prevtbprefix . "approvals ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "approvals_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "approvals (name,doctype,conditions,active,comments,userquery) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // approvals_producers
        $sql = "DELETE FROM " . $tbprefix . "approvals_producers";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT approval,producer_type,producer,exclude FROM " . $prevtbprefix . "approvals_producers";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "approvals_producers (approval,producer_type,producer,exclude) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // approvals_stages
        $sql = "DELETE FROM " . $tbprefix . "approvals_stages";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT approval,confirmation_level,sortid FROM " . $prevtbprefix . "approvals_stages";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "approvals_stages (approval,confirmation_level,sortid) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // approvals_terms
        $sql = "DELETE FROM " . $tbprefix . "approvals_terms";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,approval,fieldname,ratio,value,compare_fields FROM " . $prevtbprefix . "approvals_terms ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "approvals_terms_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "approvals_terms (approval,fieldname,ratio,value,compare_fields) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // assignments
        $sql = "DELETE FROM " . $tbprefix . "assignments";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,task,owner,assigned_to,comments,assigned FROM " . $prevtbprefix . "assignments ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "assignments_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "assignments (task,owner,assigned_to,comments,assigned) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // attendees
        $sql = "DELETE FROM " . $tbprefix . "attendees";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT meeting,member,attended,authorized,required,schedule_string,string_validto FROM " . $prevtbprefix . "attendees";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "attendees (meeting,member,attended,authorized,required,schedule_string,string_validto) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // bookmarks
        $sql = "DELETE FROM " . $tbprefix . "bookmarks";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT title,url,created,cuser FROM " . $prevtbprefix . "bookmarks";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $id = sprintf("%024d", $row[3]) . sprintf("%024d", $row[2]) . sprintf("%020d", 0);
                $sql = "INSERT INTO " . $tbprefix . "bookmarks (id,title,url,created,cuser) VALUES ('$id','" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // calendar
        $sql = "DELETE FROM " . $tbprefix . "calendars";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT owner,target,target_id FROM " . $prevtbprefix . "calendars";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "calendars (owner,target,target_id) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // calendar_events
        $sql = "DELETE FROM " . $tbprefix . "calendar_events";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,target,target_id,subject,description,time_start,time_end,reminder_time1,reminder_time2,created,cuser,modified,muser FROM " . $prevtbprefix . "calendar_events";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "calendar_events_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "calendar_events (target,target_id,subject,description,time_start,time_end,reminder_time1,reminder_time2,created,cuser,modified,muser) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "','" . $row[9] . "','" . $row[10] . "','" . $row[11] . "','" . $row[12] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // calendar_events - History
        $sql = "DELETE FROM " . $tbprefix . "calendar_events_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,target,target_id,subject,description,time_start,time_end,reminder_time1,reminder_time2,created,cuser,modified,muser FROM " . $prevtbprefix . "calendar_events_hist";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "calendar_events_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "calendar_events_hist (action,id,target,target_id,subject,description,time_start,time_end,reminder_time1,reminder_time2,created,cuser,modified,muser) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "','" . $row[9] . "','" . $row[10] . "','" . $row[11] . "','" . $row[12] . "','" . $row[13] . "','" . $row[14] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // claim_type
        $sql = "DELETE FROM " . $tbprefix . "claim_type";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "claim_type ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "claim_type_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "claim_type (name,locked,created,cuser,modified,muser) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // claim_type_hist
        $sql = "DELETE FROM " . $tbprefix . "claim_type_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "claim_type_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "claim_type_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "claim_type_hist (action,id,name,locked,created,cuser,modified,muser) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // confirmation_levels
        $sql = "DELETE FROM " . $tbprefix . "confirmation_levels";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,comments,max_requirements FROM " . $prevtbprefix . "confirmation_levels ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "confirmation_levels_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "confirmation_levels (name,comments,max_requirements) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // confirmation_levels_detail
        $sql = "DELETE FROM " . $tbprefix . "confirmation_levels_detail";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT confirmation_level,authorizer_type,authorizer,required FROM " . $prevtbprefix . "confirmation_levels_detail";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "confirmation_levels_detail (confirmation_level,authorizer_type,authorizer,required) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // contacts
        $sql = "DELETE FROM " . $tbprefix . "contacts";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,salutation,first_name,last_name,title,organization,department,do_not_call,email_work,email_home,email_other,phone_work,phone_home,phone_other,mobile,fax,address_street,address_city,address_state,address_postalcode,address_country,alt_address_street,alt_address_city,alt_address_state,alt_address_postalcode,alt_address_country,assistant,assistant_phone,birthdate,description,shared,member,created,cuser,modified,muser FROM " . $prevtbprefix . "contacts ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "contacts_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "contacts (salutation,first_name,last_name,title,organization,department,do_not_call,email_work,email_home,email_other,phone_work,phone_home,phone_other,mobile,fax,address_street,address_city,address_state,address_postalcode,address_country,alt_address_street,alt_address_city,alt_address_state,alt_address_postalcode,alt_address_country,assistant,assistant_phone,birthdate,description,shared,member,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // contacts_hist
        $sql = "DELETE FROM " . $tbprefix . "contacts_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,salutation,first_name,last_name,title,organization,department,do_not_call,email_work,email_home,email_other,phone_work,phone_home,phone_other,mobile,fax,address_street,address_city,address_state,address_postalcode,address_country,alt_address_street,alt_address_city,alt_address_state,alt_address_postalcode,alt_address_country,assistant,assistant_phone,birthdate,description,shared,member,created,cuser,modified,muser FROM " . $prevtbprefix . "contacts_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "contacts_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "contacts_hist (action,id,salutation,first_name,last_name,title,organization,department,do_not_call,email_work,email_home,email_other,phone_work,phone_home,phone_other,mobile,fax,address_street,address_city,address_state,address_postalcode,address_country,alt_address_street,alt_address_city,alt_address_state,alt_address_postalcode,alt_address_country,assistant,assistant_phone,birthdate,description,shared,member,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // currency_rates
        $sql = "DELETE FROM " . $tbprefix . "currency_rates";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT currency_from,currency_to,reference_date,exchange_rate,created,cuser,modified,muser FROM " . $prevtbprefix . "currency_rates";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "currency_rates (currency_from,currency_to,reference_date,exchange_rate,created,cuser,modified,muser) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // denyips
        $sql = "DELETE FROM " . $tbprefix . "denyips";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT ipaddr,reason,created,cuser FROM " . $prevtbprefix . "denyips";
        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "denyips (ipaddr,reason,created,cuser) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // departments
        $sql = "DELETE FROM " . $tbprefix . "departments";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,organization,name,location,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "departments ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "departments_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "departments (organization,name,location,locked,created,cuser,modified,muser) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // departments_hist
        $sql = "DELETE FROM " . $tbprefix . "departments_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,organization,name,location,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "departments_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "departments_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "departments_hist (action,id,organization,name,location,locked,created,cuser,modified,muser) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "','" . $row[9] . "','" . $row[10] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // documents_approval
        $sql = "DELETE FROM " . $tbprefix . "documents_approval";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,approval,owner,docid,doctype,docdate,current_stage,status,canceled,closed,comments,userquery,created,cuser,modified,muser FROM " . $prevtbprefix . "documents_approval ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "documents_approval_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "documents_approval (approval,owner,docid,doctype,docdate,current_stage,status,canceled,closed,comments,userquery,created,cuser,modified,muser) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "','" . $row[9] . "','" . $row[10] . "','" . $row[11] . "','" . $row[12] . "','" . $row[13] . "','" . $row[14] . "','" . $row[15] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // documents_approval_authorizers
        $sql = "DELETE FROM " . $tbprefix . "documents_approval_authorizers";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT document_approval,confirmation_level,authorizer,required,stage,status,approve_string,not_approve_string,string_validto,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "documents_approval_authorizers";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "documents_approval_authorizers (document_approval,confirmation_level,authorizer,required,stage,status,approve_string,not_approve_string,string_validto,comments,created,cuser,modified,muser) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "','" . $row[6] . "','" . $row[7] . "','" . $row[8] . "','" . $row[9] . "','" . $row[10] . "','" . $row[11] . "','" . $row[12] . "','" . $row[13] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // documents_approval_terms
        $sql = "DELETE FROM " . $tbprefix . "documents_approval_terms";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,document_approval,fieldname,ratio,value,compare_fields FROM " . $prevtbprefix . "documents_approval_terms ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "documents_approval_terms_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                $sql = "INSERT INTO " . $tbprefix . "documents_approval_terms (document_approval,fieldname,ratio,value,compare_fields) VALUES ('" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // equipment_category
        $sql = "DELETE FROM " . $tbprefix . "equipment_category";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "equipment_category ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "equipment_category_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "equipment_category (name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // equipment_category_hist
        $sql = "DELETE FROM " . $tbprefix . "equipment_category_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "equipment_category_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "equipment_category_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "equipment_category_hist (action,id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // equipment_type
        $sql = "DELETE FROM " . $tbprefix . "equipment_type";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "equipment_type ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "equipment_type_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "equipment_type (name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // equipment_type_hist
        $sql = "DELETE FROM " . $tbprefix . "equipment_type_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "equipment_type_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "equipment_type_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "equipment_type_hist (action,id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // equipments
        $sql = "DELETE FROM " . $tbprefix . "equipments";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'equipments', $qrysql);
        $sql = "SELECT id,equipment_id,equipment_category,equipment_type,name,owner,location,status,manufacturer,model,serial_number,purchase_date,warranty,cost_price,cost_currency,sell_price,sell_currency,photo,locked,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "equipments ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "equipments_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "equipments (equipment_id,equipment_category,equipment_type,name,owner,location,status,manufacturer,model,serial_number,purchase_date,warranty,cost_price,cost_currency,sell_price,sell_currency,photo,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // equipments_hist
        $sql = "DELETE FROM " . $tbprefix . "equipments_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'equipments', $qrysql);
        $sql = "SELECT hid,action,id,equipment_id,equipment_category,equipment_type,name,owner,location,status,manufacturer,model,serial_number,purchase_date,warranty,cost_price,cost_currency,sell_price,sell_currency,photo,locked,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "equipments_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "equipments_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "equipments_hist (action,id,equipment_id,equipment_category,equipment_type,name,owner,location,status,manufacturer,model,serial_number,purchase_date,warranty,cost_price,cost_currency,sell_price,sell_currency,photo,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // expense_type
        $sql = "DELETE FROM " . $tbprefix . "expense_type";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,claimtype,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "expense_type ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "expense_type_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "expense_type (name,claimtype,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // expense_type_hist
        $sql = "DELETE FROM " . $tbprefix . "expense_type_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,claimtype,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "expense_type_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "expense_type_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "expense_type_hist (action,id,name,claimtype,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // expenses
        $sql = "DELETE FROM " . $tbprefix . "expenses";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'expenses', $qrysql);
        $sql = "SELECT id,project,status,status_approval,printed,owner,description,docdate,billable,claimtype,expensetype,amount,currency,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "expenses ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "expenses_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "expenses (project,status,status_approval,printed,owner,description,docdate,billable,claimtype,expensetype,amount,currency,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // expenses_hist
        $sql = "DELETE FROM " . $tbprefix . "expenses_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'expenses', $qrysql);
        $sql = "SELECT hid,action,id,project,status,status_approval,printed,owner,description,docdate,billable,claimtype,expensetype,amount,currency,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "expenses_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "expenses_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "expenses_hist (action,id,project,status,status_approval,printed,owner,description,docdate,billable,claimtype,expensetype,amount,currency,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // extensions
        $sql = "DELETE FROM " . $tbprefix . "extensions";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,description,version,active,created,cuser,modified,muser FROM " . $prevtbprefix . "extensions ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "extensions_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "extensions (name,description,version,active,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // extensions_hist
        $sql = "DELETE FROM " . $tbprefix . "extensions_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,description,version,active,created,cuser,modified,muser FROM " . $prevtbprefix . "extensions_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "extensions_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "extensions_hist (action,id,name,description,version,active,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // field_filters
        $sql = "DELETE FROM " . $tbprefix . "field_filters";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT member,tablename,dbfieldname,pattern_begin,pattern_mid,pattern_end FROM " . $prevtbprefix . "field_filters";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "field_filters (member,tablename,dbfieldname,pattern_begin,pattern_mid,pattern_end) VALUES ('" . $row[0] . "','" . $row[1] . "','" . $row[2] . "','" . $row[3] . "','" . $row[4] . "','" . $row[5] . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // files
        $sql = "DELETE FROM " . $tbprefix . "files";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'files', $qrysql);
        $sql = "SELECT id,owner,doc_type,doc_type_ex,doc_id,name,size,extension,comments,published,status,vc_status,vc_version,vc_parent,dir,parent,fullname,project,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "files ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "files_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "files (owner,doc_type,doc_type_ex,doc_id,name,size,extension,comments,published,status,vc_status,vc_version,vc_parent,dir,parent,fullname,project,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // files_hist
        $sql = "DELETE FROM " . $tbprefix . "files_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'files', $qrysql);
        $sql = "SELECT hid,action,id,owner,doc_type,doc_type_ex,doc_id,name,size,extension,comments,published,status,vc_status,vc_version,vc_parent,dir,parent,fullname,project,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "files_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "files_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "files_hist (action,id,owner,doc_type,doc_type_ex,doc_id,name,size,extension,comments,published,status,vc_status,vc_version,vc_parent,dir,parent,fullname,project,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // flextime_adjustment
        $sql = "DELETE FROM " . $tbprefix . "flextime_adjustment";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,owner,refdate,hours,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "flextime_adjustment ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "flextime_adjustment_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "flextime_adjustment (owner,refdate,hours,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // flextime_adjustment_hist
        $sql = "DELETE FROM " . $tbprefix . "flextime_adjustment_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,owner,refdate,hours,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "flextime_adjustment_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "flextime_adjustment_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "flextime_adjustment_hist (action,id,owner,refdate,hours,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // history
        $sql = "DELETE FROM " . $tbprefix . "history";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,title,uri,created,cuser FROM " . $prevtbprefix . "history ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "history_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "history (title,uri,created,cuser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // hour_code
        $sql = "DELETE FROM " . $tbprefix . "hour_code";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,hourtype,sysdef,locked,service,created,cuser,modified,muser FROM " . $prevtbprefix . "hour_code ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "hour_code (id,hourtype,sysdef,locked,service,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // hour_type
        $sql = "DELETE FROM " . $tbprefix . "hour_type";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,sysdef,locked,service,created,cuser,modified,muser FROM " . $prevtbprefix . "hour_type ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "hour_type (id,sysdef,locked,service,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // invoices
        $sql = "DELETE FROM " . $tbprefix . "invoices";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'invoices', $qrysql);
        $sql = "SELECT id,invoice_no,organization,owner,status,status_approval,printed,doc_date,due_date,payment_terms,payment_method,currency,before_discount,discount_percent,discount_sum,freight,tax_sum,amount,paid,notes,bill_to,ship_to,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,tax_group,tax_group2,tax_group3,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "invoices ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "invoices_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "invoices (invoice_no,organization,owner,status,status_approval,printed,doc_date,due_date,payment_terms,payment_method,currency,before_discount,discount_percent,discount_sum,freight,tax_sum,amount,paid,notes,bill_to,ship_to,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,tax_group,tax_group2,tax_group3,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // invoices_hist
        $sql = "DELETE FROM " . $tbprefix . "invoices_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'invoices', $qrysql);
        $sql = "SELECT hid,action,id,invoice_no,organization,owner,status,status_approval,printed,doc_date,due_date,payment_terms,payment_method,currency,before_discount,discount_percent,discount_sum,freight,tax_sum,amount,paid,notes,bill_to,ship_to,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,tax_group,tax_group2,tax_group3,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "invoices_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "invoices_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "invoices_hist (action,id,invoice_no,organization,owner,status,status_approval,printed,doc_date,due_date,payment_terms,payment_method,currency,before_discount,discount_percent,discount_sum,freight,tax_sum,amount,paid,notes,bill_to,ship_to,recurring,recurring_start,recurring_finish,recurring_next,recurring_type,recurring_frequency,recurring_enabled,tax_group,tax_group2,tax_group3,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // invoice_rows
        $sql = "DELETE FROM " . $tbprefix . "invoice_rows";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,invoice,description,project,quantity,price,unit,discount_percent,discount_sum,tax_code,tax_sum,tax_code2,tax_sum2,tax_code3,tax_sum3,line_total,created,cuser,modified,muser FROM " . $prevtbprefix . "invoice_rows ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "invoice_rows_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "invoice_rows (invoice,description,project,quantity,price,unit,discount_percent,discount_sum,tax_code,tax_sum,tax_code2,tax_sum2,tax_code3,tax_sum3,line_total,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // invoice_rows_hist
        $sql = "DELETE FROM " . $tbprefix . "invoice_rows_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,invoice,description,project,quantity,price,unit,discount_percent,discount_sum,tax_code,tax_sum,tax_code2,tax_sum2,tax_code3,tax_sum3,line_total,created,cuser,modified,muser FROM " . $prevtbprefix . "invoice_rows_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "invoice_rows_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "invoice_rows_hist (action,id,invoice,description,project,quantity,price,unit,discount_percent,discount_sum,tax_code,tax_sum,tax_code2,tax_sum2,tax_code3,tax_sum3,line_total,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // leaves
        $sql = "DELETE FROM " . $tbprefix . "leaves";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,owner,year,type,hours,due_date,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "leaves ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "leaves_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "leaves (owner,year,type,hours,due_date,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // leaves_hist
        $sql = "DELETE FROM " . $tbprefix . "leaves_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,owner,year,type,hours,due_date,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "leaves_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "leaves_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "leaves_hist (action,id,owner,year,type,hours,due_date,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // leave_requests
        $sql = "DELETE FROM " . $tbprefix . "leave_requests";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'leave_requests', $qrysql);
        $sql = "SELECT id,owner,type,date_start,date_end,time_start,time_end,hours_total,hours_start,hours_end,status,comments,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "leave_requests ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "leave_requests_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "leave_requests (owner,type,date_start,date_end,time_start,time_end,hours_total,hours_start,hours_end,status,comments,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // leave_requests_hist
        $sql = "DELETE FROM " . $tbprefix . "leave_requests_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'leave_requests', $qrysql);
        $sql = "SELECT hid,action,id,owner,type,date_start,date_end,time_start,time_end,hours_total,hours_start,hours_end,status,comments,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "leave_requests_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "leave_requests_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "leave_requests_hist (action,id,owner,type,date_start,date_end,time_start,time_end,hours_total,hours_start,hours_end,status,comments,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // locations
        $sql = "DELETE FROM " . $tbprefix . "locations";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,timezone,language,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "locations ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "locations_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "locations (name,timezone,language,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // locations_hist
        $sql = "DELETE FROM " . $tbprefix . "locations_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,timezone,language,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "locations_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "locations_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "locations_hist (action,id,name,timezone,language,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // loginips
        $sql = "DELETE FROM " . $tbprefix . "loginips";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT login,ipaddr,loginfail,ipclassb,compt,last_visite,approve_string,not_approve_string FROM " . $prevtbprefix . "loginips";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "loginips (login,ipaddr,loginfail,ipclassb,compt,last_visite,approve_string,not_approve_string) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // materials
        $sql = "DELETE FROM " . $tbprefix . "materials";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'materials', $qrysql);
        $sql = "SELECT id,name,unit,currency,costrate,locked,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "materials ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "materials_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "materials (name,unit,currency,costrate,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // materials_hist
        $sql = "DELETE FROM " . $tbprefix . "materials_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'materials', $qrysql);
        $sql = "SELECT hid,action,id,name,unit,currency,costrate,locked,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "materials_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "materials_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "materials_hist (action,id,name,unit,currency,costrate,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // material_issue
        $sql = "DELETE FROM " . $tbprefix . "material_issue";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,task,owner,material,date,quantity,created,cuser,modified,muser FROM " . $prevtbprefix . "material_issue ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "material_issue_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "material_issue (project,task,owner,material,date,quantity,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // material_issue_hist
        $sql = "DELETE FROM " . $tbprefix . "material_issue_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,project,task,owner,material,date,quantity,created,cuser,modified,muser FROM " . $prevtbprefix . "material_issue_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "material_issue_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "material_issue_hist (action,id,project,task,owner,material,date,quantity,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // meetings
        $sql = "DELETE FROM " . $tbprefix . "meetings";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'meetings', $qrysql);
        $sql = "SELECT id,project,priority,status,owner,name,agenda,location,minutes,duration,start_time,reminder,reminder_time1,reminder_time2,confirmed,allreplied,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "meetings ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "meetings_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "meetings (project,priority,status,owner,name,agenda,location,minutes,duration,start_time,reminder,reminder_time1,reminder_time2,confirmed,allreplied,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // meetings_hist
        $sql = "DELETE FROM " . $tbprefix . "meetings_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'meetings', $qrysql);
        $sql = "SELECT hid,action,id,project,priority,status,owner,name,agenda,location,minutes,duration,start_time,reminder,reminder_time1,reminder_time2,confirmed,allreplied,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "meetings_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "meetings_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "meetings_hist (action,id,project,priority,status,owner,name,agenda,location,minutes,duration,start_time,reminder,reminder_time1,reminder_time2,confirmed,allreplied,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // meetings_available
        $sql = "DELETE FROM " . $tbprefix . "meetings_available";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT meeting,member,schedule FROM " . $prevtbprefix . "meetings_available";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "meetings_available (meeting,member,schedule) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // meetings_schedule
        $sql = "DELETE FROM " . $tbprefix . "meetings_schedule";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,meeting,start_time,canceled FROM " . $prevtbprefix . "meetings_schedule ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "meetings_schedule_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "meetings_schedule (meeting,start_time,canceled) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // meetings_time
        $sql = "DELETE FROM " . $tbprefix . "meetings_time";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,meeting,owner,date,hours,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "meetings_time ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "meetings_time_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "meetings_time (project,meeting,owner,date,hours,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // members
        $sql = "DELETE FROM " . $tbprefix . "members";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'members', $qrysql);
        $sql = "SELECT id,organization,department,login,password,name,title,email_work,email_home,phone_work,phone_home,mobile,fax,comments,profil,language,logout_time,start_page,location,timezone,date_format,time_format,manager,deputy,theme,palette,gender,extension_picture,active,workcalendar,enable_flextime,begin_flextime,initial_flexibalance,invalid_logon,export_format,ldap_cn,enable_timeclock,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "members ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "members_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "members (organization,department,login,password,name,title,email_work,email_home,phone_work,phone_home,mobile,fax,comments,profil,language,logout_time,start_page,location,timezone,date_format,time_format,manager,deputy,theme,palette,gender,extension_picture,active,workcalendar,enable_flextime,begin_flextime,initial_flexibalance,invalid_logon,export_format,ldap_cn,enable_timeclock,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // members_hist
        $sql = "DELETE FROM " . $tbprefix . "members_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'members', $qrysql);
        $sql = "SELECT hid,action,id,organization,department,login,password,name,title,email_work,email_home,phone_work,phone_home,mobile,fax,comments,profil,language,logout_time,start_page,location,timezone,date_format,time_format,manager,deputy,theme,palette,gender,extension_picture,active,workcalendar,enable_flextime,begin_flextime,initial_flexibalance,invalid_logon,export_format,ldap_cn,enable_timeclock,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "members_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "members_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "members_hist (action,id,organization,department,login,password,name,title,email_work,email_home,phone_work,phone_home,mobile,fax,comments,profil,language,logout_time,start_page,location,timezone,date_format,time_format,manager,deputy,theme,palette,gender,extension_picture,active,workcalendar,enable_flextime,begin_flextime,initial_flexibalance,invalid_logon,export_format,ldap_cn,enable_timeclock,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // messages
        $sql = "DELETE FROM " . $tbprefix . "messages";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,from_id,to_id,message_type,message,extension_picture,subject,seen,archived,created FROM " . $prevtbprefix . "messages ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "messages_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "messages (from_id,to_id,message_type,message,extension_picture,subject,seen,archived,created) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // notes
        $sql = "DELETE FROM " . $tbprefix . "notes";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'notes', $qrysql);
        $sql = "SELECT id,project,owner,topic,subject,description,date,published,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "notes ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "notes_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "notes (project,owner,topic,subject,description,date,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // notes_hist
        $sql = "DELETE FROM " . $tbprefix . "notes_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'notes', $qrysql);
        $sql = "SELECT hid,action,id,project,owner,topic,subject,description,date,published,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "notes_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "notes_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "notes_hist (action,id,project,owner,topic,subject,description,date,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // notifications
        $sql = "DELETE FROM " . $tbprefix . "notifications";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT member,notiid,block,block_string,string_validto FROM " . $prevtbprefix . "notifications";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "notifications (member,notiid,block,block_string,string_validto) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // organizations
        $sql = "DELETE FROM " . $tbprefix . "organizations";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'organizations', $qrysql);
        $sql = "SELECT id,name,address,zip_code,city,country,phone,fax,url,email,comments,sscomments,extension_logo,owner,currency,workcalendar,payment_method,payment_terms,locked,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "organizations ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "organizations_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "organizations (name,address,zip_code,city,country,phone,fax,url,email,comments,sscomments,extension_logo,owner,currency,workcalendar,payment_method,payment_terms,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // organizations_hist
        $sql = "DELETE FROM " . $tbprefix . "organizations_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'organizations', $qrysql);
        $sql = "SELECT hid,action,id,name,address,zip_code,city,country,phone,fax,url,email,comments,sscomments,extension_logo,owner,currency,workcalendar,payment_method,payment_terms,locked,created,cuser,modified,muser" . $cfs . " FROM " . $prevtbprefix . "organizations_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "organizations_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "organizations_hist (action,id,name,address,zip_code,city,country,phone,fax,url,email,comments,sscomments,extension_logo,owner,currency,workcalendar,payment_method,payment_terms,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // organizations_extension
        $sql = "DELETE FROM " . $tbprefix . "organizations_extension";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT organization,extension,perm_read,perm_write,perm_edit,perm_admin,created,cuser FROM " . $prevtbprefix . "organizations_extension";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "organizations_extension (organization,extension,perm_read,perm_write,perm_edit,perm_admin,created,cuser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // organizations_form
        $sql = "DELETE FROM " . $tbprefix . "organizations_form";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,organization,tablename,addenabled,modenabled,created,cuser,modified,muser FROM " . $prevtbprefix . "organizations_form ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "organizations_form_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "organizations_form (organization,tablename,addenabled,modenabled,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // payment_methods
        $sql = "DELETE FROM " . $tbprefix . "payment_methods";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "payment_methods ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "payment_methods_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "payment_methods (name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // payment_methods_hist
        $sql = "DELETE FROM " . $tbprefix . "payment_methods_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "payment_methods_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "payment_methods_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "payment_methods_hist (action,id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // payment_terms
        $sql = "DELETE FROM " . $tbprefix . "payment_terms";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,start_from,extra_months,extra_days,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "payment_terms ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "payment_terms_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "payment_terms (name,start_from,extra_months,extra_days,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // payment_terms_hist
        $sql = "DELETE FROM " . $tbprefix . "payment_terms_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,start_from,extra_months,extra_days,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "payment_terms_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "payment_terms_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "payment_terms_hist (action,id,name,start_from,extra_months,extra_days,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // payments
        $sql = "DELETE FROM " . $tbprefix . "payments";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'payments', $qrysql);
        $sql = "SELECT id,invoice,currency,amount,doc_date,payment_method,notes,printed,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "payments ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "payments_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "payments (invoice,currency,amount,doc_date,payment_method,notes,printed,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // payments_hist
        $sql = "DELETE FROM " . $tbprefix . "payments_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'payments', $qrysql);
        $sql = "SELECT hid,action,id,invoice,currency,amount,doc_date,payment_method,notes,printed,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "payments_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "payments_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "payments_hist (action,id,invoice,currency,amount,doc_date,payment_method,notes,printed,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // periods
        $sql = "DELETE FROM " . $tbprefix . "periods";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,fyear,serial_no,date_start,date_end,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "periods ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "periods_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "periods (fyear,serial_no,date_start,date_end,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // phase_sets
        $sql = "DELETE FROM " . $tbprefix . "phase_sets";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,phase_num,created,cuser,modified,muser FROM " . $prevtbprefix . "phase_sets ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "phase_sets_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "phase_sets (name,phase_num,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // phases
        $sql = "DELETE FROM " . $tbprefix . "phases";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'phases', $qrysql);
        $sql = "SELECT id,project,phase_set,order_num,status,date_start,date_end,comments,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "phases ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "phases_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "phases (project,phase_set,order_num,status,date_start,date_end,comments,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // phases_hist
        $sql = "DELETE FROM " . $tbprefix . "phases_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'phases', $qrysql);
        $sql = "SELECT hid,action,id,project,phase_set,order_num,status,date_start,date_end,comments,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "phases_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "phases_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "phases_hist (action,id,project,phase_set,order_num,status,date_start,date_end,comments,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // posts
        $sql = "DELETE FROM " . $tbprefix . "posts";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'posts', $qrysql);
        $sql = "SELECT id,topic,member,message,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "posts ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "posts_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "posts (topic,member,message,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // posts_hist
        $sql = "DELETE FROM " . $tbprefix . "posts_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'posts', $qrysql);
        $sql = "SELECT hid,action,id,topic,member,message,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "posts_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "posts_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "posts_hist (action,id,topic,member,message,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // print_layouts
        $sql = "DELETE FROM " . $tbprefix . "print_layouts";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,doc_type,doc_type_ex,description,path,start_page,defaults,created,cuser,modified,muser FROM " . $prevtbprefix . "print_layouts ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "print_layouts_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "print_layouts (doc_type,doc_type_ex,description,path,start_page,defaults,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // projects
        $sql = "DELETE FROM " . $tbprefix . "projects";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'projects', $qrysql);
        $sql = "SELECT id,organization,owner,priority,status,status_next,status_approval,open_date,complete_date,name,description,published,phase_set,type,upload_dir,currency,permission_set,cost_method,workcalendar,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "projects ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "projects_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "projects (organization,owner,priority,status,status_next,status_approval,open_date,complete_date,name,description,published,phase_set,type,upload_dir,currency,permission_set,cost_method,workcalendar,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // projects_hist
        $sql = "DELETE FROM " . $tbprefix . "projects_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'projects', $qrysql);
        $sql = "SELECT hid,action,id,organization,owner,priority,status,status_next,status_approval,open_date,complete_date,name,description,published,phase_set,type,upload_dir,currency,permission_set,cost_method,workcalendar,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "projects_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "projects_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "projects_hist (action,id,organization,owner,priority,status,status_next,status_approval,open_date,complete_date,name,description,published,phase_set,type,upload_dir,currency,permission_set,cost_method,workcalendar,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // projects_budget
        $sql = "DELETE FROM " . $tbprefix . "projects_budget";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,costtype,currency,amount,created,cuser,modified,muser FROM " . $prevtbprefix . "projects_budget ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "projects_budget_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "projects_budget (project,costtype,currency,amount,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // projects_budget_hist
        $sql = "DELETE FROM " . $tbprefix . "projects_budget_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,project,costtype,currency,amount,created,cuser,modified,muser FROM " . $prevtbprefix . "projects_budget_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "projects_budget_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "projects_budget_hist (action,id,project,costtype,currency,amount,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // projects_form
        $sql = "DELETE FROM " . $tbprefix . "projects_form";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT project,tablename,publishenabled,created,cuser,modified,muser FROM " . $prevtbprefix . "projects_form";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "projects_form (project,tablename,publishenabled,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // project_permissions
        $sql = "DELETE FROM " . $tbprefix . "project_permissions";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,profil,view_project,create_customer_service_site,copy_project,import_project,export_project,modify_project_properties,delete_project,view_project_budget,view_project_cost,view_phase,modify_phase_properties,view_task,create_task,copy_task,modify_task_status,log_task_hours,material_issue,arrange_task_resource,modify_task_properties,edit_task_relationships,delete_task,configure_task_security,publish_task,view_milestone,create_milestone,modify_milestone_properties,delete_milestone,configure_milestone_security,publish_milestone,view_meeting,create_meeting,copy_meeting,modify_meeting_status,log_meeting_hours,arrange_meeting_resource,modify_meeting_properties,delete_meeting,configure_meeting_security,view_support_request,add_support_response,assign_support_request,close_support_request,delete_support_request,view_discussion,add_discussion_topic,post_reply,close_discussion_topic,publish_discussion_topic,delete_discussion_topic,view_team,send_message,add_team_member,publish_team_member,modify_team_member_properties,delete_team_member,view_file,add_file,update_file,publish_file,delete_file,view_note,add_note,modify_note_properties,delete_note,publish_note,view_calendar,add_calendar_event,modify_calendar_event,delete_calendar_event,view_role,configure_role_security,view_expense,create_expense,modify_expense_properties,delete_expense,created,cuser,modified,muser FROM " . $prevtbprefix . "project_permissions ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "project_permissions_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "project_permissions (project,profil,view_project,create_customer_service_site,copy_project,import_project,export_project,modify_project_properties,delete_project,view_project_budget,view_project_cost,view_phase,modify_phase_properties,view_task,create_task,copy_task,modify_task_status,log_task_hours,material_issue,arrange_task_resource,modify_task_properties,edit_task_relationships,delete_task,configure_task_security,publish_task,view_milestone,create_milestone,modify_milestone_properties,delete_milestone,configure_milestone_security,publish_milestone,view_meeting,create_meeting,copy_meeting,modify_meeting_status,log_meeting_hours,arrange_meeting_resource,modify_meeting_properties,delete_meeting,configure_meeting_security,view_support_request,add_support_response,assign_support_request,close_support_request,delete_support_request,view_discussion,add_discussion_topic,post_reply,close_discussion_topic,publish_discussion_topic,delete_discussion_topic,view_team,send_message,add_team_member,publish_team_member,modify_team_member_properties,delete_team_member,view_file,add_file,update_file,publish_file,delete_file,view_note,add_note,modify_note_properties,delete_note,publish_note,view_calendar,add_calendar_event,modify_calendar_event,delete_calendar_event,view_role,configure_role_security,view_expense,create_expense,modify_expense_properties,delete_expense,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $i++;
            }

            @pg_free_result($result);
        }

        // project_permissions_hist
        $sql = "DELETE FROM " . $tbprefix . "project_permissions_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,project,profil,view_project,create_customer_service_site,copy_project,import_project,export_project,modify_project_properties,delete_project,view_project_budget,view_project_cost,view_phase,modify_phase_properties,view_task,create_task,copy_task,modify_task_status,log_task_hours,material_issue,arrange_task_resource,modify_task_properties,edit_task_relationships,delete_task,configure_task_security,publish_task,view_milestone,create_milestone,modify_milestone_properties,delete_milestone,configure_milestone_security,publish_milestone,view_meeting,create_meeting,copy_meeting,modify_meeting_status,log_meeting_hours,arrange_meeting_resource,modify_meeting_properties,delete_meeting,configure_meeting_security,view_support_request,add_support_response,assign_support_request,close_support_request,delete_support_request,view_discussion,add_discussion_topic,post_reply,close_discussion_topic,publish_discussion_topic,delete_discussion_topic,view_team,send_message,add_team_member,publish_team_member,modify_team_member_properties,delete_team_member,view_file,add_file,update_file,publish_file,delete_file,view_note,add_note,modify_note_properties,delete_note,publish_note,view_calendar,add_calendar_event,modify_calendar_event,delete_calendar_event,view_role,configure_role_security,view_expense,create_expense,modify_expense_properties,delete_expense,created,cuser,modified,muser FROM " . $prevtbprefix . "project_permissions_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "project_permissions_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "project_permissions_hist (action,id,project,profil,view_project,create_customer_service_site,copy_project,import_project,export_project,modify_project_properties,delete_project,view_project_budget,view_project_cost,view_phase,modify_phase_properties,view_task,create_task,copy_task,modify_task_status,log_task_hours,material_issue,arrange_task_resource,modify_task_properties,edit_task_relationships,delete_task,configure_task_security,publish_task,view_milestone,create_milestone,modify_milestone_properties,delete_milestone,configure_milestone_security,publish_milestone,view_meeting,create_meeting,copy_meeting,modify_meeting_status,log_meeting_hours,arrange_meeting_resource,modify_meeting_properties,delete_meeting,configure_meeting_security,view_support_request,add_support_response,assign_support_request,close_support_request,delete_support_request,view_discussion,add_discussion_topic,post_reply,close_discussion_topic,publish_discussion_topic,delete_discussion_topic,view_team,send_message,add_team_member,publish_team_member,modify_team_member_properties,delete_team_member,view_file,add_file,update_file,publish_file,delete_file,view_note,add_note,modify_note_properties,delete_note,publish_note,view_calendar,add_calendar_event,modify_calendar_event,delete_calendar_event,view_role,configure_role_security,view_expense,create_expense,modify_expense_properties,delete_expense,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // report_category
        $sql = "DELETE FROM " . $tbprefix . "report_category";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,created,cuser,modified,muser FROM " . $prevtbprefix . "report_category ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "report_category_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "report_category (name,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // reports
        $sql = "DELETE FROM " . $tbprefix . "reports";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,category,name,url,tablename,sysdef,userquery,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "reports ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "reports_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "reports (category,name,url,tablename,sysdef,userquery,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // reports_rel_tables
        $sql = "DELETE FROM " . $tbprefix . "reports_rel_tables";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,report,tablename,table_left FROM " . $prevtbprefix . "reports_rel_tables ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "reports_rel_tables_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "reports_rel_tables (report,tablename,table_left) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // reports_rel_fields
        $sql = "DELETE FROM " . $tbprefix . "reports_rel_fields";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,report,rel_table,field_left,field_right FROM " . $prevtbprefix . "reports_rel_fields ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "reports_rel_fields_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "reports_rel_fields (report,rel_table,field_left,field_right) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // reports_criteria
        $sql = "DELETE FROM " . $tbprefix . "reports_criteria";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,report,rel_table,fieldname,fieldtype,ratio,value,table_right,compare_fields,parameter FROM " . $prevtbprefix . "reports_criteria ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "reports_criteria_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "reports_criteria (report,rel_table,fieldname,fieldtype,ratio,value,table_right,compare_fields,parameter) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // reports_output
        $sql = "DELETE FROM " . $tbprefix . "reports_output";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,report,rel_table,fieldname,fieldtype,title,disp_order,sort_order,sort_type,field_group,field_function FROM " . $prevtbprefix . "reports_output ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "reports_output_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "reports_output (report,rel_table,fieldname,fieldtype,title,disp_order,sort_order,sort_type,field_group,field_function) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // services
        $sql = "DELETE FROM " . $tbprefix . "services";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'services', $qrysql);
        $sql = "SELECT id,name,hourly_rate,hourly_rate_i,currency,locked,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "services ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "services_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "services (name,hourly_rate,hourly_rate_i,currency,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // services_hist
        $sql = "DELETE FROM " . $tbprefix . "services_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'services', $qrysql);
        $sql = "SELECT hid,action,id,name,hourly_rate,hourly_rate_i,currency,locked,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "services_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "services_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "services_hist (action,id,name,hourly_rate,hourly_rate_i,currency,locked,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // sorting
        $sql = "DELETE FROM " . $tbprefix . "sorting";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        // support_posts
        $sql = "DELETE FROM " . $tbprefix . "support_posts";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'support_posts', $qrysql);
        $sql = "SELECT id,request_id,message,date,owner,project,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "support_posts ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "support_posts_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "support_posts (request_id,message,date,owner,project,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // support_posts_hist
        $sql = "DELETE FROM " . $tbprefix . "support_posts_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'support_posts', $qrysql);
        $sql = "SELECT hid,action,id,request_id,message,date,owner,project,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "support_posts_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "support_posts_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "support_posts_hist (action,id,request_id,message,date,owner,project,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // support_requests
        $sql = "DELETE FROM " . $tbprefix . "support_requests";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'support_requests', $qrysql);
        $sql = "SELECT id,owner,status,reproducibility,severity,priority,subject,message,date_open,date_close,project,assigned_to,resolution,auto_close,published,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "support_requests ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "support_requests_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "support_requests (owner,status,reproducibility,severity,priority,subject,message,date_open,date_close,project,assigned_to,resolution,auto_close,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // support_requests_hist
        $sql = "DELETE FROM " . $tbprefix . "support_requests_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'support_requests', $qrysql);
        $sql = "SELECT hid,action,id,owner,status,reproducibility,severity,priority,subject,message,date_open,date_close,project,assigned_to,resolution,auto_close,published,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "support_requests_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "support_requests_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "support_requests_hist (action,id,owner,status,reproducibility,severity,priority,subject,message,date_open,date_close,project,assigned_to,resolution,auto_close,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // tasks
        $sql = "DELETE FROM " . $tbprefix . "tasks";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'tasks', $qrysql);
        $sql = "SELECT id,project,priority,status,owner,assigned_to,name,description,start_date,due_date,estimated_time,actual_time,comments,completion,assigned,published,parent_phase,complete_date,service,fixed_duration,milestone,type,task_group,level,parent_task,subtask,support_request,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "tasks ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tasks_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "tasks (project,priority,status,owner,assigned_to,name,description,start_date,due_date,estimated_time,actual_time,comments,completion,assigned,published,parent_phase,complete_date,service,fixed_duration,milestone,type,task_group,level,parent_task,subtask,support_request,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // tasks_hist
        $sql = "DELETE FROM " . $tbprefix . "tasks_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'tasks', $qrysql);
        $sql = "SELECT hid,action,id,project,priority,status,owner,assigned_to,name,description,start_date,due_date,estimated_time,actual_time,comments,completion,assigned,published,parent_phase,complete_date,service,fixed_duration,milestone,type,task_group,level,parent_task,subtask,support_request,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "tasks_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tasks_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "tasks_hist (action,id,project,priority,status,owner,assigned_to,name,description,start_date,due_date,estimated_time,actual_time,comments,completion,assigned,published,parent_phase,complete_date,service,fixed_duration,milestone,type,task_group,level,parent_task,subtask,support_request,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // tasks_predecessor
        $sql = "DELETE FROM " . $tbprefix . "tasks_predecessor";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,task,predecessor,type,lag,created,cuser,modified,muser FROM " . $prevtbprefix . "tasks_predecessor ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tasks_predecessor_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "tasks_predecessor (task,predecessor,type,lag,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // tasks_predecessor_hist
        $sql = "DELETE FROM " . $tbprefix . "tasks_predecessor_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,task,predecessor,type,lag,created,cuser,modified,muser FROM " . $prevtbprefix . "tasks_predecessor_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tasks_predecessor_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "tasks_predecessor_hist (action,id,task,predecessor,type,lag,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // tasks_resource
        $sql = "DELETE FROM " . $tbprefix . "tasks_resource";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,task,resource,type,quantity,created,cuser,modified,muser FROM " . $prevtbprefix . "tasks_resource ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tasks_resource_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "tasks_resource (project,task,resource,type,quantity,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // tasks_resource_hist
        $sql = "DELETE FROM " . $tbprefix . "tasks_resource_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,project,task,resource,type,quantity,created,cuser,modified,muser FROM " . $prevtbprefix . "tasks_resource_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tasks_resource_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "tasks_resource_hist (action,id,project,task,resource,type,quantity,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // tasks_time
        $sql = "DELETE FROM " . $tbprefix . "tasks_time";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,task,owner,date,hours,comments,billing,service,created,cuser,modified,muser FROM " . $prevtbprefix . "tasks_time ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tasks_time_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "tasks_time (project,task,owner,date,hours,comments,billing,service,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // tax_codes
        $sql = "DELETE FROM " . $tbprefix . "tax_codes";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,tax_group,name,rate,created,cuser,modified,muser FROM " . $prevtbprefix . "tax_codes ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tax_codes_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "tax_codes (tax_group,name,rate,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // tax_codes_hist
        $sql = "DELETE FROM " . $tbprefix . "tax_codes_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,tax_group,name,rate,created,cuser,modified,muser FROM " . $prevtbprefix . "tax_codes_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tax_codes_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "tax_codes_hist (action,id,tax_group,name,rate,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // tax_groups
        $sql = "DELETE FROM " . $tbprefix . "tax_groups";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "tax_groups ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tax_groups_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "tax_groups (name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // tax_groups_hist
        $sql = "DELETE FROM " . $tbprefix . "tax_groups_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,name,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "tax_groups_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "tax_groups_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "tax_groups_hist (action,id,name,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // teams
        $sql = "DELETE FROM " . $tbprefix . "teams";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,member,published,authorized,profil,service,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "teams ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "teams_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "teams (project,member,published,authorized,profil,service,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // teams_hist
        $sql = "DELETE FROM " . $tbprefix . "teams_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,project,member,published,authorized,profil,service,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "teams_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "teams_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "teams_hist (action,id,project,member,published,authorized,profil,service,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // timeclock
        $sql = "DELETE FROM " . $tbprefix . "timeclock";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,owner,finish,project,task,meeting,work,clockin,clockout,hours,comments,billing,service FROM " . $prevtbprefix . "timeclock ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "timeclock_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "timeclock (owner,finish,project,task,meeting,work,clockin,clockout,hours,comments,billing,service) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // timesheet
        $sql = "DELETE FROM " . $tbprefix . "timesheet";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,owner,period,standard_hours,actual_hours,billable_hours,overtime_hours,status,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "timesheet ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "timesheet_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "timesheet (owner,period,standard_hours,actual_hours,billable_hours,overtime_hours,status,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // timesheet_hist
        $sql = "DELETE FROM " . $tbprefix . "timesheet_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT hid,action,id,owner,period,standard_hours,actual_hours,billable_hours,overtime_hours,status,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "timesheet_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "timesheet_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "timesheet_hist (action,id,owner,period,standard_hours,actual_hours,billable_hours,overtime_hours,status,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // topics
        $sql = "DELETE FROM " . $tbprefix . "topics";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'topics', $qrysql);
        $sql = "SELECT id,project,owner,subject,status,last_post,posts,published,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "topics ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "topics_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "topics (project,owner,subject,status,last_post,posts,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // topics_hist
        $sql = "DELETE FROM " . $tbprefix . "topics_hist";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $qrysql = '';
        $cfs = setupCustomFieldsSQLpg($pgsqlp, $prevtbprefix, 'topics', $qrysql);
        $sql = "SELECT hid,action,id,project,owner,subject,status,last_post,posts,published,created,cuser,modified,muser" . $qrysql . " FROM " . $prevtbprefix . "topics_hist ORDER BY hid";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $hid = -1;

            while (($row = pg_fetch_row($result))) {
                if ($hid != $row[0]) {
                    $hid = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "topics_hist_hid_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "topics_hist (action,id,project,owner,subject,status,last_post,posts,published,created,cuser,modified,muser" . $cfs . ") VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $hid++;
            }

            @pg_free_result($result);
        }

        // updates
        $sql = "DELETE FROM " . $tbprefix . "updates";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,type,item,member,comments,created FROM " . $prevtbprefix . "updates ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "updates_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "updates (type,item,member,comments,created) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // user_profiles
        $sql = "DELETE FROM " . $tbprefix . "user_profiles";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,department,location,language,date_format,time_format,workcalendar,enable_flextime,readonly,sysdef,locked,created,cuser,modified,muser FROM " . $prevtbprefix . "user_profiles ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "user_profiles_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "user_profiles (name,department,location,language,date_format,time_format,workcalendar,enable_flextime,readonly,sysdef,locked,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // user_profiles_detail
        $sql = "DELETE FROM " . $tbprefix . "user_profiles_detail";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT user_profile,function_name,access_level,granted FROM " . $prevtbprefix . "user_profiles_detail";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            while (($row = pg_fetch_row($result))) {
                $sql = "INSERT INTO " . $tbprefix . "user_profiles_detail (user_profile,function_name,access_level,granted) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }
            }

            @pg_free_result($result);
        }

        // work_hours
        $sql = "DELETE FROM " . $tbprefix . "work_hours";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,project,work,owner,date,hours,comments,billing,service,baseref,created,cuser,modified,muser FROM " . $prevtbprefix . "work_hours ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "work_hours_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "work_hours (project,work,owner,date,hours,comments,billing,service,baseref,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // workcalendar
        $sql = "DELETE FROM " . $tbprefix . "workcalendars";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,name,description,created,cuser,modified,muser FROM " . $prevtbprefix . "workcalendars ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "workcalendars_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "workcalendars (name,description,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // workcalendars_weekday
        $sql = "DELETE FROM " . $tbprefix . "workcalendars_weekday";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,workcalendar,year,day_type,hours,start_time,end_time,created,cuser,modified,muser FROM " . $prevtbprefix . "workcalendars_weekday ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "workcalendars_weekday_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "workcalendars_weekday (workcalendar,year,day_type,hours,start_time,end_time,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        // workcalendars_detail
        $sql = "DELETE FROM " . $tbprefix . "workcalendars_detail";

        if (!pg_query($pgsqln, $sql)) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqln));
            throw new Exception('', 1);
        }

        $sql = "SELECT id,workcalendar,year,date,type,hours,comments,created,cuser,modified,muser FROM " . $prevtbprefix . "workcalendars_detail ORDER BY id";

        if (!($result = pg_query($pgsqlp, $sql))) {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
            $block1->contentRow("", pg_last_notice($pgsqlp));
            throw new Exception('', 1);
        } else {
            $id = -1;

            while (($row = pg_fetch_row($result))) {
                if ($id != $row[0]) {
                    $id = $row[0];
                    $sql = "ALTER SEQUENCE " . $tbprefix . "workcalendars_detail_id_seq RESTART WITH " . $row[0];

                    if (!pg_query($pgsqln, $sql)) {
                        $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                        $block1->contentRow("", pg_last_notice($pgsqln));
                        throw new Exception('', 1);
                    }
                }

                unset($row[0]);
                $sql = "INSERT INTO " . $tbprefix . "workcalendars_detail (workcalendar,year,date,type,hours,comments,created,cuser,modified,muser) VALUES ('" . implode("','", $row) . "')";

                if (!pg_query($pgsqln, $sql)) {
                    $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $sql, "false", "");
                    $block1->contentRow("", pg_last_notice($pgsqln));
                    throw new Exception('', 1);
                }

                $id++;
            }

            @pg_free_result($result);
        }

        pg_query($pgsqln, 'COMMIT');  // Commits the current transaction
    } catch (Exception $e) {
        $error = true;

        if ($e->getCode() == '0') {
            $block1->contentRow("<img src=\"../themes/" . THEME . "/error.png\">", $e->getMessage(), "false", "");
        }

        @pg_query($pgsqln, 'ROLLBACK');// Rolls back current transaction
    }

    pg_close($pgsqlp);
    pg_close($pgsqln);
}
