<?php // $Revision: 1.5 $
/* vim: set expandtab ts=4 sw=4 sts=4: */

/**
 * $Id: exportcontacts.php,v 1.5 2013/01/29 08:46:19 luiswang Exp $
 *
 * Copyright (c) 2018 by the webTareas developers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

/**
 * Includes
 */
$checkSession = true;
require_once("../includes/library.php");

/**
 * Permission check
 */
if (!$SW_Permissions['Contact']['Read'][$profilSession]) {
    header("Location: ../general/home.php?msg=permissiondenied");
    exit;
}

/**
 * Variables init
 */
$id = AddSlashesNOD(isset($_GET['id'])?$_GET['id']:'0');
$ids = "'" . str_replace('**', "','", $id) . "'";

/**
 * Generate vCard address
 * 
 * @param array $adda
 *      the post office box;
 *      the extended address (e.g., apartment or suite number);
 *      the street address;
 *      the locality (e.g., city);
 *      the region (e.g., state or province);
 *      the postal code;
 *      the country name
 * @return string
 */
function vCardAddress($adda) {
    $addr = implode('', $adda);
    
    if ($addr != '') {
        $addr = implode(';', $adda);
    }
    
    return $addr;
}

/**
 * Load contacts
 */
$querymore = " WHERE con.id IN ($ids)";
$listContacts = new request();
$listContacts->openContacts($querymore);
$comptListContacts = count($listContacts->con_id);
$output = '';

for ($i=0; $i<$comptListContacts; $i++) {
    $output .= "BEGIN:VCARD\nVERSION:3.0";
    /**
     * A structured representation of the name of the person
     */
    // Family Names (also known as surnames), Given Names, Additional Names, Honorific Prefixes, and Honorific Suffixes
    $output .= "\nN:" . $listContacts->con_last_name[$i] . ';' . $listContacts->con_first_name[$i] . ';;' . $listContacts->con_salutation[$i] . ';';
    /**
     * To specify the formatted text corresponding to the name of the object the vCard represents
     */
    $fn = $listContacts->con_formatted_name[$i];
    $output .= "\nFN:". $fn;
    /**
     * To specify the position or job of the object the vCard represents
     */
    if ($listContacts->con_title[$i] != '') {
        $output .= "\nTITLE:" . $listContacts->con_title[$i];
    }
    /**
     * To specify the organizational name and units associated with the vCard
     */
    if ($listContacts->con_organization[$i] > 1) {
        $output .= "\nORG:" . $listContacts->con_org_name[$i];
        
        if ($listContacts->con_department[$i] != '') {
            $output .= ';' . $listContacts->con_department[$i];
        }
    }
    /**
     * To specify the electronic mail address for communication with the object the vCard represents
     */
    if ($listContacts->con_email_work[$i] != '') {
        $output .= "\nEMAIL;TYPE=WORK:" . $listContacts->con_email_work[$i];
    }
    
    if ($listContacts->con_email_home[$i] != '') {
        $output .= "\nEMAIL;TYPE=HOME:" . $listContacts->con_email_home[$i];
    }
    
    if ($listContacts->con_email_other[$i] != '') {
        $output .= "\nEMAIL:" . $listContacts->con_email_other[$i];
    }
    /**
     * To specify the telephone number for telephony communication with the object the vCard represents
     */
    if ($listContacts->con_phone_work[$i] != '') {
        $output .= "\nTEL;TYPE=WORK,VOICE:" . $listContacts->con_phone_work[$i];
    }
    
    if ($listContacts->con_phone_home[$i] != '') {
        $output .= "\nTEL;TYPE=HOME,VOICE:" . $listContacts->con_phone_home[$i];
    }
    
    if ($listContacts->con_mobile[$i] != '') {
        $output .= "\nTEL;TYPE=CELL:" . $listContacts->con_mobile[$i];
    }
    
    if ($listContacts->con_fax[$i] != '') {
        $output .= "\nTEL;TYPE=FAX:" . $listContacts->con_fax[$i];
    }
    
    if ($listContacts->con_phone_other[$i] != '') {
        $output .= "\nTEL;TYPE=VOICE:" . $listContacts->con_phone_other[$i];
    }
    /**
     * To specify the components of the delivery address for the vCard object
     */
    $adr = vCardAddress($adda = array('', '', $listContacts->con_address_street[$i], $listContacts->con_address_city[$i], $listContacts->con_address_state[$i], $listContacts->con_address_postalcode[$i], $listContacts->con_address_country[$i]));
    
    if ($adr != '') {
        $output .= "\nADR;TYPE=WORK:" . $adr;
    }
    
    $adr1 = vCardAddress($adda = array('', '', $listContacts->con_alt_address_street[$i], $listContacts->con_alt_address_city[$i], $listContacts->con_alt_address_state[$i], $listContacts->con_alt_address_postalcode[$i], $listContacts->con_alt_address_country[$i]));
    
    if ($adr1 != '') {
        $output .= "\nADR:" . $adr1;
    }
    /**
     * To specify the birth date of the object the vCard represents
     */
    if ($listContacts->con_birthdate[$i] != '0') {
        $output .= "\nBDAY:" . date('Ymd', $listContacts->con_birthdate[$i]);
    }
    /**
     * To specify supplemental information or a comment that is associated with the vCard
     */
    if ($listContacts->con_description[$i] != '') {
        $output .= "\nNOTE:" . $listContacts->con_description[$i];
    }
    
    $output .= "\nEND:VCARD\n\n";
}

if ($output != '') {
    $filename = 'contacts.vcf';
    Header("Content-Disposition: attachment; filename=\"$filename\"");
    Header("Content-Length: " . strlen($output));
    Header("Content-Type: text/x-vCard; charset=utf-8; name=\"$filename\"");
    echo $output;
}
