<?php // $Revision: 1.25 $
/* vim: set expandtab ts=4 sw=4 sts=4: */

/**
 * $Id: block.class.php,v 1.25 2013/01/29 08:46:24 luiswang Exp $
 *
 * Copyright (c) 2018 by the webTareas developers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */
class cardviewColumn {
    public $title = '';
    public $field_type = '0';
    public $linked_table = '';
    public $table_name = '';
    public $field_name = '';
    public $tooltip = '';
}

class block {
    public $theme;
    public $pathImg;
    public $sortingFields;
    public $sortingOrders;
    public $sortingArrows;
    public $sortingStyles;
    public $sortingRef;
    public $sortingDefault;
    private $headingBlock;
    
    function block()
    {
        global $path_prefix;
        $this->theme = THEME;
        $this->pathImg = $path_prefix . "themes";
        $this->sortingFields = '';
        $this->sortingOrders = '';
        $this->sortingArrows = '';
        $this->sortingStyles = '';
        $this->sortingRef = '';
        $this->sortingDefault = '';
        $this->headingBlock = false;
    }

    /**
     * Print tooltips
     *
     * @param string $item Text printed in tooltip
     * @param boolean $plain Plain text
     * @param string $align
     * @return string
     * @access public
     */
    function printHelp($item, $plain = false, $align = 'right')
    {
        static $id = 1;
        $helpStr = (!$plain)?lang_help($item):$item;
        $rc = '<div id="dialog' . $id . '" title="' . addslashes(lang_strings('help')) . '" class="hide">' . addslashes($helpStr) . '</div><a onclick="$(\'#dialog' . $id . '\').dialog({position: {my: \'left center\', at: \'right center\', of: \'#help' . $id . '\' }, clickOutside: true, clickOutsideTrigger: \'#help' . $id . '\'});"><div id="help' . $id . '" class="solid-ball ' . $align . '">?</div></a>';
        $id++;
        return $rc;
    }

    /**
     * Add a note
     *
     * @param string $content Text printed in note
     * @access public
     */
    function note($content)
    {
        echo '<div class="note">' . $content . '</div>';
    }

    /**
     * Print heading with view options
     *
     * @param string $title Text printed in heading
     * @access public
     */
    function openHeading($title)
    {
        echo "<div class=\"heading-group\"><div class=\"title\"><h1>" . $title . "</h1></div>";
        $this->headingBlock = true;
    }
    
    /**
     * End of heading with view options block
     *
     * @see block::openHeading()
     * @access public
     */
    function closeHeading()
    {
        echo "</div>";
        $this->headingBlock = false;
    }
    
    /**
     * Print standard heading
     *
     * @param string $title Text printed in heading
     * @access public
     */
    function heading($title)
    {
        echo "<div class=\"heading\"><h1>" . $title . "</h1></div>";
    }

    /**
     * Print toggle heading (with collapse/expand arrow)
     *
     * @param string $title Text printed in heading
     * @param boolean $collapse
     * @see block::closeToggle()
     * @access public
     */
    function headingToggle($title, $collapse = true)
    {
        static $id = 1;
        echo '<input class="toggle-box" id="toggle-box' . $id . '" type="checkbox"' . ($collapse?'':' checked') . '><label for="toggle-box' . $id . '">' . $title . '</label><div>';
        $id++;
    }

    /**
     * Close toggle block
     *
     * @see block::headingToggle()
     * @access public
     */
    function closeToggle()
    {
        echo '</div>';
    }

    function returnBorne($current)
    {
        global ${'borne'.$current};

        if (${'borne'.$current} == "") {
            $borneValue = "0";
        } else {
            $borneValue = ${'borne'.$current};
        }

        return $borneValue;
    }

    /**
     * Print page-per-page in bottom of list block
     *
     * @param string $current Borne number for concerned block
     * @param string $total Total bornes number
     * @param string $parameters Optional parameters to transmit between pages
     * @access public
     */
    function bornesFooter($current, $total, $parameters = '')
    {
        if ($this->rowsLimit < $this->recordsTotal) {
            $pageNumber = ceil($this->recordsTotal / $this->rowsLimit);
            $pageCurrent = ceil($this->borne / $this->rowsLimit);

            if ($pageNumber <= 5) {
                $pageStart = 0;
                $pageEnd = $pageNumber - 1;
            } else {
                $pageStart = $pageCurrent - 2;
                $pageEnd = $pageCurrent + 2;

                if ($pageStart < 0) {
                    $pageStart = 0;
                    $pageEnd = 4;
                } else if ($pageEnd >= $pageNumber) {
                    $pageStart = $pageNumber - 5;
                    $pageEnd = $pageNumber - 1;
                }
            }

            echo '<div class="footerCell"><ul><li>';

            if ($pageCurrent == 0) {
                echo '<div class="pageNumber-gray">&laquo;</div>';
            } else {
                echo '<a href="' . $_SERVER['PHP_SELF'] . '?' . $parameters;

                for ($k=1; $k<=$total; $k++) {
                    global ${'borne'.$k};

                    if ($k != $current) {
                        echo "&amp;borne$k=" . ${'borne'.$k};
                    } else if ($k == $current) {
                        echo "&amp;borne$k=0";
                    }
                }

                echo '#' . $this->form . 'Anchor"><div class="pageNumber">&laquo;</div></a>';
            }

            for ($i=$pageStart, $rowOffset=($this->rowsLimit * $pageStart); $i<=$pageEnd; $i++, $rowOffset+=($this->rowsLimit)) {
                echo '</li><li>';

                if ($this->borne == $rowOffset) {
                    echo '<div class="pageNumber-highlight">' . ($i+1) . '</div>';
                } else {
                    echo '<a href="' . $_SERVER['PHP_SELF'] . '?' . $parameters;

                    for ($k=1; $k<=$total; $k++) {
                        global ${'borne'.$k};

                        if ($k != $current) {
                            echo "&amp;borne$k=" . ${'borne'.$k};
                        } else if ($k == $current) {
                            echo "&amp;borne$k=$rowOffset";
                        }
                    }

                    echo '#' . $this->form . 'Anchor"><div class="pageNumber">' . ($i+1) . '</div></a>';
                }
            }

            echo '</li><li>';

            if ($pageCurrent == ($pageNumber -1)) {
                echo '<div class="pageNumber-gray">&raquo;</div>';
            } else {
                echo '<a href="' . $_SERVER['PHP_SELF'] . '?' . $parameters;

                for ($k=1; $k<=$total; $k++) {
                    global ${'borne'.$k};

                    if ($k != $current) {
                        echo "&amp;borne$k=" . ${'borne'.$k};
                    } else if ($k == $current) {
                        echo "&amp;borne$k=" . (($pageNumber -1) * $this->rowsLimit);
                    }
                }

                echo '#' . $this->form . 'Anchor"><div class="pageNumber">&raquo;</div></a>';
            }

            echo '</li></ul></div>';
        }
    }

    /**
     * Print Message table (deprecated)
     */
    function messageBox($msgType, $msgText) {}

    /**
     * Open icons table
     *
     * @see block::closePaletteIcon()
     * @see block::paletteIcon()
     * @see block::paletteScript()
     * @access public
     */
    function openPaletteIcon()
    {
        echo "<div class=\"icons\">";
    }

    /**
     * Close icons table
     *
     * @see block::openPaletteIcon()
     * @see block::paletteIcon()
     * @see block::paletteScript()
     * @access public
     */
    function closePaletteIcon()
    {
        echo "</div>";
    }

    /**
     * Open icons script
     *
     * @see block::openPaletteScript()
     * @access public
     */
    function openPaletteScript()
    {
        echo "<script type=\"text/JavaScript\">
document." . $this->form . "Form.buttons = new Array();";
    }

    /**
     * Close icons script
     *
     * @param integer $compt Total records
     * @param array $values First row
     * @see block::closePaletteScript()
     * @access public
     */
    function closePaletteScript($compt, $values)
    {
        echo "MM_updateButtons(document." . $this->form . "Form, 0);document." . $this->form . "Form.checkboxes = new Array();";
        for ($i = 0;$i < $compt;$i++) {
            echo "document." . $this->form . "Form.checkboxes[document." . $this->form . "Form.checkboxes.length] = new MMCheckbox('$values[$i]',document." . $this->form . "Form,'" . $this->form . "cb$values[$i]');";
        }
        echo "document." . $this->form . "Form.tt = '" . $this->form . "tt';
</script>";
    }

    /**
     * Define sorting to apply on a list block
     *
     * @param string $sortingRef Row reference in sorting table
     * @param string $sortingValue Row value in sorting table
     * @param string $sortingDefault Default sorting value
     * @param array $sortingFields Array with sorted fields on each column
     * @access public
     */
    function sorting($sortingRef, $sortingValue, $sortingDefault, $sortingFields)
    {
        $sortingOrders = array();
        $sortingArrows = array();
        $sortingStyles = array();
        $this->sortingRef = $sortingRef;
        $this->sortingValue = $sortingValue;
        $this->sortingDefault = $sortingDefault;
        $this->sortingFields = $sortingFields;

        if (isset($this->sortingValue) && $this->sortingValue != '') {
            $explode = explode(" ", $this->sortingValue);
        } else {
            $this->sortingValue = $this->sortingDefault;
            $explode = explode(" ", $this->sortingValue);
        }

        for ($i=0; $i<count($sortingFields); $i++) {
            if ($sortingFields[$i] == $explode[0] && $explode[1] == "DESC") {
                $sortingOrders[$i] = "ASC";
                $sortingArrows[$i] = "&#160;<img border=\"0\" src=\"$this->pathImg/$this->theme/icon_sort_za.gif\" alt=\"\" width=\"11\" height=\"11\">";
                $sortingStyles[$i] = "active";
            } else if ($sortingFields[$i] == $explode[0] && $explode[1] == "ASC") {
                $sortingOrders[$i] = "DESC";
                $sortingArrows[$i] = "&#160;<img border=\"0\" src=\"$this->pathImg/$this->theme/icon_sort_az.gif\" alt=\"\" width=\"11\" height=\"11\">";
                $sortingStyles[$i] = "active";
            } else {
                $sortingOrders[$i] = "ASC";
                $sortingArrows[$i] = "";
                $sortingStyles[$i] = "";
            }
        }

        $this->sortingOrders = $sortingOrders;
        $this->sortingArrows = $sortingArrows;
        $this->sortingStyles = $sortingStyles;
    }

    function openLoginForm($address)
    {
        global $path, $path_prefix;
        echo '<div class="login-page"><div class="form"><div class="logo"><img src="';

        if ($path != '') {
            echo $path;
        } else {
            echo $path_prefix . 'general/webTareasLogo.png';
        }

        echo '"><h1>webTareas</h1></div>';
        $this->openForm($address);
    }
    /**
     * Open a standard form
     *
     * @param string $address Action form value
     * @param string $onsubmit Script to be run when a form is submitted
     * @see block::closeFormResults()
     * @see block::closeForm()
     * @access public
     */
    function openForm($address, $onsubmit = 'return __default_checkformdata(this)')
    {
        echo "<a name=\"" . $this->form . "Anchor\"></a><form accept-charset=\"UNKNOWN\" method=\"POST\" action=\"$address\" name=\"" . $this->form . "Form\" enctype=\"multipart/form-data\"";

        if ($onsubmit != '') {
            echo " onsubmit=\"";
            echo "tinyMCE.triggerSave();";
            echo "$onsubmit\"";
        }

        echo "><noscript><input type=\"hidden\" name=\"js_disabled\" value=\"1\"></noscript>";
    }

    /**
     * Close a form used with a list block
     *
     * @access public
     */
    function closeFormResults()
    {
        echo "<input name=\"sor_cible\" type=\"HIDDEN\" value=\"$this->sortingRef\"><input name=\"sor_champs\" type=\"HIDDEN\" value=\"\"><input name=\"sor_ordre\" type=\"HIDDEN\" value=\"\">
</form>\n\n";
    }

    /**
     * Define column labels in a list block
     *
     * @param array $labels Array with labels strings
     * @param string $published Show/hide a published column
     * @param string $sorting Disable sorting
     * @param mixed $sortingOff Array with label number (from $labels) and order (ASC/DESC)
     * @param boolean $hideCB hide checkbox
     * @access public
     */
    function labels($labels, $published, $sorting = "true", $sortingOff = "", $hideCB = false)
    {
        global $strings, $sitePublish;
        $sortingFields = $this->sortingFields;
        $sortingOrders = $this->sortingOrders;
        $sortingArrows = $this->sortingArrows;
        $sortingStyles = $this->sortingStyles;
        $this->dataTitle = array();
        echo '<thead><tr>';

        if ($this->columnCheckbox) {
            if ($hideCB) {
                echo '<th>&nbsp;</th>';
            } else {
                echo "<th><a data-checkbox=\"yes\" href=\"javascript:MM_toggleSelectedItems(document." . $this->form . "Form,'$this->theme')\"><img border=\"0\" src=\"$this->pathImg/$this->theme/checkbox_off.png\" alt=\"\" vspace=\"3\" hspace=\"3\"></a></th>";
            }
        }

        if ($sitePublish != "true" && $published == "true") {
            $comptLabels = count($labels) - 1;
        } else {
            $comptLabels = count($labels);
        }

        for ($i=0; $i<$comptLabels; $i++) {
            if ($sorting == "true") {
                $apos = strpos($labels[$i], "<a href=");

                if ($apos === false) {
                    echo "<th class=\"" . $sortingStyles[$i] . "\"><a href=\"javascript:document." . $this->form . "Form.sor_cible.value='" . $this->sortingRef . "';document." . $this->form . "Form.sor_champs.value='" . $sortingFields[$i] . "';document." . $this->form . "Form.sor_ordre.value='" . $sortingOrders[$i] . "';document." . $this->form . "Form.submit();\" onMouseOver=\"javascript:window.status='" . lang_strings("sort_by") . " " . addslashes($labels[$i]) . "'; return true;\" onMouseOut=\"javascript:window.status=''; return true\">" . $labels[$i] . $sortingArrows[$i] . "</a></th>";
                    $this->dataTitle[] = $labels[$i];
                } else {
                    $label1 = substr($labels[$i], 0, $apos);
                    $label2 = substr($labels[$i], $apos);
                    echo "<th class=\"" . $sortingStyles[$i] . "\"><a href=\"javascript:document." . $this->form . "Form.sor_cible.value='" . $this->sortingRef . "';document." . $this->form . "Form.sor_champs.value='" . $sortingFields[$i] . "';document." . $this->form . "Form.sor_ordre.value='" . $sortingOrders[$i] . "';document." . $this->form . "Form.submit();\" onMouseOver=\"javascript:window.status='" . $strings["sort_by"] . " " . addslashes($label1) . "'; return true;\" onMouseOut=\"javascript:window.status=''; return true\">" . $label1 . $sortingArrows[$i] . "</a>$label2</th>";
                    $this->dataTitle[] = $label1;
                }
            } else {
                if (is_array($sortingOff)) {
                    if ($sortingOff[1] == "ASC") {
                        $sortingArrow = "&#160;<img border=\"0\" src=\"" . $this->pathImg . "/" . $this->theme . "/icon_sort_az.gif\" alt=\"\" width=\"11\" height=\"11\">";
                    } else if ($sortingOff[1] == "DESC") {
                        $sortingArrow = "&#160;<img border=\"0\" src=\"" . $this->pathImg . "/" . $this->theme . "/icon_sort_za.gif\" alt=\"\" width=\"11\" height=\"11\">";
                    }

                    if ($i == $sortingOff[0]) {
                        echo '<th class="active">' . $labels[$i] . $sortingArrow . "</th>";
                    } else {
                        echo '<th>' . $labels[$i] . "</th>";
                    }
                } else {
                    echo '<th>' . $labels[$i] . "</th>";
                }

                $this->dataTitle[] = $labels[$i];
            }
        }

        echo '</tr></thead><tbody>';
    }

    /**
     * Start a block to display results
     *
     * @param boolean $checkbox display checkbox
     * @access public
     */
    function openResults($checkbox = true)
    {
        $this->columnCheckbox = $checkbox;
        echo '<div class="listing"><table>';
    }

    /**
     * End of results block
     */
    function closeResults()
    {
        echo '</tbody></table></div>';
    }

    function noresults()
    {
        echo '<div class="clear"><p>' . lang_strings("no_items") . '</p></div>';
    }

    /**
     * Print an icon (html)
     *
     * @global array $strings
     * @param integer $num Icon number
     * @param string $type Icon name (used in graphic file name)
     * @param string $text Text used in info-tip
     * @see block::openPaletteIcon()
     * @access public
     */
    function paletteIcon($num, $type, $text)
    {
        global $strings;
        $apos = strpos($text, '<a href=');

        if ($apos === false) {
            if ($type === 'search') {   // search icon
                $text = $strings['search_for'] . ' ' . tableNameString($text);
            }

            echo "<div><div class=\"commandBtn\"><a href=\"javascript:var b = MM_getButtonWithName(document." . $this->form . "Form, '" . $this->form . "$num'); if (b) b.click();\" onMouseOver=\"var over = MM_getButtonWithName(document." . $this->form . "Form, '" . $this->form . "$num'); if (over) over.over(); return true; \" onMouseOut=\"var out = MM_getButtonWithName(document." . $this->form . "Form, '" . $this->form . "$num'); if (out) out.out(); return true; \"><img title=\"$text\" name=\"" . $this->form . "$num\" src=\"$this->pathImg/$this->theme/btn_" . $type . "_norm.png\" alt=\"$text\"></a></div></div>\n";
        } else {
            echo "<div><div class=\"commandBtn\"><a href=\"javascript:var b = MM_getButtonWithName(document." . $this->form . "Form, '" . $this->form . "$num'); if (b) b.click();\" onMouseOver=\"var over = MM_getButtonWithName(document." . $this->form . "Form, '" . $this->form . "$num'); if (over) over.over(); return true; \" onMouseOut=\"var out = MM_getButtonWithName(document." . $this->form . "Form, '" . $this->form . "$num'); if (out) out.out(); return true; \"><img title=\"" . substr($text, 0, $apos) . "\" name=\"" . $this->form . "$num\" src=\"$this->pathImg/$this->theme/btn_" . $type . "_norm.png\" alt=\"" . substr($text, 0, $apos) . "\"></a>" . substr($text, $apos) . "</div></div>\n";
        }
    }

    /**
     * Print an icon (JavaScript)
     *
     * @global array $strings
     * @param integer $num Icon number
     * @param string $type Icon name (used in graphic file name)
     * @param string $link action url/search text used in roll-over layer
     * @param string $options JavaScript options enableOnNoSelection, enableOnSingleSelection, enableOnMultipleSelection
     * @param string $text Text used in roll-over layer
     * @see block::openPaletteIcon()
     * @access public
     */
    function paletteScript($num, $type, $link, $options = '', $text = '')
    {
        global $strings;

        if ($type === 'search') {   // search icon
            $text = $strings['search_for'] . ' ' . tableNameString($link);
            $link = 'javascript:var x=document.seaForm.searchonly;x.value="' . $link . '";var y=document.seaForm.searchfor;y.placeholder="' . $text . '";y.focus();';
            $options = 'true,true,true';
        }

        $text1 = HtmlEntitiesNOD($text);
        echo "document." . $this->form . "Form.buttons[document." . $this->form . "Form.buttons.length] = new MMCommandButton('" . $this->form . "$num',document." . $this->form . "Form,'" . $link . "','$this->pathImg/$this->theme/btn_" . $type . "_norm.png','$this->pathImg/$this->theme/btn_" . $type . "_over.png','$this->pathImg/$this->theme/btn_" . $type . "_down.png','$this->pathImg/$this->theme/btn_" . $type . "_dim.png',$options,'','" . $text1 . "',false,'');\n";
    }

    /**
     * Start a block to display sheet/form
     *
     * @param boolean $multiColumns
     * @see block::contentRow()
     * @access public
     */
    function openContent($multiColumns = false)
    {
        if ($multiColumns) {
            echo '<div class="content multi-columns">';
        } else {
            echo '<div class="content">';
        }
    }

    /**
     * Print a line in sheet/form
     *
     * @param string $left Text in left cell
     * @param string $right Text in right cell
     * @access public
     */
    function contentRow($left, $right = '__emPty_vAlue__')
    {
        if ($right === '__emPty_vAlue__') {
            echo '<div class="contentRow"><div class="contentValue">' . $left . '</div></div>';
        } else if ($left != '') {
            echo '<div class="contentRow"><div class="leftValue">' . $left . '</div><div class="rightValue">' . $right . '</div></div>';
        } else {
            echo '<div class="contentRow"><div class="leftValue">&nbsp;</div><div class="rightValue">' . $right . '</div></div>';
        }
    }

    /**
     * Start a result row
     *
     * @param string $ref row id
     * @param boolean $hideCB hide checkbox
     */
    function openRow($ref = '', $hideCB = false)
    {
        echo '<tr>';

        if ($this->columnCheckbox) {    // Display a checkbox in row
            if ($hideCB) {
                echo '<td data-title="">&nbsp;</td>';
            } else {
                echo "<td data-title=\"\"><a data-checkbox=\"yes\" href=\"javascript:MM_toggleItem(document." . $this->form . "Form, '" . $ref . "', '" . $this->form . "cb" . $ref . "','$this->theme')\"><img name=\"" . $this->form . "cb" . $ref . "\" border=\"0\" src=\"$this->pathImg/$this->theme/checkbox_off.png\" alt=\"\" vspace=\"3\"></a></td>";
            }
        }

        $this->columnIdx = 0;
    }

    function cellRow($content)
    {
        echo '<td data-title="' . (isset($this->dataTitle[$this->columnIdx])?$this->dataTitle[$this->columnIdx]:'') . '">' . $content . '</td>';
        $this->columnIdx++;
    }

    function closeRow()
    {
        echo '</tr>';
    }

    function contentTitle($title)
    {
        echo '<div class="contentTitle">' . $title . '</div>';
    }

    /**
     * End of sheet/form block
     *
     * @see block::contentRow()
     * @access public
     */
    function closeContent()
    {
        echo '</div><div class="content-bottom"></div>';
    }

    function closeLoginForm()
    {
        global $version;
        $this->closeForm();
        echo '<p id="footer">Powered by webTareas v' . $version . '</p></div></div>';
    }

    function closeForm()
    {
        echo "</form>";
    }

    function openBreadcrumbs()
    {
        echo '<div id="wtBreadcrumbs"><ul>';
    }

    function itemBreadcrumbs($content)
    {
        if (preg_match('/<a\s+href=/i', $content)) {
            echo '<li>' . $content . '</li>';
        } else {
            echo '<li><a href="#">' . $content . '</a></li>';
        }
    }

    function closeBreadcrumbs()
    {
        echo '</ul></div>';
    }

    function buildLink($url, $label, $type)
    {
        global $idSession, $nameSession, $genderSession, $extSession;
        $apos = strpos($label, '<a href=');

        if ($type == 'in') {
            if ($apos === false) {
                return('<a href="' . $url . '">' . $label . '</a>');
            } else {
                return('<a href="' . $url . '">' . substr($label, 0, $apos) . '</a>' . substr($label, $apos));
            }
        } else if ($type == 'instrike') {
            if ($apos === false) {
                return('<a href="' . $url . '" class="instrike">' . $label . '</a>');
            } else {
                return('<a href="' . $url . '" class="instrike">' . substr($label, 0, $apos) . '</a>' . substr($label, $apos));
            }
        } else if ($type == 'inblank') {
            if ($apos === false) {
                return('<a href="' . $url . '" target="_blank">' . $label . '</a>');
            } else {
                return('<a href="' . $url . '" target="_blank">' . substr($label, 0, $apos) . '</a>' . substr($label, $apos));
            }
        } else if ($type == 'out') {
            if ($apos === false) {
                return('<a href="' . $url . '" target="_blank">' . $label . '</a>');
            } else {
                return('<a href="' . $url . '" target="_blank">' . substr($label, 0, $apos) . '</a>' . substr($label, $apos));
            }
        } else if ($type == 'icone') {
            if ($apos === false) {
                return('<a href="' . $url . '"><img src="../interface/icones/' . $label . '" border="0" alt=""></a>');
            } else {
                return('<a href="' . $url . '"><img src="../interface/icones/' . substr($label, 0, $apos) . '" border="0" alt=""></a>' . substr($label, $apos));
            }
        } else if ($type == 'powered') {
            if ($apos === false) {
                return('Powered by <a href="' . $url . '" target="_blank">' . $label . '</a>');
            } else {
                return('Powered by <a href="' . $url . '" target="_blank">' . substr($label, 0, $apos) . '</a>' . substr($label, $apos));
            }
        } else if ($type == 'mail') {
            if ($apos === false) {
                return('<a href="mailto:' . $url . '">' . $label . '</a>');
            } else {
                return('<a href="mailto:' . $url . '">' . substr($label, 0, $apos) . '</a>' . substr($label, $apos));
            }
        } else if ($type == 'message') {
            if ($url == $idSession) {
                $userFound = true;
                $name = $nameSession;
                $gender = $genderSession;
                $ext = $extSession;
                $active = enumBoolean::bTrue;
            } else {
                $querymore = " WHERE mem.id='$url'";
                $userDetail = new request();
                $userDetail->openMembers($querymore);

                if (count($userDetail->mem_id) > 0) {
                    $userFound = true;
                    $name = $userDetail->mem_name[0];
                    $gender = $userDetail->mem_gender[0];
                    $ext = $userDetail->mem_extension_picture[0];
                    $active = $userDetail->mem_active[0];
                } else {
                    $userFound = false;
                }
            }

            if ($userFound) {
                $avatar = getAvatar($url, $ext, $gender);

                if ($url == $idSession) {
                    return '<div class="avatar-name"><img src="' . $avatar . '"><label>' . $name . '</label></div>';
                } elseif ($active == enumBoolean::bFalse) {
                    return '<div class="avatar-name"><img class="inactive" src="' . $avatar . '"><label>' . $name . '</label></div>';
                } else {
                    return '<a data-avatar="yes" href="javascript:openChatTab(' . $url . ',\'P\');"><div class="avatar-name"><img src="' . $avatar . '"><label>' . $name . '</label></div></a>';
                }
            } else {
                return '&nbsp;';
            }
        }
    }

    function openViewOptions()
    {
        if ($this->headingBlock) {
            echo '<div class="options"><ul>';
        } else {
            echo '<div class="view-options"><ul>';
        }
    }

    function itemViewOptions($content)
    {
        echo '<li>'. $content . '</li>';
    }

    function closeViewOptions()
    {
        echo '</ul></div>';
    }

    function openCardView($opt)
    {
        global $strings;
        $this->cv_object = '';
        $this->cv_tblnam = '';
        $this->cv_columns = array();
        $this->cv_addurl = '';
        $this->cv_importurl = '';
        $this->cv_multi = true;
        $this->cv_search = false;
        $this->cv_extraparam = '';
        $this->cv_extensionactionpath = '';
        $this->cv_trimtrailemptycol = false;

        foreach ($opt as $k => $v) {
            switch ($k) {
                case enumCardviewOption::oObject:
                    $this->cv_object = $v;
                    break;
                case enumCardviewOption::oTableName:
                    $this->cv_tblnam = $v;
                    break;
                case enumCardviewOption::oColumns:
                    $this->cv_columns = $v;
                    break;
                case enumCardviewOption::oAddUrl:
                    $this->cv_addurl = $v;
                    break;
                case enumCardviewOption::oImportUrl:
                    $this->cv_importurl = $v;
                    break;
                case enumCardviewOption::oMultipleSelection:
                    $this->cv_multi = $v;
                    break;
                case enumCardviewOption::oSearch:
                    $this->cv_search = $v;
                    break;
                case enumCardviewOption::oExtraParameters:
                    $this->cv_extraparam = $v;
                    break;
                case enumCardviewOption::oExtensionActionPath:
                    $this->cv_extensionactionpath = $v;
                    break;
                case enumCardviewOption::oTrimTrailEmptyCol:
                    $this->cv_trimtrailemptycol = $v;
                    break;
            }
        }

        $this->sorting_index = 0;
        $this->row_count = 0;
        $sortingItems = array();

        if (is_array($this->sortingFields)) {
            foreach ($this->cv_columns as $k => $column) {
                if (isset($this->sortingFields[$k]) && ($this->sortingFields[$k] != 'none')) {
                    $sortingItems[] = '<li><a href="#" data-cible="' . $this->sortingRef . '" data-champs="' . $this->sortingFields[$k] . '" data-ordre="' . $this->sortingOrders[$k] . '"' . (($column->tooltip=='')?'':' title="' . $column->tooltip . '"') . '>' . $column->title . $this->sortingArrows[$k] . '</a></li>';

                    if ($this->sorting_index ==0 && $this->sortingArrows[$k] != '') {
                        $this->sorting_index = $k;
                    }
                }
            }
        }

        if ($this->cv_addurl !== '') {
            $sortingItems[] = '<li><a href="' . $this->cv_addurl . '" title="' . $strings['add'] . '">+</a></li>';
        }

        if ($this->cv_importurl !== '') {
            $sortingItems[] = '<li><a href="' . $this->cv_importurl . '" title="' . $strings['import'] . '">&darr;</a></li>';
        }

        if ($this->cv_search) {
            $text = $strings['search_for'] . ' ' . tableNameString($this->cv_tblnam);
            $link = "javascript:var x=document.seaForm.searchonly;x.value='" . $this->cv_tblnam . "';var y=document.seaForm.searchfor;y.placeholder='" . $text . "';y.focus();";
            $sortingItems[] = '<li><a href="' . $link . '" title="' . $strings['search'] . '">?</a></li>';
        }

        if (count($sortingItems) > 0) {
            echo '<ul class="cardview-sorting">';
            echo implode('', $sortingItems);
            echo '</ul>';
        }

        echo '<ul class="cardview" data-object="' . $this->cv_object . '" data-tblnam="' . $this->cv_tblnam . '" data-extra="' . $this->cv_extraparam . '" data-extpath="' . urlencode($this->cv_extensionactionpath) . '">';
    }

    function itemCardView($id, $content) {
        echo '<li data-id="' . $id . '">';

        if ($this->cv_multi) {
            echo '<div class="cardview-select"><input type="checkbox"></div>';
        }

        echo '<ul class="cardview-item">';

        if ($this->cv_trimtrailemptycol) {
            for ($k=0; $k<count($content); $k++) {
                $column = $this->cv_columns[$k];
                echo '<li title="' . (isset($column->title)?$column->title:'') . '"' . (($this->sorting_index == $k)?' data-sort="X"':'') . '>';
                echo setupFieldHTML($this, $content[$k], $column->field_type, $column->linked_table, $column->table_name, $column->field_name);
                echo '</li>';
            }
        } else {
            foreach ($this->cv_columns as $k => $column) {
                echo '<li title="' . $column->title . '"' . (($this->sorting_index == $k)?' data-sort="X"':'') . '>';

                if (is_array($content[$k])) {
                    for ($l=0; $l<count($content[$k]); $l++) {
                        if ($l>0) echo "&nbsp;&nbsp;";
                        echo setupFieldHTML($this, $content[$k][$l], $column->field_type, $column->linked_table, $column->table_name, $column->field_name);
                    }
                } else {
                    echo setupFieldHTML($this, $content[$k], $column->field_type, $column->linked_table, $column->table_name, $column->field_name);
                }

                echo '</li>';
            }
        }

        echo '</ul></li>';
        $this->row_count++;
    }

    function closeCardView() {
        if ($this->row_count == 0) {
            echo '<li><p>' . lang_strings("no_items") . '</p></li>';
        }

        echo '</ul>';
    }
}
