<?php // $Revision: 1.3 $
/* vim: set expandtab ts=4 sw=4 sts=4: */

/**
 * $Id: massdownload.php,v 1.3 2013/02/01 02:25:47 luiswang Exp $
 *
 * Copyright (c) 2018 by the webTareas developers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

$datasec = array(); // array to store compressed data
$ctrl_dir = array(); // central directory
$eof_ctrl_dir = "\x50\x4b\x05\x06\x00\x00\x00\x00"; //end of Central directory record
$old_offset = 0;

/**
 * Convert name to client character encoding
 *
 * @param string $name
 * @return string
 */
function name_iconv($name)
{
    global $lang;

    if (WEBTAREAS_USR_OS == 'Win') {
        switch ($lang) {
            case 'zh-tw':
                return iconv('UTF-8', 'BIG5', $name);
            case 'zh':
                return iconv('UTF-8', 'GB2312', $name);
            default:
                return $name;
        }
    } else {
        return $name;
    }
}

/**
 * adds "directory" to archive
 *
 * @param string $name name of directory
 * @access public
 */
function add_dir($name)
{
    global $datasec, $ctrl_dir, $eof_ctrl_dir, $old_offset;
    $name = str_replace("\\", "/", $name);
    $name = name_iconv($name);

    $fr = "\x50\x4b\x03\x04";
    $fr .= "\x0a\x00";    // ver needed to extract
    $fr .= "\x00\x00";    // gen purpose bit flag
    $fr .= "\x00\x00";    // compression method
    $fr .= "\x00\x00\x00\x00"; // last mod time and date

    $fr .= pack("V",0); // crc32
    $fr .= pack("V",0); //compressed filesize
    $fr .= pack("V",0); //uncompressed filesize
    $fr .= pack("v", strlen($name) ); //length of pathname
    $fr .= pack("v", 0 ); //extra field length
    $fr .= $name;
    // end of "local file header" segment

    // "data descriptor" segment (optional but necessary if archive is not served as file)
    $fr .= pack("V",$crc); //crc32
    $fr .= pack("V",$c_len); //compressed filesize
    $fr .= pack("V",$unc_len); //uncompressed filesize

    // add this entry to array
    $datasec[] = $fr;

    $new_offset = strlen(implode("", $datasec));

    // now add to central record
    $cdrec = "\x50\x4b\x01\x02";
    $cdrec .="\x00\x00";    // version made by
    $cdrec .="\x0a\x00";    // version needed to extract
    $cdrec .="\x00\x00";    // gen purpose bit flag
    $cdrec .="\x00\x00";    // compression method
    $cdrec .="\x00\x00\x00\x00"; // last mod time & date
    $cdrec .= pack("V",0); // crc32
    $cdrec .= pack("V",0); //compressed filesize
    $cdrec .= pack("V",0); //uncompressed filesize
    $cdrec .= pack("v", strlen($name) ); //length of filename
    $cdrec .= pack("v", 0 ); //extra field length
    $cdrec .= pack("v", 0 ); //file comment length
    $cdrec .= pack("v", 0 ); //disk number start
    $cdrec .= pack("v", 0 ); //internal file attributes
    $ext = "\x00\x00\x10\x00";
    $ext = "\xff\xff\xff\xff";
    $cdrec .= pack("V", 16 ); //external file attributes  - 'directory' bit set

    $cdrec .= pack("V", $old_offset ); //relative offset of local header
    $old_offset = $new_offset;

    $cdrec .= $name;
    // save to array
    $ctrl_dir[] = $cdrec;
}

/**
 * adds "file" to archive
 *
 * @param binary $data file contents
 * @param string $name name of file in archive
 * @access public
 */
function add_file($data, $name)
{
    global $datasec, $ctrl_dir, $eof_ctrl_dir, $old_offset;
    $name = str_replace("\\", "/", $name);
    $name = name_iconv($name);

    $fr = "\x50\x4b\x03\x04";
    $fr .= "\x14\x00";    // ver needed to extract
    $fr .= "\x00\x00";    // gen purpose bit flag
    $fr .= "\x08\x00";    // compression method
    $fr .= "\x00\x00\x00\x00"; // last mod time and date

    $unc_len = strlen($data);
    $crc = crc32($data);
    $zdata = gzcompress($data);
    $zdata = substr( substr($zdata, 0, strlen($zdata) - 4), 2); // fix crc bug
    $c_len = strlen($zdata);
    $fr .= pack("V",$crc); // crc32
    $fr .= pack("V",$c_len); //compressed filesize
    $fr .= pack("V",$unc_len); //uncompressed filesize
    $fr .= pack("v", strlen($name) ); //length of filename
    $fr .= pack("v", 0 ); //extra field length
    $fr .= $name;
    // end of "local file header" segment

    // "file data" segment
    $fr .= $zdata;

    // "data descriptor" segment (optional but necessary if archive is not served as file)
    $fr .= pack("V",$crc); //crc32
    $fr .= pack("V",$c_len); //compressed filesize
    $fr .= pack("V",$unc_len); //uncompressed filesize

    // add this entry to array
    $datasec[] = $fr;

    $new_offset = strlen(implode("", $datasec));

    // now add to central directory record
    $cdrec = "\x50\x4b\x01\x02";
    $cdrec .="\x00\x00";    // version made by
    $cdrec .="\x14\x00";    // version needed to extract
    $cdrec .="\x00\x00";    // gen purpose bit flag
    $cdrec .="\x08\x00";    // compression method
    $cdrec .="\x00\x00\x00\x00"; // last mod time & date
    $cdrec .= pack("V",$crc); // crc32
    $cdrec .= pack("V",$c_len); //compressed filesize
    $cdrec .= pack("V",$unc_len); //uncompressed filesize
    $cdrec .= pack("v", strlen($name) ); //length of filename
    $cdrec .= pack("v", 0 ); //extra field length
    $cdrec .= pack("v", 0 ); //file comment length
    $cdrec .= pack("v", 0 ); //disk number start
    $cdrec .= pack("v", 0 ); //internal file attributes
    $cdrec .= pack("V", 32 ); //external file attributes - 'archive' bit set

    $cdrec .= pack("V", $old_offset ); //relative offset of local header
    $old_offset = $new_offset;

    $cdrec .= $name;
    // save to central directory
    $ctrl_dir[] = $cdrec;
}

@set_time_limit(600);
$created_dirs = array();

for ($i=0; $i<count($names); $i++) {
    for ($j=0; $j<count($names[$i]); $j++) {
        if (!fileExists($fullpaths[$i][$j])) continue;

        if (!in_array($dirnames[$i][$j], $created_dirs)) {
            $created_dirs[] = $dirnames[$i][$j];
            add_dir($dirnames[$i][$j]);
        }

        if ($mkdirMethod == 'FTP') {
            $tmpfname = tempnam("../files", "FOO");
            $ftp = ftp_connect(FTPSERVER);
            ftp_login($ftp, FTPLOGIN, FTPPASSWORD);
            ftp_get($ftp, $tmpfname, $ftpRoot . "/" . $fullpaths[$i][$j], FTP_BINARY);
            ftp_quit($ftp);
            $path = $tmpfname;
        } else {
            $path = "../files/" . $fullpaths[$i][$j];
        }

        $filedata = implode('', file($path));
        add_file($filedata, $names[$i][$j]);

        if ($mkdirMethod == 'FTP') {
            unlink($path);
        }
    }
}

header("Content-type: application/octet-stream");
header("Content-disposition: attachment; filename=$zipfile.zip");
$data = implode("", $datasec);
$ctrldir = implode("", $ctrl_dir);
echo $data . $ctrldir . $eof_ctrl_dir .
    pack("v", sizeof($ctrl_dir)).     // total # of entries "on this disk"
    pack("v", sizeof($ctrl_dir)).     // total # of entries overall
    pack("V", strlen($ctrldir)).      // size of central dir
    pack("V", strlen($data)).         // offset to start of central dir
    "\x00\x00";                       // .zip file comment length

?>