//GLOBALS
var chatState = '0';    // chatbox status, 0 - not ready, 1 - ready, 2 - busy
var chatTo = '0';       // user id chat to / meeting id
var chatType = '';      // chat type, P - 1-1, M - Meeting
var chatFirst = '0';    // first loaded message id
var chatPrev = '0';     // last loaded message id
var hTimeout = null;
var chatLast = 0;       // last send message timestamp
var chatIdle = 300000;  // 5 min idle force close chatbox
var chatSess = 0;       // last update chat sessions timestamp
var chatOnline = '';    // online user list
var timeoutDelay = 1000;    // number of milliseconds to wait
// check new notifications & chat messages
function checkNewNotifications() {
    var prefix = $('#path-prefix').val();
    var now = $.now();

    if (chatState !== '0') {            // chat tab opened
        var diff = now - chatLast;

        if (chatLast > 0 && diff > chatIdle) {
            closeChatTab();             // long idle force close chatbox
        }
    }

    if (chatState === '1') {            // ready -> check new messages (1 ~ 5 sec)
        var diff = now - chatSess;
        var updSess = (chatSess > 0 && diff > 60000)?'1':'0';
        var interval = Math.floor((Math.random() * 1000) + timeoutDelay);
        timeoutDelay = ((timeoutDelay<5000)?(timeoutDelay+=1000):5000);
        hTimeout = setTimeout(function() {
            $.ajax({url: prefix+'includes/chattab_serv.php',
                type: 'post',
                data: {'action': 'load', 'chatTo': chatTo, 'chatType': chatType, 'chatPrev': chatPrev, 'updSess': updSess, 'chatOnline': chatOnline},
                success: function(json) {
                    if (json.content.length > 0) {
                        $('#wtChatDialog').append(json.content);
                        $('#wtChatDialog').animate({ scrollTop: $('#wtChatDialog').prop('scrollHeight')}, 1000);
                    }

                    if (chatOnline !== json.online) {
                        chatOnline = json.online;                   // online user list
                        $('#wtChatOnline>div').html(json.onlineHtml);   // chat online content
                    }

                    chatPrev = json.prev;               // last loaded message id

                    if (updSess === '1') {
                        chatSess = $.now();
                    }
                },
                complete: function() {
                    checkNewNotifications();
                }
            });
        }, interval);
    } else if (chatState === '2') {     // busy -> do nothing (1 ~ 5 sec)
        var interval = Math.floor((Math.random() * 1000) + timeoutDelay);
        timeoutDelay = ((timeoutDelay<5000)?(timeoutDelay+=1000):5000);
        hTimeout = setTimeout(function() {
            checkNewNotifications();
        }, interval);
    } else {                            // not ready -> check new notifications (1 sec ~ 5 min)
        var interval = Math.floor((Math.random() * 1000) + timeoutDelay);
        timeoutDelay = ((timeoutDelay<300000)?(timeoutDelay+=5000):300000);
        hTimeout = setTimeout(function() {
            $.ajax({url: prefix+'includes/chattab_serv.php',
                type: 'post',
                data: {'action': 'unread'},
                success: function(json) {
                    document.title = json.title;
                    $('#wtNotifications>span').html(json.unread);

                    if (json.unread > 0) {
                        if ($('#wtNotifications>span').hasClass('hide')) {
                            $('#wtNotifications>span').removeClass("hide");
                        }
                    } else {
                        if (!$('#wtNotifications>span').hasClass('hide')) {
                            $('#wtNotifications>span').addClass("hide");
                        }
                    }
                },
                complete: function() {
                    checkNewNotifications();
                }
            });
        }, interval);
    }
}
// open chat tab
function openChatTab(chat_to, chat_type) {
    if (hTimeout !== null) {
        clearTimeout(hTimeout);
    }

    var prefix = $('#path-prefix').val();
    $('#wtChatTitleText').html('');
    $('#wtChatDialog').html('');
    $('#wtChatText>textarea').val('');
    $('#wtChatText>textarea').focus();
    $('#wtChatOverlay').fadeIn(333);
    $('#wtChatBox').fadeIn(666);
    chatTo = chat_to;
    chatType = chat_type;
    chatLast = $.now();
    chatSess = $.now();
    $.ajax({url: prefix+'includes/chattab_serv.php',
        type: 'post',
        data: {'action': 'initial', 'chatTo': chatTo, 'chatType': chatType},
        success: function(json) {
            $('#wtChatTitleText').html(json.title);     // user name or meeting name
            $('#wtChatDialog').html(json.content);      // dialog content
            var height = $('#wtChatDialog').prop('scrollHeight');
            $('#wtChatDialog').scrollTop(height);
            chatFirst = json.firstId;                   // first loaded message id
            chatPrev = json.prev;                       // last loaded message id
            chatOnline = json.online;                   // online user list
            $('#wtChatOnline>div').html(json.onlineHtml);   // chat online content
            $('#wtChatText>textarea').focus();
            chatState = '1';
            timeoutDelay = 1000;
            checkNewNotifications();
        }
    });
}
// close chat tab
function closeChatTab() {
    var prefix = $('#path-prefix').val();
    $.ajax({url: prefix+'includes/chattab_serv.php',
        type: 'post',
        data: {'action': 'leave'}
    });
    chatState = '0';
    chatOnline = '';
    $('#wtChatBox').fadeOut(333);
    $('#wtChatOverlay').fadeOut(666);
}
// chat dialog scroll to very top
function scrollTopChatDialog() {
    var prefix = $('#path-prefix').val();
    chatState = '2';
    var height = $('#wtChatDialog').prop('scrollHeight');       // store original scroll height
    $.ajax({url: prefix+'includes/chattab_serv.php',
        type: 'post',
        data: {'action': 'history', 'chatTo': chatTo, 'chatType': chatType, 'chatFirst': chatFirst},
        success: function(json) {
            if (json.content.length > 0) {
                $('#wtChatDialog').prepend(json.content);
                var scrolltop = $('#wtChatDialog').prop('scrollHeight') - height;   // calculate new scroll top
                $('#wtChatDialog').scrollTop(scrolltop);
            }

            chatFirst = json.firstId;           // change first loaded message id
        },
        complete: function() {
            chatState = '1';
        }
    });
    chatLast = $.now();
}
// send a message
function sendChatMessage() {
    var prefix = $('#path-prefix').val();
    var msg = $('#wtChatText>textarea').val();
    msg = msg.trim();

    if (msg.length > 0) {
        timeoutDelay = 1000;
        chatState = '2';
        $.ajax({url: prefix+'includes/chattab_serv.php',
            type: 'post',
            data: {'action': 'sendMessage', 'chatTo': chatTo, 'chatType': chatType, 'message': msg},
            success: function(json) {
                $('#wtChatDialog').append(json.content);
                $('#wtChatDialog').animate({ scrollTop: $('#wtChatDialog').prop('scrollHeight')}, 1000);
                $('#wtChatText>textarea').val('');
                $('#wtChatText>textarea').focus();
            },
            complete: function() {
                chatState = '1';
            }
        });
    }
    chatLast = $.now();
}
// send photos
function sendChatPhotos() {
    var prefix = $('#path-prefix').val();
    var files = $('#chatPhotos')[0].files;

    if (files.length > 0) {
        timeoutDelay = 1000;
        chatState = '2';
        var data = new FormData();
        data.append('action', 'sendPhotos');
        data.append('chatTo', chatTo);
        data.append('chatType', chatType);

        for (var i=0; i<files.length; i++) {
            data.append("chatPhotos" + i, files[i]);
        }

        $.ajax({url: prefix+'includes/chattab_serv.php',
            type: 'post',
            contentType: false,
            processData: false,
            data: data,
            success: function(json) {
                $('#wtChatDialog').append(json.content);
                $('#wtChatDialog').animate({ scrollTop: $('#wtChatDialog').prop('scrollHeight')}, 1000);
                $('#wtChatText>textarea').focus();
            },
            complete: function() {
                chatState = '1';
            }
        });
    }

    chatLast = $.now();
}

$(document).ready(function() {
    var prefix = $('#path-prefix').val();
    // close chatbox
    $('#wtChatOverlay,#wtChatTitleClose').click(function() {
        closeChatTab();
    });
    // scroll chat dialog
    $('#wtChatDialog').scroll(function() {
        timeoutDelay = 1000;

        if ($('#wtChatDialog').scrollTop() == 0) {
            scrollTopChatDialog();
        }
    });
    // chat textarea key down
    $('#wtChatText>textarea').keydown(function(event) {
        if (event.which == 13) {
            // press Enter
            sendChatMessage();
            event.preventDefault();
        } else if (event.which == 27) {
            // press Esc
            closeChatTab();
            event.preventDefault();
        }
    });
    // open photos dialog
    $('#wtChatPhoto>img').click(function() {
        $('#chatPhotos').click();
    });
    // select photos and send
    $('#chatPhotos').change(function() {
        sendChatPhotos();
    });
    // save chat
    $('#wtChatSave>img').click(function() {
        window.open(prefix+'includes/download_chat.php?to=' + chatTo + '&type=' + chatType + '&first=' + chatFirst, '_blank');
    });
    // click new notifications icon
    $('#wtNotifications').click(function() {
        if ($('#wtSearch').length) {
            window.location.href = prefix+"general/newnotifications.php";
        } else {
            window.location.href = prefix+"service_site/newnotifications.php";
        }
    });
    // click user new messages box
    $('.user-unread-messages ul li').click(function() {
        var chatTo = $(this).find('input').val();
        openChatTab(chatTo, 'P');
        $(this).remove();
    });

    if ($('#wtNotifications').length) {
        // check new notifications
        checkNewNotifications();
    }
});