<?php // $Revision: 1.4 $
/* vim: set expandtab ts=4 sw=4 sts=4: */

/**
 * $Id: exportea.php,v 1.4 2013/01/18 09:50:30 luiswang Exp $
 *
 * Copyright (c) 2018 by the webTareas developers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

/**
 * Includes
 */
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
require_once('../includes/PhpSpreadsheet/vendor/autoload.php');
$checkSession = true;
$extractImportVariables = 'false';
require_once("../includes/library.php");

/**
 * Variables init
 */
if (!isset($_SESSION['TARGET']) || $_SESSION['TARGET'] != 'exportea') {
    exit;
}

// set up the equipment selection
$S_equ = $_SESSION['PARAMS']['S_EQUSEL'];
// handle date ranges
$S_date1 = $_SESSION['PARAMS']['S_DATE1'];
$S_date2 = $_SESSION['PARAMS']['S_DATE2'];
// end of configuration
// this is the filename the 'save as' file window will bring up by default
$filename = "equipment_allocation";
@set_time_limit(600);

/**
 * Prepare data
 */
// Query equipments that was selected
if ($S_equ === '') {
    $querymore0 = " WHERE equ.id='0' ORDER BY equ.equipment_id";
} else if (is_array($S_equ)) {
    $querymore0 = " WHERE equ.id IN ('" . implode("','", $S_equ) . "') ORDER BY equ.equipment_id";
} else {
    $querymore0 = " WHERE equ.id='$S_equ'";
}

$listEquipments = new request();
$listEquipments->openEquipments($querymore0);
$comptListEquipments = count($listEquipments->equ_id);

if ($comptListEquipments == '0') {
    exit;
}

// Create new Spreadsheet object
$objSpreadsheet = new Spreadsheet();
// Set document properties
$objSpreadsheet->getProperties()->setCreator("webTareas")
                             ->setLastModifiedBy("webTareas")
                             ->setTitle($strings["equipment_allocation"])
                             ->setSubject($strings["equipment_allocation"])
                             ->setDescription($strings["equipment_allocation"])
                             ->setKeywords($strings["equipment_allocation"]);
/**
 * Construct the headers
 */
$objSpreadsheet->setActiveSheetIndex(0)
            ->setCellValue('A1', $strings["equipment_allocation"])
            ->setCellValue('B1', $S_date1)
            ->setCellValue('C1', '~')
            ->setCellValue('D1', $S_date2);
/**
 * Construct the header row
 */
$objSpreadsheet->setActiveSheetIndex(0)
            ->setCellValue('A2', $strings["equipment"])
            ->setCellValue('B2', $strings["project"])
            ->setCellValue('C2', $strings["task"]);
$ts1 = date2Timestamp($S_date1);
$ts2 = date2Timestamp($S_date2);
$d1 = $ts1;
$col = 3;

while ($d1 <= $ts2) {
    $objSpreadsheet->setActiveSheetIndex(0)
                ->setCellValueByColumnAndRow($col++, 2, createDate($d1, 'date', false, true));
    $d1 = mktime(0, 0, 0, intval(date('n', $d1)), intval(date('j', $d1))+1, intval(date('Y', $d1)));
}

/*
 * Construct the content
 */
for ($i=0, $pRow=3; $i<$comptListEquipments; $i++, $pRow+=2) {
    $objSpreadsheet->setActiveSheetIndex(0)
                ->setCellValueByColumnAndRow(0, $pRow, $listEquipments->equ_equipment_id[$i]);
    if ($listEquipments->equ_status[$i] === enumEquipmentStatus::stBroken || $listEquipments->equ_status[$i] === enumEquipmentStatus::stSold) {
        $objSpreadsheet->setActiveSheetIndex(0)
                    ->setCellValueByColumnAndRow(1, $pRow, $equipmentStatusNames[$listEquipments->equ_status[$i]]);
    } else {
        // initial equipment status summary
        $d1 = $ts1;
        $equipmentStatus = array();

        while ($d1 <= $ts2) {
            $x = createDate($d1, 'date', false, true);
            $equipmentStatus[$x] = 0;
            $d1 = mktime(0, 0, 0, intval(date('n', $d1)), intval(date('j', $d1))+1, intval(date('Y', $d1)));
        }
            
        // Query to list only open or not started tasks (currently being worked on or scheduled to be worked on).
        $querymore1 = " AND tas.status IN ('" . enumStatus::stNotStarted . "','" . enumStatus::stOpen . "') AND tas.start_date<'" . mktime(0,0,0,intval(date('n',$ts2)),intval(date('j',$ts2))+1,intval(date('Y',$ts2))) . "' AND tas.due_date>='$ts1' AND res.resource='" . $listEquipments->equ_id[$i] . "' ORDER BY pro.name,tas.name";
        $listEquipmentResources = new request();
        $listEquipmentResources->openTaskEquipmentResource($querymore1);
        $comptListEquipmentResources = count($listEquipmentResources->res_id);
        
        for ($j=0; $j<$comptListEquipmentResources; $j++, $pRow++) {
            $objSpreadsheet->setActiveSheetIndex(0)
                        ->setCellValueByColumnAndRow(1, $pRow, $listEquipmentResources->res_pro_name[$j])   // project name
                        ->setCellValueByColumnAndRow(2, $pRow, $listEquipmentResources->res_tas_name[$j]);  // task name
            $d1 = $ts1;
            $col = 3;

            while ($d1 <= $ts2) {
                $x = createDate($d1, 'date', false, true);

                if ($d1 < $listEquipmentResources->res_tas_start_date[$j] || $d1 >= addDays($listEquipmentResources->res_tas_due_date[$j], 1)) {
                    $objSpreadsheet->setActiveSheetIndex(0)
                                ->setCellValueByColumnAndRow($col++, $pRow, 'A');     // available
                } else {
                    $objSpreadsheet->setActiveSheetIndex(0)
                                ->setCellValueByColumnAndRow($col++, $pRow, 'O');     // occupied
                    $equipmentStatus[$x] += 1;
                }

                $d1 = mktime(0, 0, 0, intval(date('n', $d1)), intval(date('j', $d1))+1, intval(date('Y', $d1)));
            }
        }
        
        $d2 = $ts1;
        $col = 3;

        while ($d2 <= $ts2) {
            $x = createDate($d2, 'date', false, true);

            if ($equipmentStatus[$x] === 0) {
                $objSpreadsheet->setActiveSheetIndex(0)
                            ->setCellValueByColumnAndRow($col++, $pRow, 'A');     // available
            } else if ($equipmentStatus[$x] === 1) {
                $objSpreadsheet->setActiveSheetIndex(0)
                            ->setCellValueByColumnAndRow($col++, $pRow, 'O');     // occupied
            } else {
                $objSpreadsheet->setActiveSheetIndex(0)
                            ->setCellValueByColumnAndRow($col++, $pRow, 'X');     // occupied by more than one task
            }

            $d2 = mktime(0, 0, 0, intval(date('n', $d2)), intval(date('j', $d2))+1, intval(date('Y', $d2)));
        }
    }
}

// Set active sheet index to the first sheet, so Excel opens this as the first sheet
$objSpreadsheet->setActiveSheetIndex(0);
// Redirect output to a client’s web browser
header('Content-Type: ' . $exportFormats[$expfmtSession]['Content-Type']);
$content_disp = (WEBTAREAS_USR_BROWSER_AGENT == 'IE') ? 'inline' : 'attachment';
header('Content-Disposition:  ' . $content_disp . '; filename="' . $filename . '.' . $exportFormats[$expfmtSession]['File-Extension'] . '"');
header('Pragma: no-cache');
header('Expires: 0');
$objWriter = IOFactory::createWriter($objSpreadsheet, $exportFormats[$expfmtSession]['PHPExcel-Writer-Type']);
$objWriter->save('php://output');
