<?php // $Revision: 1.4 $
/* vim: set expandtab ts=4 sw=4 sts=4: */

/**
 * $Id: exportflexibalance.php,v 1.4 2013/01/18 09:50:29 luiswang Exp $
 *
 * Copyright (c) 2018 by the webTareas developers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

/**
 * Includes
 */
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
require_once('../includes/PhpSpreadsheet/vendor/autoload.php');
$checkSession = true;
$extractImportVariables = 'false';
require_once("../includes/library.php");

/**
 * Permission check
 */
if (!$SW_Permissions['Report']['Read'][$profilSession]) {
    exit;
}

if (!$SW_Permissions['User']['Write'][$profilSession]) {
    exit;
}

// check if there are users with flexibalance enabled
$querymore = " WHERE mem.organization='1' AND mem.enable_flextime='" . enumBoolean::bTrue . "' ORDER BY mem.name";
$listMembers = new request();
$listMembers->openMembers($querymore);
$comptListMembers = count($listMembers->mem_id);

if ($comptListMembers == '0') {
    exit;
}

/**
 * Variables init
 */
if (!isset($_SESSION['TARGET']) || $_SESSION['TARGET'] != 'exportflexibalance') {
    exit;
}

// set up the department selection
$S_dep = $_SESSION['PARAMS']['S_DEPSEL'];
// set up the member selection
$S_mem = $_SESSION['PARAMS']['S_MEMSEL'];
// set up the date selection
$dtI = $_SESSION['PARAMS']['dt'];
$dt = date2Timestamp($dtI);
// end of configuration
// this is the filename the 'save as' file window will bring up by default
$filename = "flexibalance";
@set_time_limit(600);
$tmpquery = "SELECT T1.* FROM (
SELECT T0.id,T0.name,T0.department,((CASE WHEN T0.th IS NULL THEN 0 ELSE T0.th END) + (CASE WHEN T0.mh IS NULL THEN 0 ELSE T0.mh END) +
(CASE WHEN T0.wh IS NULL THEN 0 ELSE T0.wh END) - (CASE WHEN T0.wc IS NULL THEN 0 ELSE T0.wc END) + (CASE WHEN T0.ftah IS NULL THEN 0 ELSE T0.ftah END) + T0.initial_flexibalance) AS balance
FROM (SELECT mem.id,mem.name,dep.name AS department,
(SELECT SUM(tim.hours) FROM " . $tablePrefix . "tasks_time tim WHERE tim.owner=mem.id AND tim.date>=mem.begin_flextime AND tim.date<='$dt') AS th,
(SELECT SUM(mti.hours) FROM " . $tablePrefix . "meetings_time mti WHERE mti.owner=mem.id AND mti.date>=mem.begin_flextime AND mti.date<='$dt') AS mh,
(SELECT SUM(wkh.hours) FROM " . $tablePrefix . "work_hours wkh WHERE wkh.owner=mem.id AND wkh.date>=mem.begin_flextime AND wkh.date<='$dt') AS wh,
(SELECT SUM(wcd.hours) FROM " . $tablePrefix . "workcalendars_detail wcd WHERE wcd.workcalendar=mem.workcalendar AND wcd.date>=mem.begin_flextime AND wcd.date<='$dt') AS wc,
(SELECT SUM(fta.hours) FROM " . $tablePrefix . "flextime_adjustment fta WHERE fta.owner=mem.id AND fta.refdate<='$dt') AS ftah,
mem.initial_flexibalance
FROM " . $tablePrefix . "members mem LEFT OUTER JOIN " . $tablePrefix . "departments dep ON dep.id = mem.department WHERE mem.begin_flextime<='$dt' AND mem.organization='1' AND mem.enable_flextime='" . enumBoolean::bTrue . "'";

// a department selection was made
if ($S_dep != '') {
    $tmpquery .= " AND mem.department" . ((is_array($S_dep))?" IN ('" . implode("','", $S_dep) . "')":"='$S_dep'");
}
// a member selection was made
if ($S_mem == '') {
    $tmpquery .= " AND mem.id='0'";
} else {
    $tmpquery .= " AND mem.id" . ((is_array($S_mem))?" IN ('" . implode("','", $S_mem) . "')":"='$S_mem'");
}

$tmpquery .= ") T0) T1 ORDER BY T1.name";
$listFlexiBalance = new request();
$listFlexiBalance->connectClass();
$listFlexiBalance->query($tmpquery);
$listFlexiBalance->fb_id = array();

while ($listFlexiBalance->fetch()) {
    $listFlexiBalance->fb_id[] = ($row[0]);
    $listFlexiBalance->fb_name[] = HtmlEntityDecodeNOD($row[1]);
    $listFlexiBalance->fb_department[] = HtmlEntityDecodeNOD($row[2]);
    $hours0 = ($row[3]);
    $hours = (is_numeric($hours0)?round($hours0, $hourAccuracy):'0');
    $listFlexiBalance->fb_balance[] = $hours;
}

$listFlexiBalance->close();
$comptListFlexiBalance = count($listFlexiBalance->fb_id);

if ($comptListFlexiBalance == '0') {
    exit;
}

// Create new Spreadsheet object
$objSpreadsheet = new Spreadsheet();
// Set document properties
$objSpreadsheet->getProperties()->setCreator("webTareas")
                             ->setLastModifiedBy("webTareas")
                             ->setTitle($strings["flexibalance"])
                             ->setSubject($strings["flexibalance"])
                             ->setDescription($strings["flexibalance"])
                             ->setKeywords($strings["flexibalance"]);
/**
 * Construct the headers
 */
$objSpreadsheet->setActiveSheetIndex(0)
            ->setCellValue('A1', $strings["flexibalance"]);
/**
 * Construct the header row
 */
$objSpreadsheet->setActiveSheetIndex(0)
            ->setCellValue('A2', $strings["name"])
            ->setCellValue('B2', $strings["department"])
            ->setCellValue('C2', $strings["hours"]);
/*
 * Construct the content
 */
for ($i=0, $pRow=3; $i<$comptListFlexiBalance; $i++, $pRow++) {
    $objSpreadsheet->setActiveSheetIndex(0)
                ->setCellValueByColumnAndRow(0, $pRow, $listFlexiBalance->fb_name[$i])          // member name
                ->setCellValueByColumnAndRow(1, $pRow, $listFlexiBalance->fb_department[$i])    // department
                ->setCellValueByColumnAndRow(2, $pRow, NumberFormatNOD($listFlexiBalance->fb_balance[$i], $hourAccuracy, false));   // hours
}

// Set active sheet index to the first sheet, so Excel opens this as the first sheet
$objSpreadsheet->setActiveSheetIndex(0);
// Redirect output to a client’s web browser
header('Content-Type: ' . $exportFormats[$expfmtSession]['Content-Type']);
$content_disp = (WEBTAREAS_USR_BROWSER_AGENT == 'IE') ? 'inline' : 'attachment';
header('Content-Disposition:  ' . $content_disp . '; filename="' . $filename . '.' . $exportFormats[$expfmtSession]['File-Extension'] . '"');
header('Pragma: no-cache');
header('Expires: 0');
$objWriter = IOFactory::createWriter($objSpreadsheet, $exportFormats[$expfmtSession]['PHPExcel-Writer-Type']);
$objWriter->save('php://output');
