<?php // $Revision: 1.15 $
/* vim: set expandtab ts=4 sw=4 sts=4: */

/**
 * $Id: exportfta.php,v 1.15 2009/11/03 14:29:12 luiswang Exp $
 *
 * Copyright (c) 2018 by the webTareas developers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

/**
 * Includes
 */
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
require_once('../includes/PhpSpreadsheet/vendor/autoload.php');
$checkSession = true;
$extractImportVariables = 'false';
require_once("../includes/library.php");

/**
 * Permission check
 */
if (!$SW_Permissions['Report']['Read'][$profilSession]) {
    exit;
}

if (($enableTimesheet != 'true') || ($enableFlextime != 'true')) {
    exit;
}

// check if there are users with flextime enabled
$querymore1 = " WHERE mem.organization='1' AND mem.enable_flextime='" . enumBoolean::bTrue . "' ORDER BY mem.name";
$listMembers = new request();
$listMembers->openMembers($querymore1);
$comptListMembers = count($listMembers->mem_id);

if ($comptListMembers == '0') {
    exit;
}

/**
 * Variables init
 */
if (!isset($_SESSION['TARGET']) || $_SESSION['TARGET'] != 'exportfta') {
    exit;
}
// set up the department selection
$S_dep = $_SESSION['PARAMS']['S_DEPSEL'];
// set up the member selection
$S_mem = $_SESSION['PARAMS']['S_MEMSEL'];
// a date range was selected
$S_date1 = $_SESSION['PARAMS']['S_DATE1'];
$S_date2 = $_SESSION['PARAMS']['S_DATE2'];
// end of configuration
// this is the filename the 'save as' file window will bring up by default
$filename = "flextime_adjustments";
@set_time_limit(600);

/**
 * Load data
 */
// a member selection was made
if ($S_mem == '') {
    $querymore = " WHERE fta.owner='0'";
} else {
    $querymore = " WHERE fta.owner" . ((is_array($S_mem))?" IN ('" . implode("','", $S_mem) . "')":"='$S_mem'");
}
// a department selection was made
if ($S_dep != '') {
    $querymore .= " AND mem.department" . ((is_array($S_dep))?" IN ('" . implode("','", $S_dep) . "')":"='$S_dep'");
}
// a date range was selected
if ($S_date1 !== '') {
    $ts1 = date2Timestamp($S_date1);
    $querymore .= " AND fta.refdate>='" . mktime(0, 0, 0, date('n', $ts1), date('j', $ts1), date('Y', $ts1)) . "'";
}

if ($S_date2 !== '') {
    $ts2 = date2Timestamp($S_date2);
    $querymore .= " AND fta.refdate<'" . mktime(0, 0, 0, date('n', $ts2), intval(date('j', $ts2))+1, date('Y', $ts2)) . "'";
}

$listFlextimeAdjustment = new request();
$listFlextimeAdjustment->openFlextimeAdjustment($querymore);
$comptListFlextimeAdjustment = count($listFlextimeAdjustment->fta_id);

if ($comptListFlextimeAdjustment == 0) {
    exit;
}

// Create new Spreadsheet object
$objSpreadsheet = new Spreadsheet();
// Set document properties
$objSpreadsheet->getProperties()->setCreator("webTareas")
                             ->setLastModifiedBy("webTareas")
                             ->setTitle($strings["flextime_adjustments_report"])
                             ->setSubject($strings["flextime_adjustments_report"])
                             ->setDescription($strings["flextime_adjustments_report"])
                             ->setKeywords($strings["flextime_adjustments_report"]);
/**
 * Construct the headers
 */
$objSpreadsheet->setActiveSheetIndex(0)
            ->setCellValue('A1', $strings["flextime_adjustments_report"]);
/**
 * Construct the header row
 */
$objSpreadsheet->setActiveSheetIndex(0)
            ->setCellValue('A2', $strings["name"])
            ->setCellValue('B2', $strings["date"])
            ->setCellValue('C2', $strings["time"])
            ->setCellValue('D2', $strings["note"]);
/*
 * Construct the content
 */
for ($i=0, $pRow=3; $i<$comptListFlextimeAdjustment; $i++, $pRow++) {
    /**
     * The individual fields
     */
    $objSpreadsheet->setActiveSheetIndex(0)
                ->setCellValueByColumnAndRow(0, $pRow, $listFlextimeAdjustment->fta_mem_name[$i])   // member name
                ->setCellValueByColumnAndRow(1, $pRow, createDate($listFlextimeAdjustment->fta_refdate[$i], 'date', false, true))   // date
                ->setCellValueByColumnAndRow(2, $pRow, $listFlextimeAdjustment->fta_hours[$i])      // hours
                ->setCellValueByColumnAndRow(3, $pRow, $listFlextimeAdjustment->fta_comments[$i]);  // note
}

// Set active sheet index to the first sheet, so Excel opens this as the first sheet
$objSpreadsheet->setActiveSheetIndex(0);
// Create PHPExcel writer
$objWriter = IOFactory::createWriter($objSpreadsheet, $exportFormats[$expfmtSession]['PHPExcel-Writer-Type']);

if ($generate_mail_attach == 'Y') {
    $attfname = $filename . '.' . $exportFormats[$expfmtSession]['File-Extension'];
    $tmpfname = tempnam("../files", "FOO");
    $objWriter->save($tmpfname);
} else {
    // Redirect output to a client’s web browser
    header('Content-Type: ' . $exportFormats[$expfmtSession]['Content-Type']);
    $content_disp = (WEBTAREAS_USR_BROWSER_AGENT == 'IE') ? 'inline' : 'attachment';
    header('Content-Disposition:  ' . $content_disp . '; filename="' . $filename . '.' . $exportFormats[$expfmtSession]['File-Extension'] . '"');
    header('Pragma: no-cache');
    header('Expires: 0');
    $objWriter->save('php://output');
}
