<?php // $Revision: 1.4 $
/* vim: set expandtab ts=4 sw=4 sts=4: */

/**
 * $Id: exportmu.php,v 1.4 2013/01/18 09:50:30 luiswang Exp $
 *
 * Copyright (c) 2018 by the webTareas developers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

/**
 * Includes
 */
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
require_once('../includes/PhpSpreadsheet/vendor/autoload.php');
$checkSession = true;
$extractImportVariables = 'false';
require_once("../includes/library.php");

/**
 * Variables init
 */
if (!isset($_SESSION['TARGET']) || $_SESSION['TARGET'] != 'exportmu') {
    exit;
}

// set up the project selection
$S_prj = $_SESSION['PARAMS']['S_PRJSEL'];
// set up the material selection
$S_mat = $_SESSION['PARAMS']['S_MATSEL'];
// a date range was selected
$S_date1 = $_SESSION['PARAMS']['S_DATE1'];
$S_date2 = $_SESSION['PARAMS']['S_DATE2'];
// end of configuration
// this is the filename the 'save as' file window will bring up by default
$filename = "material_usage";
@set_time_limit(600);

/**
 * Prepare data
 */
$tmpquery = "SELECT iss.project,iss.material,iss.date,SUM(iss.quantity) AS quantity,mat.name,mat.unit,pro.name
FROM " . $tablePrefix . "material_issue iss
LEFT OUTER JOIN " . $tablePrefix . "materials mat ON mat.id=iss.material
LEFT OUTER JOIN " . $tablePrefix . "projects pro ON pro.id=iss.project
";
$conditions = array();
// a project selection was made
if ($S_prj != '') {
    $conditions[] = "iss.project" . ((is_array($S_prj))?" IN ('" . implode("','", $S_prj) . "')":"='$S_prj'");
}
// a material selection was made
if ($S_mat != '') {
    $conditions[] = "iss.material" . ((is_array($S_mat))?" IN ('" . implode("','", $S_mat) . "')":"='$S_mat'");
}
// a date range was selected

if ($S_date1 !== '') {
    $ts1 = date2Timestamp($S_date1);
    $conditions[] = "iss.date>='" . mktime(0, 0, 0, date('n', $ts1), date('j', $ts1), date('Y', $ts1)) . "'";
}

if ($S_date2 !== '') {
    $ts2 = date2Timestamp($S_date2);
    $conditions[] = "iss.date<'" . mktime(0, 0, 0, date('n', $ts2), intval(date('j', $ts2))+1, date('Y', $ts2)) . "'";
}

if (count($conditions) > 0) {
    $tmpquery .= " WHERE " . implode(" AND ", $conditions);
}

$tmpquery .= " GROUP BY iss.project,iss.material,iss.date,mat.name,mat.unit,pro.name ORDER BY pro.name,mat.name,iss.date";
$listMaterialIssue = new request();
$listMaterialIssue->connectClass();
$listMaterialIssue->query($tmpquery);
$listMaterialIssue->iss_project = array();

while ($listMaterialIssue->fetch()) {
    $i = 0;
    $listMaterialIssue->iss_project[] = ($row[$i++]);
    $listMaterialIssue->iss_material[] = ($row[$i++]);
    $listMaterialIssue->iss_date[] = ($row[$i++]);
    $val = ($row[$i++]);
    $qty = (is_numeric($val))?round($val, $quantityAccuracy):0;
    $listMaterialIssue->iss_quantity[] = $qty;
    $matn = ($row[$i++]);
    $listMaterialIssue->iss_mat_name_print[] = $strings['MAT__' . $matn];
    $listMaterialIssue->iss_mat_unit[] = HtmlEntityDecodeNOD($row[$i++]);
    $listMaterialIssue->iss_pro_name[] = HtmlEntityDecodeNOD($row[$i++]);
}

$listMaterialIssue->close();
$comptListMaterialIssue = count($listMaterialIssue->iss_project);

if ($comptListMaterialIssue == '0') {
    exit;
}

// Create new Spreadsheet object
$objSpreadsheet = new Spreadsheet();
// Set document properties
$objSpreadsheet->getProperties()->setCreator("webTareas")
                             ->setLastModifiedBy("webTareas")
                             ->setTitle($strings["material_usage"])
                             ->setSubject($strings["material_usage"])
                             ->setDescription($strings["material_usage"])
                             ->setKeywords($strings["material_usage"]);
/**
 * Construct the headers
 */
$objSpreadsheet->setActiveSheetIndex(0)
            ->setCellValue('A1', $strings["material_usage"]);
/**
 * Construct the header row
 */
$objSpreadsheet->setActiveSheetIndex(0)
            ->setCellValue('A2', $strings["project"])
            ->setCellValue('B2', $strings["material"])
            ->setCellValue('C2', $strings["date"])
            ->setCellValue('D2', $strings["quantity"])
            ->setCellValue('E2', $strings["unit"]);
/*
 * Construct the content
 */
for ($i=0, $pRow=3; $i<$comptListMaterialIssue; $i++, $pRow++) {
    $objSpreadsheet->setActiveSheetIndex(0)
                ->setCellValueByColumnAndRow(0, $pRow, $listMaterialIssue->iss_pro_name[$i])    // project name
                ->setCellValueByColumnAndRow(1, $pRow, $listMaterialIssue->iss_mat_name_print[$i])  // material name print
                ->setCellValueByColumnAndRow(2, $pRow, createDate($listMaterialIssue->iss_date[$i], 'date', false, true))   // date
                ->setCellValueByColumnAndRow(3, $pRow, $listMaterialIssue->iss_quantity[$i])    // quantity
                ->setCellValueByColumnAndRow(4, $pRow, $listMaterialIssue->iss_mat_unit[$i]);   // unit
}

// Set active sheet index to the first sheet, so Excel opens this as the first sheet
$objSpreadsheet->setActiveSheetIndex(0);
// Redirect output to a client’s web browser
header('Content-Type: ' . $exportFormats[$expfmtSession]['Content-Type']);
$content_disp = (WEBTAREAS_USR_BROWSER_AGENT == 'IE') ? 'inline' : 'attachment';
header('Content-Disposition:  ' . $content_disp . '; filename="' . $filename . '.' . $exportFormats[$expfmtSession]['File-Extension'] . '"');
header('Pragma: no-cache');
header('Expires: 0');
$objWriter = IOFactory::createWriter($objSpreadsheet, $exportFormats[$expfmtSession]['PHPExcel-Writer-Type']);
$objWriter->save('php://output');
